#include <iostream>
#include <fstream> 
#include <string>
#include "edg/workload/checkpointing/client/checkpointing.h"

using namespace std;
using namespace edg::workload::common;

USING_CHKPT_NAMESPACE; 

enum err {
  OK                =  0,              
  EmptyState        =  3,     
  SaveFailed        =  5,     
  GetFailed         =  7,      
  AppendFailed      = 11,  
  CheckFailed       = 13,   
  StepperFailed     = 17,  
  ConstructFailed   = 19, 
  WrongExcept       = 23,                
  UnexpectedExcept  = 29   
};
  
// We set some global values	
bool bval = true;
int ival = 23;
double dval = 3.14;
string sval = "This is a string";
string names[] = {"Bool", "Int", "Double", "String", "ListBool", "ListInt", "ListDouble", "ListString"};
ofstream logfile;

int check_save_value(JobState* js) {

  // We try to insert 4 differents values
  logfile << "Test the saveValue methods." << endl;
  logfile << "Save the pairs:\n" 
	  << "<" << names[0] << ", " << bval << ">\n"
	  << "<" << names[1] << ", " << ival << ">\n"
	  << "<" << names[2] << ", " << dval << ">\n"
	  << "<" << names[3] << ", " << sval << ">" << endl;
  logfile << "Check if saving the pairs: \n" 
	  << "< -this-is-not-a-valid-id, 42 >\n"
	  << "returns the 'CHKPT_SyntaxError'" << endl;
  try {
    
    if ( js->saveValue(names[0], bval) ||
	 js->saveValue(names[1], ival) ||
	 js->saveValue(names[2], dval) ||
	 js->saveValue(names[3], sval) ||
	 js->saveValue("-this-is-not-a-valid-id", 42) != CHKPT_SyntaxError)
      return SaveFailed;
  } catch ( ESException ) { 
    return EmptyState;
  } catch ( ChkptException ) {
    return UnexpectedExcept;
  };
  
  logfile << "\n OK, test is passed!!!" << endl;
  return OK;  
}

int check_get_value(JobState* js) {
  
  logfile << "Test the getTYPEDValue methods.\n" 
	  << "We retrieve only the simple values stored before..." << endl; 
  try { 
    if ( ( js->getIntValue("Int")[0] != ival ) ||
	 ( js->getBoolValue("Bool")[0] != bval ) ||
	 ( js->getDoubleValue("Double")[0] != dval ) ||
	 ( js->getStringValue("String")[0] != sval ) )
      return GetFailed;
  } catch ( ESException ) { 
    return EmptyState;
  } catch ( ChkptException ) {
    return UnexpectedExcept;
  };

  logfile << "... done!" << endl;

  logfile << "Test if the exception 'WTException' is correctly thrown..." << endl; 
  int count = 0;
  try {
    for (int i = 0; i < 4; i++) 
      try {
	js->getBoolValue(names[i])[0];
      } catch ( WTException ) {
	count++;
      } 
    if ( count != 3 ) return WrongExcept;
    else count = 0;
    for (int i = 0; i < 4; i++) 
      try {
	js->getIntValue(names[i])[0];
      } catch ( WTException ) {
	count++;
      } 
    if ( count != 3 ) return WrongExcept;
    else count = 0;
    for (int i = 0; i < 4; i++) 
      try {
	js->getDoubleValue(names[i])[0];
      } catch ( WTException ) {
	count++;
      } 
    if ( count != 3 ) return WrongExcept;
    else count = 0;
    for (int i = 0; i < 4; i++) 
      try {
	js->getStringValue(names[i])[0];
      } catch ( WTException ) {
	count++;
      } 
  } catch ( ChkptException ) {
    return UnexpectedExcept;
  };
  if ( count != 3 ) return WrongExcept;
  
  logfile << "... done!" << endl;
  
  logfile << "Test if the exception 'ULException' is correctly thrown..." << endl; 
  
  try {
    try { 
      js->getBoolValue("UndefinedLabel")[0];
    } catch ( ULException ) {}
    try { 
      js->getIntValue("UndefinedLabel")[0];
    } catch ( ULException ) {} 
    try { 
      js->getDoubleValue("UndefinedLabel")[0];
    } catch ( ULException ) {} 
    try { 
      js->getStringValue("UndefinedLabel")[0];
    } catch ( ULException ) {} 
  } catch ( ChkptException ) {
    return WrongExcept;
  };

  logfile << "... done!" << endl;

  logfile << "\n OK, test is passed!!!" << endl;
  return OK;  
}

int check_append_value(JobState* js) {
    
  int i;
  bool   vb[] = {true, false, true, false, true};
  int    vi[] = {1, 2, 3, 4, 5};
  double vd[] = {1.2, 2.3, 3.4, 4.5, 5.6};
  std::string vs[] = {"a", "b", "c", "d", "e"};

  logfile << "Test the appendValue methods." << endl;
  logfile << "Save the pairs:\n" << "<" << names[4] << ", {";  
  for (i = 0; i < 4; i++ ) 
    logfile << vb[i] << ", ";
  logfile << vb[i] << "}>\n" << "<" << names[5] << ", {"; 
  for (i = 0; i < 4; i++ ) 
    logfile << vi[i] << ", ";
  logfile << vi[i] << "}>\n" << "<" << names[6] << ", {";
  for (i = 0; i < 4; i++ ) 
    logfile << vd[i] << ", ";
  logfile << vd[i] << "}>\n" << "<" << names[7] << ", {";
  for (i = 0; i < 4; i++ ) 
    logfile << vs[i] << ", ";
  logfile << vs[i] << "}>" << endl;

  try {
    for ( i = 0; i < 5; i++ ) 
      if ( js->appendValue(names[4], vb[i] ) ||
	   js->appendValue(names[5], vi[i] ) ||
	   js->appendValue(names[6], vd[i] ) ||
	   js->appendValue(names[7], vs[i] ) )
	return AppendFailed;
  } catch ( ESException ) { 
    return EmptyState;
  } catch ( ChkptException ) {
    return UnexpectedExcept;
  };

  logfile << "... done!" << endl;
  
  logfile << "Test the return values CHKPT_SyntaxError"
	  << " saving the pairs < -this-is-not-a-valid-id, 42 >" << endl;

  try {  
    if ( js->appendValue("-this-is-not-a-valid-id", 42) != CHKPT_SyntaxError )
      return AppendFailed; 
  } catch ( ESException ) { 
    return EmptyState;
  } catch ( ChkptException ) {
    return UnexpectedExcept;
  };

  logfile << "... done!" << endl;
  
  logfile << "Test the return values CHKPT_WrongType..." << endl;
  
  try {
    for ( i = 0; i < 4; i++) 
      if ( ( i != 0 &&  js->appendValue( names[i], bval ) != CHKPT_WrongType ) ||
	   ( i != 1 &&  js->appendValue( names[i], ival ) != CHKPT_WrongType ) ||
	   ( i != 2 &&  js->appendValue( names[i], dval ) != CHKPT_WrongType ) ||
	   ( i != 3 &&  js->appendValue( names[i], sval ) != CHKPT_WrongType ) )
	return AppendFailed;
  } catch ( ESException ) { 
    return EmptyState;
  } catch ( ChkptException ) {
    return UnexpectedExcept;
  };

  logfile << "... done!" << endl;
  logfile << "\n OK, test is passed!!!" << endl;

  return OK;
}

int check_get_multiple_values(JobState* js) {

  int i;
  bool   vb[] = {true, false, true, false, true};
  int    vi[] = {1, 2, 3, 4, 5};
  double vd[] = {1.2, 2.3, 3.4, 4.5, 5.6};
  std::string vs[] = {"a", "b", "c", "d", "e"};

  std::vector<bool>   bret;
  std::vector<int>    iret;
  std::vector<double> dret;
  std::vector<std::string> sret;

  logfile << "Test the getTYPEDValue methods on list values.\n" 
	  << "We retrieve and check the pairs stored through the append methods." << endl;

  try {
    bret = js->getBoolValue(names[4]);
    iret = js->getIntValue(names[5]);
    dret = js->getDoubleValue(names[6]);
    sret = js->getStringValue(names[7]);
			    
    for ( i = 0; i < 5; i++ )
      if ( bret[i] != vb[i] ||
	   iret[i] != vi[i] ||
	   dret[i] != vd[i] ||
	   sret[i] != vs[i] )
	return GetFailed;
  } catch ( ESException ) { 
    return EmptyState;
  } catch ( ChkptException ) {
    return UnexpectedExcept;
  };
  
  logfile << "\n OK, test is passed!!!" << endl;
  return OK;
}

int check_is_typed(JobState* js) {

  logfile << "Test the isTYPEDValue methods." << endl; 

  try { 
    for ( int i = 0; i < 7; i++ )  
      if ( ( js->isBoolValue(names[i])   && ( i != 0 ) && ( i != 4 ) ) ||
	   ( js->isIntValue(names[i])    && ( i != 1 ) && ( i != 5 ) ) ||
	   ( js->isDoubleValue(names[i]) && ( i != 2 ) && ( i != 6 ) ) ||
	   ( js->isStringValue(names[i]) && ( i != 3 ) && ( i != 7 ) ) ||
	   ( js->isListValue(names[i])   && ( i != 4 ) && ( i != 5 ) && ( i != 6 ) && ( i != 7 ) ) )
	return CheckFailed;
  } catch ( ESException ) { 
    return EmptyState;
  } catch ( ChkptException ) {
    return UnexpectedExcept;
  };
     
  logfile << "Test if the exception 'ULException' is correctly thrown..." << endl; 
  
  try {
    try { 
      js->isBoolValue("UndefinedLabel");
    } catch ( ULException ) {}
    try { 
      js->isIntValue("UndefinedLabel");
    } catch ( ULException ) {} 
    try { 
      js->isDoubleValue("UndefinedLabel");
    } catch ( ULException ) {} 
    try { 
      js->isStringValue("UndefinedLabel");
    } catch ( ULException ) {} 
  } catch ( ChkptException ) {
    return WrongExcept;
  };

  logfile << "... done!" << endl;
  logfile << "\n OK, test is passed!!!" << endl;

  return OK;
}

int check_stepper(JobState* js) {
  
  std::vector<std::string> labels;
  labels.push_back("label1");
  labels.push_back("label2");
  labels.push_back("label3");
  labels.push_back("label4");

  logfile << "Test the stepper methods." << endl;
  logfile << "\n The JobState is: " << js->toString() << endl;

  int   count = 0;
  Step *step;
  step = (Step *) malloc( sizeof( Step *) );

  try {
    (*step) = js->getCurrentStep();
    if ( step->isLabel() ) {
      logfile << "We have a list of strings as stepper; these are the labels" << endl;
      while ( 1 ) {
	logfile << step->getLabel() << " ";
	if ( step->getLabel() != labels[count] ) 
	  return StepperFailed;
	count++;
	(*step) = js->getNextStep();
      } 
    } else { // Numeric stepper
      logfile << "We have a numeric stepper; these are the labels" << endl;
      while ( 1 ) {
	logfile << step->getInteger() << " ";
	if ( step->getInteger() != count ) 
	  return StepperFailed;
	count++;
	(*step) = js->getNextStep();
      }
    }
    logfile << endl;
  } catch ( ESException ) {
    free(step);
    return EmptyState;
  } catch ( WTException &ex ) {
    free(step);
    return UnexpectedExcept;
  } catch ( EoSException ) { 
    if ( ( step->isLabel() && (count != 4) ) ||
	 ( step->isInteger() && (count != 33 ) ) ) {
      free(step);
      return StepperFailed;
    }
  }
  
  logfile << "\n\n OK, test is passed!!!" << endl;
  free(step);
  return OK;
}


int main(int argc, char* argv[]) {

  // Command line options: -log <logfile>
  // Default is: logfile = /dev/null 
  
  int result = 1, tmp = 0;
  std::string logname = "/dev/null";

  // try with an empty state => check the catch of the ESException
  logfile.open(logname.c_str()); // don't log please!!!
  try {
    JobState empty;
    if ( ( !EmptyState == check_save_value(&empty) ) ||
	 ( !EmptyState == check_get_value(&empty) ) ||
	 ( !EmptyState == check_append_value(&empty) ) ||
	 ( !EmptyState == check_is_typed(&empty) ) ||
	 ( !EmptyState == check_get_multiple_values(&empty) ) )
      result =  EmptyState;
  } catch ( ChkptException ) {
    result = ConstructFailed;
  } 
  logfile.close();
  
  // Now we can begin to log!!! ----> if required!!!!
  
  if ( (argc == 3) &&  (strcmp(argv[1],"-log") == 0) ) 
    logname = argv[2];
  else {
    if ( argc != 1 ) {
      cout << "Usage : " << argv[0]
	   << " [-log logfile_name]\n" 
	   << "or simply " << argv[0] 
	   << " if you do not want a log file!" << endl;
      exit(-1);
    }
  }

  logfile.open(logname.c_str());

  logfile << "Test of the checkpointing API\n\n"
	  << "Test if the exception 'EmptyState' is correctly thrown..." << endl;
  if ( result == ConstructFailed ) 
    logfile << "\nFAILED to build an empty object!" << endl; 
  if ( result == EmptyState )
    logfile << "\nFAILED!" << endl;
  else 
    logfile << "... done!" << endl;

  logfile << "\n-----" << endl;
  logfile << "Test the API methods.\n" << endl;

  const std::string statest = "[ UserData = [  ]; StateId = \"This is the job_ID\"]";

  // Normal test
  try {          
    JobState state( statest );
    if ( putenv("EDG_WL_JOBID=This is the job_ID") == ENOMEM) {
      logfile << "A system call returns ENOMEM => ABORTED" << endl;
      return -1; // we can't do any tests
    }
    logfile << "\nJobState is: " << state.toString() << endl;
    logfile << "\n-----" << endl;
    tmp = check_save_value(&state);
    if ( tmp ) {
      result*= tmp;
      logfile << "\nFAILED! (code error: " << tmp <<  ")." << endl;
    } else {
      logfile << "\nJobState is: " << state.toString() << endl;
      logfile << "\n-----" << endl;
      tmp = check_get_value(&state); 
      if ( tmp ) {
	result*= tmp;
	logfile << "\nFAILED! (code error: " << tmp <<  ")." << endl;
      }
      logfile << "\nJobState is: " << state.toString() << endl;
    }
    
    logfile << "\n-----" << endl;
    tmp = check_append_value(&state);
    if ( tmp ) {
      result*= tmp;
      logfile << "\nFAILED! (code error: " << tmp <<  ")." << endl;
    } else {
      logfile << "\nJobState is: " << state.toString() << endl;
      logfile << "\n-----" << endl;
      tmp = check_get_multiple_values(&state);
      if ( tmp ) {
	result*= tmp;
	logfile << "\nFAILED! (code error: " << tmp <<  ")." << endl;
	logfile << "\nJobState is: " << state.toString() << endl;
      }
    }
    
    // execute this test only if the previous ones have been passed
    if ( ( result == EmptyState ) || 
	 ( result == ConstructFailed ) || 
	 (result == 1) ) {
      logfile << "\n-----" << endl;
      tmp = check_is_typed(&state);
      if ( tmp ) {
	result*= tmp;
	logfile << "\nFAILED! (code error: " << tmp <<  ")." << endl;
      }
    } else {
      logfile << "\n-----" << endl;
      logfile << "Methods isTYPEDvalue can not be tested!" << endl;
    }

    // test the stepper
    logfile << "\n-----" << endl;
    const std::string statest3 = "[ UserData = [  ]; StateId = \"This is the job_ID\"; JobSteps = {\"label1\", \"label2\", \"label3\", \"label4\" } ; CurrentStep = 0;]";
    JobState stepperlab( statest3 );
    tmp = check_stepper(&stepperlab);
    if ( tmp ) {
      result*= tmp;
      logfile << "\nFAILED! (code error: " << tmp <<  ")." << endl;
    }

    const std::string statest2 = "[ UserData = [ ]; StateId = \"This is the job_ID\"; JobSteps = 32; CurrentStep = 0;]";
    JobState stepperint( statest2 );

    tmp = check_stepper(&stepperint);
    if ( tmp ) {
      result*= tmp;
      logfile << "\nFAILED! (code error: " << tmp <<  ")." << endl;
    }

  } catch ( ChkptException &ex) {
    logfile << "\nFAILED to build a JobState object!" 
	    << "DBGmsg: " << ex.dbgMessage() << endl;
    return result * ConstructFailed;
  };

  if ( result == 1 ) {
    logfile << "\n\n TEST has been SUCCESSFULLY executed!!! " << endl;
    logfile.close();
    return 0;
  } else {
    logfile << "\n\n TEST FAILED! Error code: " << result << endl;
    logfile.close();
    return result;
  }  
} ///:~

