#ifndef EDG_WORKLOAD_COMMON_LOGGER_CONTAINER_TS_H
#define EDG_WORKLOAD_COMMON_LOGGER_CONTAINER_TS_H

#include <boost/thread/mutex.hpp>
#include <boost/thread/tss.hpp>

#include "common.h"

COMMON_NAMESPACE_BEGIN {

namespace logger {

class DataContainerMulti : public DataContainerImpl {
public:
  DataContainerMulti( const char *format );
  ~DataContainerMulti( void );

  // Setters
  virtual void date( bool d );
  virtual void multiline( bool d, const char *prefix );
  virtual void next_level( level_t lev );
  virtual void time_format( const char *format );
  virtual void function( const char *func );
  virtual void clear_function( void );

  // Constant extractors
  virtual bool date( void ) const;
  virtual bool multiline( void ) const;
  virtual level_t next_level( void ) const;
  virtual const std::string &time_format( void ) const;
  virtual const std::string &function( void ) const;
  virtual const std::string &multiline_prefix( void ) const;
  virtual bool current_tid(std::string &tid) const;

  // Extractors
  virtual bool date( void );
  virtual bool multiline( void );
  virtual level_t next_level( void );
  virtual const std::string &time_format( void );
  virtual const std::string &function( void );
  virtual const std::string &multiline_prefix( void );
  virtual bool current_tid(std::string &tid);

private:
  struct data_s {
    data_s( const DataContainerSingle &dcs );

    const std::string current_tid( void );

    bool         d_date, d_multiline;
    level_t      d_next;
    std::string  d_format, d_function, d_multiprefix;

    static unsigned long int d_next_tid;
    static boost::mutex d_tid_mutex;
    static boost::thread_specific_ptr<unsigned long int> d_current_tid;
  };

  inline void createData( void ) const
  { if( this->dcm_data.get() == NULL ) this->dcm_data.reset( new data_s(this->dcm_single) ); return; }

  mutable boost::thread_specific_ptr<data_s>       dcm_data;
  DataContainerSingle                              dcm_single;
};

}; // Namespace logger

} COMMON_NAMESPACE_END;

#endif /* EDG_WORKLOAD_COMMON_LOGGER_CONTAINER_TS_H */

// Local Variables:
// mode: c++
// End:
