/* **************************************************************************
*  filename  : JobAd.cpp
*  author    : Alessandro Maraschini <alessandro.maraschini@datamat.it>
*  copyright : (C) 2002 by DATAMAT
***************************************************************************/
//
// $Id: JobAd.cpp,v 1.79 2004/11/10 09:51:58 dhsmith Exp $
//

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include "edg/workload/common/requestad/JobAd.h"
#include "edg/workload/common/requestad/JobAdExceptions.h"
#include "edg/workload/common/requestad/exception_codes.h"
// new attributes list
#include "edg/workload/common/requestad/JDLAttributes.h"
#include "edg/workload/common/requestad/jdl_attributes.h"
#include "edg/workload/common/utilities/classad_utils.h"
#ifdef HAVE_STRINGSTREAM
#include <sstream>
#else
#include <strstream>
#endif

#include <glob.h>

COMMON_NAMESPACE_BEGIN{ //Defining UserInterFace NameSpace
namespace requestad{

using namespace classad;
using namespace std ;
using namespace edg::workload::common::utilities ;


#define WIN_SEP "\\"
#define UNIX_SEP "/"

// Static Variable definition:
// pthread_mutex_t JobAd::mutex = PTHREAD_MUTEX_INITIALIZER;
// int JobAd::CHAR_BUFFER_SIZE = 1024 ;
/******************************************************************
 methods : Constructors
*******************************************************************/
JobAd::JobAd() {
   checking = false ;
   defaultReq= NULL ;
   defaultRank = NULL ;
   schema = NULL ;
   // user = new ClassAd ();
};
//Constuctor From  string
JobAd::JobAd(const string& jdl_string) {
   defaultReq= NULL ;
   defaultRank = NULL ;
   schema = NULL ;
   EDG_STACK_TRY("JobAd::JobAd(const string& jdl_string)") ;
   checking = false ;
   fromString(jdl_string);
   EDG_STACK_CATCH() ; //Exiting from method: remove line from stack trace
};

JobAd::JobAd(   const classad::ClassAd& classAd  ){
	defaultReq= NULL ;
	defaultRank = NULL ;
	schema = NULL ;
	vector < pair < string, ExprTree* > >  vect ;
	classAd.GetComponents( vect ) ;
	for (unsigned int i = 0 ; i < vect.size() ; i++ )
		insertAttribute (  vect[i].first , vect[i].second->Copy()  );
}

JobAd::~JobAd() throw(){

	if (schema != NULL)  delete schema ;
} ;


JobAd::JobAd(const JobAd& jobad) {
	EDG_STACK_TRY("JobAd::JobAd(const JobAd& )") ;
	// cout <<  "JobAd called copy constructor begin" << endl ;
	vector< pair< string, ExprTree *> > vec ;
	jobad.GetComponents(vec ) ;
	for (unsigned int i = 0 ; i < vec.size() ; i ++ ) Insert ( vec[i].first , vec[i].second->Copy() ) ;
	if (jobad.defaultRank!=NULL)  defaultRank = jobad.defaultRank->Copy() ;
	else defaultRank = NULL ;
	if (jobad.defaultReq!=NULL) defaultReq = jobad.defaultReq->Copy() ;
	else defaultReq = NULL ;
	if (jobad.schema!=NULL) schema =  jobad.schema->Copy() ;
	else schema = NULL ;
	EDG_STACK_CATCH() ; //Exiting from method: remove line from stack trace
	// cout <<  "JobAd called copy constructor end" << endl ;
};

void JobAd::operator=(const JobAd& jobad) {
	EDG_STACK_TRY("JobAd::operator=(const JobAd& jobad)") ;
	vector< pair< string, ExprTree *> > vec ;
	jobad.GetComponents(vec ) ;
	for (unsigned int i = 0 ; i < vec.size() ; i ++ ) Insert ( vec[i].first , vec[i].second->Copy() ) ;
	if (jobad.defaultRank!=NULL)  defaultRank = jobad.defaultRank->Copy() ;
	else defaultRank = NULL ;
	if (jobad.defaultReq!=NULL) defaultReq = jobad.defaultReq->Copy() ;
	else defaultReq = NULL ;
	if (jobad.schema!=NULL) schema =  jobad.schema->Copy() ;
	else schema = NULL ;
	EDG_STACK_CATCH() ; //Exiting from method: remove line from stack trace
};




/******************************************************************
 method :   toString
*******************************************************************/
string JobAd::toString(){
   string       buffer="";
   restore() ;
   PrettyPrint  unp;
   //flat the classAd into one single line
   unp.SetClassAdIndentation(0);
   unp.SetListIndentation(0);
   //Unparse the ClassAd instance
   unp.Unparse(buffer, this );
   return buffer;
};
string JobAd::toString( const string& attr_name ){
  restore() ;
  return Ad::toString (  attr_name ) ;
}
/******************************************************************
 method :   toSubmissionString
*******************************************************************/
string JobAd::toSubmissionString(){
   EDG_STACK_TRY("JobAd::toSubmissionString()") ;
   check();
   string       buffer="";
   PrettyPrint  unp;
   //flat the classAd into one single line
   unp.SetClassAdIndentation(0);  //TBD set to 0
   unp.SetClassAdIndentation(0);  //TBD set to 0
   unp.SetListIndentation(0);
   //Unparse the ClassAd instance
   unp.Unparse(buffer, this );
   return buffer;
   EDG_STACK_CATCH() ; //Exiting from method: remove line from stack trace
}

/******************************************************************
 method : fromStream
*******************************************************************/
void JobAd::fromStream(istream& jdl_in)  {
  EDG_STACK_TRY("JobAd::fromStream(istream& jdl_in)") ;
#ifdef HAVE_STRINGSTREAM
  stringstream buffer ;
#else
  strstream buffer;
#endif
  string result = "", tmpStr ;
  int blank ;
  while (  getline(jdl_in, tmpStr,'\n')  ){
          //find the first no-blank character
          blank = 0 ;
          while ( tmpStr[blank]==' ' )
               blank++;
          //Check For Commented lines:
          if ( tmpStr[blank] =='#' ){
              //header += "\n" + tmpStr;
              //Python Comment: do nothing
          }else if ( tmpStr[blank] =='/' && tmpStr[blank+1] =='/' ){
              //header += "\n" + tmpStr;
              //C/C++ Comment: do nothing
          }else
              result += " " + tmpStr;
  }
  // add Par when needed (first delete initial blanks) TBD move this part to fromString method
  while ( result.length() !=0  ){
        if (  result.substr(0,1)==" " )
	     result = result.substr(1);
	else
	    break;
  }
  if (  result.substr(0,1)==" " )
           result = "[" + result   +   " ]"  ;
  fromString ( result );
  EDG_STACK_CATCH() ; //Exiting from method: remove line from stack trace
};
/******************************************************************
 method :   toFile
 Print the jdl in a string format in the specified file
*******************************************************************/
void JobAd::toFile(const string& file_path)
  {
  EDG_STACK_TRY("JobAd::toFile(const string& file_path)") ;
  string jdl=toLines() ;
  ofstream fout (file_path.c_str());
  fout << jdl ;
  EDG_STACK_CATCH() ; //Exiting from method: remove line from stack trace
};

/******************************************************************
 methods : setAttribute
 Add A couple attr_name = attr_value to the jdl
 where value-type could be:
 -int
 -double
 -bool
 -string
 -generic ClassAd expression
*******************************************************************/
void JobAd::insertAttribute(const std::string& attr_name ,ExprTree* tree){
    string METHOD ="insertAttribute(const std::string attr_name , ExprTree* val)" ;
        //Empty attribute -> try to fill
        checkSyntax (attr_name , tree ) ;
        if (!   Insert (attr_name  , tree)     )
              throw AdSyntaxException (__FILE__ , __LINE__ ,METHOD,   WL_JDLSYN ,attr_name);
        // fill the attribute in the user's classAd (if necessary)
        if (  jdlist.findCheck(attr_name)   && !checking   ){
             user.Insert (attr_name  , tree->Copy()) ;
        }
}


void JobAd::setDefaultRank (const std::string& attr_value ){
	ClassAdParser par ;
	defaultRank =par.ParseExpression(attr_value, true) ;
	if (  defaultRank==NULL)
	throw  AdClassAdException (__FILE__ , __LINE__ ,
		" JobAd::setDefaultRank (const string& attr_value)", WL_JDLSYN , "ClassAdParser::ParseExpression");

} ;

void JobAd::setDefaultReq   (const std::string& attr_value ){
	ClassAdParser par ;
	defaultReq = par.ParseExpression(attr_value, true) ;
	if (  defaultReq==NULL)
	throw  AdClassAdException (__FILE__ , __LINE__ ,
		"JobAd::setDefaultReq(const string& attr_value)", WL_JDLSYN , "ClassAdParser::ParseExpression");
} ;



void JobAd::setAttributeExpr (const string& attr_name, const string& attr_value) {
   EDG_STACK_TRY("JobAd::setAttributeExpr (const string& attr_name, const string& attr_value)") ;
    if (  hasAttribute(attr_name)   )
        throw AdEmptyException (__FILE__ , __LINE__ ,METHOD,   WL_JDLFULL ,attr_name);
   jdlist.checkExpr (attr_name , attr_value) ;
   Ad::setAttributeExpr (attr_name, attr_value) ;
   EDG_STACK_CATCH() ; //Exiting from method: remove line from stack trace
};

/******************************************************************
 methods :   is<type>value
 Retreive the value of an attribute
 where <type> could be:
 -int
 -double
 -bool
 -string
*******************************************************************/
 std::string JobAd::getAd(const std::string& attr_name) {
       string METHOD ="getAd(const std::string& attr_name)";
       ExprTree* tree = Lookup (attr_name);
       if ( tree== NULL )
           throw AdEmptyException (__FILE__ , __LINE__ ,METHOD,   WL_JDLFULL ,attr_name);;
       string       buffer="";
       PrettyPrint  unp;
       //flat the classAd into one single line
       unp.SetClassAdIndentation(0);
       unp.SetListIndentation(0);
       //Unparse the ClassAd instance
       unp.Unparse(buffer, tree );
       return buffer;
 };

 std::string JobAd::getString(const std::string& attr_name) {
 EDG_STACK_TRY("JobAd::getString(const std::string& attr_name) ");
   JdlAttributeList jdlist ;
   if (   jdlist.findList(attr_name)   )
      throw  AdListException (__FILE__ , __LINE__ ,METHOD, EPERM , attr_name);
   return getStringValue (attr_name)[0];
 EDG_STACK_CATCH() ; //Exiting from method: remove line from stack trace
 };
 int JobAd::getInt(const std::string& attr_name) {
 EDG_STACK_TRY("JobAd::getInt(const std::string& attr_name)");
  JdlAttributeList jdlist ;
  if (   jdlist.findList(attr_name)   )
   throw  AdListException (__FILE__ , __LINE__ ,METHOD, EPERM , attr_name);
  return getIntValue (attr_name)[0];
 EDG_STACK_CATCH() ; //Exiting from method: remove line from stack trace
 };
 double JobAd::getDouble(const std::string& attr_name) {
 EDG_STACK_TRY("JobAd::getDouble(const std::string& attr_name)");
  JdlAttributeList jdlist ;
  if (   jdlist.findList(attr_name)   )
   throw  AdListException (__FILE__ , __LINE__ ,METHOD, EPERM , attr_name);
  return getDoubleValue (attr_name)[0];
 EDG_STACK_CATCH() ; //Exiting from method: remove line from stack trace
 };
 bool JobAd::getBool(const std::string& attr_name) {
 EDG_STACK_TRY("JobAd::getBool(const std::string& attr_name)");
  JdlAttributeList jdlist ;
  if (   jdlist.findList(attr_name)   )
   throw  AdListException (__FILE__ , __LINE__ ,METHOD, EPERM , attr_name);
  return getBoolValue (attr_name)[0];
 EDG_STACK_CATCH() ; //Exiting from method: remove line from stack trace
 };

/******************************************************************
 method :   delAttribute
*******************************************************************/
classad::ExprTree* JobAd::delAttribute(const string& attr_name) {
   EDG_STACK_TRY("JobAd::delAttribute(const string& attr_name)");
   //Call superclass method
   ExprTree* attr_value = Ad::delAttribute(attr_name) ;
   user.Delete(attr_name) ;  //TBD should it be possible shift this operation before the superclass Method...
   return attr_value;
   EDG_STACK_CATCH() ; //Exiting from method: remove line from stack trace
};

/******************************************************************
 method :  check
 Check the jobAd syntax and semantics built classAd
*******************************************************************/
void JobAd::check(bool restoring)  {
   EDG_STACK_TRY("JobAd::check()") ;
   checking = true ;  //Set the checking tag to true
   // Restore to values inserted by user:
   if (restoring) restore() ;
   //Check for semantic Error in classAd
   checkSemantic();
   checkSpecials() ;
   checking = false ;
   EDG_STACK_CATCH() ; //Exiting from method: remove line from stack trace
};
/******************************************************************
 PRIVATE methods :  checkSyntax
 Check the jobAd syntax
******************************************************************/
/**
* checkSyntax(const string attr_name , Value val)
* This method is used by checkSyntax( const string attr_name ,   ExprTree  *attr_expr )*/
void JobAd::checkSyntax(const string& attr_name , Value val)    {
    EDG_STACK_TRY("JobAd::checkSyntax(const string& attr_name , const Value attr_value)");
    //Check the Value to the ClassAd
    switch ( val.GetType() ){
       case Value::BOOLEAN_VALUE :{
          bool b ;
          if (val.IsBooleanValue( b )  )    jdlist.checkBool (attr_name , b) ;
          else throw AdMismatchException (__FILE__ , __LINE__ ,METHOD, WL_JDLMISMATCH , attr_name);
	  }
          break;
       case Value::INTEGER_VALUE:{
          int i;
          if (val.IsIntegerValue( i )  )     jdlist.checkInt (attr_name , i) ;
          else throw AdMismatchException (__FILE__ , __LINE__ ,METHOD, WL_JDLMISMATCH , attr_name);
	  }
          break;
       case Value::REAL_VALUE:{
          double d ;
          if (val.IsRealValue( d )  )       jdlist.checkDouble (attr_name , d) ;
          else throw AdMismatchException (__FILE__ , __LINE__ ,METHOD, WL_JDLMISMATCH , attr_name);
	  }
          break;
       case Value::STRING_VALUE:{
          string s ;
          if ( val.IsStringValue( s )  )      jdlist.checkString ( attr_name , s) ;
          else throw AdMismatchException (__FILE__ , __LINE__ ,METHOD, WL_JDLMISMATCH , attr_name);
	  }
          break;
       case Value::CLASSAD_VALUE:{
	  cout << "\n\nFATAL ERROR: please Contact the Developers: " << __FILE__  << " at line: " << __LINE__ << endl << flush ;
	  throw AdMismatchException (__FILE__ , __LINE__ ,METHOD, WL_JDLMISMATCH , attr_name);
	  /*
          ClassAd* ad  ;
          if (val.IsClassAdValue( ad )  )  jdlist.checkAd (attr_name , ad) ;
          else throw AdMismatchException (__FILE__ , __LINE__ ,METHOD, WL_JDLMISMATCH , attr_name);
	  */
	  }
          break;
       default:
          throw AdMismatchException (__FILE__ , __LINE__ ,METHOD, WL_JDLMISMATCH , attr_name);
    }
   EDG_STACK_CATCH() ; //Exiting from method: remove line from stack trace
};
/******************************************************************
 Check the jobAd syntax
*******************************************************************/
void JobAd::checkSyntax( const string& attr_name ,   ExprTree  *attr_expr ) {
   EDG_STACK_TRY("JobAd::checkSyntax(const string& attr_name ,   ExprTree  *attr_expr)");
      Value              val ;
      if (   jdlist.findAttribute(attr_name)){
         // The Attribute is known: check needed
         //Converts the expression to a Value
         if (!EvaluateExpr(attr_expr, val ))
             throw AdClassAdException (__FILE__ , __LINE__ ,METHOD, WL_JDLMISMATCH , "ClassAd::EvaluateExpr" , attr_name );
         switch ( val.GetType()  ){
           case Value::LIST_VALUE:{
                 if (   !jdlist.findList(attr_name)   )
                     throw AdListException (__FILE__ , __LINE__ ,METHOD,  WL_JDLSYN,attr_name );
                 const ExprList    *el;
                 vector<ExprTree*> vectList ;
                 val.IsListValue(el) ;
                 el->GetComponents(vectList) ;
                 for ( int i = 0; i< (int)vectList.size() ; i++){
                     EvaluateExpr(  vectList[i] , val ) ;
                     if ( val.GetType() == Value::CLASSAD_VALUE)
                           jdlist.checkAd (attr_name , (ClassAd*) vectList[i] ) ;
                     else
                           checkSyntax(attr_name , val) ;
                 }
              }
              break;
           case  Value::STRING_VALUE:
           case  Value::BOOLEAN_VALUE:
           case  Value::INTEGER_VALUE:
           case  Value::REAL_VALUE:
                           checkSyntax(attr_name , val) ;
                           break;
           case  Value::CLASSAD_VALUE:
		     jdlist.checkAd (attr_name , (ClassAd*) attr_expr) ; // checkSyntax(attr_name , val) ;
                     break ;
         default:
                 jdlist.checkExpr(  attr_name , attr_name  );
                 break ;
         }  // END switch
      } // END findAttribute
   // cout << "JobAd::checkSyntax(const string& attr_name ,   ExprTree  *attr_expr) DONE" <<attr_name << endl << flush ;
   EDG_STACK_CATCH() ; //Exiting from method: remove line from stack trace
};

/******************************************************************
 PRIVATE method :  checkSemantic, checkSpecials
 Check the jobAd semantic attribute expressions
*******************************************************************/
void JobAd::checkSpecials(  ) {
	EDG_STACK_TRY("JobAd::checkSemantic()");
	/**
	*    JDL::VIRTUAL_ORGANISATION  attribute check
	*/
	if (! hasAttribute(JDL::VIRTUAL_ORGANISATION) )
		throw AdSemanticMandatoryException (__FILE__ , __LINE__ ,METHOD , WL_JDLMANDATORY ,JDL::VIRTUAL_ORGANISATION);
	EDG_STACK_CATCH() ; //Exiting from method: remove line from stack trace
};
//Check the Semantic of a classAd
void JobAd::checkSemantic() {
    //This Method check the co-existance of the attributes in classAd
    EDG_STACK_TRY("JobAd::checkSemantic()");
    // JdlAttributeList jdlist ;
    bool stdinput_tofind = true ;
    bool exec_tofind = true ;
    int index =0 ;
    // Initialise default Schema (if null)
    if (schema==NULL)
	schema = JobAdSchema::glueSchema () ;
    // This variable stores the value of extracted inputsandbox files:
    vector<string> extracted   ;
    /**
    *    JDL::REQUIREMENTS,
    */
    if (!  hasAttribute(JDL::REQUIREMENTS)  ){
       if ( defaultReq== NULL)
          throw AdSemanticMandatoryException (__FILE__ , __LINE__ ,METHOD , WL_JDLMANDATORY ,JDL::REQUIREMENTS);
       else     insertAttribute ( JDL::REQUIREMENTS, defaultReq->Copy() ) ;
    } else {
    	// Append value TBD
    }
   /*
   * JDL::RANK attribute check
   */
    if (!  hasAttribute(JDL::RANK)  ){
       if (defaultRank == NULL)
          throw AdSemanticMandatoryException (__FILE__ , __LINE__ ,METHOD , WL_JDLMANDATORY ,JDL::RANK);
       else   insertAttribute ( JDL::RANK , defaultRank->Copy() ) ;
    }
    /**
    *     TYPE attribute check
    */
    if (hasAttribute(JDL::TYPE) ){
       const string type = getString(JDL::TYPE);
       Delete (JDL::TYPE);
       setAttribute ( JDL::TYPE ,  toLower(type)) ;
       if (!  ( toLower( type )== JDL_TYPE_JOB) )
           throw AdSyntaxException (__FILE__ , __LINE__ ,METHOD,   WL_JDLSYN ,  JDL::TYPE );
    }else
       setAttribute ( JDL::TYPE , (string)JDL_TYPE_JOB  );
    /**
    *    JDL::INPUTSB  attribute check
    */
	if ( hasAttribute(JDL::INPUTSB) ){
		checkInputSandbox (  extracted )  ;
	}

    if ( hasAttribute(JDL::HLR_LOCATION) ){
        string hlr = string (JDL_HLR_LOCATION_ENV ) + "=" +  getString (JDL::HLR_LOCATION) ;
        addAttribute (JDL::ENVIRONMENT , hlr   ) ;
    }
    /**
    *     JDL::EXECUTABLE attribute check
    */
    if (hasAttribute(JDL::EXECUTABLE) ){
	const string exec = getStringValue(JDL::EXECUTABLE)[0];
	if ( isAbsolutePath(  exec , false )  )
          exec_tofind= false ;
	else{
#ifdef WIN
	  // Windows Check
	   index =exec.find(WIN_SEP) ;
#else
           // Linux Unix Check
	   index = exec.find(UNIX_SEP) ;
#endif
	   if (   (int) index <0 || (int) index > (int)exec.length()-1 )
	     exec_tofind= false ;
	}
	if ( exec_tofind ){
		// JdlAttributeList jdlist ;
		if (   !jdlist.findAttr(exec , extracted )   )
			throw AdSemanticPathException (__FILE__ , __LINE__ ,METHOD, WL_JDLMANDATORY, JDL::EXECUTABLE, exec);
	}
    } else
         // Mandator Executable attribute not found
         throw AdSemanticMandatoryException (__FILE__ , __LINE__ ,METHOD , WL_JDLMANDATORY ,JDL::EXECUTABLE);
    /**
    *   JDL::STDINPUT   attribute check
    */
    if ( hasAttribute(JDL::STDINPUT) ){
        const string  stdInput=getStringValue(JDL::STDINPUT)[0];
        // stinput has to be found if and only if the path is not absolute:
        stdinput_tofind =  ! isAbsolutePath(  stdInput , false ) ;
        if (  stdinput_tofind  )  //It's not a simple path:  must be contained in InputSandbox
              if (   !jdlist.findAttr(stdInput , extracted )   ){
                     // throw AdSemanticPathException (__FILE__ , __LINE__ ,METHOD, WL_JDLMANDATORY , JDL::STDINPUT, stdInput );
              }
    }
    /**
    *   JDL::INPUTDATA, JDL::REPLICA_CATALOG , JDL::DATA_ACCESS  attributes check
    * REPLICA_CATALOG attribute DEPRECATED
    */
    bool id = hasAttribute(JDL::INPUTDATA);
    // bool rc = hasAttribute(JDL::REPLICA_CATALOG);
    bool dp = hasAttribute(JDL::DATA_ACCESS);
    if (id || dp)
       //At least one out of three has been specified:
       // if one out of these attribute has been specified then all of them have to be set
       if (!(id && dp)){
                  string errStr = string ( JDL::INPUTDATA) + ", " +
                                                // string(JDL::REPLICA_CATALOG) + ", " +
                                                string(JDL::DATA_ACCESS )  ;
                  throw AdSemanticGroupException (__FILE__ , __LINE__ ,METHOD,WL_JDLMANDATORY, errStr );
       }
    /**
    *    STDOUTPUT - STDERROR check:
    * StdOutput & StdError can have the same file name
    * only if the absolute path is the same*/
    if (  hasAttribute(JDL::STDOUTPUT) && hasAttribute(JDL::STDERROR) ){
	string stdOutput = getStringValue(JDL::STDOUTPUT)[0];
	string stdError = getStringValue(JDL::STDERROR)[0];
	if (   getName (stdOutput) == getName (stdError)   )
	   if (stdOutput != stdError){
	      string errMsg = JDL::STDOUTPUT +","+JDL::STDERROR+": can have the same file name only if the absolute path is the same" ;
	      throw AdSemanticGroupException (__FILE__ , __LINE__ ,METHOD,WL_JDLMANDATORY, errMsg );
	   }
    }
    /**
    *    JDL::OUTPUSB attribute check
    */
    if (  hasAttribute(JDL::OUTPUTSB)){
      vector<string> outputFiles = getStringValue(JDL::OUTPUTSB);
      vector<string> outMap ;  // keep track of simple file names
      vector<string> outDupli;  // keep track of duplicated file names
      vector<string>::iterator it ;
      vector<string>::iterator iter ;
      string outSimple ;
      //Create a list of all inputsandbox files
      for (iter = outputFiles.begin() ; iter!=outputFiles.end() ; ++iter){
         outSimple = getName (*iter) ;
         for (it = outMap.begin() ; it!=outMap.end() ; ++it){
	    // if (   it->compare (outSimple  )  ) {//TBC (to be checked) //DEPRECATED!!! it works bad!!
	    if (   *it ==outSimple  )   {//TBC (to be checked)
	        outDupli.push_back(   outSimple  ) ;
	        break;
	    }
	 }
	 if (it== outMap.end())  //TBC+
	    outMap.push_back( outSimple ) ;
      }
      for (it = outDupli.begin() ; it!=outDupli.end() ; ++it)
         throw AdSemanticPathException (__FILE__ , __LINE__ ,METHOD, WL_JDLDUPLICATE , JDL::OUTPUTSB, *it );
    }
    /**
    *    JDL::JOBTYPE attribute check
    */
    if (  hasAttribute(JDL::JOBTYPE))   checkJobType( getStringValue(JDL::JOBTYPE ) );
    else setAttribute( JDL::JOBTYPE , string(JDL_JOBTYPE_NORMAL ) );
    /**
    *    JDL_DEFAULT_RANK private attribute check
    */
    if (  hasAttribute(JDL_DEFAULT_RANK))
       delAttribute ( JDL_DEFAULT_RANK ) ;
    setAttributeExpr (JDL_DEFAULT_RANK , JDL_RANK_DEFAULT ) ;
    /*
    * DataAccessManagemente check
    */
    if (  hasAttribute(JDL::RANK) )    checkDataManagement () ;
 EDG_STACK_CATCH() ; //Exiting from method: remove line from stack trace
};
/******************************************************************
 PRIVATE methods :  checkOutputData
 check the OutputData attribute value (it's a list of classAd )
*******************************************************************/

void JobAd::checkMultiAttribute ( const vector<string> &multi ){
   EDG_STACK_TRY("JobAd::checkMultiAttribute (const vector<string> &multi )")
   std::string requirements ;
   check(false) ;
   for (unsigned int  att = 0 ; att<2 ; att++){
        if ( att==1) requirements =  getAttributeExpr ( JDL::RANK) ;
        else requirements =  getAttributeExpr ( JDL::REQUIREMENTS) ;
        const int  DIM = 128 ;
        string values  [ DIM ]  ;
        string lists      [ DIM ]  ;
        int len = requirements.length() ;
        //find Keyword member:
        int position = requirements.find ("Member") ;
        int openPar, comma1, comma2 , closedPar ;
        int i = 0 ;
        while (position < len   && position >=0){
            openPar   = requirements.find ("("  , position) +1;
            while (   requirements[openPar] == ' '  )
               openPar++;
            comma1     = requirements.find (","  , position) - 1 ;
            comma2 =comma1+2 ;
            while (   requirements[comma1] == ' '  )
               comma1-- ;
            while (   requirements[comma2] == ' '  )
               comma2++ ;
            closedPar = requirements.find (")"  , position) -1 ;
            while (   requirements[closedPar] == ' '  )
                closedPar--;
            //check Consistency:
            if (  openPar < len && closedPar < len  && comma1 < len && openPar<comma1 && comma2 < closedPar ){
		values[i] = requirements.substr (openPar , comma1 - openPar +1 )   ;
		lists[i] =       requirements.substr (comma2 , closedPar - comma2 +1)   ;
            }
            position = requirements.find ("Member", closedPar ) ;
            i++ ;
            if (i==DIM)
               break ;
        } //end while position...
        string* ip ; //  used to find string
        // vector<string>::const_iterator it  = multi.begin() ;
        for  (  vector<string>::const_iterator it  =  multi.begin() ; it !=multi.end() ; it++ ) {
              if (  (int)requirements.find ("other."+*it)>=0  && (int) requirements.find ("other."+*it)<len   ){
                    //The Attribute cannot be present in the values
                    ip = std::find(  values, values + DIM , "other."+*it );
                    if  (ip != values + DIM){ // found
                              throw AdSyntaxException (__FILE__ , __LINE__ ,METHOD,  WL_JDL_MEMBER  ,*it);
                    }
                    //The Attribute must be present in the lists
                    ip = std::find(  lists, lists + DIM , "other."+*it );
                    if  (ip == lists + DIM) {  //not found
                               throw AdSyntaxException (__FILE__ , __LINE__ ,METHOD,  WL_JDL_MEMBER  ,*it);
                    }
             } // end if
        } // end for (const iterator )
   } // end for (int i )
   restore () ;
   EDG_STACK_CATCH() ; //Exiting from method: remove line from stack trace
};
/******************************************************************
 PRIVATE methods :checkDataManagement
 Check the Rank attribute, if contains DataAccessCost then
 1) InputDAta, DataAccessProtocol Are mandatory
 2) it is the one value
*******************************************************************/
void JobAd::checkDataManagement( ){
EDG_STACK_TRY("JobAd::checkDataManagement (  )");
vector<string> attrs;
utilities::insertAttributeInVector(&attrs, Lookup(JDL::RANK) , utilities::is_reference_to("other"));
string dac = string (JDL_RANK_DATA_ACC_COST);
for (unsigned int i = 0 ; i< attrs.size() ; i++){
   if ( toLower(attrs[i] ) == toLower(dac) ){
      // DataAccessCost found
      if (   (! hasAttribute( JDL::INPUTDATA  ) ) ||  (! hasAttribute( JDL::DATA_ACCESS ) )  )
         throw   AdSemanticGroupException(__FILE__ , __LINE__ ,METHOD, WL_JDLGROUP ,"DataAccessCost rank, InputData, DataAccessProtocol") ;
     else if ( attrs.size() >1 )
         throw   AdSemanticGroupException(__FILE__ , __LINE__ ,METHOD, WL_JDLGROUP ,"DataAccessCost rank not alone") ;
      else    break;
   }
}
EDG_STACK_CATCH() ; //Exiting from method: remove line from stack trace
} ;

/******************************************************************
 PRIVATE methods :checkInputSandbox
Check The inputSandbox attribute and extract the absolute file names
*******************************************************************/
void JobAd::checkInputSandbox(  std::vector<std::string>& extracted ){
	// This Vector will replace the old InputSanbox with the extracted values
	vector<ExprTree*> isVect ;
	// This vector contains all ISB string values
	vector<string> inputFiles ;
	inputFiles = getStringValue(JDL::INPUTSB);
	// Old files have to be replaced
	vector<string>::iterator it ;
	//Create a list of all inputsandbox files
	for (it = inputFiles.begin() ; it!=inputFiles.end() ; ++it)
		vector<string> tmpList = extractFiles (JDL::INPUTSB, *it , extracted) ;
	//Update InputSandbox with new files:
	vector<string>::iterator iter ;
	// put the extracted files back into the InputSandbox attribute
	Value  val ;
	for (iter = extracted.begin(); iter!=extracted.end(); iter++){
		val.SetStringValue( *iter) ;
		isVect.push_back( Literal::MakeLiteral( val ) ) ;
	}
	delAttribute( JDL::INPUTSB  );
	insertAttribute ( JDL::INPUTSB ,(ExprTree*) ExprList::MakeExprList( isVect  ) ) ;
} ;

/******************************************************************
 PRIVATE methods :  checkJobType
 check the type of the job and update several involved attributes such as Rank, Requirements, Environment...
*******************************************************************/
void JobAd::checkJobType(const vector<string>& attr_value) {
EDG_STACK_TRY("JobAd::checkJobType( const vector<string>& attr_value   )");
    // bool req = hasAttribute(JDL::REQUIREMENTS);
    // bool rk = hasAttribute(JDL::RANK);
    Delete (JDL::JOBTYPE) ;
    for (unsigned int i = 0 ; i<attr_value.size() ; i++) addAttribute (JDL::JOBTYPE  ,  toLower(attr_value [i] )   ) ;
    string requirements = "" ;
    string rank = "" ;
    /* Initial checks:
      - NodeNumber ->Mpi
      - ShPort -> interacive
      - JobSteps-> checkponintable
    */
	if ( hasAttribute(JDL::NODENUMB) && !hasAttribute ( JDL::JOBTYPE, JDL_JOBTYPE_MPICH) )
		throw   AdSemanticGroupException(__FILE__ , __LINE__ ,METHOD,WL_JDLMANDATORY,"Nodenumber, MPICH jobs") ;
	if ( hasAttribute(JDL::SHPORT) && !hasAttribute ( JDL::JOBTYPE, JDL_JOBTYPE_INTERACTIVE ) )
		throw   AdSemanticGroupException(__FILE__ , __LINE__ ,METHOD,WL_JDLMANDATORY,"ShadowPort, INTERACTIVE jobs") ;
	if ( hasAttribute(JDL::CHKPT_STEPS)  )
		// If chkpt steps is present the job can be either partitionable or checkpointing
		if (    !hasAttribute ( JDL::JOBTYPE, JDL_JOBTYPE_PARTITIONABLE)  && !hasAttribute ( JDL::JOBTYPE, JDL_JOBTYPE_CHECKPOINTABLE )   )
			throw   AdSemanticGroupException(__FILE__ , __LINE__ ,METHOD,WL_JDLMANDATORY,"JobSteps, CHECKPOINTABLE|INTERACTIVE jobs") ;
    /**
    *                        MPICH  Jobs
    */
    if (hasAttribute ( JDL::JOBTYPE, JDL_JOBTYPE_MPICH) ){
	// TBD todavia faltan parentesis al cabo y al final (logica booleana)
        if (!hasAttribute(JDL::NODENUMB) )
                throw AdSemanticGroupException (__FILE__ , __LINE__ ,METHOD,WL_JDLMANDATORY, "Nodenumber, MPICH jobs") ;
	// Modify Requirements:
        string buffer  = ""; 
        char NodeNum [1024] ;
        sprintf ( NodeNum, "%i" , getInt(JDL::NODENUMB)) ;

        buffer = "( " + getAttributeExpr (JDL::REQUIREMENTS) + " )" \
                 + " && Member(\"MPICH\", " + schema->get( JobAdSchema::SCHEMA_RTE ) + ")" \
                 + " && " + schema->get( JobAdSchema::SCHEMA_TCPU ) + " >= " + NodeNum;  

	Delete ( JDL::REQUIREMENTS);
	setAttributeExpr (JDL::REQUIREMENTS , buffer) ;
	// Rank attribute value: if not present initialise:
	if (  !hasAttribute (JDL::RANK) ){
		setAttributeExpr(JDL::RANK , schema->get( JobAdSchema::SCHEMA_FCPU )  ) ;
	}

   /**
   *                        INTERACTIVE  Jobs
   */
    }else if (    hasAttribute ( JDL::JOBTYPE, JDL_JOBTYPE_INTERACTIVE ) ){
         if (      (  hasAttribute (JDL::STDINPUT )   )
              ||   (  hasAttribute ( JDL::STDOUTPUT)   )
              || (  hasAttribute (JDL::STDERROR )   )    )
         throw   AdSemanticGroupException(__FILE__ , __LINE__ ,METHOD,WL_JDLGROUP,"StdInput/Ouput/Error attributes not allowed for interactive jobs") ;
         // Interactive Job Specified
         // Update Requirements attribute:
         requirements = getAttributeExpr (JDL::REQUIREMENTS) + " && " + schema->get( JobAdSchema::SCHEMA_OIP );

         Delete(JDL::REQUIREMENTS);
         setAttributeExpr(JDL::REQUIREMENTS , requirements);

         // Update Environment Attributes: ShadowHost & Shadow Port DEPRECATED (Job Instance do it )
         /*
         ExprTree* shExpr = Lookup ( JDL::SHPORT ) ;
         if ( shExpr != NULL ){
              char shPort [1024] ;
              sprintf ( shPort, "%i" , getInt(JDL::SHPORT)) ;
              // int shPort =    getIntValue(JDL::SHPORT)[0] ;
              string envPort  = string (JDL_INTERACTIVE_SHADOWPORT) +"="+string (  shPort ) ;
              addAttribute( JDL::ENVIRONMENT, envPort) ;
         }
         int CHAR_BUFFER_SIZE = 1024;
         char hostName[CHAR_BUFFER_SIZE];
         if ( gethostname(hostName , CHAR_BUFFER_SIZE) )
             throw FatalErrorException (__FILE__ , __LINE__ ,METHOD )  ;
         setAttribute( JDL_INTERACTIVE_LISTENERHOST , string(hostName));
         */

    }else if (  hasAttribute ( JDL::JOBTYPE, JDL_JOBTYPE_CHECKPOINTABLE ) ){
      if (!     hasAttribute(JDL::CHKPT_CURRENTSTEP)   )
               setAttribute(JDL::CHKPT_CURRENTSTEP  , 0) ;
    }
EDG_STACK_CATCH() ;   //Exiting from method: remove line from stack trace
};
/******************************************************************
 PRIVATE method :   restore ()
  restore values inserted by the user which could have been changed by previous
  check() method callls
*******************************************************************/
void JobAd::restore(){
   // JdlAttributeList jdlist ;
   string       buffer="";
   PrettyPrint  unp;
   unp.Unparse(buffer, &user );
   // cout << "\n\nRestoring from : " << buffer ;
   for (unsigned int i = 0 ; i< jdlist.checkAttrs.size() ; i++ ) {
       string attr_name =  jdlist.checkAttrs[i] ;
       ExprTree* tree = user.Lookup (  attr_name  );
       if (tree != NULL){
           // the user has set this attribute, restoring the previous value
           Delete(   attr_name    ) ;
           insertAttribute (attr_name , tree->Copy()) ;
           // cout << " ... done " ;
       }
   }
}
/******************************************************************
 PRIVATE method :   extractFiles
 Extract all files from a specified path
 Since the method is not thread safe resources will be locked while writing-files /executing system commands
*******************************************************************/
vector<string>  JobAd::extractFiles(const string& attr_name , const string& path , vector<string> &extracted){
	string METHOD = "extractFiles(const string& path , const string& path , vector<string> &extracted     )";
	// Check for Environment Variables:
	string mpath= path;
	if ( path.find("$") ==0 ){
	   unsigned int last_char = path.find("/") ;

	   if (last_char > path.size () ) last_char =path.size() -1;
	   // cout << "JobAd::extractFiles adding parts :" << path.substr(1, last_char-1) << endl << flush ;
	   // cout <<"JobAd::extractFiles adding env  :" << getenv ( path.substr(1, last_char-1).c_str() ) << endl << flush ;
	   // cout <<"JobAd::extractFiles adding substr :" << path.substr ( last_char , path.size()-last_char ) << endl << flush ;
	   char* environ =getenv ( path.substr(1, last_char-1).c_str() ) ;
	   if (environ == NULL)
	      throw AdSemanticPathException (__FILE__ , __LINE__ ,METHOD, WL_JDLMISMATCH, attr_name , path);
	   mpath= string(getenv ( path.substr(1, last_char-1).c_str() ) ) + path.substr ( last_char , path.size()-last_char);
	   // cout << "JobAd::extractFiles >Mpath changed: " << mpath << endl << flush ;
	}
	vector<string> simple;
	glob_t  *pglob ;
	pglob = (glob_t  *) malloc (sizeof(glob_t)) ;
	// Retrieve files from path
	int gl = glob(   mpath.c_str(), GLOB_ERR ,  NULL, pglob);
	if  (gl!=0)
               throw AdSemanticPathException (__FILE__ , __LINE__ ,METHOD, WL_JDLMISMATCH, attr_name , path);
	// Retrieve the absolute current path
	char *cwd = NULL;
	char *buff ;
	long size ;
	size = pathconf(".", _PC_PATH_MAX);
	if ((buff = (char *)malloc((size_t)size)) != NULL)
		cwd = getcwd(buff, (size_t)size);
	if (buff == NULL)
		throw AdSemanticPathException (__FILE__ , __LINE__ ,METHOD, WL_JDLMISMATCH, attr_name , path);
	string abs = string (buff) ;
	vector<string>::iterator iter ;
	for  (int i=0 ; i<(int) pglob->gl_pathc ; i++)  {
		string attr_value =pglob->gl_pathv[i] ;
		if (attr_value.find("./") ==0)
			attr_value = abs + attr_value.substr(1) ;
		if (attr_value.find("/") !=0)
			attr_value = abs +"/"+  attr_value ;
			// attr_value.replace("./" , abs) ;
		// cout << "Adding this:" <<  attr_value << endl ;
		// Check if path has been repeated
		for ( iter=extracted.begin() ; iter != extracted.end() ; ++iter ){
			      if (   *iter == attr_value.substr( attr_value.find_last_of("/")+1   )){
			         simple.push_back    ( attr_value.substr( attr_value.find_last_of("/")+1    ) );
				 break;
			      }
		}
		// cout << "found to add:" << attr_value ;
		extracted.push_back(  attr_value  );
		if (  simple.size() >  0 )
		         for ( iter=simple.begin(); iter != simple.end(); ++iter )
				throw AdSemanticPathException (__FILE__ , __LINE__ , METHOD , WL_JDLDUPLICATE, attr_name , *iter );
        }
        //globfree(pglob);       // 36 bytes lose
        free (pglob) ;            //   valgrind lose 32 bytes
	free (buff);
   return  extracted;
};
/******************
* getName (const std::string &path , bool platform_dependence  )
* return the simple filename of the path
******************/
const std::string JobAd::getName (const std::string &path , bool platform_dependence  )  {
  int unix_sep = path.find_last_of (UNIX_SEP );
  int win_sep = path.find_last_of ( WIN_SEP );
  // TBD check it !!! find may not return negative number while fail
  if ( unix_sep + win_sep == -2)
        //no sep found, return the simple file
        return path ;
  if (platform_dependence){
     if (  unix_sep >  win_sep)
        return path.substr ( unix_sep) ;
     else
        return path.substr ( win_sep) ;
  }
#ifndef WIN
  // Linux Unix Check
  return   path.substr ( unix_sep)    ;
#else
  // Windows Check
  return path.substr ( win_sep) ;
#endif
};



/**
* This method verifies wheter the specified path is absolute or not
* if the optional platform_dependence is set to false then the check doesn't depend on the system operator running
*/
bool JobAd::isAbsolutePath(const string &path , bool platform_dependence ) {
  if ( path.empty() )
     return false;
  // platform un-dependendent check
  if (! platform_dependence){
     int unix_sep = path.find (UNIX_SEP  );
     int win_sepa = path.find ( WIN_SEP );
     int win_sepb = path.find (":\\" );
     if (unix_sep > (int)path.length() -1)
        unix_sep = -1 ;
     if (  win_sepa> (int) path.length() -1)
         win_sepa= -1 ;
     if ( win_sepb> (int)path.length() -1)
         win_sepb= -1 ;
     if ( unix_sep==0 && win_sepa<0 )
        return true ;
     if ( win_sepa==0 && unix_sep <0 )
        return true ;
     if ( win_sepb==1 && unix_sep <0 )
        return true ;
     // if this poiunt is reached no absolute path found:
     return false;
  }
#ifdef WIN
	// Windows Check
	return ( path.find (WIN_SEP)==1 || path.find ("\\")==0);
#else
        // Linux Unix Check
	return (    path.find ( UNIX_SEP ) == 0   ) ;
#endif

return false;
}

} // requestad namespace
COMMON_NAMESPACE_END } //Closing  UserInterFace NameSpace
