/* **************************************************************************
*  filename  : JobAdExecptions.cpp
*  author    : Alessandro Maraschini <alessandro.maraschini@datamat.it>
*  copyright : (C) 2002 by DATAMAT
***************************************************************************/
#include "edg/workload/common/requestad/JobAdExceptions.h"
#include "edg/workload/common/requestad/exception_codes.h"
using namespace std ;
COMMON_NAMESPACE_BEGIN{
namespace requestad{


string JobAdException::what() throw(){
     if (error_description!="")
            error_message += "\n" + error_description ;
     if ( error_message.empty() )
        return "";
     else
        return error_message.c_str() ;
}


JobAdException::JobAdException (string file,
                                   int line,
                                   string method,
                                   int code,
                                   string exception_name):
   Exception(file,line,method,code , exception_name ){};

AdClassAdException::AdClassAdException (string file,
                                      int line,
                                      string method,
                                      int code ,
                                      string method_name,
                                      std::string error_description ):
  JobAdException(file , line , method , code , "AdClassAdException"){
      error_message = method_name+ ": ClassAd returned syntax error";
      if (error_description!= "") error_message +=" for " + error_description ;
  }
AdSyntaxException::AdSyntaxException (string file,
                                      int line,
                                      string method,
                                      int code ,
                                      string attr_name ):
  JobAdException(file , line , method , code, "AdSyntaxException"){
    switch (code){
      case WL_JDL_MEMBER:
         error_message = attr_name+ ": wrong usage of MultiAttribute function.";
         error_message +=" Syntax is: Member/IsMember(Value,List)";
         break;
      case WL_JDLPARSE:
         error_message = "The following parsing error(s) have been found:\n" + attr_name;
         break;
      case WL_DAG_CHECK:
         error_message = "The following error(s) have been found while checking the dag nodes:\n" + attr_name;
         break;	 
      default:
         error_message = attr_name+ ": syntax Error caught";
    }
 }
AdSemanticException::AdSemanticException (string file,
                                      int line,
                                      string method,
                                      int code,
                                      string exception_name ):
  JobAdException(file , line , method , code  , exception_name){
  }


AdSemanticMandatoryException::AdSemanticMandatoryException (string file,
                                        int line,
                                        string method,
                                        int code,
                                        string attr_name ):
        AdSemanticException(file , line , method , code, "AdSemanticMandatoryException"){  // WL_JDLMANDATORY err code
        error_message = attr_name + ": Jdl mandatory attribute is missing";
        }

AdSemanticPathException::AdSemanticPathException (string file,
                                        int line,
                                        string method,
                                        int code,
                                        string attr_name,
					string path_name ):
        AdSemanticException(file , line , method , code, "AdSemanticPathException"){
            switch (code){
            case  WL_JDLDUPLICATE:
	        error_message = attr_name + ":  filename conflict found while extracting files. The following file is repeated more than once: " + path_name;
                break;
            default: // WL_JDLMANDATORY err code
	        error_message = attr_name + ": Jdl specified path '"+path_name+"' is missing";
                break;
            }
        }

AdSemanticGroupException::AdSemanticGroupException (string file,
                                        int line,
                                        string method,
                                        int code,
                                        string attrs_name ):
        AdSemanticException(file , line , method , code, "AdSemanticGroupException"){ // WL_JDLMANDATORY err code
            switch (code){
            case  WL_JDLMANDATORY:
	       error_message = attrs_name+ ": attributes group not complete, the attributes have to be specified all togheter";
               break;
            default: // WL_JDLGROUP err code
               error_message = attrs_name+ ": wrong combination of values" ;
               break;
            }
        }

AdAttributeException::AdAttributeException (string file,
                                      int line,
                                      string method,
                                      int code,
                                      string exception_name ):
      JobAdException(file , line , method , code  , exception_name){
      }

AdMismatchException::AdMismatchException (string file,
                                        int line,
                                        string method,
                                        int code,
                                        string attr_name,
                                        string error_description ):
	AdAttributeException(file , line , method , code, "AdMismatchException"){  // WL_JDLMISMATCH ,WL_JDLSYN  err codes
		switch (code){
		case  WL_JDLSYN:
			error_message = attr_name + ": wrong value caught for attribute" ;
			break;
		default:  // WL_JDLMISMATCH
		error_message = attr_name + ": wrong type caught for attribute" ;
		}
		if (error_description!="") error_message += " (" + error_description +")";
	}
AdFormatException::AdFormatException(string file,
                                        int line,
                                        string method,
                                        int code,
                                        string attr_name,
                                        string format ):
AdAttributeException(file , line , method , code, "AdFormatException"){ // WL_JDLMISMATCH err  code
	switch (code){
		case WL_JDLMISMATCH:
			error_message = attr_name + ": wrong format caught for attribute" ;
			error_message += ". The right format is:\n'" +format+"'" ;
			break;
		default : // WL_JOBADSCHEMA:
			error_message= "JobAd Schema constructor dimension exceeded" ;
	}
}


AdListException::AdListException(string file,
                                        int line,
                                        string method,
                                        int code,
                                        string attr_name ):
        AdAttributeException(file , line , method , code, "AdListException"){
	switch (code){
	case  WL_JDLSYN:
             error_message = attr_name +": the attribute cannot be of list type";
	     break;
	default: // EPERM error code
             error_message = attr_name +": the attribute is of list type, operation not allowed";
	}
        }


AdEmptyException::AdEmptyException(string file,
                                        int line,
                                        string method,
                                        int code,
                                        string attr_name ):
        AdAttributeException(file , line , method , code, "AdEmptyException"){
            error_message = attr_name + ": unable to complete the operation: the attribute" ;
            switch (code){
            case  WL_JDLEMPTY:
                error_message += " has not been initialised yet";
                break;
            default:
                error_message +=" has already been set";
                break;
            }
}
} COMMON_NAMESPACE_END }

