// $Id: hlrFund.cpp,v 1.1 2003/04/04 10:29:54 guarise Exp $
// -------------------------------------------------------------------------
// Copyright (c) 2001-2002, The DataGrid project, INFN, 
// All rights reserved. See LICENSE file for details.
// -------------------------------------------------------------------------
// Authors: Andrea Guarise <andrea.guarise@to.infn.it>
/***************************************************************************
 * Code borrowed from:
 *  authors   : 
 *             
 *  
 ***************************************************************************/


#include<hlrFund.h>

#define E_GET_FDESC 	1
#define E_GET_ROLLBACK 	3
#define E_PUT_FDESC	4
#define E_DEL_FDESC	6
#define E_GET_KEYS 	7
#define E_CREDIT	8
#define E_CREDIT_FDESC	9
#define E_DEBIT		10
#define E_DEBIT_FDESC	11

extern const char * hlr_sql_server;
extern const char * hlr_sql_user;
extern const char * hlr_sql_password;
extern const char * hlr_sql_dbname;

hlrFund::hlrFund( string _fid,
		string _descr,
		int _total,
		int _spent
		)
{
	fid = _fid;
	descr = _descr;
	total = _total;
	spent = _spent;
	#ifdef DEBUG
	 cerr << "hlrFund::hlrFund():fid=" << fid;
	 cerr << "descr=" << descr;
	 cerr << "total=" << total;
	 cerr << "spent=" << spent;
	#endif
}


int hlrFund::get()
{
	hlrFdesc FdescBuff(fid,descr);
	if ( FdescBuff.get() != 0 )
	{
		#ifdef DEBUG
		 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 cerr << "Couldn't get hlrFdesc info from the Database." << endl;
		#endif
		 return E_GET_FDESC;
	}
	fid = FdescBuff.fid;
	descr = FdescBuff.descr;
	total = FdescBuff.total;
	spent = FdescBuff.spent;
	return 0;
}


int hlrFund::get(vector<hlrFund>& fv)
{
	vector<hlrFdesc> Fdescv;
	hlrFdesc FdescBuff(fid,descr);
	//populate the Fdescv vector with the matching records
	if ( FdescBuff.get(Fdescv) != 0 )
	{
		#ifdef DEBUG
		 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 cerr << "Couldn't get hlrFdesc info from the Database." << endl;
		#endif
		 return E_GET_FDESC;
	}
	//get every Ug record corresponding to the matching Udesc records
	vector<hlrFdesc>::iterator it = Fdescv.begin();
	while ( it != Fdescv.end() )
	{
	
		hlrFund f(
				(*it).fid,
				(*it).descr,
				(*it).total,
				(*it).spent
						);
		fv.push_back(f);
		it++;	
	}
	return 0;
}


int hlrFund::put()
{
	hlrFdesc FdescBuff(fid,descr,total,spent);
	if ( FdescBuff.put() != 0 )
	{
		#ifdef DEBUG
		cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
	 	cerr << "Couldn't put Fdesc info in the Database." << endl;
		#endif
		return E_PUT_FDESC;
		
	}
	//ok, everithing went fine.
	#ifdef DEBUG
	 cerr << "DEBUG:hlrFund::put():";
	 cerr << "fid="<< fid;
	 cerr << "descr="<< descr;
	 cerr << "total="<< total;
	 cerr << "spent="<< spent << endl;
	#endif
	return 0;
}

int hlrFund::del()
{
	hlrFdesc FdescBuff(fid);
	if ( FdescBuff.del() != 0 )
	{
		#ifdef DEBUG
		cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
	 	cerr << "Couldn't delete Fdesc info in the Database." << endl;
		#endif
		return E_DEL_FDESC;
	}
	//ok, everithing went fine.
	return 0;
}

bool hlrFund::exists()
{
	hlrFdesc FdescBuff(fid,descr);
	return FdescBuff.exists();
}

int hlrFund::getKeys(vector<string>& keys)
{
	hlrFdesc FdescBuff(fid,descr);
	//populate the Udescv vector with the matching records
	if ( FdescBuff.getKeys(keys) != 0 )
	{
		#ifdef DEBUG
		 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 cerr << "Couldn't get hlrFdesc info from the Database." << endl;
		#endif
		 return E_GET_KEYS;
	}
	return 0;
}

int hlrFund::credit(int v)
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING " << __BASE_FILE__  <<"::credit()..." << endl;
	#endif
	if ( fid == "" )
	{
		#ifdef DEBUG
		cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		cerr << "In the credit method fid is a mandatory info." << endl;
		#endif
		#ifdef DEBUG
		cerr << "DEBUG: EXITING "<< __BASE_FILE__ <<"::credit()." << endl << endl;
		#endif
		return E_CREDIT;
	}
	hlrFdesc fBuff(fid);
	if ( fBuff.credit(v) != 0 )
	{
		#ifdef DEBUG
		 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 cerr << "Couldn't update info in the Database." << endl;
		#endif
		 //error updating the info, since nothing has been
		 //modified yet we can simply return with an error
		#ifdef DEBUG
		 cerr << "DEBUG: EXITING "<< __BASE_FILE__ <<"::credit()." << endl << endl;
		#endif
		 return E_CREDIT_FDESC;
	}
	#ifdef DEBUG
	cerr << "DEBUG: EXITING "<< __BASE_FILE__ <<"::credit()." << endl << endl;
	#endif
	return 0;
}

int hlrFund::debit(int v)
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING " << __BASE_FILE__  <<"::debit()..." << endl;
	#endif
	if ( fid == "" )
	{
		#ifdef DEBUG
		cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		cerr << "In the credit method fid is a mandatory info." << endl;
		#endif
		#ifdef DEBUG
		cerr << "DEBUG: EXITING "<< __BASE_FILE__ <<"::debit()." << endl << endl;
		#endif
		return E_DEBIT;
	}
	hlrFdesc fBuff(fid);
	if ( fBuff.debit(v) != 0 )
	{
		#ifdef DEBUG
		 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 cerr << "Couldn't update info in the Database." << endl;
		#endif
		 //error updating the info, since nothing has been
		 //modified yet we can simply return with an error
		#ifdef DEBUG
		 cerr << "DEBUG: EXITING "<< __BASE_FILE__ <<"::debit()." << endl << endl;
		#endif
		 return E_DEBIT_FDESC;
	}
	#ifdef DEBUG
	cerr << "DEBUG: EXITING "<< __BASE_FILE__ <<"::debit()." << endl << endl;
	#endif
	return 0;
}
