// $Id: hlrGenericAccount.cpp,v 1.2 2003/12/02 12:00:57 guarise Exp $
// -------------------------------------------------------------------------
// Copyright (c) 2001-2002, The DataGrid project, INFN, 
// All rights reserved. See LICENSE file for details.
// -------------------------------------------------------------------------
// Authors: Andrea Guarise <andrea.guarise@to.infn.it>
/***************************************************************************
 * Code borrowed from:
 *  authors   : 
 *             
 *  
 ***************************************************************************/


//TODO log db events in the log file.

#include<hlrGenericAccount.h>

#define E_GET_ADESC 	1
#define E_GET_UG 	2
#define E_GET_ROLLBACK 	3
#define E_PUT_UG	4
#define E_ROLLBACK 	5
#define E_DEL_UG	6
#define E_GET_KEYS 	7
#define E_DEL 		8
#define E_CREDIT 	9
#define E_CREDIT_UG	10
#define E_DEBIT 	11
#define E_DEBIT_UG	12
#define E_PUT_ADESC 	13

#define PRTDBG "DEBUG:"<<__FILE__<<":"<<__LINE__<<":"

extern const char * hlr_sql_server;
extern const char * hlr_sql_user;
extern const char * hlr_sql_password;
extern const char * hlr_sql_dbname;

hlrGenericAccount::hlrGenericAccount( string _id,
		string _type,
		string _email,
		string _descr,
		string _certSubject
		)
{
	id = _id;
	type = _type;
	email = _email;
	descr = _descr;
	certSubject = _certSubject;
	#ifdef DEBUG
	 cerr << "hlrGenericAccount::hlrGenericAccount():id=" << id;
	 cerr << ",type=" << email;
	 cerr << ",email=" << email;
	 cerr << ",descr=" << descr;
	 cerr << ",certSubject=" << certSubject << endl;
	#endif
}


int hlrGenericAccount::get()
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrGenericAccount::get()..." << endl;
	#endif
	hlrAcctdesc AcctdescBuff(id,type,email,descr,certSubject);
	if ( AcctdescBuff.get() != 0 )
	{
		#ifdef DEBUG
		 cerr << PRTDBG << "Couldn't get hlrAcctdesc info from the Database." << endl;
		#endif
		 return E_GET_ADESC;
	}
	#ifdef DEBUG
	 cerr << "DEBUG:" << AcctdescBuff << endl;
	#endif
	id = AcctdescBuff.id;
	type = AcctdescBuff.type;
	email = AcctdescBuff.email;
	descr = AcctdescBuff.descr;
	certSubject = AcctdescBuff.certSubject;
	return 0;
}

int hlrGenericAccount::get(vector<hlrGenericAccount>& av)
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrGenericAccount::get()..." << endl;
	#endif
	vector<hlrAcctdesc> Acctdescv;
	hlrAcctdesc AcctdescBuff(id,type,email,descr,certSubject);
	//populate the Acctdescv vector with the matching records
	if ( AcctdescBuff.get(Acctdescv) != 0 )
	{
		#ifdef DEBUG
		 cerr << PRTDBG << "Couldn't get hlrAcctdesc info from the Database." << endl;
		#endif
		 return E_GET_ADESC;
	}
	//get every Ug record corresponding to the matching Acctdesc records
	vector<hlrAcctdesc>::iterator it = Acctdescv.begin();
	while ( it != Acctdescv.end() )
	{
			hlrGenericAccount u(
					(*it).id,
					(*it).type,
					(*it).email,
					(*it).descr,
					(*it).certSubject
							);
			av.push_back(u);
		it++;	
	}
	return 0;
}

int hlrGenericAccount::put()
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrGenericAccount::put()..." << endl;
	#endif
	hlrAcctdesc AcctdescBuff(id,type,email,descr,certSubject);
	if ( AcctdescBuff.put() != 0 )
	{
		#ifdef DEBUG
		 cerr << PRTDBG << "Error inserting account" << endl;
		#endif	
		 return E_PUT_ADESC;
	}
	//ok, everithing went fine.
	#ifdef DEBUG
	 cerr << "DEBUG:hlrGenericAccount::put():";
	 cerr << ",id="<< id;
	 cerr << ",type="<< type;
	 cerr << ",email="<< email;
	 cerr << ",descr="<< descr;
	 cerr << ",certSubject="<< certSubject << endl;
	cerr << "DEBUG: EXITING hlrGenericAccount::put()." << endl << endl;
	#endif
	return 0;
}

int hlrGenericAccount::del()
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrGenericAccount::del()..." << endl;
	#endif
	if ( id == "" )
	{
		#ifdef DEBUG
		 	cerr << PRTDBG << "In the delete method uid, gid and fid are mandatory info." << endl;
			cerr << "DEBUG: EXITING hlrGenericAccount::del()." << endl << endl;
		#endif
		 return E_DEL;
	}
	hlrAcctdesc AcctdescBuff(id,type);
	if ( AcctdescBuff.del() != 0 )
	{
		#ifdef DEBUG
			cerr << PRTDBG << "Error deleting the record" << endl;
			return E_DEL;
		#endif	
	}
	//ok, everithing went fine.
	#ifdef DEBUG
	cerr << "DEBUG: EXITING hlrGenericAccount::del()." << endl << endl;
	#endif
	return 0;
}

bool hlrGenericAccount::exists()
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrGenericAccount::exists()..." << endl;
	#endif
	hlrAcctdesc AcctdescBuff(id,type,email,descr,certSubject);
	return AcctdescBuff.exists();
}

int hlrGenericAccount::getKeys(vector<string>& keys)
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrGenericAccount::getKeys()..." << endl;
	#endif
	hlrAcctdesc AcctdescBuff(id,type,email,descr,certSubject);
	//populate the Acctdescv vector with the matching records
	if ( AcctdescBuff.getKeys(keys) != 0 )
	{
		#ifdef DEBUG
		 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 cerr << "Couldn't get hlrAcctdesc info from the Database." << endl;
		#endif
		#ifdef DEBUG
		cerr << "DEBUG: EXITING hlrGenericAccount::getKeys()." << endl << endl;
		#endif
		 return E_GET_KEYS;
	}
		#ifdef DEBUG
		cerr << "DEBUG: EXITING hlrGenericAccount::getKeys()." << endl << endl;
		#endif
	return 0;
}

ostream& operator << ( ostream& os, const hlrGenericAccount& u )
{
	os << "id=" << u.id;
	os << "type=" << u.type;
	os << ",email=" << u.email;
	os << ",descr=" << u.descr;
	os << ",certSubject=" << u.certSubject;
	return os;

}



