// $Id: hlrGroup.cpp,v 1.1 2003/04/04 10:29:54 guarise Exp $
// -------------------------------------------------------------------------
// Copyright (c) 2001-2002, The DataGrid project, INFN, 
// All rights reserved. See LICENSE file for details.
// -------------------------------------------------------------------------
// Authors: Andrea Guarise <andrea.guarise@to.infn.it>
/***************************************************************************
 * Code borrowed from:
 *  authors   : 
 *             
 *  
 ***************************************************************************/


#include<hlrGroup.h>

#define E_GET_GRDESC 	1
#define E_GET_GF 	2
#define E_GET_ROLLBACK 	3
#define E_PUT_GF	4
#define E_ROLLBACK 	5
#define E_DEL_GF	6
#define E_GET_KEYS 	7
#define E_CREDIT 	8
#define E_CREDIT_GF 	9
#define E_DEBIT 	10
#define E_DEBIT_GF 	11

extern const char * hlr_sql_server;
extern const char * hlr_sql_user;
extern const char * hlr_sql_password;
extern const char * hlr_sql_dbname;

hlrGroup::hlrGroup( string _gid,
		string _descr,
		string _fid,
		int _total,
		int _booked,
		int _spent
		)
{
	gid = _gid;
	descr = _descr;
	fid = _fid;
	total = _total;
	booked = _booked;
	spent = _spent;
	#ifdef DEBUG
	 cerr << "hlrUser::hlrUser():gid=" << gid;
	 cerr << "descr=" << descr;
	 cerr << "fid=" << fid;
	 cerr << "total=" << total;
	 cerr << "booked=" << booked;
	 cerr << "spent=" << spent;
	#endif
}


int hlrGroup::get()
{
	hlrGrdesc GrdescBuff(gid,descr);
	if ( GrdescBuff.get() != 0 )
	{
		#ifdef DEBUG
		 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 cerr << "Couldn't get hlrGrdesc info from the Database." << endl;
		#endif
		 return E_GET_GRDESC;
	}
	hlrGf	 GfBuff(gid,fid,total,booked,spent);
	if ( GfBuff.get() != 0 )
	{
		#ifdef DEBUG
		 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 cerr << "Couldn't get hlrUg info from the Database." << endl;
		#endif
		 return E_GET_GF;
	}
	gid = GrdescBuff.gid;
	descr = GrdescBuff.descr;
	fid = GfBuff.fid;
	total = GfBuff.total;
	booked = GfBuff.booked;
	spent = GfBuff.spent;
	return 0;
}

int hlrGroup::get(vector<hlrGroup>& gv)
{
	vector<hlrGrdesc> Grdescv;
	hlrGrdesc GrdescBuff(gid,descr);
	//populate the Udescv vector with the matching records
	if ( GrdescBuff.get(Grdescv) != 0 )
	{
		#ifdef DEBUG
		 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 cerr << "Couldn't get hlrGrdesc info from the Database." << endl;
		#endif
		 return E_GET_GRDESC;
	}
	//get every Ug record corresponding to the matching Udesc records
	vector<hlrGrdesc>::iterator it = Grdescv.begin();
	while ( it != Grdescv.end() )
	{
		vector<hlrGf> Gfv;
		hlrGf    GfBuff((*it).gid);
	
		if ( GfBuff.get(Gfv) != 0 )
		{
			#ifdef DEBUG
			cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 	cerr << "Couldn't get hlrGf info from the Database." << endl;
			#endif
		 	return E_GET_GF;
		}
		//Now we have the info.	
		vector<hlrGf>::iterator GfvIt = Gfv.begin();
		while ( GfvIt != Gfv.end() )
		{
			hlrGroup g(
					(*it).gid,
					(*it).descr,
					(*GfvIt).fid,
					(*GfvIt).total,
					(*GfvIt).booked,
					(*GfvIt).spent
							);
			gv.push_back(g);
			GfvIt++;
		}
		it++;	
	}
	return 0;
}


int hlrGroup::put()
{
	hlrGf gfBuff(gid,fid,total,booked,spent); 
	hlrGf gfRollBack = gfBuff; // Buffer to store Ug rollback
	//check if this Ug record already exists;
	bool GfExists = gfBuff.exists();
	if ( GfExists )
	{
		// store info for rollback in case of error
		if ( gfRollBack.get() != 0 )
		{
			#ifdef DEBUG
			cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 	cerr << "Couldn't get rollback info from the Database." << endl;
			#endif
		 	return E_GET_ROLLBACK;
		}
		// we have info for eventual rollback	
	}
	if ( gfBuff.put() != 0 )
	{
			#ifdef DEBUG
			cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 	cerr << "Couldn't put info in the Database." << endl;
			#endif
			//error inserting the info, since nothing has been
			//modified yet we can simply return with an error
			return E_PUT_GF;
	}
	// The gf has been inserted/upadted correctly, go on inserting the 
	// Udesc.
	hlrGrdesc GrdescBuff(gid,descr);
	if ( GrdescBuff.put() != 0 )
	{
		//there was an error inserting/updating the Udesc,
		//if there was already an UG entry it's better to roll 
		//back the UG transaction
		if ( GfExists )
		{
			//roll back
			if ( gfRollBack.put() != 0 )
			{
				//Oh oh, can't roll back!
				#ifdef DEBUG
				cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 		cerr << "Couldn't rollback info in the Database." << endl;
				#endif
				return E_ROLLBACK;
				
			}
		}
		
	}
	//ok, everithing went fine.
	#ifdef DEBUG
	 cerr << "DEBUG:hlrGroup::put():";
	 cerr << "gid="<< gid;
	 cerr << "fid="<< fid;
	 cerr << "descr="<< descr;
	 cerr << "total="<< total;
	 cerr << "booked="<< booked;
	 cerr << "spent="<< spent << endl;
	#endif
	return 0;
}

int hlrGroup::del()
{
	hlrGf gfBuff(gid,fid); 
	hlrGf gfRollBack = gfBuff; // Buffer to store Ug rollback
	//check if this Gf record already exists;
	bool GfExists = gfBuff.exists();
	if ( GfExists )
	{
		// store info for rollback in case of error
		if ( gfRollBack.get() != 0 )
		{
			#ifdef DEBUG
			cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 	cerr << "Couldn't get rollback info from the Database." << endl;
			#endif
		 	return E_GET_ROLLBACK;
		}
		// we have info for eventual rollback	
	}
	if ( gfBuff.del() != 0 )
	{
			#ifdef DEBUG
			cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 	cerr << "Couldn't delete info in the Database." << endl;
			#endif
			//error deleting the info, since nothing has been
			//modified yet we can simply return with an error
			return E_DEL_GF;
	}
	// The gf has been deleted correctly, go on deleting the 
	// Udesc.
	hlrGrdesc GrdescBuff(gid);
	if ( GrdescBuff.del() != 0 )
	{
		//there was an error deleting the Grdesc,
		//if there was already an GF entry it's better to roll 
		//back the GF transaction
		if ( GfExists )
		{
			//roll back
			if ( gfRollBack.put() != 0 )
			{
				//Oh oh, can't roll back!
				#ifdef DEBUG
				cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 		cerr << "Couldn't rollback info in the Database." << endl;
				#endif
				return E_ROLLBACK;
				
			}
		}
		
	}
	//ok, everithing went fine.

	return 0;
}

bool hlrGroup::exists()
{
	hlrGrdesc GrdescBuff(gid,descr);
	hlrGf	 GfBuff(gid,fid);
	return (GrdescBuff.exists() && GfBuff.exists());
}

int hlrGroup::getKeys(vector<string>& keys)
{
	hlrGrdesc GrdescBuff(gid,descr);
	//populate the Udescv vector with the matching records
	if ( GrdescBuff.getKeys(keys) != 0 )
	{
		#ifdef DEBUG
		 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 cerr << "Couldn't get hlrUdesc info from the Database." << endl;
		#endif
		 return E_GET_KEYS;
	}
	return 0;
}

int hlrGroup::credit(int v)
{
	#ifdef DEBUG
        cerr << endl <<"DEBUG: ENTERING " << __BASE_FILE__  <<"::credit()..." << endl;
        #endif
	if ( gid == "" || fid == "" )
	{
               #ifdef DEBUG
               cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
               cerr << "In the credit method gid and fid are mandatory info." << endl;
		#endif
                #ifdef DEBUG 
                cerr << "DEBUG: EXITING "<< __BASE_FILE__ <<"::credit()." << endl << endl;
	        #endif
	        return E_CREDIT;
	}
	hlrGf gfBuff(gid,fid);
	if ( gfBuff.credit(v) != 0 )
	{
		#ifdef DEBUG
	        cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
	        cerr << "Couldn't update info in the Database." << endl;
	        #endif
		//error updating the info, since nothing has been
		//modified yet we can simply return with an error
		#ifdef DEBUG
                cerr << "DEBUG: EXITING "<< __BASE_FILE__ <<"::credit()." << endl << endl;
                #endif
		return E_CREDIT_GF;					
	}
	#ifdef DEBUG
        cerr << "DEBUG: EXITING "<< __BASE_FILE__ <<"::credit()." << endl << endl;
        #endif
	return 0;
}

int hlrGroup::debit(int v)
{
	#ifdef DEBUG
        cerr << endl <<"DEBUG: ENTERING " << __BASE_FILE__  <<"::debit()..." << endl;
        #endif
	if ( gid == "" || fid == "" )
	{
               #ifdef DEBUG
               cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
               cerr << "In the debit method gid and fid are mandatory info." << endl;
		#endif
                #ifdef DEBUG 
                cerr << "DEBUG: EXITING "<< __BASE_FILE__ <<"::debit()." << endl << endl;
	        #endif
	        return E_DEBIT;
	}
	hlrGf gfBuff(gid,fid);
	if ( gfBuff.debit(v) != 0 )
	{
		#ifdef DEBUG
	        cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
	        cerr << "Couldn't update info in the Database." << endl;
	        #endif
		//error updating the info, since nothing has been
		//modified yet we can simply return with an error
		#ifdef DEBUG
                cerr << "DEBUG: EXITING "<< __BASE_FILE__ <<"::debit()." << endl << endl;
                #endif
		return E_DEBIT_GF;					
	}
	#ifdef DEBUG
        cerr << "DEBUG: EXITING "<< __BASE_FILE__ <<"::debit()." << endl << endl;
        #endif
	return 0;
}
