// $Id: hlrResource.cpp,v 1.1 2003/04/04 10:29:54 guarise Exp $
// -------------------------------------------------------------------------
// Copyright (c) 2001-2002, The DataGrid project, INFN, 
// All rights reserved. See LICENSE file for details.
// -------------------------------------------------------------------------
// Authors: Andrea Guarise <andrea.guarise@to.infn.it>
/***************************************************************************
 * Code borrowed from:
 *  authors   : 
 *             
 *  
 ***************************************************************************/


#include<hlrResource.h>

#define E_GET_ADESC 	1
#define E_GET_RGF 	2
#define E_GET_ROLLBACK 	3
#define E_PUT_RGF	4
#define E_ROLLBACK 	5
#define E_DEL_RGF	6
#define E_GET_KEYS 	7
#define E_DEL 		8
#define E_CREDIT 	9
#define E_CREDIT_RGF	10
#define E_DEBIT 	11
#define E_DEBIT_RGF	12

extern const char * hlr_sql_server;
extern const char * hlr_sql_user;
extern const char * hlr_sql_password;
extern const char * hlr_sql_dbname;

hlrResource::hlrResource( string _rid,
		string _email,
		string _descr,
		string _certSubject,
		string _gid,
		string _fid,
		int _total
		)
{
	rid = _rid;
	email = _email;
	descr = _descr;
	certSubject = _certSubject;
	gid = _gid;
	fid = _fid;
	total = _total;
	#ifdef DEBUG
	 cerr << "hlrResource::hlrResource():rid=" << rid;
	 cerr << ",descr=" << descr;
	 cerr << ",certSubject=" << certSubject;
	 cerr << ",gid=" << gid;
	 cerr << ",fid=" << fid;
	 cerr << ",total=" << total << endl;
	#endif
}


int hlrResource::get()
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrResource::get()..." << endl;
	#endif
	hlrAcctdesc AcctdescBuff(rid,"resource",email,descr,certSubject);
	if ( AcctdescBuff.get() != 0 )
	{
		#ifdef DEBUG
		 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 cerr << "Couldn't get hlrAcctdesc info from the Database." << endl;
		#endif
		#ifdef DEBUG
		cerr << "DEBUG: EXITING hlrResource::get()." << endl << endl;
		#endif
		 return E_GET_ADESC;
	}
	hlrRgf	 RgfBuff(AcctdescBuff.id,gid,fid);
	if ( RgfBuff.get() != 0 )
	{
		#ifdef DEBUG
		 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 cerr << "Couldn't get hlrRgf info from the Database." << endl;
		#endif
		#ifdef DEBUG
		cerr << "DEBUG: EXITING hlrResource::get()." << endl << endl;
		#endif
		 return E_GET_RGF;
	}
	rid = AcctdescBuff.id;
	email = AcctdescBuff.email;
	descr = AcctdescBuff.descr;
	certSubject = AcctdescBuff.certSubject;
	gid = RgfBuff.gid;
	fid = RgfBuff.fid;
	total = RgfBuff.total;
	#ifdef DEBUG
	cerr << "DEBUG: EXITING hlrResource::get()." << endl << endl;
	#endif
	return 0;
}

int hlrResource::get(vector<hlrResource>& rv)
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrResource::get()..." << endl;
	#endif
	vector<hlrAcctdesc> Adescv;
	hlrAcctdesc AcctdescBuff(rid,"resource",email,descr,certSubject);
	//populate the Udescv vector with the matching records
	if ( AcctdescBuff.get(Adescv) != 0 )
	{
		#ifdef DEBUG
		 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 cerr << "Couldn't get hlrAcctdesc info from the Database." << endl;
		#endif
		#ifdef DEBUG
		cerr << "DEBUG: EXITING hlrResource::get()." << endl << endl;
		#endif
		 return E_GET_ADESC;
	}
	//get every Rgf record corresponding to the matching Rdesc records
	vector<hlrAcctdesc>::iterator it = Adescv.begin();
	while ( it != Adescv.end() )
	{
		vector<hlrRgf> Rgfv;
		hlrRgf    RgfBuff((*it).id);
	
		if ( RgfBuff.get(Rgfv) != 0 )
		{
			#ifdef DEBUG
			cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 	cerr << "Couldn't get hlrRgf info from the Database." << endl;
			#endif
			#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrResource::get()." << endl << endl;
			#endif
		 	return E_GET_RGF;
		}
		//Now we have the info.	
		vector<hlrRgf>::iterator RgfvIt = Rgfv.begin();
		while ( RgfvIt != Rgfv.end() )
		{
			hlrResource r(
					(*it).id,
					(*it).email,
					(*it).descr,
					(*it).certSubject,
					(*RgfvIt).gid,
					(*RgfvIt).fid,
					(*RgfvIt).total
							);
			rv.push_back(r);
			RgfvIt++;
		}
		it++;	
	}
	#ifdef DEBUG
	cerr << "DEBUG: EXITING hlrResource::get()." << endl << endl;
	#endif
	return 0;
}

int hlrResource::put()
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrResource::put()..." << endl;
	#endif
	hlrRgf rgfBuff(rid,gid,fid,total); 
	hlrRgf rgfRollBack = rgfBuff; // Buffer to store Rgf rollback
	//check if this Rgf record already exists;
	bool RgfExists = rgfBuff.exists();
	if ( RgfExists )
	{
		// store info for rollback in case of error
		if ( rgfRollBack.get() != 0 )
		{
			#ifdef DEBUG
			cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 	cerr << "Couldn't get rollback info from the Database." << endl;
			#endif
			#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrResource::put()." << endl << endl;
			#endif
		 	return E_GET_ROLLBACK;
		}
		// we have info for eventual rollback	
	}
	if ( rgfBuff.put() != 0 )
	{
			#ifdef DEBUG
			cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 	cerr << "Couldn't put info in the Database." << endl;
			#endif
			//error inserting the info, since nothing has been
			//modified yet we can simply return with an error
			#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrResource::put()." << endl << endl;
			#endif
			return E_PUT_RGF;
	}
	// The rgf has been inserted/upadted correctly, go on inserting the 
	// Acctdesc.
	hlrAcctdesc AcctdescBuff(rid,"resource",email,descr,certSubject);
	if ( AcctdescBuff.put() != 0 )
	{
		//there was an error inserting/updating the Rdesc,
		//if there was already an Rgf entry it's better to roll 
		//back the Rgf transaction
		if ( RgfExists )
		{
			//roll back
			if ( rgfRollBack.put() != 0 )
			{
				//Oh oh, can't roll back!
				#ifdef DEBUG
				cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 		cerr << "Couldn't rollback info in the Database." << endl;
				#endif
				#ifdef DEBUG
				cerr << "DEBUG: EXITING hlrResource::put()." << endl << endl;
				#endif
				return E_ROLLBACK;
				
			}
		}
		
	}
	//ok, everithing went fine.
	#ifdef DEBUG
	 cerr << "DEBUG:hlrResource::put():";
	 cerr << ",rid="<< rid;
	 cerr << ",email="<< email;
	 cerr << ",descr="<< descr;
	 cerr << ",certSubject="<< certSubject;
	 cerr << ",gid="<< gid;
	 cerr << ",fid="<< fid;
	 cerr << ",total="<< total << endl;
	#endif
	#ifdef DEBUG
	cerr << "DEBUG: EXITING hlrResource::put()." << endl << endl;
	#endif
	return 0;
}

int hlrResource::del()
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrResource::del()..." << endl;
	#endif
	if ( rid == "" || gid == "" || fid == "" )
	{
		#ifdef DEBUG
		cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
	 	cerr << "In the delete method rid, gid and fid are mandatory items." << endl;
		#endif
		#ifdef DEBUG
		cerr << "DEBUG: EXITING hlrResource::del()." << endl << endl;
		#endif
		return E_DEL;
	}
	hlrRgf rgfBuff(rid,gid,fid); 
	hlrRgf rgfRollBack = rgfBuff; // Buffer to store Rgf rollback
	//check if this Ug record already exists;
	bool RgfExists = rgfBuff.exists();
	if ( RgfExists )
	{
		// store info for rollback in case of error
		if ( rgfRollBack.get() != 0 )
		{
			#ifdef DEBUG
			cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 	cerr << "Couldn't get rollback info from the Database." << endl;
			#endif
			#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrResource::del()." << endl << endl;
			#endif
		 	return E_GET_ROLLBACK;
		}
		// we have info for eventual rollback	
	}
	if ( rgfBuff.del() != 0 )
	{
			#ifdef DEBUG
			cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 	cerr << "Couldn't delete info in the Database." << endl;
			#endif
			//error deleting the info, since nothing has been
			//modified yet we can simply return with an error
			#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrResource::del()." << endl << endl;
			#endif
			return E_DEL_RGF;
	}
	// The rgf has been deleted correctly, go on deleting the 
	// Rdesc.
	hlrAcctdesc AcctdescBuff(rid,"resource");
	if ( AcctdescBuff.del() != 0 )
	{
		//there was an error inserting/updating the Udesc,
		//if there was already an UG entry it's better to roll 
		//back the UG transaction
		if ( RgfExists )
		{
			//roll back
			if ( rgfRollBack.put() != 0 )
			{
				//Oh oh, can't roll back!
				#ifdef DEBUG
				cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 		cerr << "Couldn't rollback info in the Database." << endl;
				#endif
				#ifdef DEBUG
				cerr << "DEBUG: EXITING hlrResource::del()." << endl << endl;
				#endif
				return E_ROLLBACK;
			}
		}
	}
	//ok, everithing went fine.
	#ifdef DEBUG
	cerr << "DEBUG: EXITING hlrResource::del()." << endl << endl;
	#endif
	return 0;
}

bool hlrResource::exists()
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrResource::exists()..." << endl;
	#endif
	hlrAcctdesc AcctdescBuff(rid,"resource",email,descr,certSubject);
	if ( AcctdescBuff.get()  == 0 )
	{
		hlrRgf	 RgfBuff(AcctdescBuff.id,gid,fid);
		#ifdef DEBUG
		cerr << "DEBUG: EXITING hlrResource::exists()." << endl << endl;
		#endif
		return RgfBuff.exists();
	}
	else
	{
		#ifdef DEBUG
		cerr << "DEBUG: EXITING hlrResource::exists()." << endl << endl;
		#endif
		return false;
	}
}

int hlrResource::getKeys(vector<string>& keys)
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrResource::getKeys()..." << endl;
	#endif
	hlrAcctdesc AcctdescBuff(rid,"resource",email,descr,certSubject);
	//populate the Rdescv vector with the matching records
	if ( AcctdescBuff.getKeys(keys) != 0 )
	{
		#ifdef DEBUG
		 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 cerr << "Couldn't get hlrAcctdesc info from the Database." << endl;
		#endif
		#ifdef DEBUG
		cerr << "DEBUG: EXITING hlrResource::getKeys()." << endl << endl;
		#endif
		 return E_GET_KEYS;
	}
	#ifdef DEBUG
	cerr << "DEBUG: EXITING hlrResource::getKeys()." << endl << endl;
	#endif
	return 0;
}

ostream& operator << ( ostream& os, const hlrResource& r )
{
	os << "rid=" << r.rid;
	os << ",email=" << r.email;
	os << ",descr=" << r.descr;
	os << ",certSubject=" << r.certSubject;
	os << ",gid=" << r.gid;
	os << ",fid=" << r.fid;
	os << ",total=" << r.total;
	return os;
}

int hlrResource::credit(int v)
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrResource::credit()..." << endl;
	#endif
	if ( rid == "" || gid == "" || fid == "" )
	{
		#ifdef DEBUG
			cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 	cerr << "In the credit method rid, gid and fid are mandatory info." << endl;
		#endif
		#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrResource::credit()." << endl << endl;
		#endif
		 return E_CREDIT;
	}
	hlrRgf rgfBuff(rid,gid,fid); 
	hlrRgf rgfRollBack = rgfBuff; // Buffer to store Ug rollback
	//check if this Ug record already exists;
	bool RgfExists = rgfBuff.exists();
	if ( RgfExists )
	{
		// store info for rollback in case of error
		if ( rgfRollBack.get() != 0 )
		{
			#ifdef DEBUG
			cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 	cerr << "Couldn't get rollback info from the Database." << endl;
			#endif
			#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrResource::credit()." << endl << endl;
			#endif
		 	return E_GET_ROLLBACK;
		}
		// we have info for eventual rollback	
	}
	if ( rgfBuff.credit(v) != 0 )
	{
			#ifdef DEBUG
			cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 	cerr << "Couldn't delete info in the Database." << endl;
			#endif
			//error deleting the info, since nothing has been
			//modified yet we can simply return with an error
			#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrResource::credit()." << endl << endl;
			#endif
			return E_CREDIT_RGF;
	}
	return 0;

}

int hlrResource::debit(int v)
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrResource::debit()..." << endl;
	#endif
	if ( rid == "" || gid == "" || fid == "" )
	{
		#ifdef DEBUG
			cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 	cerr << "In the debit method rid,gid and fid are mandatory info." << endl;
		#endif
		#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrResource::debit()." << endl << endl;
		#endif
		 return E_DEBIT;
	}
	hlrRgf rgfBuff(rid,gid,fid); 
	hlrRgf rgfRollBack = rgfBuff; // Buffer to store Ug rollback
	//check if this Ug record already exists;
	bool RgfExists = rgfBuff.exists();
	if ( RgfExists )
	{
		// store info for rollback in case of error
		if ( rgfRollBack.get() != 0 )
		{
			#ifdef DEBUG
			cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 	cerr << "Couldn't get rollback info from the Database." << endl;
			#endif
			#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrResource::debit()." << endl << endl;
			#endif
		 	return E_GET_ROLLBACK;
		}
		// we have info for eventual rollback	
	}
	if ( rgfBuff.debit(v) != 0 )
	{
			#ifdef DEBUG
			cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 	cerr << "Couldn't update info in the Database." << endl;
			#endif
			//error deleting the info, since nothing has been
			//modified yet we can simply return with an error
			#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrResource::debit()." << endl << endl;
			#endif
			return E_DEBIT_RGF;
	}
	return 0;

}
