#include<hlrTransaction.h>

#define E_GET_TRANSIN 	1
#define E_GET_RESOURCE 	2
#define E_GET_USER 	3
#define E_GET_TRANSOUT 	4
#define E_PUT_TRANSIN 	5
#define E_PUT_TRANSOUT 	6
#define E_PUT 		7
#define E_DEL_TRANSOUT 	8
#define E_DEL_TRANSIN 	9
#define E_DEL 		10
#define E_GET 		11
#define E_PROCESS_IN  	12
#define E_PROCESS_OUT 	13
#define E_PROCESS 	14

extern const char * hlr_def_port;
extern const char * hlr_server_contact;

extern ofstream logStream;

hlrTransaction::hlrTransaction(
			       int _tid,
			       type _tType,
			       string _id,
			       string _fromId,
			       string _toId,
			       string _fromHlrUrl,
			       string _toHlrUrl,
			       int _amount,
			       string _timeStamp,
			       string _logData
			     )
{
	tid = _tid;
	tType = _tType;
	id =_id;
	fromId=_fromId;
	toId=_toId;
	fromHlrUrl=_fromHlrUrl;
	toHlrUrl=_toHlrUrl;
	amount=_amount;
	timeStamp=_timeStamp;
	logData=_logData;
	#ifdef DEBUG
	cerr << "DEBUG:hlrTransaction::hlrTransaction():" << *this <<endl;
	#endif
		
}

int hlrTransaction::get()
{
	#ifdef DEBUG
	cerr << "DEBUG: ENTERING hlrTransaction::get()..." << endl;
	#endif
	int res = E_GET;
	switch ( tType )
	{
		case out:
			res = this->getOut(); break;
		case in:
			res = this->getIn(); break;
		default: break;
	}
	#ifdef DEBUG
	cerr << "DEBUG: EXITING hlrTransaction::get()." << endl << endl;
	#endif
	return res;
}


int hlrTransaction::get(vector<hlrTransaction>& tv)
{
	//FIXME this algorithm is very expansive, reimplement
	//it adding two private methods getIn(tv) and getOut(tv)
	#ifdef DEBUG
	cerr << "DEBUG: ENTERING hlrTransaction::get(vector<hlrTransaction>&)..." << endl;
	#endif
	int res = E_GET;
	bool isIn = false;
	bool isOut = false;
	switch ( tType )
	{
		case out:
			isOut = true;
			break;
		case in:
			isIn = true;
			break;
		case nd:
			isOut = true;
			isIn = true;
			break;
		default:
			break;
	}
	if ( isOut )
	{
		vector <int> kOutV;
		res = this->getKeysOut(kOutV);
		vector<int>::const_iterator kOutV_it = kOutV.begin();
		while ( kOutV_it != kOutV.end() )
		{
			hlrTransaction t(*kOutV_it,out);
			if ( t.get() != 0 );
			tv.push_back(t);
			kOutV_it++;
		}
	}
	if ( isIn )
	{
		vector <int> kInV;
		res = this->getKeysIn(kInV);
		vector<int>::const_iterator kInV_it = kInV.begin();
		while ( kInV_it != kInV.end() )
		{
			hlrTransaction t(*kInV_it,in);
			if ( t.get() != 0 );
			tv.push_back(t);
			kInV_it++;
		}
	}
	
	#ifdef DEBUG
	cerr << "DEBUG: EXITING hlrTransaction::get(vector<hlrTransaction>&)." << endl << endl;
	#endif
	
	return res;
}


int hlrTransaction::put()
{
	#ifdef DEBUG
	cerr << "DEBUG: ENTERING hlrTransaction::put()..." << endl;
	#endif
	int res = E_PUT;
	switch ( tType )
	{
		case out:
			res = this->putOut(); break;
		case in:
			res = this->putIn(); break;
		default: break;
	}
	#ifdef DEBUG
	cerr << "DEBUG: EXITING hlrTransaction::put()." << endl << endl;
	#endif
	return res;
}


int hlrTransaction::getKeys(vector<int>& keys)
{
	int res = E_GET;
	switch ( tType )
	{
		case out:
			res = this->getKeysOut(keys); break;
		case in:
			res = this->getKeysIn(keys); break;
		case nd:
			res = this->getKeysOut(keys);
			if ( res != 0 )
			{
				res = this->getKeysIn(keys);
			}
			break;
		default: break;
	}
	return res;
}


int hlrTransaction::del()
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrTransaction::del()..." << endl;
	#endif
	int res = E_DEL;
	switch ( tType )
	{
		case out: 
			res = this->delOut(); break;
		case in:
			res = this->delIn(); break;
		default: break;
	}
	#ifdef DEBUG
	cerr << "DEBUG: EXITING hlrTransaction::del()." << endl << endl;
	#endif
	return res;
}


int hlrTransaction::del(int& start, int& end)
{
	return 0;
}


bool hlrTransaction::exists()
{
	#ifdef DEBUG
	cerr << "DEBUG: ENTERING hlrTransaction::exists()..." << endl;
	#endif
	bool res = false;
        string logBuff;
	switch ( tType )
	{
		case out: 
			res = this->existsOut(); break;
		case in:
			res = this->existsIn(); break;
		default: break;
	}
	 if ( res )
	 {
		logBuff="hlrTransaction::exists():, transaction for job: " +
			this->id + " exists.";
		hlr_log(logBuff, &logStream, 4);
	 }
	 else
	 {
		logBuff="hlrTransaction::exists():, transaction for job: " +
			this->id + " doesn't exists.";
		hlr_log(logBuff, &logStream, 4);
	 }
	#ifdef DEBUG
	cerr << "DEBUG: EXITING hlrTransaction::exists()." << endl << endl;
	#endif
	return res;
}

int hlrTransaction::process()
{
	bool processed = false;
	ostringstream streamBuff;
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrTransaction::process()..." << endl;
	#endif
	string acctType = "";
	streamBuff << "Transaction:" << *this;	
	hlr_log(streamBuff.str(),&logStream);
	//check if the transaction is incoming
	//processIn only if tType is not forced to out
	if ( (tType != out) && this->isIn(acctType) )
	{
		hlr_log("hlrTransaction::process(): processing incoming transaction",&logStream);
		//process the transaction
		if ( this->processIn(acctType) != 0 )
		{
			//there was an error processing the transaction
			hlr_log("hlrTransaction::process(): Error processing the incoming transaction",&logStream);
			#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrTransaction::process()." << endl << endl;
			#endif
			return E_PROCESS;
		}
		else
		{
			//insert transaction record in the Db
			this->tType=in;
			if ( this->put() != 0 )
			{
				hlr_log("hlrTransaction::process():ERROR transaction not inserted!",&logStream);
			}
			processed = true;
		}
	}
	//check if the transaction is outcoming
	//processOut only if tType is not forced to in
	if ( (tType != in) && this->isOut(acctType) )
	{
		hlr_log("hlrTransaction::process(): processing outcoming transaction",&logStream);
		//process the transaction
		if ( this->processOut(acctType) != 0 )
		{
			//there was an error processing the transaction
			hlr_log("hlrTransaction::process(): Error processing the outcoming transaction",&logStream);
			#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrTransaction::process()." << endl << endl;
			#endif
			return E_PROCESS;
		}
		else
		{
			this->tType=out;
			if ( this->put() != 0 )
			{
				hlr_log("hlrTransaction::process():ERROR transaction not inserted!",&logStream);
			}	
			processed = true;
		}
	}
	if ( !processed )
	{
		hlr_log("hlrTransaction::process(): WARNING Transaction not processed!", &logStream, 4);
		return E_PROCESS;
	}
	#ifdef DEBUG
	cerr << "DEBUG: EXITING hlrTransaction::process()." << endl << endl;
	#endif
	hlr_log("hlrTransaction::process(): Transaction processed!", &logStream, 4);
	return 0;
}


//Private methods


int hlrTransaction::getIn()
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrTransaction::getIn()..." << endl;
	#endif
	//retrieve rid and gid from toId
	hlrResource rBuff;
	if ( toId != "" )
	{
		rBuff.certSubject=toId;
		if ( rBuff.get() != 0 ) //get the object containing
		{			//the resource info
			#ifdef DEBUG
			 cerr <<"DEBUG:"<<__FILE__<<":"<<__LINE__<<":";
			 cerr <<"Error retrieving the hlrResource data."<<endl;
			#endif
			#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrTransaction::getIn()." << endl << endl;
			#endif
			 return E_GET_RESOURCE;
		}
		#ifdef DEBUG
		 cerr <<"DEBUG:hlrTransaction::getIn(),toId=";
		 cerr << toId << "==>rid=" << rBuff.rid << "." << endl;
		#endif
	}
	hlrTransIn TransInBuff(
			tid,
			rBuff.rid, 
			rBuff.gid,
			fromId,
			fromHlrUrl,
			amount,
			timeStamp,
			id
			);
	if ( TransInBuff.get() != 0 )
	{
		//there were errors retrieving the transaction
		#ifdef DEBUG
		 cerr <<"DEBUG:"<<__FILE__<<":"<<__LINE__<<":";
		 cerr <<"Error retrieving the hlrTransIn data."<<endl;
		#endif
		#ifdef DEBUG
		cerr << "DEBUG: EXITING hlrTransaction::getIn()." << endl << endl;
		#endif
		return E_GET_TRANSIN;
	}
	tid = TransInBuff.tid;
	id = TransInBuff.dgJobId;
	fromId = TransInBuff.fromDn;
	fromHlrUrl = TransInBuff.fromUrl;
	amount = TransInBuff.amount;
	timeStamp = TransInBuff.timeStamp;
	if ( toId == "" )
	{
		//we don't have the toId, so we can find it from 
		//the rid value
		rBuff.rid = TransInBuff.rid;
		if ( rBuff.get() != 0 )
		{
			#ifdef DEBUG
			 cerr <<"DEBUG:"<<__FILE__<<":"<<__LINE__<<":";
			 cerr <<"Error retrieving the hlrResource data."<<endl;
			#endif
			#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrTransaction::getIn()." << endl << endl;
			#endif
			return E_GET_RESOURCE;
		}
	}
	toId = rBuff.certSubject;
	//FIXME toHlrUrl !!!!
	tType = in;
	#ifdef DEBUG
	cerr << "DEBUG: EXITING hlrTransaction::getIn()." << endl << endl;
	#endif
	return 0;
}

int hlrTransaction::getOut()
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrTransaction::getOut()..." << endl;
	#endif
	//retrieve uid from fromId
	hlrUser uBuff;
	if ( fromId != "" )
	{
		uBuff.certSubject = fromId;
		if ( uBuff.get() != 0 ) //get the object containing
		{			//the suer info
			#ifdef DEBUG
			 cerr <<"DEBUG:"<<__FILE__<<":"<<__LINE__<<":";
			 cerr <<"Error retrieving the hlrUser data."<<endl;
			#endif
			#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrTransaction::getOut()." << endl << endl;
			#endif
			 return E_GET_USER;
		}
		#ifdef DEBUG
		 cerr <<"DEBUG:hlrTransaction::getOut(),fromId=";
		 cerr << fromId << "==>uid=" << uBuff.uid << "." << endl;
		#endif
	}
	hlrTransOut TransOutBuff(
			tid,
			uBuff.uid,
			uBuff.gid,
			toId,
			toHlrUrl,
			amount,
			timeStamp,
			id
			);
	if ( TransOutBuff.get() != 0 )
	{
		//there were errors retrieving the transaction
		#ifdef DEBUG
		 cerr <<"DEBUG:"<<__FILE__<<":"<<__LINE__<<":";
		 cerr <<"Error retrieving the hlrTransOut data."<<endl;
		#endif
		#ifdef DEBUG
		cerr << "DEBUG: EXITING hlrTransaction::getOut()." << endl << endl;
		#endif
		 return E_GET_TRANSOUT;
		
	}
	tid = TransOutBuff.tid;
	id = TransOutBuff.dgJobId;
	toId = TransOutBuff.toDn;
	toHlrUrl = TransOutBuff.toUrl;
	amount = TransOutBuff.amount;
	timeStamp = TransOutBuff.timeStamp;
	if ( fromId == "" )
	{
		//we don't have the fromId, so we can find it from 
		//the uid value
		uBuff.uid = TransOutBuff.uid;
		if ( uBuff.get() != 0 )
		{
			#ifdef DEBUG
			 cerr <<"DEBUG:"<<__FILE__<<":"<<__LINE__<<":";
			 cerr <<"Error retrieving the hlrUser data."<<endl;
			#endif
			#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrTransaction::getOut()." << endl << endl;
			#endif
			return E_GET_USER;
		}
	}
	fromId = uBuff.certSubject;
	string localh = "localhost:";
	//fromHlrUrl = localh + hlr_def_port +":" +hlr_server_contact ;
	tType = out;
	#ifdef DEBUG
	cerr << "DEBUG: EXITING hlrTransaction::getOut()." << endl << endl;
	#endif
	return 0;
}

int hlrTransaction::getKeysOut(vector<int>& keys)
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrTransaction::getKeysOut()..." << endl;
	#endif
	//retrieve uid from fromId
	hlrUser uBuff;
	if ( fromId != "" )
	{
		uBuff.certSubject = fromId;
		if ( uBuff.get() != 0 ) //get the object containing
		{			//the suer info
			#ifdef DEBUG
			 cerr <<"DEBUG:"<<__FILE__<<":"<<__LINE__<<":";
			 cerr <<"Error retrieving the hlrUser data."<<endl;
			#endif
			#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrTransaction::getKeysOut()." << endl << endl;
			#endif
			 return E_GET_USER;
		}
		#ifdef DEBUG
		 cerr <<"DEBUG:hlrTransaction::getKeysOut(),fromId=";
		 cerr << fromId << "==>uid=" << uBuff.uid << "." << endl;
		#endif
	}
	hlrTransOut TransOutBuff(
			tid,
			uBuff.uid,
			uBuff.gid,
			toId,
			toHlrUrl,
			amount,
			timeStamp,
			id
			);
	if ( TransOutBuff.getKeys(keys) != 0 )
	{
		//there were errors retrieving the transaction
		#ifdef DEBUG
		 cerr <<"DEBUG:"<<__FILE__<<":"<<__LINE__<<":";
		 cerr <<"Error retrieving the hlrTransOut keys."<<endl;
		#endif
		#ifdef DEBUG
		cerr << "DEBUG: EXITING hlrTransaction::getKeysOut()." << endl << endl;
		#endif
		 return E_GET_TRANSOUT;
		
	}
	#ifdef DEBUG
	cerr << "DEBUG: EXITING hlrTransaction::getKeysOut()." << endl << endl;
	#endif
	return 0;
}

int hlrTransaction::getKeysIn(vector<int>& keys)
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrTransaction::getKeysIn()..." << endl;
	#endif
	//retrieve rid from toId
	hlrResource rBuff;
	if ( toId != "" )
	{
		rBuff.certSubject = toId;
		if ( rBuff.get() != 0 ) //get the object containing
		{			//the suer info
			#ifdef DEBUG
			 cerr <<"DEBUG:"<<__FILE__<<":"<<__LINE__<<":";
			 cerr <<"Error retrieving the hlrResource data."<<endl;
			#endif
			#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrTransaction::getKeysIn()." << endl << endl;
			#endif
			 return E_GET_RESOURCE;
		}
		#ifdef DEBUG
		 cerr <<"DEBUG:hlrTransaction::getKeysIn(),toId=";
		 cerr << toId << "==>rid=" << rBuff.rid << "." << endl;
		#endif
	}
	hlrTransIn TransInBuff(
			tid,
			rBuff.rid,
			rBuff.gid,
			fromId,
			fromHlrUrl,
			amount,
			timeStamp,
			id
			);
	if ( TransInBuff.getKeys(keys) != 0 )
	{
		//there were errors retrieving the transaction
		#ifdef DEBUG
		 cerr <<"DEBUG:"<<__FILE__<<":"<<__LINE__<<":";
		 cerr <<"Error retrieving the hlrTransIn keys."<<endl;
		#endif
		#ifdef DEBUG
		cerr << "DEBUG: EXITING hlrTransaction::getKeysIn()." << endl << endl;
		#endif
		 return E_GET_TRANSIN;
		
	}
	#ifdef DEBUG
	cerr << "DEBUG: EXITING hlrTransaction::getKeysIn()." << endl << endl;
	#endif
	return 0;
}

int hlrTransaction::putIn()
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrTransaction::putIn()..." << endl;
	#endif
	//retrieve rid and gid of the resource (to item )from toId
	hlrResource rBuff;
	if ( toId != "" )
	{
		rBuff.certSubject=toId;
		if ( rBuff.get() != 0 ) //get the object containing
		{			//the resource info
			#ifdef DEBUG
			 cerr <<"DEBUG:"<<__FILE__<<":"<<__LINE__<<":";
			 cerr <<"Error retrieving the hlrResource data."<<endl;
			#endif
			 return E_GET_RESOURCE;
		}
		#ifdef DEBUG
		 cerr <<"DEBUG:hlrTransaction::getIn(),toId=";
		 cerr << toId << "==>rid=" << rBuff.rid << "." << endl;
		#endif
	}
	hlrTransIn transInBuff(0,
			rBuff.rid,
			rBuff.gid,
			fromId,
			fromHlrUrl,
			amount,
			timeStamp,
			id);
	if ( transInBuff.put() != 0 )
	{
		//there was an error inserting the transaction.
		#ifdef DEBUG
		 cerr <<"DEBUG:"<<__FILE__<<":"<<__LINE__<<":";
		 cerr <<"Error inserting the hlrTransIn data."<<endl;
		#endif
		 return E_PUT_TRANSIN;
	}
			
	#ifdef DEBUG
	cerr << "DEBUG: EXITING hlrTransaction::putIn()." << endl << endl;
	#endif
			
	return 0;
}

int hlrTransaction::putOut()
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrTransaction::putOut()..." << endl;
	#endif
	//retrieve uid and gid from fromId
	hlrUser uBuff;
	if ( fromId != "" )
	{
		uBuff.certSubject = fromId;
		if ( uBuff.get() != 0 ) //get the object containing
		{			//the user info
			#ifdef DEBUG
			 cerr <<"DEBUG:"<<__FILE__<<":"<<__LINE__<<":";
			 cerr <<"Error retrieving the hlrUser data."<<endl;
			#endif
			 return E_GET_USER;
		}
		#ifdef DEBUG
		 cerr <<"DEBUG:hlrTransaction::getOut(),fromId=";
		 cerr << fromId << "==>uid=" << uBuff.uid << "." << endl;
		#endif
	}
	hlrTransOut transOutBuff(
			0,
			uBuff.uid,
			uBuff.gid,
			toId,
			toHlrUrl,
			amount,
			timeStamp,
			id
			);
	if ( transOutBuff.put() != 0 )
	{
		//there was an error inserting the transaction.
		#ifdef DEBUG
		 cerr <<"DEBUG:"<<__FILE__<<":"<<__LINE__<<":";
		 cerr <<"Error inserting the hlrTransOut data."<<endl;
		#endif
		 return E_PUT_TRANSOUT;
	}

	#ifdef DEBUG
	cerr << "DEBUG: EXITING hlrTransaction::putOut()." << endl << endl;
	#endif
	return 0;
}

int hlrTransaction::delIn()
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrTransaction::delIn()..." << endl;
	#endif
	//retrieve uid from fromId
	hlrResource rBuff;
	if ( toId != "" )
	{
		rBuff.certSubject = toId;
		if ( rBuff.get() != 0 ) //get the object containing
		{			//the suer info
			#ifdef DEBUG
			 cerr <<"DEBUG:"<<__FILE__<<":"<<__LINE__<<":";
			 cerr <<"Error retrieving the hlrResource data."<<endl;
			#endif
			#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrTransaction::delIn()." << endl << endl;
			#endif
			 return E_GET_RESOURCE;
		}
		#ifdef DEBUG
		 cerr <<"DEBUG:hlrTransaction::delIn(),toId=";
		 cerr << toId << "==>rid=" << rBuff.rid << "." << endl;
		#endif
	}
	hlrTransIn TransInBuff(
			tid,
			rBuff.rid,
			rBuff.gid,
			fromId,
			fromHlrUrl,
			amount,
			timeStamp,
			id
			);
	if ( TransInBuff.del() != 0 )
	{
		//there was an error deleting the transaction(s).
		#ifdef DEBUG
		 cerr <<"DEBUG:"<<__FILE__<<":"<<__LINE__<<":";
		 cerr <<"Error deleting the hlrTransIn data."<<endl;
		#endif
		#ifdef DEBUG
		cerr << "DEBUG: EXITING hlrTransaction::delIn()." << endl << endl;
		#endif
		 return E_DEL_TRANSIN;
	}
	#ifdef DEBUG
	cerr << "DEBUG: EXITING hlrTransaction::delIn()." << endl << endl;
	#endif

	return 0;
}

int hlrTransaction::delOut()
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrTransaction::delOut()..." << endl;
	#endif
	//retrieve uid from fromId
	hlrUser uBuff;
	if ( fromId != "" )
	{
		uBuff.certSubject = fromId;
		if ( uBuff.get() != 0 ) //get the object containing
		{			//the suer info
			#ifdef DEBUG
			 cerr <<"DEBUG:"<<__FILE__<<":"<<__LINE__<<":";
			 cerr <<"Error retrieving the hlrUser data."<<endl;
			#endif
			#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrTransaction::delOut()." << endl << endl;
			#endif
			 return E_GET_USER;
		}
		#ifdef DEBUG
		 cerr <<"DEBUG:hlrTransaction::getOut(),fromId=";
		 cerr << fromId << "==>uid=" << uBuff.uid << "." << endl;
		#endif
	}
	hlrTransOut TransOutBuff(
			tid,
			uBuff.uid,
			uBuff.gid,
			toId,
			toHlrUrl,
			amount,
			timeStamp,
			id
			);
	if ( TransOutBuff.del() != 0 )
	{
		//there was an error deleting the transaction(s).
		#ifdef DEBUG
		 cerr <<"DEBUG:"<<__FILE__<<":"<<__LINE__<<":";
		 cerr <<"Error deleting the hlrTransOut data."<<endl;
		#endif
		#ifdef DEBUG
		cerr << "DEBUG: EXITING hlrTransaction::delOut()." << endl << endl;
		#endif
		 return E_DEL_TRANSOUT;
	}

	#ifdef DEBUG
	cerr << "DEBUG: EXITING hlrTransaction::delOut()." << endl << endl;
	#endif
	return 0;
}

bool hlrTransaction::existsIn()
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrTransaction::existsIn()..." << endl;
	#endif
	//retrieve rid from toId
	hlrResource rBuff;
	if ( toId != "" )
	{
		rBuff.certSubject = toId;
		if ( rBuff.get() != 0 ) //get the object containing
		{			//the resource info
			#ifdef DEBUG
			 cerr <<"DEBUG:"<<__FILE__<<":"<<__LINE__<<":";
			 cerr <<"Error retrieving the hlrResource data."<<endl;
			#endif
			 return false;
		}
		#ifdef DEBUG
		 cerr <<"DEBUG:hlrTransaction::existsIn(),toId=";
		 cerr << toId << "==>rid=" << rBuff.rid << "." << endl;
		#endif
	}
	hlrTransIn TransInBuff(
			tid,
			rBuff.rid,
			rBuff.gid,
			fromId,
			fromHlrUrl,
			amount,
			timeStamp,
			id
			);
	#ifdef DEBUG
	cerr << "DEBUG: EXITING hlrTransaction::existsIn()." << endl << endl;
	#endif

	return TransInBuff.exists();
}

bool hlrTransaction::existsOut()
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrTransaction::existsOut()..." << endl;
	#endif
	//retrieve uid from fromId
	hlrUser uBuff;
	if ( fromId != "" )
	{
		uBuff.certSubject = fromId;
		if ( uBuff.get() != 0 ) //get the object containing
		{			//the suer info
			#ifdef DEBUG
			 cerr <<"DEBUG:"<<__FILE__<<":"<<__LINE__<<":";
			 cerr <<"Error retrieving the hlrUser data."<<endl;
			#endif
			 return false;
		}
		#ifdef DEBUG
		 cerr <<"DEBUG:hlrTransaction::existOut(),fromId=";
		 cerr << fromId << "==>uid=" << uBuff.uid << "." << endl;
		#endif
	}
	hlrTransOut TransOutBuff(
			tid,
			uBuff.uid,
			uBuff.gid,
			toId,
			toHlrUrl,
			amount,
			timeStamp,
			id
			);
	#ifdef DEBUG
	cerr << "DEBUG: EXITING hlrTransaction::existsOut()." << endl << endl;
	#endif

	return TransOutBuff.exists();
}

bool hlrTransaction::isIn(string& acctType)
{
	string logBuff;
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrTransaction::isIn()..." << endl;
	#endif
	bool result = false;
	// if there is a resource corresponding to the one
	// in toId the transaction is directed toward this HLR
	hlrResource rBuff;
	rBuff.certSubject=toId;
	if ( rBuff.exists() )
	{
		acctType = "resource";
		result = true;
	}
	else
	{
		hlrUser uBuff;
		uBuff.certSubject=toId;
		if ( uBuff.exists() )
		{
			acctType = "user";
			result = true;
		}
		else
		{
			hlrFund fBuff;
			fBuff.fid = toId;
			if ( fBuff.exists() )
			{
				acctType = "fund";
				result = true;
			}
			else
			{
				//FIXME insert group check here
				//and delete next line
				result = false;
			}
		}
	}
	if ( result ) 
	{
		logBuff = "The account type has been correctly identified as:" + acctType;
	}
	else
	{
		logBuff = "The account type hasn't been corrctly identified";
	}
	hlr_log(logBuff, &logStream, 4);
	#ifdef DEBUG
	cerr << "DEBUG: hlrTransaction::isIn(), result=" << result;
	cerr << ",acctType=" << acctType << endl;
	cerr << "DEBUG: EXITING hlrTransaction::isIn()." << endl << endl;
	#endif
	return result; 
}

bool hlrTransaction::isOut(string& acctType)
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrTransaction::isOut()..." << endl;
	#endif
	bool result = false;
	// if there is a user corresponding to the one
	// in fromId the transaction is directed outside this HLR
	hlrUser uBuff;
	uBuff.certSubject=fromId;
	if ( uBuff.exists() )
	{
		acctType = "user";
		result = true;
	}
	else
	{
		hlrResource rBuff;
		rBuff.certSubject=fromId;
		if ( rBuff.exists() )
		{
			acctType = "resource";
			result = true;
		}
		else
		{
			hlrFund fBuff;
			fBuff.fid = fromId;
			if ( fBuff.exists() )
			{
				acctType = "fund";
				result = true;
			}
			else
			{
				//FIXME insert group check here
				//and delete next line
				result = false;
			}
		}
	}
	#ifdef DEBUG
	cerr << "DEBUG: hlrTransaction::isOut(), result=" << result;
	cerr << ",acctType=" << acctType << endl;
	cerr << "DEBUG: EXITING hlrTransaction::isOut()." << endl << endl;
	#endif
	return result; 

}

int hlrTransaction::processIn(string& acctType)
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrTransaction::processIn()..." << endl;
	#endif
	if ( acctType == "resource" )
	{
		//credit the resource
		hlrResource r;
		r.certSubject = toId;
		if ( r.get() != 0 )
		{
			#ifdef DEBUG
			 cerr <<"DEBUG:"<<__FILE__<<":"<<__LINE__<<":";
			 cerr <<"Error getting the resource  data."<<endl;
			#endif
			#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrTransaction::processIn()." << endl << endl;
			#endif
			hlr_log("hlrTransaction::processIn(): Error retrieving resource info.",&logStream,4);
			return E_PROCESS_IN;
		}
		else
		{
			//Branch0: we can credit the resource.
			if ( r.credit(amount) != 0 )
			{
				#ifdef DEBUG
				 cerr <<"DEBUG:"<<__FILE__<<":"<<__LINE__<<":";
				 cerr <<"Error crediting the resource."<<endl;
				#endif
				#ifdef DEBUG
				cerr << "DEBUG: EXITING hlrTransaction::processIn()." << endl << endl;
				#endif
				hlr_log("hlrTransaction::processIn(): Error crediting the resource account.",&logStream,4);
				return E_PROCESS_IN;
			}
		}
	}
	else
	{
		if ( acctType == "user" )
		{
			//Branch1: credit the user
			hlrUser u;
			u.certSubject = toId;
			if ( u.get() != 0 )
			{
				#ifdef DEBUG
				 cerr <<"DEBUG:"<<__FILE__<<":"<<__LINE__<<":";
				 cerr <<"Error getting the resource  data."<<endl;
				#endif
				#ifdef DEBUG
				 cerr << "DEBUG: EXITING hlrTransaction::processIn()." << endl << endl;
				#endif
				 return E_PROCESS_IN;
			}
			else
			{
				//we can credit the user
				if ( u.credit(amount) != 0 )
				{
					#ifdef DEBUG
					 cerr <<"DEBUG:"<<__FILE__<<":"<<__LINE__<<":";
					 cerr <<"Error crediting the resource."<<endl;
					#endif
					#ifdef DEBUG
					cerr << "DEBUG: EXITING hlrTransaction::processIn()." << endl << endl;
					#endif
					return E_PROCESS_IN;
				}
			}
		}
		else
		{
			if ( acctType == "fund" )
			{
				//Branch2: credit the fund;
				hlrFund f;
				f.fid = toId;
				if ( f.get() != 0 )
				{
					#ifdef DEBUG
					 cerr <<"DEBUG:"<<__FILE__<<":"<<__LINE__<<":";
					 cerr <<"Error getting the fund data."<<endl;
					#endif
					#ifdef DEBUG
					 cerr << "DEBUG: EXITING hlrTransaction::processIn()." << endl << endl;
					#endif
					 return E_PROCESS_IN;
				}
				else
				{
					//we can credit the fund
					if ( f.credit(amount) != 0 )
					{
						#ifdef DEBUG
						 cerr <<"Error crediting the fund."<<endl;
						#endif
						#ifdef DEBUG
						cerr << "DEBUG: EXITING hlrTransaction::processIn()." << endl << endl;
						#endif
						return E_PROCESS_IN;
					}
					
				}
			}
			else
			{
				if ( acctType == "group" )
				{
					//FIXME implement
				}
				else
				{
					//we shouldn't be here! so there's an error
					hlr_log("hlrTransaction::processIn: Error: account type not supported!",&logStream);
					return E_PROCESS_IN;
				}
			}
		}
	}
	hlr_log("hlrTransaction::processIn(): Account credited.",&logStream,4);
	#ifdef DEBUG
	cerr << "DEBUG: EXITING hlrTransaction::processIn()." << endl << endl;
	#endif
	return 0;
	
}

int hlrTransaction::processOut(string& acctType)
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrTransaction::processOut()..." << endl;
	#endif
	if ( acctType == "user" )
	{
		//Branch0: credit the user
		hlrUser u;
		u.certSubject = fromId;
		if ( u.get() != 0 )
		{
			#ifdef DEBUG
			 cerr <<"DEBUG:"<<__FILE__<<":"<<__LINE__<<":";
			 cerr <<"Error getting the user data."<<endl;
			#endif
			#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrTransaction::processOut()." << endl << endl;
			#endif
			return E_PROCESS_OUT;
		}
		else
		{
			// we can credit the user.
			if ( u.debit(amount) != 0 )
			{
				#ifdef DEBUG
				 cerr <<"DEBUG:"<<__FILE__<<":"<<__LINE__<<":";
				 cerr <<"Error debiting the user."<<endl;
				#endif
				#ifdef DEBUG
				cerr << "DEBUG: EXITING hlrTransaction::processOut()." << endl << endl;
				#endif
				return E_PROCESS_OUT;
			}
		}
	}
	else
	{
		if ( acctType == "resource" )
		{
			//Branch1: debit the resource
			hlrResource r;
			r.certSubject = fromId;
			if ( r.get() != 0 )
			{
				#ifdef DEBUG
				 cerr <<"DEBUG:"<<__FILE__<<":"<<__LINE__<<":";
				 cerr <<"Error getting the resource  data."<<endl;
				#endif
				#ifdef DEBUG
				 cerr << "DEBUG: EXITING hlrTransaction::processOut()." << endl << endl;
				#endif
				 return E_PROCESS_OUT;
			}
			else
			{
				//we can debit the resource
				if ( r.debit(amount) != 0 )
				{
					#ifdef DEBUG
					 cerr <<"DEBUG:"<<__FILE__<<":"<<__LINE__<<":";
					 cerr <<"Error debiting the user."<<endl;
					#endif
					#ifdef DEBUG
					cerr << "DEBUG: EXITING hlrTransaction::processOut()." << endl << endl;
					#endif
					return E_PROCESS_OUT;
				}
			}
		}
		else
		{
			if ( acctType == "fund" )
			{
				//Branch2: debit the fund;
				hlrFund f;
				f.fid = fromId;
				if ( f.get() != 0 )
				{
					#ifdef DEBUG
					 cerr <<"DEBUG:"<<__FILE__<<":"<<__LINE__<<":";
					 cerr <<"Error getting the fund data."<<endl;
					#endif
					#ifdef DEBUG
					 cerr << "DEBUG: EXITING hlrTransaction::processOut()." << endl << endl;
					#endif
					 return E_PROCESS_OUT;
				}
				else
				{
					//we can debit the fund
					if ( f.debit(amount) != 0 )
					{
						#ifdef DEBUG
						 cerr <<"Error debiting the fund."<<endl;
						#endif
						#ifdef DEBUG
						cerr << "DEBUG: EXITING hlrTransaction::processOut()." << endl << endl;
						#endif
						return E_PROCESS_OUT;
					}
					
				}
			}
			else
			{
				if ( acctType == "group" )
				{
					//FIXME implement
				}
			}
		}
	}
	#ifdef DEBUG
	cerr << "DEBUG: EXITING hlrTransaction::processOut()." << endl << endl;
	#endif
	return 0;
}



ostream& operator << ( ostream& os, const hlrTransaction& tr )
{
	os << "tid=" << tr.tid;
	os << ",tType=" << tr.tType;
	os << ",id=" << tr.id;
	os << ",fromId=" << tr.fromId;
	os << ",toId=" << tr.toId;
	os << ",fromHlrUrl=" << tr.fromHlrUrl;
	os << ",toHlrUrl=" << tr.toHlrUrl;
	os << ",amount=" << tr.amount;
	os << ",timeStamp=" << tr.timeStamp;
	os << ",logData=" << tr.logData;
	return os;

}

