// $Id: hlrUser.cpp,v 1.3 2003/12/09 14:14:08 guarise Exp $
// -------------------------------------------------------------------------
// Copyright (c) 2001-2002, The DataGrid project, INFN, 
// All rights reserved. See LICENSE file for details.
// -------------------------------------------------------------------------
// Authors: Andrea Guarise <andrea.guarise@to.infn.it>
/***************************************************************************
 * Code borrowed from:
 *  authors   : 
 *             
 *  
 ***************************************************************************/


//TODO log db events in the log file.

#include<hlrUser.h>

#define E_GET_UDESC 	1
#define E_GET_UG 	2
#define E_GET_ROLLBACK 	3
#define E_PUT_UG	4
#define E_ROLLBACK 	5
#define E_DEL_UG	6
#define E_GET_KEYS 	7
#define E_DEL 		8
#define E_CREDIT 	9
#define E_CREDIT_UG	10
#define E_DEBIT 	11
#define E_DEBIT_UG	12

extern const char * hlr_sql_server;
extern const char * hlr_sql_user;
extern const char * hlr_sql_password;
extern const char * hlr_sql_dbname;

extern ofstream logStream;

hlrUser::hlrUser( string _uid,
		string _gid,
		string _fid,
		string _email,
		string _descr,
		string _certSubject,
		int _assigned,
		int _booked,
		int _spent
		)
{
	uid = _uid;
	gid = _gid;
	fid = _fid;
	email = _email;
	descr = _descr;
	certSubject = _certSubject;
	assigned = _assigned;
	booked = _booked;
	spent = _spent;
	#ifdef DEBUG
	 cerr << "hlrUser::hlrUser():uid=" << uid;
	 cerr << ",gid=" << gid;
	 cerr << ",fid=" << fid;
	 cerr << ",email=" << email;
	 cerr << ",descr=" << descr;
	 cerr << ",certSubject=" << certSubject;
	 cerr << ",assigned=" << assigned;
	 cerr << ",booked=" << booked;
	 cerr << ",spent=" << spent << endl;
	#endif
}


int hlrUser::get()
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrUser::get()..." << endl;
	#endif
	hlrAcctdesc AcctdescBuff(uid,"user",email,descr,certSubject);
	if ( AcctdescBuff.get() != 0 )
	{
		#ifdef DEBUG
		 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 cerr << "Couldn't get hlrAcctdesc info from the Database." << endl;
		#endif
		 return E_GET_UDESC;
	}
	#ifdef DEBUG
	 cerr << "DEBUG: uid=" << AcctdescBuff << endl;
	#endif
	hlrUgf	 UgBuff(AcctdescBuff.id,gid,fid,assigned,booked,spent);
	if ( UgBuff.get() != 0 )
	{
		#ifdef DEBUG
		 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 cerr << "Couldn't get hlrUgf info from the Database." << endl;
		#endif
		 return E_GET_UG;
	}
	uid = AcctdescBuff.id;
	email = AcctdescBuff.email;
	descr = AcctdescBuff.descr;
	certSubject = AcctdescBuff.certSubject;
	gid = UgBuff.gid;
	fid = UgBuff.fid;
	assigned = UgBuff.assigned;
	booked = UgBuff.booked;
	spent = UgBuff.spent;
	return 0;
}

int hlrUser::get(vector<hlrUser>& uv)
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrUser::get()..." << endl;
	#endif
	vector<hlrAcctdesc> Acctdescv;
	hlrAcctdesc AcctdescBuff(uid,"user",email,descr,certSubject);
	//populate the Acctdescv vector with the matching records
	if ( AcctdescBuff.get(Acctdescv) != 0 )
	{
		#ifdef DEBUG
		 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 cerr << "Couldn't get hlrAcctdesc info from the Database." << endl;
		#endif
		 return E_GET_UDESC;
	}
	//get every Ug record corresponding to the matching Acctdesc records
	vector<hlrAcctdesc>::iterator it = Acctdescv.begin();
	while ( it != Acctdescv.end() )
	{
		vector<hlrUgf> Ugv;
		hlrUgf    UgBuff((*it).id);
	
		if ( UgBuff.get(Ugv) != 0 )
		{
			#ifdef DEBUG
			cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 	cerr << "Couldn't get hlrUgf info from the Database." << endl;
			#endif
		 	return E_GET_UG;
		}
		//Now we have the info.	
		vector<hlrUgf>::iterator UgvIt = Ugv.begin();
		while ( UgvIt != Ugv.end() )
		{
			hlrUser u(
					(*it).id,
					(*UgvIt).gid,
					(*UgvIt).fid,
					(*it).email,
					(*it).descr,
					(*it).certSubject,
					(*UgvIt).assigned,
					(*UgvIt).booked,
					(*UgvIt).spent
							);
			uv.push_back(u);
			UgvIt++;
		}
		it++;	
	}
	return 0;
}

int hlrUser::put()
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrUser::put()..." << endl;
	#endif
	hlrUgf ugBuff(uid,gid,fid,assigned,booked,spent); 
	hlrUgf ugRollBack = ugBuff; // Buffer to store Ug rollback
	//check if this Ug record already exists;
	bool UgExists = ugBuff.exists();
	if ( UgExists )
	{
		// store info for rollback in case of error
		if ( ugRollBack.get() != 0 )
		{
			#ifdef DEBUG
			cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 	cerr << "Couldn't get rollback info from the Database." << endl;
			#endif
			#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrUser::put()." << endl << endl;
			#endif
		 	return E_GET_ROLLBACK;
		}
		// we have info for eventual rollback	
	}
	if ( ugBuff.put() != 0 )
	{
			#ifdef DEBUG
			cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 	cerr << "Couldn't put info in the Database." << endl;
			#endif
			//error inserting the info, since nothing has been
			//modified yet we can simply return with an error
			#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrUser::put()." << endl << endl;
			#endif
			return E_PUT_UG;
	}
	// The ug has been inserted/upadted correctly, go on inserting the 
	// Acctdesc.
	hlrAcctdesc AcctdescBuff(uid,"user",email,descr,certSubject);
	if ( AcctdescBuff.put() != 0 )
	{
		//there was an error inserting/updating the Udesc,
		//if there was already an UG entry it's better to roll 
		//back the UG transaction
		if ( UgExists )
		{
			//roll back
			if ( ugRollBack.put() != 0 )
			{
				//Oh oh, can't roll back!
				#ifdef DEBUG
				cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 		cerr << "Couldn't rollback info in the Database." << endl;
				#endif
				#ifdef DEBUG
				cerr << "DEBUG: EXITING hlrUser::put()." << endl << endl;
				#endif
				return E_ROLLBACK;
				
			}
		}
		
	}
	//ok, everithing went fine.
	#ifdef DEBUG
	 cerr << "DEBUG:hlrUser::put():";
	 cerr << ",uid="<< uid;
	 cerr << ",gid="<< gid;
	 cerr << ",fid="<< fid;
	 cerr << ",email="<< email;
	 cerr << ",descr="<< descr;
	 cerr << ",certSubject="<< certSubject;
	 cerr << ",assigned="<< assigned;
	 cerr << ",booked="<< booked;
	 cerr << ",spent="<< spent << endl;
	#endif
	#ifdef DEBUG
	cerr << "DEBUG: EXITING hlrUser::put()." << endl << endl;
	#endif
	return 0;
}

int hlrUser::del()
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrUser::del()..." << endl;
	#endif
	if ( uid == "" || gid == "" || fid == "" )
	{
		#ifdef DEBUG
			cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 	cerr << "In the delete method uid, gid and fid are mandatory info." << endl;
		#endif
		#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrUser::del()." << endl << endl;
		#endif
		 return E_DEL;
	}
	hlrUgf ugBuff(uid,gid,fid); 
	hlrUgf ugRollBack = ugBuff; // Buffer to store Ug rollback
	//check if this Ug record already exists;
	bool UgExists = ugBuff.exists();
	if ( UgExists )
	{
		// store info for rollback in case of error
		if ( ugRollBack.get() != 0 )
		{
			#ifdef DEBUG
			cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 	cerr << "Couldn't get rollback info from the Database." << endl;
			#endif
			#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrUser::del()." << endl << endl;
			#endif
		 	return E_GET_ROLLBACK;
		}
		// we have info for eventual rollback	
	}
	if ( ugBuff.del() != 0 )
	{
			#ifdef DEBUG
			cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 	cerr << "Couldn't delete info in the Database." << endl;
			#endif
			//error deleting the info, since nothing has been
			//modified yet we can simply return with an error
			#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrUser::del()." << endl << endl;
			#endif
			return E_DEL_UG;
	}
	// The ug has been deleted correctly, go on deleting the 
	// Acctdesc.
	hlrAcctdesc AcctdescBuff(uid,"user");
	if ( AcctdescBuff.del() != 0 )
	{
		//there was an error inserting/updating the Acctdesc,
		//if there was already an UG entry it's better to roll 
		//back the UG transaction
		if ( UgExists )
		{
			//roll back
			if ( ugRollBack.put() != 0 )
			{
				//Oh oh, can't roll back!
				#ifdef DEBUG
				cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 		cerr << "Couldn't rollback info in the Database." << endl;
				#endif	
				#ifdef DEBUG
					cerr << "DEBUG: EXITING hlrUser::del()." << endl << endl;
				#endif
				return E_ROLLBACK;
				
			}
		}
		
	}
	//ok, everithing went fine.
	#ifdef DEBUG
	cerr << "DEBUG: EXITING hlrUser::del()." << endl << endl;
	#endif

	return 0;
}

bool hlrUser::exists()
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrUser::exists()..." << endl;
	#endif
	hlrAcctdesc AcctdescBuff(uid,"user",email,descr,certSubject);
	if ( AcctdescBuff.get() == 0 )
	{
		hlrUgf UgBuff(AcctdescBuff.id,gid,fid);
		#ifdef DEBUG
		cerr << "DEBUG: EXITING hlrUser::exists()." << endl << endl;
		#endif
		return UgBuff.exists();
	}
	else
	{
		#ifdef DEBUG
		cerr << "DEBUG: EXITING hlrUser::exists()." << endl << endl;
		#endif
		return false;
	}
}

int hlrUser::getKeys(vector<string>& keys)
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrUser::getKeys()..." << endl;
	#endif
	hlrAcctdesc AcctdescBuff(uid,"user",email,descr,certSubject);
	//populate the Acctdescv vector with the matching records
	if ( AcctdescBuff.getKeys(keys) != 0 )
	{
		#ifdef DEBUG
		 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 cerr << "Couldn't get hlrAcctdesc info from the Database." << endl;
		#endif
		#ifdef DEBUG
		cerr << "DEBUG: EXITING hlrUser::getKeys()." << endl << endl;
		#endif
		 return E_GET_KEYS;
	}
		#ifdef DEBUG
		cerr << "DEBUG: EXITING hlrUser::getKeys()." << endl << endl;
		#endif
	return 0;
}

ostream& operator << ( ostream& os, const hlrUser& u )
{
	os << "uid=" << u.uid;
	os << ",email=" << u.email;
	os << ",descr=" << u.descr;
	os << ",certSubject=" << u.certSubject;
	os << ",gid=" << u.gid;
	os << ",fid=" << u.fid;
	os << ",assigned=" << u.assigned;
	os << ",booked=" << u.booked;
	os << ",spent=" << u.spent;
	return os;

}


int hlrUser::credit(int v)
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrUser::credit()..." << endl;
	#endif
	string logBuff;
	logBuff = "Crediting user account: user=" + uid + "group=" + gid + "fid=" + fid + "by " + int2string(v);   
	hlr_log(logBuff,&logStream);
	if ( uid == "" || gid == "" || fid == "" )
	{
		#ifdef DEBUG
			cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 	cerr << "In the credit method uid and gid are mandatory info." << endl;
		#endif
		#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrUser::credit()." << endl << endl;
		#endif
		 return E_CREDIT;
	}
	hlrUgf ugBuff(uid,gid,fid); 
	hlrUgf ugRollBack = ugBuff; // Buffer to store Ug rollback
	//check if this Ug record already exists;
	bool UgExists = ugBuff.exists();
	if ( UgExists )
	{
		// store info for rollback in case of error
		if ( ugRollBack.get() != 0 )
		{
			#ifdef DEBUG
			cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 	cerr << "Couldn't get rollback info from the Database." << endl;
			#endif
			#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrUser::credit()." << endl << endl;
			#endif
			hlr_log("hlrUser::credit(): Error crediting the account (E_GET_ROLLBACK)!", &logStream);
		 	return E_GET_ROLLBACK;
		}
		// we have info for eventual rollback	
	}
	if ( ugBuff.credit(v) != 0 )
	{
			#ifdef DEBUG
			cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 	cerr << "Couldn't update info in the Database." << endl;
			#endif
			//error updating the info, since nothing has been
			//modified yet we can simply return with an error
			#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrUser::credit()." << endl << endl;
			#endif
			hlr_log("hlrUser::credit(): Error crediting the account (E_CREDIT_UG)!", &logStream);
			return E_CREDIT_UG;
	}
	#ifdef DEBUG
	cerr << "DEBUG: EXITING hlrUser::credit()." << endl << endl;
	#endif
	hlr_log("Account credited succesfully!", &logStream);
	return 0;
}

int hlrUser::debit(int v)
{
	#ifdef DEBUG
	cerr << endl <<"DEBUG: ENTERING hlrUser::debit()..." << endl;
	#endif
	string logBuff;
	logBuff = "Debiting user account: user=" + uid + "group=" + gid + "fid=" + fid + "by " + int2string(v);   
	hlr_log(logBuff,&logStream);
	if ( uid == "" || gid == "" || fid == "" )
	{
		#ifdef DEBUG
			cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 	cerr << "In the debit method uid gid fid are mandatory info." << endl;
		#endif
		#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrUser::del()." << endl << endl;
		#endif
		 return E_DEBIT;
	}
	hlrUgf ugBuff(uid,gid,fid); 
	hlrUgf ugRollBack = ugBuff; // Buffer to store Ug rollback
	//check if this Ug record already exists;
	bool UgExists = ugBuff.exists();
	if ( UgExists )
	{
		// store info for rollback in case of error
		if ( ugRollBack.get() != 0 )
		{
			#ifdef DEBUG
			cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 	cerr << "Couldn't get rollback info from the Database." << endl;
			#endif
			#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrUser::debit()." << endl << endl;
			#endif
			hlr_log("hlrUser::debit(): Error crediting the account (E_GET_ROLLBACK)!", &logStream);
		 	return E_GET_ROLLBACK;
		}
		// we have info for eventual rollback	
	}
	if ( ugBuff.debit(v) != 0 )
	{
			#ifdef DEBUG
			cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << ":";
		 	cerr << "Couldn't update info in the Database." << endl;
			#endif
			//error deleting the info, since nothing has been
			//modified yet we can simply return with an error
			#ifdef DEBUG
			cerr << "DEBUG: EXITING hlrUser::debit()." << endl << endl;
			#endif
			hlr_log("hlrUser::debit(): Error crediting the account (E_DEBIT_UG)!", &logStream);
			return E_DEBIT_UG;
	}
	return 0;
}

