/***************************************************************************
 *  filename  : LDAPForwardIterator.h
 *  authors   : Salvatore Monforte <salvatore.monforte@ct.infn.it>
 *  copyright : (C) 2001 by INFN
 ***************************************************************************/

/**
 * @file LDAPForwardIterator.h
 * @brief Contains definitions for LDAP Forward Iterator.
 * @author Salvatore Monforte salvatore.monforte@ct.infn.it
 * @author comments by Marco Pappalardo marco.pappalardo@ct.infn.it  and Salvatore Monforte
 */

// $Id:

#ifndef _LDAP_FORWARD_ITERATOR_
#define _LDAP_FORWARD_ITERATOR_

/** LDAP header file. */
#include "lber.h"
/** LDAP header file. */
#include "ldap.h"

#include <string>

using namespace std;

/** A class representing LDIF Object. */
class LDIFObject;
/** A class representing LDAP Query Object. */
class LDAPQuery;

/**
 * A forward iterator for LDAP Objects.
 * It applies to LDAP Query results a powerful means of reading.
 * @author Salvatore Monforte salvatore.monforte@ct.infn.it
 * @author comments by Marco Pappalardo marco.pappalardo@ct.infn.it and Salvatore Monforte
 */
class LDAPForwardIterator
{
 public:
  /**
   * Constructor
   * @param q a reference to an LDAPQuery result object.
   */
  LDAPForwardIterator(LDAPQuery*);
  /**
   * Rewinds the LDAP forward iterator.
   * After use of this method, the forward iterator is addressed 
   * to the first query result record.
   * @return true if first query entry is reached, false otherwise.
   */
  bool rewind();
  /**
   * Moves to next record.
   * Redefines '++' operator in order to provide a simple way to
   * move forward inside the LDAP Query.
   * @return true if next entry exists, false otherwise.
   */
  bool operator++();
  
  /** 
   * Returns the DN attribute for LDAP.
   */ 
  const std::string& ldap_dn();

  /**
   * 
   */
  const LDIFObject& operator*();

  /**
   * Destructor.
   */
  ~LDAPForwardIterator();

 private:  
  /** An LDAP Message. */  
  LDAPMessage* ldresult;
  /** The LDAP current result message. */ 
  LDAPMessage* ld_current_result;
  /** A query result. */
  LDAPQuery* query;
  /** */
  LDIFObject *object;
  /** The DN of this iterator. */
  std::string dn;
};

#endif

/*
  Local Variables:
  mode: c++
  End:
*/







