/***************************************************************************
 *  filename  : LDAPQuery.h
 *  authors   : Salvatore Monforte <salvatore.monforte@ct.infn.it>
 *  copyright : (C) 2001 by INFN
 ***************************************************************************/

// $Id:

/**
 * @file LDAPQuery.cpp
 * @brief This files contains definition for an LDAP Query Object.
 * @author Salvatore Monforte salvatore.monforte@ct.infn.it
 * @author commetns by Marco Pappalardo marco.pappalardo@ct.infn.it and Salvatore Monforte
 */


#ifndef _LDAP_QUERY_
#define _LDAP_QUERY_

#include <string>
#include <vector>
#include "lber.h"	/**< LDAP header file. */
#include "ldap.h"       /**< LDAP header file. */

using namespace std;

/** An LDAP Connection. */
class LDAPConnection;

/**
 * Queries a Database.
 * This class uses LDAP Connection base class to query a database for info.
 * @author Salvatore Monforte salvatore.monforte@ct.infn.it
 * @author commetns by Marco Pappalardo marco.pappalardo@ct.infn.it and Salvatore Monforte
 */
class LDAPQuery
{
 public:
  /**
   * Constructor.
   * This object queries a database through an LDAP Connection 
   * for record satisfing a filter, having a specified scope.
   */ 
  LDAPQuery(LDAPConnection*, const string&, const vector<string>&, int s = LDAP_SCOPE_SUBTREE);

  /**
   * Destructor.
   */
  ~LDAPQuery();
  
  /**
   * Returns scope attribute value.
   * @return the scope.
   */
  int Scope() const { return scope; }
  /**
   * Returns the applied filter.
   * @return the filter.
   */
  const string& Filter() const { return filter; }
  /**
   * Returns the topic attribute.
   * @return the topic.
   */
  char** Topic() const { return topic; }

 private:
  /** The forward iterator to move through the query result. */
  friend class LDAPForwardIterator;
  /** The LDAP Connection to data. */
  LDAPConnection* connection;

  string filter;  /**< A string representation of the filter to apply in the search. */
  char** topic;   /**< A null-terminated array of  attribute  types  to  return  from entries that  match filter. */
  int scope;      /**< the scope of the search  and  should  be  one  of LDAP_SCOPE_BASE,  to  search the object itself, LDAP_SCOPE_ONELEVEL, to search the  object's  immediate  children,  or  LDAP_SCOPE_SUBTREE,  to search the object and all its descendents. */
};

#endif

/*
  Local Variables:
  mode: c++
  End:
*/







