/***************************************************************************
 *  filename  : LDAPSynchConnection.h
 *  authors   : Salvatore Monforte <salvatore.monforte@ct.infn.it>
 *  copyright : (C) 2001 by INFN
 ***************************************************************************/

// $Id:

/**
 * @file LDAPSynchConnection.h
 * @brief This files contains definitions for LDAP Synchronous connections.
 * @author Salvatore Monforte salvatore.monforte@ct.infn.it
 * @author comments by Marco Pappalardo marco.pappalardo@ct.infn.it and Salvatore Monforte
 */

#ifndef _LDAPSYNCHCONNECTION_
#define _LDAPSYNCHCONNECTION_ 

#include <sys/time.h>

/** Includes the superclass's definitions. */
#include "LDAPConnection.h"

using namespace std;

/**
 * A Synchronous connection to an LDAP Server.
 * This class implements all virtual superclass methods in order
 * to provide a synchronous connection to the LDAP Server.
 * @see LDAPConnection
 * @author Salvatore Monforte salvatore.monforte@ct.infn.it
 * @author comments by Marco Pappalardo marco.pappalardo@ct.infn.it
 */
class LDAPSynchConnection : public LDAPConnection
{
 public:
  /**
   * Constructor.
   */
  LDAPSynchConnection(const std::string&, const std::string&, int, long = 15);
  /**
   * Distructor.
   */
  virtual ~LDAPSynchConnection();

  /**
   * Opens a connection to the Server. 
   * This function overrides virtual LDAP Connection one.
   * @return true on success, false otherwise.
   */
  bool Open();
  /**
   * Closes a connection to the Server. 
   * This function overrides virtual LDAP Connection one.
   * @return true on success, false otherwise.
   */
  bool Close();
  /**
   * Performs LDAP search  operations using ldap_search_st() 
   * allowing a timeout to be specified.
   * @param filter is a string representation of the filter to apply in the search.
   * @param scope is the scope of the searchitself.
   * @param topics is a null-terminated array of attribute types to return from entries that match filter.
   * @param results LDAPMessage filled back by the ldap_search_st()
   * @return wheter the search operation has compled succesfully, or not.
   */
  bool Search(const std::string& filter, int scope, char** topics, LDAPMessage** results);
 
 private:
  /** A connection timeout. */
  struct timeval timeout;
};

#endif

/*
  Local Variables:
  mode: c++
  End:
*/






