/***************************************************************************
 *  filename  : LDIFObject.cpp
 *  authors   : Salvatore Monforte <salvatore.monforte@ct.infn.it>
 *  copyright : (C) 2001 by INFN
 ***************************************************************************/

// $Id:

#include <sstream>


#include "LDIFObject.h"

LDIFObject::LDIFObject()
{
}

LDIFObject::LDIFObject(const LDIFObject& o)
{
  attributes = o.attributes;
}

LDIFObject& LDIFObject::operator=(const LDIFObject& o )
{
  attributes = o.attributes;
  return *this;
}
void LDIFObject::add(const string& att, const string& val)
{
  attributes[att].push_back(val);
}
 
bool LDIFObject::EvaluateAttribute(const string& name, string& value) const
{
 bool result;
 if( result = !attributes[name].empty() )
 	value = attributes[name].front();

 return result;
}

bool LDIFObject::EvaluateAttribute(const string& name, vector<string>& values) const
{
 bool result; 
 if( result = !attributes[name].empty() )
        values = vector<string>(attributes[name].begin(), attributes[name].end());

 return result;
}

ostream& operator << (ostream& s, const LDIFObject& o)
{
  for (LDIFAttributes::const_iterator it = o.attributes.begin(); 
       it != o.attributes.end(); it++) {

    s << (*it).first << " = <" << endl;
    
    for(LDIFValue::const_iterator v_it = (*it).second.begin();
	v_it != (*it).second.end(); v_it++) cout << (*v_it) << endl;
    s << ">" << endl;
  }
  return s;
}

void LDIFObject::ParseValue(const string& v, stringstream& s) const
{
 stringstream vstream;
 string value, unit; 
  
 vstream << v;
 vstream >> value >> unit;

  if(isdigit(value[0])) { //...if numeric value... 
   
    // append .0 if needed
    if( value.find(".")==string::npos ) value.append(".0");
   
    s << value;

    switch(unit[0] ) { //...parse the unit factor, if any.
	  
    case 'B': 
    case 'K': 
    case 'M': 
    case 'G': 
    case 'T': s << unit[0]; break;
    
    }
  }
  else {				 				

#ifdef DONT_QUOTE_TRUE_FALSE
    // Change everything back into upper case, but store the
   // result in a different string
   string  lower_v(v);
   std::transform (lower_v.begin(), lower_v.end(), lower_v.begin(), tolower);
   
   if( lower_v == "true" || lower_v == "false" ) {
	s << lower_v;
    }
    else {
#endif // DONT_QUOTE_TRUE_FALSE

    // Quotes the value for the attribute if alphanumeric...
    s << "\"" << v << "\""; 

#ifdef DONT_QUOTE_TRUE_FALSE
    }
#endif // DONT_QUOTE_TRUE_FALSE
  }
}

void LDIFObject::ParseMultiValue(const LDIFValue& v, stringstream& s) const
{
  s << "{";  
  
  for(LDIFValue::const_iterator it = v.begin(); it != v.end(); ) {
    
    ParseValue(*it, s);
    
    s << ((++it != v.end()) ? "," : "}" );     
  }
}

 
