// $Id: hlr_add_fund.cpp,v 1.8 2003/12/02 13:50:05 guarise Exp $
// -------------------------------------------------------------------------
// Copyright (c) 2001-2002, The DataGrid project, INFN, 
// All rights reserved. See LICENSE file for details.
// -------------------------------------------------------------------------
// Author: Andrea Guarise <andrea.guarise@to.infn.it>
/***************************************************************************
 * Code borrowed from:
 *  authors   : 
 *             
 *  
 ***************************************************************************/

 /* Version 0.2 
 * revision a 
 * first release 26/07/2001 
 */

#include <iostream>
#include <ctype.h>
#include <getopt.h>
#include "edg/workload/dgas/hlr_api/hlrFund.h"
#include "edg/workload/dgas/common/dgas_config.h"
#include "edg/workload/dgas/common/dgasVersion.h"

#define OPTION_STRING "ihFf:d:t:C:"
#define DGAS_DEF_CONF_FILE "/opt/edg/etc/edg-wl-dgas-hlr.conf"

using namespace std;

const char * hlr_sql_server;
const char * hlr_sql_user;
const char * hlr_sql_password;
const char * hlr_sql_dbname;
ofstream logStream;
bool interactive = false;
bool needs_help = false;
bool force = false;
string fid_buff = "";
string descr_buff = "";
string total_buff = "";
string confFileName = DGAS_DEF_CONF_FILE;
string hlr_logFileName;

int options ( int argc, char **argv )
{
	int option_char;
	int option_index = 0;
	static struct option long_options[] =
	{
		{"interactive",0,0,'i'},
		{"fid",1,0,'f'},
		{"Force",0,0,'F'},
		{"descr",1,0,'d'},
		{"total",1,0,'t'},
		{"Conf",1,0,'C'},
		{"help",0,0,'h'},
		{0,0,0,0}
	};
	while (( option_char = getopt_long( argc, argv, OPTION_STRING,
					long_options, &option_index)) != EOF)
		switch (option_char)
		{
			case 'i': interactive=true; break;
			case 'f': fid_buff = optarg; break;
			case 'F': force = true; break;
			case 'd': descr_buff = optarg; break;
			case 't': total_buff = optarg; break;
			case 'C': confFileName = optarg; break;
			case 'h': needs_help =true; break;		  
			default : break;
		}
	return 0;
}

//Data query
int get_fund_info ()
{
	if ( interactive ) {
	if ( fid_buff == "" )
	{	
		cout << "fid: ";
		getline(cin ,fid_buff,'\n');
	}	
	if ( descr_buff == "" )
	{
		cout << "descr: ";
		getline(cin,descr_buff, '\n');
	}
	if ( total_buff == "" )
	{
		cout << "total: ";
		getline(cin, total_buff, '\n');
	}
	}	
	return 0;
}

int help (const char *progname)
{
        cerr << "\n " << progname << " " << VERSION << endl;
	cerr << " Author: Andrea Guarise " << endl;
	cerr << " usage: " << endl;
	cerr << " " << progname << " [OPTIONS] " << endl;
	cerr << "-i --interactive" << endl;
	cerr << "-C --Conf\t Configuration file name" << endl;
        cerr << "-f --fid \"fid\"" << endl;
        cerr << "-F --Force \"Force the record insertion\"" << endl;
	cerr << "-d --descr \"description\"" << endl;
	cerr << "-t --total \"total funds\"";
	cerr << endl;
	return 0;	
}

int main (int argc, char **argv)
{
	options ( argc, argv );
	map <string,string> confMap;
	dgas_conf_read ( confFileName, &confMap );
	hlr_sql_server = (confMap["hlr_sql_server"]).c_str();
	hlr_sql_user = (confMap["hlr_sql_user"]).c_str();
	hlr_sql_password = (confMap["hlr_sql_password"]).c_str();
	hlr_sql_dbname = (confMap["hlr_sql_dbname"]).c_str();
	hlr_logFileName = (confMap["hlr_def_log"]).c_str();
	if (needs_help)
	{
		help(argv[0]);
		exit(0);
	}
	logStream.open( hlr_logFileName.c_str() , ios::app);
	get_fund_info();
	
	hlrFund f;
	
	f.fid = fid_buff;
	f.descr = descr_buff;
	f.total = atoi(total_buff.c_str());
	f.spent = 0; //total amount spent by the whole fund, shouldn't be set up manually.

	if ( f.fid == "" )
	{
		cerr << "ERROR: fid is a mandatory parameter" << endl;
		help(argv[0]);
		logStream.close();
		return 1;
	}
	else
	{
		if ( force || !f.exists() )
		{
			int res = f.put();
			if ( res == 0 )
			{
				cout << "Record added!" << endl;
				logStream.close();
				return 0;
			}
			else
			{
				cout << "Error adding fund!" << endl;
				logStream.close();
				return res;
			}
		}
		else
		{
			cout << "fund already exixts!" << endl;
			logStream.close();
			return 1;
		}
	}
		
}
