// $Id: hlr_add_group.cpp,v 1.8 2003/12/02 13:50:05 guarise Exp $
// -------------------------------------------------------------------------
// Copyright (c) 2001-2002, The DataGrid project, INFN, 
// All rights reserved. See LICENSE file for details.
// -------------------------------------------------------------------------
// Author: Andrea Guarise <andrea.guarise@to.infn.it>
/***************************************************************************
 * Code borrowed from:
 *  authors   : 
 *             
 *  
 ***************************************************************************/

 /* Version 0.2 
 * revision a 
 * first release 26/07/2001 
 */

#include <iostream>
#include <getopt.h>
#include "edg/workload/dgas/hlr_api/hlrGroup.h"
#include "edg/workload/dgas/common/dgas_config.h"
#include "edg/workload/dgas/common/dgasVersion.h"

#define OPTION_STRING "ihFg:d:f:t:b:s:C:"
#define DGAS_DEF_CONF_FILE "/opt/edg/etc/edg-wl-dgas-hlr.conf"

using namespace std;

const char * hlr_sql_server;
const char * hlr_sql_user;
const char * hlr_sql_password;
const char * hlr_sql_dbname;
ofstream logStream;
bool interactive = false;
bool needs_help = false;
bool force = false;
string gid_buff = "";
string descr_buff = "";
string fid_buff = "";
string total_buff = "";
string booked_buff = "";
string spent_buff = "";
string confFileName = DGAS_DEF_CONF_FILE;
string hlr_logFileName;

int options ( int argc, char **argv )
{
	int option_char;
	int option_index = 0;
	static struct option long_options[] =
	{
		{"interactive",0,0,'i'},
		{"Force",0,0,'F'},
		{"gid",1,0,'g'},
		{"descr",1,0,'d'},
		{"fid",1,0,'f'},
		{"total",1,0,'t'},
		{"booked",1,0,'b'},
		{"spent",1,0,'s'},
		{"Conf",1,0,'C'},
		{"help",0,0,'h'},
		{0,0,0,0}
	};
	while (( option_char = getopt_long( argc, argv, OPTION_STRING,
					long_options, &option_index)) != EOF)
		switch (option_char)
		{
			case 'i': interactive=true; break;
			case 'F': force=true; break;
			case 'g': gid_buff = optarg; break;
			case 'd': descr_buff = optarg; break;
			case 'f': fid_buff = optarg; break;
			case 't': total_buff = optarg; break;
			case 'b': booked_buff = optarg; break;
			case 's': spent_buff = optarg; break;
			case 'C': confFileName = optarg; break;
			case 'h': needs_help =true; break;		  
			default : break;
		}
	return 0;
}

//Data query
int get_group_info ()
{
	if ( interactive ) {
	if ( gid_buff == "" )
	{	
		cout << "gid: ";
		getline(cin ,gid_buff,'\n');
	}	
	if ( descr_buff == "" )
	{
		cout << "descr: ";
		getline(cin,descr_buff, '\n');
	}
	if ( fid_buff == "" )
	{
		cout << "fid: ";
		getline(cin, fid_buff, '\n');
	}
	if ( total_buff == "" )
	{
		cout << "total: ";
		getline(cin, total_buff, '\n');
	}
	if ( booked_buff == "" )
	{
		cout << "booked: ";
		getline (cin, booked_buff, '\n');
	}
	if ( spent_buff == "" )
	{
		cout << "spent: ";
		getline(cin, spent_buff,'\n');
	}
	}	
	return 0;
}

int help (const char *progname)
{
        cerr << "\n " << progname << " " << VERSION << endl;
	cerr << " Author: Andrea Guarise " << endl;
	cerr << " usage: " << endl;
	cerr << " " << progname << " [OPTIONS]" << endl;
	cerr << "-i --interactive" << endl;
	cerr << "-F --Force"<<endl;
	cerr << "-C --Conf\t Configuration file name"<<endl;
        cerr << "-g --gid \"gid\""<<endl;
	cerr << "-d --descr \"description\""<<endl;
	cerr << "-f --fid \"fid\""<<endl;
	cerr << "-t --total \"total funds\""<<endl;
	cerr << "-b --booked \"booked funds\" shouldn't be specified manually"<<endl;
	cerr << "-s --spent \"spent funds\" shouldn't be specified manually";
	cerr << endl;
	return 0;	
}

int main (int argc, char **argv)
{
	options ( argc, argv );
	map <string,string> confMap;
	dgas_conf_read ( confFileName, &confMap );
	hlr_sql_server = (confMap["hlr_sql_server"]).c_str();
	hlr_sql_user = (confMap["hlr_sql_user"]).c_str();
	hlr_sql_password = (confMap["hlr_sql_password"]).c_str();
	hlr_sql_dbname = (confMap["hlr_sql_dbname"]).c_str();
	hlr_logFileName = (confMap["hlr_def_log"]).c_str();
	if (needs_help)
	{
		help(argv[0]);
		exit(0);
	}
	logStream.open( hlr_logFileName.c_str() , ios::app);
	get_group_info ();
	
	hlrGroup g;
	
	g.gid = gid_buff;
	g.descr = descr_buff;
	g.fid = fid_buff;
	g.total = atoi(total_buff.c_str());
	g.booked = atoi(booked_buff.c_str());
	g.spent = atoi(spent_buff.c_str());

	if ( g.gid == "" || g.fid == "")
	{
		cerr << "ERROR: fid and gid are mandatory parameters!" << endl;
		help(argv[0]);
		logStream.close();
		return 1;
	}
	else
	{
		if ( force || !g.exists() )
		{
			int res = g.put();
			if ( res == 0 )
			{
				cout << "Record added!" << endl;
				logStream.close();
				return 0;
			}
			else
			{
				cout << "Error adding group!" << endl;
				cout << "received error code: " << res << endl;
				logStream.close();
				return res;
			}
		}
		else
		{
			cout << "group already exists!" << endl;
			logStream.close();
			return 1;
		}
	}	
	
}
