// $Id: hlr_del_fund.cpp,v 1.8 2003/12/02 13:50:05 guarise Exp $
// -------------------------------------------------------------------------
// Copyright (c) 2001-2002, The DataGrid project, INFN, 
// All rights reserved. See LICENSE file for details.
// -------------------------------------------------------------------------
// Author: Stefano Barale <stefano.barale@to.infn.it>
/***************************************************************************
 * Code borrowed from: 
 *  authors   : 
 *             
 *  
 ***************************************************************************/


#include <iostream>
#include <getopt.h>
#include "edg/workload/dgas/hlr_api/hlrFund.h"
#include "edg/workload/dgas/common/dgas_config.h"
#include "edg/workload/dgas/common/dgasVersion.h"

#define OPTION_STRING "ihf:d:C:"
#define DGAS_DEF_CONF_FILE "/opt/edg/etc/edg-wl-dgas-hlr.conf"

using namespace std;

const char * hlr_sql_server;
const char * hlr_sql_user;
const char * hlr_sql_password;
const char * hlr_sql_dbname;
ofstream logStream;
bool interactive = false;
bool needs_help = false;
string fid_buff = "";
string descr_buff = "";
string total_buff = "";
string confFileName = DGAS_DEF_CONF_FILE;
string hlr_logFileName;

int options ( int argc, char **argv )
{
	int option_char;
	int option_index = 0;
	static struct option long_options[] =
	{
		{"interactive",0,0,'i'},
		{"fid",1,0,'f'},
		{"descr",1,0,'d'},
		{"Conf",1,0,'C'},
		{"help",0,0,'h'},
		{0,0,0,0}
	};
	while (( option_char = getopt_long( argc, argv, OPTION_STRING,
					long_options, &option_index)) != EOF)
		switch (option_char)
		{
			case 'i': interactive = true; break;
			case 'f': fid_buff = optarg; break;
			case 'd': descr_buff = optarg; break;
			case 'C': confFileName = optarg; break;
			case 'h': needs_help = true; break;		  
			default : break;
		}
	return 0;
}

//Data query
int get_fund_info ()
{
	if ( interactive ) {
	  if ( fid_buff == "" )
	    {	
	      cout << "fid: ";
	      getline(cin ,fid_buff,'\n');
	    }	
	  if ( descr_buff == "" )
	    {
	      cout << "descr: ";
	      getline(cin,descr_buff, '\n');
	    }
	}	
	return 0;
}

int help (const char *progname)
{
        cerr << "\n " << progname << " " << VERSION << endl;
	cerr << " Author: Andrea Guarise, Stefano Barale " << endl;
	cerr << " usage: " << endl;
	cerr << " " << progname << " [OPTIONS] " << endl;
	cerr << "-i --interactive"<< endl;
	cerr << "-C --Conf\t Configuration file name" << endl;
        cerr << "-f --fid \"fid\" (mandatory)" << endl;
	cerr << "-d --descr \"description\"";
	cerr << endl;
	return 0;	
}

int main (int argc, char **argv)
{
	options ( argc, argv );
	map <string,string> confMap;
	dgas_conf_read ( confFileName, &confMap );
	hlr_sql_server = (confMap["hlr_sql_server"]).c_str();
	hlr_sql_user = (confMap["hlr_sql_user"]).c_str();
	hlr_sql_password = (confMap["hlr_sql_password"]).c_str();
	hlr_sql_dbname = (confMap["hlr_sql_dbname"]).c_str();
	hlr_logFileName = (confMap["hlr_def_log"]).c_str();
	
	if (needs_help)
	{
		help(argv[0]);
		exit(0);
	}
	logStream.open( hlr_logFileName.c_str() , ios::app);
	get_fund_info();

	if ( fid_buff == "" )
	{
		cerr << "The fid is a mandatory paramater" << endl;
		help(argv[0]);
		logStream.close();
		exit(1);
	}
	else
	{
	
		hlrFund f;
	
		f.fid = fid_buff;
		f.descr = descr_buff;

		if ( f.exists() )
		{
			int res = f.del();
			if ( res == 0 )
			{
				cout << "Record deleted!" << endl;
				logStream.close();
				return 0;
			}
			else
			{
				cerr << "Error deleting fund, exit code: "<< res << endl;
				logStream.close();
			        return res;	
			}
			
		}
		else
		{
			cerr << "Error, the fund doesn't exixts!" << endl;
			logStream.close();
			exit(1);
		}

	}
		
}
