// $Id: hlr_del_group.cpp,v 1.8 2003/12/02 13:50:05 guarise Exp $
// -------------------------------------------------------------------------
// Copyright (c) 2001-2002, The DataGrid project, INFN, 
// All rights reserved. See LICENSE file for details.
// -------------------------------------------------------------------------
// Authors: Stefano Barale <stefano.barale@to.infn.it>
/***************************************************************************
 * Code borrowed from: Andrea Guarise <andrea.guarise@to.infn.it>
 *  authors   : 
 *             
 *  
 ***************************************************************************/

 /* Version 0.1
 * revision a 
 * first release 26/08/2002 
 */

#include <iostream>
#include <getopt.h>
#include "edg/workload/dgas/hlr_api/hlrGroup.h"
#include "edg/workload/dgas/common/dgas_config.h"
#include "edg/workload/dgas/common/dgasVersion.h"

#define OPTION_STRING "ihg:d:f:C:"
#define DGAS_DEF_CONF_FILE "/opt/edg/etc/edg-wl-dgas-hlr.conf"

using namespace std;

const char * hlr_sql_server;
const char * hlr_sql_user;
const char * hlr_sql_password;
const char * hlr_sql_dbname;
ofstream logStream;
bool interactive = false;
bool needs_help = false;
string gid_buff = "";
string descr_buff = "";
string fid_buff = "";
string confFileName = DGAS_DEF_CONF_FILE;
string hlr_logFileName;

int options ( int argc, char **argv )
{
	int option_char;
	int option_index = 0;
	static struct option long_options[] =
	{
		{"interactive",0,0,'i'},
		{"gid",1,0,'g'},
		{"descr",1,0,'d'},
		{"fid",1,0,'f'},
		{"Conf",1,0,'C'},
		{"help",0,0,'h'},
		{0,0,0,0}
	};
	while (( option_char = getopt_long( argc, argv, OPTION_STRING,
					long_options, &option_index)) != EOF)
		switch (option_char)
		{
			case 'i': interactive=true; break;
			case 'g': gid_buff = optarg; break;
			case 'd': descr_buff = optarg; break;
			case 'f': fid_buff = optarg; break;
			case 'C': confFileName = optarg; break;
			case 'h': needs_help =true; break;		  
			default : break;
		}
	return 0;
}

//Data query
int get_group_info ()
{
	if ( interactive ) {
	if ( gid_buff == "" )
	{	
		cout << "gid: ";
		getline(cin ,gid_buff,'\n');
	}	
	if ( fid_buff == "" )
	{
		cout << "fid: ";
		getline(cin, fid_buff, '\n');
	}
	if ( descr_buff == "" )
	{
		cout << "descr: ";
		getline(cin,descr_buff, '\n');
	}
	}	
	return 0;
}

int help (const char *progname)
{
        cerr << "\n " << progname << " " << VERSION << endl;
	cerr << " Authors: Stefano Barale and Andrea Guarise " << endl;
	cerr << " usage: " << endl;
	cerr << " " << progname << " [OPTIONS] " << endl;
 	cerr << "-i --interactive" << endl;
	cerr << "-C --Conf\t Configuration file name" << endl;
        cerr << "-g --gid \"gid\" (mandatory)" << endl;
	cerr << "-f --fid \"fid\" (mandatory)" << endl;
	cerr << "-d --descr \"description\"";
	cerr << endl;
	return 0;	
}

int main (int argc, char **argv)
{
	options ( argc, argv );
	map <string,string> confMap;
	dgas_conf_read ( confFileName, &confMap );
	hlr_sql_server = (confMap["hlr_sql_server"]).c_str();
	hlr_sql_user = (confMap["hlr_sql_user"]).c_str();
	hlr_sql_password = (confMap["hlr_sql_password"]).c_str();
	hlr_sql_dbname = (confMap["hlr_sql_dbname"]).c_str();
	hlr_logFileName = (confMap["hlr_def_log"]).c_str();

	get_group_info();
	
	if ( needs_help )
	{
		help(argv[0]);
		exit(1);
	}
	logStream.open( hlr_logFileName.c_str() , ios::app);
	
	if ((gid_buff == "" || fid_buff == "" ) )
	{
	    cerr << "You forgot to specify one or more mandatory option(s): " << endl;
	    help(argv[0]);
	    logStream.close();
	    exit(1);
	}
	
	hlrGroup g;
	
	g.gid = gid_buff;
	g.descr = descr_buff;
	g.fid = fid_buff;

	if ( g.exists() )
	{
		int res = g.del();
		if ( res == 0 )
		{
			cout << "Record deleted!" << endl;
		}
		else
		{
			cerr <<"Error deleting the group, exit status: "<< res <<endl;
			logStream.close();
			return res;
		}
	}
	else
	{
		cerr << "Error, group doesn't exists!" << endl;
		logStream.close();
		return 1;
	}

		
}
