// $Id: hlr_del_resource.cpp,v 1.8 2003/12/02 13:50:05 guarise Exp $
// -------------------------------------------------------------------------
// Copyright (c) 2001-2002, The DataGrid project, INFN, 
// All rights reserved. See LICENSE file for details.
// -------------------------------------------------------------------------
// Authors: Stefano Barale <barale@to.infn.it> 
// Authors: Andrea Guarise <guarise@to.infn.it> 
/***************************************************************************
 * Code borrowed from: Andrea Guarise <guarise@to.infn.it>
 *  authors   : 
 *             
 *  
 ***************************************************************************/

 /* 
 * first release 26/08/2002 
 */

#include <iostream>
#include <getopt.h>
#include "edg/workload/dgas/hlr_api/hlrResource.h"
#include "edg/workload/dgas/common/dgas_config.h"
#include "edg/workload/dgas/common/dgasVersion.h"

#define OPTION_STRING "ihr:d:c:g:f:C:"
#define DGAS_DEF_CONF_FILE "/opt/edg/etc/edg-wl-dgas-hlr.conf"

using namespace std;

const char * hlr_sql_server;
const char * hlr_sql_user;
const char * hlr_sql_password;
const char * hlr_sql_dbname;
ofstream logStream;

bool interactive = false;
bool needs_help = false;
string rid_buff = "";
string descr_buff = "";
string cert_subject_buff = "";
string gid_buff = "";
string fid_buff = "";
string confFileName = DGAS_DEF_CONF_FILE;
string hlr_logFileName;

int options ( int argc, char **argv )
{
	int option_char;
	int option_index = 0;
	static struct option long_options[] =
	{
		{"interactive",0,0,'i'},
		{"rid",1,0,'r'},
		{"descr",1,0,'d'},
		{"cert",1,0,'c'},
		{"gid",1,0,'g'},
		{"fid",1,0,'f'},
		{"Conf",1,0,'C'},
		{"help",0,0,'h'},
		{0,0,0,0}
	};
	while (( option_char = getopt_long( argc, argv, OPTION_STRING,
					long_options, &option_index)) != EOF)
		switch (option_char)
		{
			case 'i': interactive=true; break;
			case 'r': rid_buff = optarg; break;
			case 'd': descr_buff = optarg; break;
		  	case 'c': cert_subject_buff = optarg; break;
			case 'g': gid_buff = optarg; break;
			case 'f': fid_buff = optarg; break;
			case 'h': needs_help =true; break;		  
			case 'C': confFileName = optarg; break;
			default : break;
		}
	return 0;
}

//Data query
int get_resource_info ()
{
	if ( interactive ) {
		if ( rid_buff == "" )
		{	
			cout << "rid: ";
			getline(cin ,rid_buff,'\n');
		}	
		if ( descr_buff == "" )
		{
			cout << "descr: ";
			getline(cin,descr_buff, '\n');
		}
		if ( cert_subject_buff == "" )
		{
			cout << "cert_subject: ";
			getline(cin, cert_subject_buff , '\n');
		}
		if ( gid_buff == "" )
		{
			cout << "gid: ";
			getline(cin, gid_buff, '\n');
		}
		if ( fid_buff == "" )
		{
			cout << "fid: ";
			getline(cin, fid_buff, '\n');
		}
	}	
	return 0;
}

int help (const char *progname)
{
        cerr << "\n " << progname << endl;
	cerr << " Version: "<< VERSION << endl;
	cerr << " Authors: Stefano Barale and Andrea Guarise" << endl;
	cerr << " usage: " << endl;
	cerr << " " << progname << " [OPTIONS] " << endl;
	cerr << "-i --interactive" << endl;
	cerr << "-C --Conf\t Configuration file name" << endl;
        cerr << "-r --rid \"rid\" (mandatory)" << endl;
	cerr << "-d --descr \"description\"" << endl;
	cerr << "-c --cert \"cert subject\"" << endl;
	cerr << "-g --gid \"gid\" (mandatory)" << endl;
	cerr << "-f --fid \"fid\" (mandatory)"; 
	cerr << endl;
	return 0;	
}

int main (int argc, char **argv)
{
	options ( argc, argv );
        map <string,string> confMap;
        dgas_conf_read ( confFileName, &confMap );
        hlr_sql_server = (confMap["hlr_sql_server"]).c_str();
        hlr_sql_user = (confMap["hlr_sql_user"]).c_str();
        hlr_sql_password = (confMap["hlr_sql_password"]).c_str();
        hlr_sql_dbname = (confMap["hlr_sql_dbname"]).c_str();
	hlr_logFileName = (confMap["hlr_def_log"]).c_str();
	if (needs_help)
	{
	    help(argv[0]);
	    exit(0);
	}
	if ((gid_buff == "" || rid_buff == "" || fid_buff == "") && !needs_help)
	{
	    cerr << "You forgot to specify on or more mandatory option(s): " << endl;
	    help(argv[0]);
	    exit(1);
	}
	logStream.open( hlr_logFileName.c_str() , ios::app);
	get_resource_info ();
	
	hlrResource r;
	
	r.rid = rid_buff;
	r.descr = descr_buff;
	r.certSubject = cert_subject_buff;
	r.gid = gid_buff;
	r.fid = fid_buff;
	int res = r.del();
	if ( res == 0 )
	{
		cout << "record deleted!" << endl;
		logStream.close();
		return 0;
	}
	else
	{
		cerr << "error!" << endl;
		logStream.close();
		return 1;
	}
		
}
