// $Id: hlr_fgetinfo.cpp,v 1.8 2003/12/02 13:50:05 guarise Exp $
// -------------------------------------------------------------------------
// Copyright (c) 2001-2002, The DataGrid project, INFN, 
// All rights reserved. See LICENSE file for details.
// -------------------------------------------------------------------------
// Author: Andrea Guarise <andrea.guarise@to.infn.it>
/***************************************************************************
 * Code borrowed from:
 *  authors   : 
 *             
 *  
 ***************************************************************************/


#include <iostream>
#include <getopt.h>
#include "edg/workload/dgas/hlr_api/hlrFund.h"
#include "edg/workload/dgas/common/dgas_config.h"
#include "edg/workload/dgas/common/dgasVersion.h"

#define OPTION_STRING "iahf:C:"
#define DGAS_DEF_CONF_FILE "/opt/edg/etc/edg-wl-dgas-hlr.conf"

using namespace std;

const char * hlr_sql_server;
const char * hlr_sql_user;
const char * hlr_sql_password;
const char * hlr_sql_dbname;

ofstream logStream;
bool interactive = false;
bool all = false;
bool needs_help = false;
string fid_buff = "";
string confFileName = DGAS_DEF_CONF_FILE;
string hlr_logFileName;

int options ( int argc, char *argv[] )
{
	int option_char;
	int option_index = 0;
	static struct option long_options[] =
	{
		{"interactive",0,0,'i'},
		{"all",0,0,'a'},
		{"fid",1,0,'f'},
		{"Conf",1,0,'C'},
		{"help",0,0,'h'},
		{0,0,0,0}
	};
	while (( option_char = getopt_long( argc, argv, OPTION_STRING,
					long_options, &option_index)) != EOF)
		switch (option_char)
		{
			case 'i': interactive=true; break;
			case 'a': all=true; break;
			case 'f': fid_buff = optarg; break;
			case 'C': confFileName = optarg; break;
			case 'h': needs_help =true; break;		  
			default : break;
		}
	return 0;
}

//Data query
int get_fund_info ()
{
	if ( interactive ) {
	if ( fid_buff == "" )
	{	
		cout << "fid: ";
		getline(cin ,fid_buff,'\n');
	}	
	}	
	return 0;
}

int help (const char* progname)
{
        cerr << "\n " << progname << " " << VERSION << endl;
	cerr << " Author: Andrea Guarise " << endl;
	cerr << " usage: " << endl;
	cerr << " " << progname << " [OPTIONS] " << endl;
	cerr << "-i --interactive" << endl;
	cerr << "-C --Conf\t Configuration file name" << endl;
	cerr << "-a --all	    (search informations about all funds)" << endl;
        cerr << "-f --fid \"fid\"";
	cerr << endl;
	return 0;	
}

ostream& operator<< ( ostream& os, const hlrFund &f)
{
	
	os << "|";
	os << f.fid << "|";
	os << f.descr << "|";
	os << f.total << "|";
	os << f.spent << "|";
  	return os;	
}



int main (int argc, char *argv[])
{
	options ( argc, argv );
        map <string,string> confMap;
        dgas_conf_read ( confFileName, &confMap );
        hlr_sql_server = (confMap["hlr_sql_server"]).c_str();
        hlr_sql_user = (confMap["hlr_sql_user"]).c_str();
        hlr_sql_password = (confMap["hlr_sql_password"]).c_str();
        hlr_sql_dbname = (confMap["hlr_sql_dbname"]).c_str();
	hlr_logFileName = (confMap["hlr_def_log"]).c_str();
	if (needs_help)
	{
		help(argv[0]);
		return 0;
	}
	logStream.open( hlr_logFileName.c_str() , ios::app);					
	get_fund_info ();
	hlrFund f(fid_buff);
	if ( !all )
	{
		int res = 0;
		res = f.get();
		if ( res != 0 )
		{
			cerr << "Error retrieving fund info!" << endl;
			return 1;
		}
		cout << f << endl;
		logStream.close();
		return 0;
	}	
	else
	{
		int res =0;
		vector <hlrFund> fv;
		res = f.get( fv );
		if ( res != 0 )
		{
			cerr << "Error retrieving data" << endl;
			logStream.close();
			return 1;
		}
		vector <hlrFund>::const_iterator fv_it = fv.begin();
		while ( fv_it != fv.end() )
		{
			cout << *fv_it << endl;
			++fv_it;
		}
		logStream.close();
		return 0;
	}
}
