// $Id: hlr_rgetinfo.cpp,v 1.8 2003/12/02 13:50:05 guarise Exp $
// -------------------------------------------------------------------------
// Copyright (c) 2001-2002, The DataGrid project, INFN, 
// All rights reserved. See LICENSE file for details.
// -------------------------------------------------------------------------
// Author: Andrea Guarise <andrea.guarise@to.infn.it>
/***************************************************************************
 * Code borrowed from:
 *  authors   : 
 *             
 *  
 ***************************************************************************/


#include <iostream>
#include <getopt.h>
#include "edg/workload/dgas/hlr_api/hlrResource.h"
#include "edg/workload/dgas/hlr_api/hlrTransaction.h"
#include "edg/workload/dgas/common/dgas_config.h"
#include "edg/workload/dgas/common/dgasVersion.h"

#define OPTION_STRING "iaTRhr:c:C:"
#define DGAS_DEF_CONF_FILE "/opt/edg/etc/edg-wl-dgas-hlr.conf"

using namespace std;

const char * hlr_sql_server;
const char * hlr_sql_user;
const char * hlr_sql_password;
const char * hlr_sql_dbname;

ofstream logStream;
bool interactive = 0;
bool all = 0;
bool trans = 0;
bool needs_help = 0;
bool resources = 0;
string rid_buff = "";
string cert_subject_buff = "";
string confFileName = DGAS_DEF_CONF_FILE;
string hlr_logFileName;

int options ( int argc, char **argv )
{
	int option_char;
	int option_index = 0;
	static struct option long_options[] =
	{
		{"interactive",0,0,'i'},
		{"all",0,0,'a'},
		{"transactions",0,0,'T'},
		{"resources",0,0,'R'},
		{"rid",1,0,'r'},
		{"cert",1,0,'c'},
		{"Conf",1,0,'C'},
		{"help",0,0,'h'},
		{0,0,0,0}
	};
	while (( option_char = getopt_long( argc, argv, OPTION_STRING,
					long_options, &option_index)) != EOF)
		switch (option_char)
		{
			case 'i': interactive=1; break;
			case 'a': all=1; break;
			case 'T': trans=1; break;
			case 'r': rid_buff = optarg; break;
			case 'R': resources= 1; break;
			case 'c': cert_subject_buff = optarg; break;
			case 'C': confFileName = optarg; break;
			case 'h': needs_help =1; break;		  
			default : break;
		}
	return 0;
}

//Data query
int get_resource_info ()
{
	if ( interactive ) {
		if ( rid_buff == "" )
		{	
			cout << "rid: ";
			getline(cin ,rid_buff,'\n');
		}	
		if ( cert_subject_buff == "" )
		{
			cout << "cert_subject: ";
			getline(cin, cert_subject_buff , '\n');
		}
	}	
	return 0;
}

int help (const char *progname)
{
        cerr << "\n " << progname << " " << VERSION << endl;
	cerr << " Author: Andrea Guarise " << endl;
	cerr << " usage: " << endl;
	cerr << " " << progname << " [OPTIONS] " << endl;
	cerr << "-i --interactive"<< endl;
	cerr << "-C --Conf\t Configuration file name" << endl;
	cerr << "-T --transactions   (prints transaction list)" << endl;
	cerr << "-R --resources 	    (prints resources info)" << endl;
	cerr << "a --all	    (search informations about all resources)" <<endl;
        cerr << "-r --rid \"rid\"" << endl;
	cerr << "-c --cert \"cert subject\"";
	cerr << endl;
	return 0;	
}

ostream& operator<< ( ostream& os, const hlrResource& r )
{
	
	os << "|";
	os << r.rid << "|";
	os << r.email << "|";
	os << r.descr << "|";
	os << r.certSubject << "|";
	os << r.gid << "|";
	os << r.fid << "|";
	os << r.total << "|";
	return os;
}

ostream& operator<< ( ostream& os, const hlrTransaction& tr)
{
	os << "|";
	os << tr.tid << "|";
	if ( tr.tType == in ) 
	{
		os << "in" << "|";
	}
	else
	{
		os << "out" << "|";
	}
	os << tr.id << "|";
	os << tr.fromId << "|";
	os << tr.fromHlrUrl << "|";
	os << tr.toId << "|";
	os << tr.toHlrUrl << "|";
	os << tr.amount << "|";
	os << tr.timeStamp << "|";
	return os;
}

int get_trans_in ( string resource_cert )
{
	vector<hlrTransaction> trv;
	hlrTransaction tr(0,in,"","",resource_cert);
	int res = tr.get(trv);
	if ( res == 0 )
	{
		vector<hlrTransaction>::const_iterator trv_it = trv.begin();
		while ( trv_it != trv.end() )
		{
			cout << *trv_it << endl;
			++trv_it;
		}
	}
	return res;
}

int main (int argc, char **argv)
{
	options ( argc, argv );
        map <string,string> confMap;
        dgas_conf_read ( confFileName, &confMap );
        hlr_sql_server = (confMap["hlr_sql_server"]).c_str();
        hlr_sql_user = (confMap["hlr_sql_user"]).c_str();
        hlr_sql_password = (confMap["hlr_sql_password"]).c_str();
        hlr_sql_dbname = (confMap["hlr_sql_dbname"]).c_str();
	hlr_logFileName = (confMap["hlr_def_log"]).c_str();
	if (needs_help)
	{
		help(argv[0]);
		return 0;
	}
	logStream.open( hlr_logFileName.c_str() , ios::app);
	get_resource_info ();
	hlrResource r(rid_buff,"","",cert_subject_buff);
	if ( !all )
	{
		int res = 0;
		res = r.get();
		if ( res != 0 )
		{
			cerr << "Error retrieving resource info" << endl;
			logStream.close();
			return res;
		}
		if ( resources )
		{
			cout << r << endl;	
		}
		if ( trans )
		{
			cout << "Last Transactions:" << endl;
			get_trans_in ( r.certSubject );
		}
	logStream.close();
	return 0;
	}
	else
	{
		int res = 0;
		vector <hlrResource> rv;
		res = r.get(rv);
		if ( res != 0 )
		{
			cerr << "Error retrieving data" << endl;
			logStream.close();
			return res;
		}
		vector <hlrResource>::const_iterator rv_it = rv.begin();
		while ( rv_it != rv.end() )
		{
			if ( resources )
			{
				cout << *rv_it << endl;
			}
			if ( trans )
			{
				cout << "Last Transactions:" << endl;
				get_trans_in ( r.certSubject );
			}
			++rv_it;
		}
		logStream.close();
		return 0;
	}
}
