// $Id: hlr_ugetinfo.cpp,v 1.8 2003/12/02 13:50:05 guarise Exp $
// -------------------------------------------------------------------------
// Copyright (c) 2001-2002, The DataGrid project, INFN, 
// All rights reserved. See LICENSE file for details.
// -------------------------------------------------------------------------
// Author: Andrea Guarise <andrea.guarise@to.infn.it>
/***************************************************************************
 * Code borrowed from:
 *  authors   : 
 *             
 *  
 ***************************************************************************/


#include <iostream>
#include <getopt.h>
#include "edg/workload/dgas/hlr_api/hlrUser.h"
#include "edg/workload/dgas/hlr_api/hlrTransaction.h"
#include "edg/workload/dgas/common/dgas_config.h"
#include "edg/workload/dgas/common/dgasVersion.h"

#define OPTION_STRING "iaTUhu:e:c:f:C:"
#define DGAS_DEF_CONF_FILE "/opt/edg/etc/edg-wl-dgas-hlr.conf"

using namespace std;

const char * hlr_sql_server;
const char * hlr_sql_user;
const char * hlr_sql_password;
const char * hlr_sql_dbname;
ofstream logStream;

bool interactive = 0;
bool all = 0;
bool trans = 0;
bool needs_help = 0;
bool users = 0;
string uid_buff = "";
string email_buff = "";
string cert_subject_buff = "";
string confFileName = DGAS_DEF_CONF_FILE;
string format_String="UEDCABS";//for future implementation
string hlr_logFileName;

int options ( int argc, char **argv )
{
	int option_char;
	int option_index = 0;
	static struct option long_options[] =
	{
		{"interactive",0,0,'i'},
		{"all",0,0,'a'},
		{"transactions",0,0,'T'},
		{"users",0,0,'U'},
		{"uid",1,0,'u'},
		{"email",1,0,'e'},
		{"cert",1,0,'c'},
		{"Conf",1,0,'C'},
		{"help",0,0,'h'},
		{"format",1,0,'f'},
		{0,0,0,0}
	};
	while (( option_char = getopt_long( argc, argv, OPTION_STRING,
					long_options, &option_index)) != EOF)
		switch (option_char)
		{
			case 'i': interactive=1; break;
			case 'a': all=1; break;
			case 'T': trans=1; break;
			case 'u': uid_buff = optarg; break;
			case 'U': users= 1; break;
			case 'e': email_buff = optarg; break;
			case 'c': cert_subject_buff = optarg; break;
			case 'C': confFileName = optarg; break;
			case 'h': needs_help =1; break;		  
			case 'f': format_String = optarg; break;		  
			default : break;
		}
	return 0;
}

//Data query
int get_user_info ()
{
	if ( interactive ) {
	if ( uid_buff == "" )
	{	
		cout << "uid: ";
		getline(cin ,uid_buff,'\n');
	}	
	if ( email_buff == "" )
	{
		cout << "email: ";
		getline(cin,email_buff, '\n');
	}
	if ( cert_subject_buff == "" )
	{
		cout << "cert_subject: ";
		getline(cin, cert_subject_buff , '\n');
	}
	}	
	return 0;
}

int help (const char *progname)
{
        cerr << "\n " << progname << endl;
	cerr << " Version: "<< VERSION << endl;
	cerr << " Author: Andrea Guarise " << endl;
	cerr << " 28/08/2001 " << endl <<endl;
	cerr << " usage: " << endl;
	cerr << " " << progname << " [OPTIONS] " << endl;
	cerr << "-i --interactive" << endl;
	cerr << "-C --Conf\t Configuration file name" << endl;
	cerr << "-T --transactions   (prints transaction list)" << endl;
	cerr << "-U --users 	    (prints user info)" << endl;
	cerr << "-a --all	    (search informatios about all users)" << endl;
        cerr << "-u --uid \"uid\"" << endl;
	cerr << "-e --email \"email\"" << endl;
	cerr << "-c --cert \"cert subject\"";
	cerr << endl;
	return 0;	
}


ostream& operator << ( ostream& os, const hlrUser& u )
{
	
	os << "|";
	os << u.uid << "|";
	os << u.email << "|";
	os << u.descr << "|";
	os << u.certSubject << "|";
	os << u.gid << "|";
	os << u.fid << "|";
	os << u.assigned << "|";
	os << u.booked << "|";
	os << u.spent << "|";
	return os;
	
}

ostream& operator << ( ostream& os, const hlrTransaction& tr )
{
	os << "|";
	os << tr.tid << "|";
	if (tr.tType == in )
	{
		os << "in" << "|";
	}
	else
	{
		os << "out" << "|";
	}
	os << tr.id << "|";
	os << tr.fromId << "|";
	os << tr.fromHlrUrl << "|";
	os << tr.toId << "|";
	os << tr.toHlrUrl << "|";
	os << tr.amount << "|";
	os << tr.timeStamp << "|";
	return os;
}

int get_trans_out ( string user_cert )
{
	vector <hlrTransaction> trv;
	hlrTransaction tr(0,out,"",user_cert);
	int res = tr.get(trv);
	if (res == 0)
	{
		vector<hlrTransaction>::const_iterator trv_it = trv.begin();
		while ( trv_it != trv.end() )
		{
			cout << *trv_it << endl;
			++trv_it;
		}
	}
	return res;	
	
}

int main (int argc, char **argv)
{
	options ( argc, argv );
        map <string,string> confMap;
        dgas_conf_read ( confFileName, &confMap );
        hlr_sql_server = (confMap["hlr_sql_server"]).c_str();
        hlr_sql_user = (confMap["hlr_sql_user"]).c_str();
        hlr_sql_password = (confMap["hlr_sql_password"]).c_str();
        hlr_sql_dbname = (confMap["hlr_sql_dbname"]).c_str();
	hlr_logFileName = (confMap["hlr_def_log"]).c_str();
	if (needs_help)
	{
	help(argv[0]);
	return 0;
	}
	logStream.open( hlr_logFileName.c_str() , ios::app);
	get_user_info ();
	hlrUser u(uid_buff,"","",email_buff,"",cert_subject_buff);
	if ( !all )
	{
		int res = 0;
		res = u.get(); 
		if ( res != 0 )
		{
			cerr << "Error retrieving user info" << endl;
			logStream.close();
			return 1;
		}
		if ( users )
		{
			cout << u << endl;	
		}
		if ( trans )
		{
			get_trans_out ( u.certSubject );
		}
		logStream.close();
		return 0;
	}
	else
	{
		int res = 0;
		vector<hlrUser> uv;
		res = u.get(uv);
		if ( res != 0 )
		{
			cerr << "Error retrieving data" << endl;
			logStream.close();
			return 1;
		}
		vector <hlrUser>::const_iterator uv_it = uv.begin();
		while ( uv_it != uv.end() )
		{
			if ( users )
			{
				cout << *uv_it << endl;
			}
			if ( trans )
			{
				get_trans_out ( (*uv_it).certSubject );
			}
			++uv_it;
		}
		logStream.close();
		return 0;
	}
}
