// $Id: dgas_paResItem.cpp,v 1.2 2003/04/28 14:44:49 guarise Exp $
// -------------------------------------------------------------------------
// Copyright (c) 2001-2002, The DataGrid project, INFN, 
// All rights reserved. See LICENSE file for details.
// -------------------------------------------------------------------------
// Author: Andrea Guarise <andrea.guarise@to.infn.it>
/***************************************************************************
 * Code borrowed from:
 *  authors   : 
 *             
 *  
 ***************************************************************************/

#include <dgas_paResItem.h>
#include "pa_db_default.h"

#define TOO_MANY_ROWS  1
#define RECORD_EXISTS  2
#define NO_RECORD      3
#define GENERIC_ERROR  4
#define QUERY_ERROR    5
#define E_NO_ID	       6 

//SQL database connectivity info defined externally.
extern const char * pa_sql_server;
extern const char * pa_sql_user;
extern const char * pa_sql_password;
extern const char * pa_sql_dbname;


paResItem::paResItem(string _resID, int _defMinTTL, int _suggPrice)
{
			#ifdef DEBUG
			 cerr << "DEBUG: paResItem constructor ";
			#endif
			 id = 0;
			 resID = _resID;
			 defMinTTL = _defMinTTL;
			 if (defMinTTL < MIN_PRICE_TTL)
			   defMinTTL = MIN_PRICE_TTL;
			 suggestedPrice = _suggPrice;
			 histTable = "";
			#ifdef DEBUG
			 cerr << "id=" << id << ","; 
			 cerr << "resID=" << resID << ","; 
			 cerr << "defMinTTL=" << defMinTTL << ",";
			 cerr << "suggestedPrice=" << suggestedPrice << endl; 
			#endif
}

int paResItem::put()
{
        if (defMinTTL < MIN_PRICE_TTL)
	  defMinTTL = MIN_PRICE_TTL;

	#ifdef DEBUG
	 cerr << "DEBUG: paResItem::put(),";
	 cerr << "id=" << id << ",";
	 cerr << "resID=" << resID << ",";
	 cerr << "defMinTTL=" << defMinTTL << ",";
	 cerr << "suggestedPrice=" << suggestedPrice << endl;
	#endif
	 db paDb(pa_sql_server,
			 pa_sql_user,
			 pa_sql_password,
			 pa_sql_dbname
			 );
	 if ( paDb.errNo == 0 )
	 {
		 //insert 
		#ifdef DEBUG
		 cerr << "DEBUG: paResItem::put(), insert ok" << endl;
		#endif
		 //insert the Item and then add a new table
		 //
		#ifdef DEBUG
		 cerr << "DEBUG: paResItem::put(), inserting:";
		 cerr << "id=" << id << ",";
 		 cerr << "resID=" << resID << ",";
 		 cerr << "defMinTTL=" << defMinTTL << ",";
		 cerr << "suggestedPrice=" << suggestedPrice << endl;
		#endif
		 string queryStr;
		 queryStr = "INSERT INTO res_index VALUES (";
		 queryStr += int2string(id);
		 queryStr += ",'";
		 queryStr += resID;
		 queryStr += "',";
		 queryStr += int2string(defMinTTL);
		 queryStr += ",";
		 queryStr += int2string(suggestedPrice);
		 queryStr += ")";
		 paDb.query(queryStr);
		 if ( paDb.errNo != 0 )
		 {
			#ifdef DEBUG
			 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << " : ";
			 cerr << "error inserting the info:" << endl;
			 cerr << paDb.errMsg << endl;
			#endif 
			 return paDb.errNo;
			 //error in query
		 }
		 paResItem itemBuff;
		 if ( (itemBuff.get(resID)) != 0 )
		 {
			#ifdef DEBUG
			 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << " : ";
			 cerr << "error retrieving the id." << endl;
			#endif
			 return GENERIC_ERROR;
		 }
		 else
		 {
			 //ok, create the table
			 if ( (itemBuff.createHist()) != 0 )
			 {
				#ifdef DEBUG
				 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << " : ";
				 cerr << "error creating the table." << endl;
				#endif
			  	return GENERIC_ERROR;
			 }
			 else
			 {
				 return 0;
			 }
		 }
	 }
	 else
	 {
		 //Error conecting to the database, reporting the error
		#ifdef DEBUG
		 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << " : ";
		 cerr << "Error connecting to the database, got error: " << endl;
		 cerr << paDb.errMsg << endl;
		#endif
		 return paDb.errNo;
	 }
}

int paResItem::replace()
{
        if (defMinTTL < MIN_PRICE_TTL)
	  defMinTTL = MIN_PRICE_TTL;

	#ifdef DEBUG
	 cerr << "DEBUG: paResItem::replace(),";
	 cerr << "id=" << id << ",";
	 cerr << "resID=" << resID << ",";
	 cerr << "defMinTTL=" << defMinTTL << ",";
	 cerr << "suggestedPrice=" << suggestedPrice << endl;
	#endif
	 if ( id  == 0 )
	 {
		#ifdef DEBUG
		 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << " : ";
		 cerr << "id can't be null in replace, use paResItem::get(_resID) first!" << endl;
		#endif
		 return E_NO_ID;
	 }
	 db paDb(pa_sql_server,
			 pa_sql_user,
			 pa_sql_password,
			 pa_sql_dbname
			 );
	 if ( paDb.errNo == 0 )
	 {
		 string queryStr;
		 queryStr = "REPLACE INTO res_index VALUES (";
		 queryStr += int2string(id);
		 queryStr += ",'";
		 queryStr += resID;
		 queryStr += "',";
		 queryStr += int2string(defMinTTL);
		 queryStr += ",";
		 queryStr += int2string(suggestedPrice);
		 queryStr += ")";
		 paDb.query(queryStr);
		 if ( paDb.errNo != 0 )
		 {
			#ifdef DEBUG
			 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << " : ";
			 cerr << "error replacing the info:" << endl;
			 cerr << paDb.errMsg << endl;
			#endif 
			 return paDb.errNo;
			 //error in query
		 }
		 else
		 {
			 return 0;
		 }
	 }
	 else
	 {
		 //Error conecting to the database, reporting the error
		#ifdef DEBUG
		 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << " : ";
		 cerr << "Error connecting to the database, got error: " << endl;
		 cerr << paDb.errMsg << endl;
		#endif
		 return paDb.errNo;
	 }
	
}

int paResItem::get(string &_resID)
{
	#ifdef DEBUG
	 cerr << "DEBUG: paResItem::get(),";
	 cerr << "resID=" << _resID << ",";
	#endif
	 db paDb(pa_sql_server,
			 pa_sql_user,
			 pa_sql_password,
			 pa_sql_dbname
			 );
	 if ( paDb.errNo == 0 )
	 {
		 //connection to the DB is OK, we can do the query
		string queryStr = "";
		queryStr = "SELECT * FROM res_index WHERE res_cert_subject='";
		queryStr += _resID;
		queryStr += "'";
		#ifdef DEBUG
	 	 cerr << "DEBUG: paResItem::get(),";
		 cerr << "DB query:" << queryStr << endl;
		#endif
		 dbResult result= paDb.query(queryStr);
		 if ( paDb.errNo == 0)
		 {
			 int numRows = result.numRows();
			 //Query ok use the results
			 if (numRows > 1)
			 {
				#ifdef DEBUG
				 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << " : ";
				 cerr << "DB mismatch, found n." << numRows << " entries." << endl;
				#endif
				 return TOO_MANY_ROWS;
			 }
			 if ( numRows == 0 )
			 {
				#ifdef DEBUG
				 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << " : ";
				 cerr << "Record not found." << endl;
				#endif
				 return NO_RECORD;
			 }
			 if ( numRows == 1 )
			 {
				id = atoi((result.getData()[0][0]).c_str());
				resID = result.getData()[0][1];
				defMinTTL = atoi((result.getData()[0][2]).c_str());
				suggestedPrice = atoi((result.getData()[0][3]).c_str());
				histTable="hist_" + int2string(id);
				#ifdef DEBUG
	 			 cerr << "DEBUG: paResItem::get(),";
	 			 cerr << "id=" << id << ",";
	 			 cerr << "resID=" << resID << ",";
				 cerr << "defMinTTL=" << defMinTTL << ",";
				 cerr << "suggestedPrice=" << suggestedPrice 
				      << endl;
				#endif
				return 0;
			 }
		 }
		 else
		 {
			 //query failed, report the error
			#ifdef DEBUG
		 	 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << " : ";
		 	 cerr << "Error querying the database, got error: " << endl;
		 	 cerr << paDb.errMsg << endl;
			#endif
			 return paDb.errNo;
		 }
		 
	 }
	 else
	 {
		 //Error conecting to the database, reporting the error
		#ifdef DEBUG
		 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << " : ";
		 cerr << "Error connecting to the database, got error: " << endl;
		 cerr << paDb.errMsg << endl;
		#endif
		 return paDb.errNo;
	 }
	 return 0;
}

bool paResItem::exists()
{
	#ifdef DEBUG
	 cerr << "DEBUG: paResItem::exists(),";
	 cerr << "id=" << id << ",";
	 cerr << "resID=" << resID << ",";
	 cerr << "defMinTTL=" << defMinTTL << ",";
	 cerr << "suggestedPrice=" << suggestedPrice << endl;
	#endif
	 db paDb(pa_sql_server,
			 pa_sql_user,
			 pa_sql_password,
			 pa_sql_dbname
			 );
	 if ( paDb.errNo == 0 )
	 {
		string queryStr = "";
		//query the DB to find if the resource with resID already exists.
		queryStr = "SELECT * FROM res_index WHERE res_cert_subject='";
		queryStr += resID;
		queryStr += "'";
		#ifdef DEBUG
	 	 cerr << "DEBUG: paResItem::exists(),";
		 cerr << "DB query:" << queryStr << endl;
		#endif
		 dbResult result= paDb.query(queryStr);
		 if ( paDb.errNo == 0)
		 {
			 if ( result.numRows() == 1 )
			 {
				 return true;
			 }
			 else
			 {
				 return false;
			 }
		 }
		 else
		 {
			 //query failed, report the error
			#ifdef DEBUG
		 	 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << " : ";
		 	 cerr << "Error querying the database, got error: " << endl;
		 	 cerr << paDb.errMsg << endl;
			#endif
			 return false;
		 }
	 }
	 else
	 {
		 //Error conecting to the database, reporting the error
		#ifdef DEBUG
		 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << " : ";
		 cerr << "Error connecting to the database, got error: " << endl;
		 cerr << paDb.errMsg << endl;
		#endif
		 return false;
	 }
}

int paResItem::del()
{
	paResItem itemBuff;
	if ( itemBuff.get(resID) == 0 )
	{
       		db paDb(pa_sql_server,
			 pa_sql_user,
			 pa_sql_password,
			 pa_sql_dbname
			 );
		if ( paDb.errNo == 0 )
	 	{
		
			//we have the info needed to purge th db
			string queryStr = "";
			queryStr = "DELETE FROM res_index WHERE id=";
			queryStr += int2string(itemBuff.id);
			paDb.query(queryStr);
			if ( paDb.errNo != 0) 
			{
				#ifdef DEBUG
		 		cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << " : ";
				cerr << "Query: " << queryStr << endl;
		 		cerr << "Error deleting the entry in  res_index, got error: " << endl;
		 		cerr << paDb.errMsg << endl;
				#endif
				return paDb.errNo; 
			}
			queryStr = "";
			queryStr = "DROP TABLE ";
			queryStr += itemBuff.histTable;
			paDb.query(queryStr);
			if ( paDb.errNo != 0) 
			{
				#ifdef DEBUG
		 		cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << " : ";
		 		cerr << "Error deleting the table" << itemBuff.histTable <<", got error: " << endl;
		 		cerr << paDb.errMsg << endl;
				#endif
				return paDb.errNo; 
			}
			return 0;
			
		}
		else
		{
			#ifdef DEBUG
		 	cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << " : ";
		 	cerr << "Error connecting to the database, got error: " << endl;
		 	cerr << paDb.errMsg << endl;
			#endif
			return paDb.errNo; 
		}
		
	}
	else
	{
			#ifdef DEBUG
			 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << " : ";
			 cerr << "error retrieving the item info." << endl;
			#endif
			 return NO_RECORD;
	}
}

//private methods

int paResItem::createHist()
{
	#ifdef DEBUG
	 cerr << "DEBUG: paResItem::createHist(),";
	 cerr << "id=" << id << ",";
	 cerr << "resID=" << resID << ",";
	 cerr << "defMinTTL=" << defMinTTL << ",";
	 cerr << "suggestedPrice=" << suggestedPrice << ",";
	 cerr << "histTable=" << histTable << endl; 
	#endif
	 db paDb(pa_sql_server,
			 pa_sql_user,
			 pa_sql_password,
			 pa_sql_dbname
			 );
	 if ( paDb.errNo == 0 )
	 {
		string queryStr = "";
		queryStr = "CREATE TABLE " + histTable;
		queryStr += " (time int not null, price int not null, primary key (time))";
		paDb.query(queryStr);
		if ( paDb.errNo != 0 )
		{
			#ifdef DEBUG
		  	 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << " : ";
		 	 cerr << "Error creating the table, got error: " << endl;
		 	 cerr << paDb.errMsg << endl;
			#endif
			 return QUERY_ERROR;
		}
		else
		{
			return 0;
		}
	 }
	 else
	 {
		#ifdef DEBUG
		 cerr << "DEBUG:" << __FILE__ << ":" << __LINE__ << " : ";
		 cerr << "Error connecting to the database, got error: " << endl;
		 cerr << paDb.errMsg << endl;
		#endif
		return paDb.errNo; 
	 }
}
