
// ---------------------------------------------------------------------------
//  Includes
// ---------------------------------------------------------------------------
#include <getopt.h>
#include <vector>
#include <string>
#include <iostream>
#include <sstream>
#include <hlr_prot_errcode.h>
#include <xmlUtil.h>
#include <stringSplit.h>
// Socket++ includes
#include "SocketClient.h"
#include "SocketAgent.h"
#ifdef GSI
#include "GSISocketClient.h"
#include "GSISocketAgent.h"
#endif
#include <dgasVersion.h>


// global definitions

#define OPTION_STRING "hf:s:"

using namespace std;
#ifndef EDG_SOCKETPP
using namespace edg::workload::common::socket_pp;
#endif

struct fund_info{
	string fid;
	string descr;
        int total;
	int spent;
};

bool needs_help = 0;
string server_name = "NULL";
string fid_buff = "";
string desc_buff = "";



// ---------------------------------------------------------------------------
//
//  Usage()
//
// ---------------------------------------------------------------------------
void usage(const char *progname)
{
  cerr << endl << " " << progname << " " << VERSION
       << endl << " Author: Andrea Guarise" << endl
       << "\n Usage:\n " 
       << progname << " [OPTIONS]\n"
       << "\n       -h --help (display this help and exit)\n"
       << "       -s --server servername\n"
       << "       -f --fid fund id\n"
       <<  endl;
}


int options ( int argc, char **argv )
{
	int option_char;
	int option_index = 0;
	static struct option long_options[] =
	{
		{"fid",1,0,'f'},
		{"server",1,0,'s'},
		{"help",0,0,'h'},
		{0,0,0,0}
	};
	while (( option_char = getopt_long( argc, argv, OPTION_STRING,
					long_options, &option_index)) != EOF)
		switch (option_char)
		{
			case 'f': fid_buff = optarg; break;
			case 's': server_name = optarg; break;
			case 'h': needs_help =1; break;		  
			default : break;
		}
	return 0;
}


int print ( fund_info f_info)
{
	cout << "|";
	cout << f_info.fid << "|";
	cout << f_info.descr << "|";
	cout << f_info.total << "|";
        cout << f_info.spent << "|";
	cout << endl;

	return 0;
}

int xml_query (string *xml, fund_info &f_info)
{
	stringstream x_buf;
	x_buf << "<HLR type=\"UI_query\">\n" 
		"<HEAD>\n"
	       	"<VER>1.0</VER>\n"
		"</HEAD>\n"
		"<BODY>\n"
		"<FUND_INFO>\n"
		"<HLR_FID>" << f_info.fid << "</HLR_FID>\n"
		"</FUND_INFO>\n"
		"</BODY>\n"
		"</HLR>\n";


	*xml = x_buf.str();
	return 0;
	
	
	
}

				
int xml_parse( string &doc, vector<fund_info> *funds)
{
	node nodeBuff;
	while ( nodeBuff.status == 0 )
	{
		string tag = "FUND_INFO";
		nodeBuff = parse (&doc, tag);
		if ( nodeBuff.status != 0 )
			break;
		fund_info fndBuff;
		node tagBuff;
		tagBuff = parse ( &nodeBuff.text, "ERROR");
		if ( tagBuff.status == 0 )
		{
			return atoi((tagBuff.text).c_str());
		}
		tagBuff = parse ( &nodeBuff.text, "HLR_FID" );
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			cerr << "Remote UI, got :"<<  tagBuff.text << endl;
			#endif
			fndBuff.fid = tagBuff.text;
		}
		tagBuff = parse ( &nodeBuff.text, "DESC" );
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			cerr << "Remote UI, got :"<<  tagBuff.text << endl;
			#endif
			fndBuff.descr = tagBuff.text;
		}
		tagBuff = parse ( &nodeBuff.text, "FUND_ASS" );
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			cerr << "Remote UI, got :"<<  tagBuff.text << endl;
			#endif
			fndBuff.total = atoi((tagBuff.text).c_str());
		}
		tagBuff = parse ( &nodeBuff.text, "FUND_SPT" );
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			cerr << "Remote UI, got :"<<  tagBuff.text << endl;
			#endif
			fndBuff.spent = atoi((tagBuff.text).c_str());
		}
		nodeBuff.release();
		funds->push_back(fndBuff);
	}
	return 0;	
}



// ---------------------------------------------------------------------------
//
//  main
//
// ---------------------------------------------------------------------------
int main(int argc, char **argv)
{
	int retval = 0;
	int res;
	options (argc, argv);
	if (needs_help)
	{
		usage(argv[0]);
		return 0;
	}
	
	vector<string> urlBuff;
	Split(':',server_name, &urlBuff);
	if ( urlBuff.size() != 3 )
	{
		return atoi(E_RUI_PARSE_ID);
	}
	string HlrHostname = urlBuff[0];
	int HlrPort = atoi((urlBuff[1]).c_str());
	string HlrContact = urlBuff[2];
   

	fund_info f_info_query = { 
		fid_buff,
                "",  
		0,
		0};
	string xml;
	res = xml_query(&xml, f_info_query);

#ifdef DEBUG
	cerr << "XML message formatted:\n";
	cerr << xml;
	cerr << "END of XML message\n";
#endif

	try{//try1
#ifdef GSI
	GSISocketClient *theClient = new GSISocketClient(HlrHostname,HlrPort);
	theClient -> ServerContact(HlrContact);
#else
	SocketClient *theClient = new SocketClient(HlrHostname,HlrPort);
#endif
	if ( !(theClient -> Open()) )
	{
	    cerr << "ERROR: can't open connection"<< endl;
	    exit (1);
	}
#ifdef DEBUG
	cerr << "Connection opened: sending message ..." <<endl;
#endif
	if ( !(theClient -> Send(xml)) )
	{
		cerr << "Error sending message" << endl;
	}
	string xml_answer;
	if ( !(theClient -> Receive(xml_answer)) )
	{
                cerr << "Error receiving message" << endl;
	}
        else
	{
		theClient -> Close();
	}
	delete theClient;

	#ifdef DEBUG
	cerr << "Answer from the server:" <<endl;
	cerr << "##" << xml_answer << "##" << endl;
	cerr << "end of the answer" << endl;
	#endif
	vector<fund_info> funds;
	res = xml_parse( xml_answer, &funds);
	if ( res > 0 ) 
	{
		if (res == ACCESS_DENIED)
		{
			cerr << "ERROR: Code " << res << endl;
			cerr << "Server answered: Access Denied" << endl;
		}
	}
	if ( res == 0 )
	{
	#ifdef DEBUG
		cerr << "Received N. " << funds.size() << " funds. " << endl;
	#endif
	}

	//now print out funds information
	if ( funds.size() != 0 )
	{
		vector < fund_info >::const_iterator f_it = funds.begin();
		while ( f_it != funds.end() )
		{
			print (*f_it );
			++f_it;
		}
	}

	
	
#ifdef DEBUG
	cerr << "Done." <<endl;
#endif
	}//try1
	catch ( std::exception& e)
	{
		 std::cerr << e.what() << std::endl;
		 retval = atoi(E_NO_CONNECTION);
	}
    return retval;
}
