
// ---------------------------------------------------------------------------
//  Includes
// ---------------------------------------------------------------------------
#include <getopt.h>
#include <vector>
#include <string>
#include <iostream>
#include <sstream>
#include <hlr_prot_errcode.h>
#include <xmlUtil.h>
#include <stringSplit.h>
// Socket++ includes
#include "SocketClient.h"
#include "SocketAgent.h"
#ifdef GSI
#include "GSISocketClient.h"
#include "GSISocketAgent.h"
#endif
#include <dgasVersion.h>

// global definitions

#define OPTION_STRING "hg:s:"

using namespace std;
#ifndef EDG_SOCKETPP
using namespace edg::workload::common::socket_pp;
#endif
struct group_info{
	string gid;
	string descr;
	string fid;
        int assigned;
        int booked;
	int spent;
};

bool needs_help = 0;
string server_name = "NULL";
string gid_buff = "";
string fid_buff = "";
string desc_buff = "";


// ---------------------------------------------------------------------------
//
//  Usage()
//
// ---------------------------------------------------------------------------
void usage(const char *progname)
{
  cerr << endl << " " << progname << " " << VERSION
       << endl << " Author: Andrea Guarise" << endl
       << "\n Usage:\n " 
       << progname << " [OPTIONS]\n"
       << "\n       -h --help (display this help and exit)\n"
       << "       -s --server servername\n"
       << "       -g --gid group id\n"
       <<  endl;
}


int options ( int argc, char **argv )
{
	int option_char;
	int option_index = 0;
	static struct option long_options[] =
	{
		{"gid",1,0,'r'},
		{"server",1,0,'s'},
		{"help",0,0,'h'},
		{0,0,0,0}
	};
	while (( option_char = getopt_long( argc, argv, OPTION_STRING,
					long_options, &option_index)) != EOF)
		switch (option_char)
		{
			case 'g': gid_buff = optarg; break;
			case 's': server_name = optarg; break;
			case 'h': needs_help =1; break;		  
			default : break;
		}
	return 0;
}

int print ( group_info g_info)
{
	cout << "|";
	cout << g_info.gid << "|";
	cout << g_info.descr << "|";
	cout << g_info.fid << "|";
	cout << g_info.assigned << "|";
	cout << g_info.booked << "|";
        cout << g_info.spent << "|";
	cout << endl;

	return 0;
}

int xml_query (string *xml, group_info &g_info)
{
	stringstream x_buf;
	x_buf << "<HLR type=\"UI_query\">\n" 
		"<HEAD>\n"
	       	"<VER>1.0</VER>\n"
		"</HEAD>\n"
		"<BODY>\n"
		"<GROUP_INFO>\n"
		"<HLR_GID>" << g_info.gid << "</HLR_GID>\n"
		"</GROUP_INFO>\n"
		"</BODY>\n"
		"</HLR>\n";


	*xml = x_buf.str();
	return 0;
	
	
	
}


int xml_parse( string &doc, vector<group_info> *groups)
{
	node nodeBuff;
	while ( nodeBuff.status == 0 )
	{
		string tag = "GROUP_INFO";
		nodeBuff = parse (&doc, tag);
		if ( nodeBuff.status != 0 )
			break;
		group_info grpBuff;
		node tagBuff;
		tagBuff = parse ( &nodeBuff.text, "ERROR");
		if ( tagBuff.status == 0 )
		{
			return atoi((tagBuff.text).c_str());
		}
		tagBuff = parse ( &nodeBuff.text, "HLR_GID" );
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			cerr << "Remote UI, got :"<<  tagBuff.text << endl;
			#endif
			grpBuff.gid = tagBuff.text;
		}
		tagBuff = parse ( &nodeBuff.text, "DESC" );
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			cerr << "Remote UI, got :"<<  tagBuff.text << endl;
			#endif
			grpBuff.descr= tagBuff.text;
		}
		tagBuff = parse ( &nodeBuff.text, "FUND_ASS" );
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			cerr << "Remote UI, got :"<<  tagBuff.text << endl;
			#endif
			grpBuff.assigned = atoi((tagBuff.text).c_str());
		}
		tagBuff = parse ( &nodeBuff.text, "FUND_BKD" );
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			cerr << "Remote UI, got :"<<  tagBuff.text << endl;
			#endif
			grpBuff.booked = atoi((tagBuff.text).c_str());
		}
		tagBuff = parse ( &nodeBuff.text, "FUND_SPT" );
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			cerr << "Remote UI, got :"<<  tagBuff.text << endl;
			#endif
			grpBuff.spent = atoi((tagBuff.text).c_str());
		}
		nodeBuff.release();
		groups->push_back(grpBuff);
	}
	return 0;	
	 
}



// ---------------------------------------------------------------------------
//
//  main
//
// ---------------------------------------------------------------------------
int main(int argc, char **argv)
{
    int retval = 0;
    int res;
    options (argc, argv);
    if (needs_help)
    {
	    usage(argv[0]);
	    return 0;
    }

	vector<string> urlBuff;
	Split(':',server_name, &urlBuff);
	if ( urlBuff.size() != 3 )
	{
		return atoi(E_RUI_PARSE_ID);
	}
	string HlrHostname = urlBuff[0];
	int HlrPort = atoi((urlBuff[1]).c_str());
	string HlrContact = urlBuff[2];
   


	group_info g_info_query = { 
		gid_buff,
                "",  
		fid_buff, 
		0,
		0,
		0};
	string xml;
	res = xml_query(&xml, g_info_query);

#ifdef DEBUG
	cerr << "XML message formatted:\n";
	cerr << xml;
	cerr << "END of XML message\n";
#endif
	try{//try1
#ifdef GSI
	GSISocketClient *theClient = new GSISocketClient(HlrHostname, HlrPort);
	theClient -> ServerContact(HlrContact);
#else
	SocketClient *theClient = new SocketClient(HlrHostname, HlrPort);
#endif

	
	if ( !(theClient -> Open()) )
	{
		cerr << "ERROR: can't open connection"<< endl;
		exit (1);
	}
#ifdef DEBUG
	cerr << "Connection opened: sending message ..." <<endl;
#endif
	if ( !(theClient -> Send(xml)) )
	{
		cerr << "Error sending message" << endl;
	}
	string xml_answer;
        if ( !(theClient -> Receive(xml_answer)) )
	{
                cerr << "Error receiving message" << endl;
	}
	else
	{
		theClient -> Close();
	}
	delete theClient;

	#ifdef DEBUG
	cerr << "Answer from the server:" <<endl;
	cerr << "##" << xml_answer << "##" << endl;
	cerr << "end of the answer" << endl;
	#endif
	vector<group_info> groups;
	res = xml_parse( xml_answer, &groups);
	if ( res > 0 ) 
	{
		if (res == ACCESS_DENIED)
		{
			cerr << "ERROR: Code " << res << endl;
			cerr << "Server answered: Access Denied" << endl;
		}
	}
	if ( res == 0 )
	{
	#ifdef DEBUG
		cerr << "Received N. " << groups.size() << " groups. " << endl;
	#endif
	}

	//now print out groups information
	
	if ( groups.size() != 0)
		{
		vector < group_info >::const_iterator g_it = groups.begin();
		while ( g_it != groups.end() )
		{
			print (*g_it );
			++g_it;
		}
	}
	
	
#ifdef DEBUG
	cerr << "Done." <<endl;
#endif
	}//try1
	catch ( std::exception& e)
	{
		std::cerr << e.what() << std::endl;
		retval = atoi(E_NO_CONNECTION);
	}
    return retval;
}
