
// ---------------------------------------------------------------------------
//  Includes
// ---------------------------------------------------------------------------
#include <getopt.h>
#include <vector>
#include <string>
#include <iostream>
#include <sstream>
#include <hlr_prot_errcode.h>
#include <xmlUtil.h>
#include <stringSplit.h>
// Socket ++ includes
#include "SocketClient.h"
#include "SocketAgent.h"
#ifdef GSI
#include "GSISocketClient.h"
#include "GSISocketAgent.h"
#endif
#include <dgasVersion.h>



// global definitions

#define OPTION_STRING "hr:c:s:"

using namespace std;
#ifndef EDG_SOCKETPP
using namespace edg::workload::common::socket_pp;
#endif

struct resource_info{
	string rid;
	string descr;
	string cert_subject;
	string gid;
	int total;
};

bool needs_help = 0;
string server_name = "NULL";
string rid_buff = "";
string cert_subject_buff = "";


// ---------------------------------------------------------------------------
//
//  Usage()
//
// ---------------------------------------------------------------------------
void usage(const char *progname)
{
  cerr << endl << " " << progname << " " << VERSION
       << endl << " Author: Andrea Guarise" << endl
       << "\n Usage:\n " 
       << progname << " [OPTIONS]\n"
       << "       -h --help (display this help and exit)\n"
       << "       -s --server HLR_BANK_ID\n"
       << "       -c --cert resource certificate header\n"
       << "       -r --rid resource id\n"
       << endl;
}


int options ( int argc, char **argv )
{
	int option_char;
	int option_index = 0;
	static struct option long_options[] =
	{
		{"rid",1,0,'r'},
		{"cert",1,0,'c'},
		{"server",1,0,'s'},
		{"help",0,0,'h'},
		{0,0,0,0}
	};
	while (( option_char = getopt_long( argc, argv, OPTION_STRING,
					long_options, &option_index)) != EOF)
		switch (option_char)
		{
			case 'r': rid_buff = optarg; break;
			case 'c': cert_subject_buff = optarg; break;
			case 's': server_name = optarg; break;
			case 'h': needs_help =1; break;		  
			default : break;
		}
	return 0;
}


int print ( resource_info r_info)
{
	cout << "|";
	cout << r_info.rid << "|";
	cout << r_info.descr << "|";
	cout << r_info.cert_subject << "|";
	cout << r_info.gid << "|";
	cout << r_info.total << "|";
	cout << endl;
	return 0;
}

int xml_query (string *xml, resource_info &r_info)
{
	stringstream x_buf;
	x_buf << "<HLR type=\"UI_query\">\n" 
		"<HEAD>\n"
	       	"<VER>1.0</VER>\n"
		"</HEAD>\n"
		"<BODY>\n"
		"<RESOURCE_INFO>\n"
		"<CERT_H>" << r_info.cert_subject << "</CERT_H>\n"
		"<RID>" << r_info.rid << "</RID>\n"
		<< "</RESOURCE_INFO>\n"
		"</BODY>\n"
		"</HLR>\n";

	*xml = x_buf.str();
	return 0;
	
}


int xml_parse( string &doc, vector<resource_info> *resources)
{
	node nodeBuff;
	while ( nodeBuff.status == 0 )
	{
		string tag = "RESOURCE_INFO";
		nodeBuff = parse (&doc, tag);
		if ( nodeBuff.status != 0 )
			break;
		resource_info resBuff;
		node tagBuff;
		tagBuff = parse ( &nodeBuff.text, "ERROR");
		if ( tagBuff.status == 0 )
		{
			return atoi((tagBuff.text).c_str());//report the error
		}
		tagBuff = parse ( &nodeBuff.text, "HLR_RID" );
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			cerr << "Remote UI, got :"<<  tagBuff.text << endl;
			#endif
			resBuff.rid = tagBuff.text;
		}
		tagBuff = parse ( &nodeBuff.text, "DESC" );
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			cerr << "Remote UI, got :"<<  tagBuff.text << endl;
			#endif
			resBuff.descr = tagBuff.text;
		}
		tagBuff = parse ( &nodeBuff.text, "CERT_H" );
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			cerr << "Remote UI, got :"<<  tagBuff.text << endl;
			#endif
			resBuff.cert_subject = tagBuff.text;
		}
		tagBuff = parse ( &nodeBuff.text, "GID" );
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			cerr << "Remote UI, got :"<<  tagBuff.text << endl;
			#endif
			resBuff.gid = tagBuff.text;
		}
		tagBuff = parse ( &nodeBuff.text, "FUND_TOT" );
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			cerr << "Remote UI, got :"<<  tagBuff.text << endl;
			#endif
			resBuff.total = atoi((tagBuff.text).c_str());
		}
		nodeBuff.release();
		resources->push_back(resBuff);
	}
	return 0;
	
	 
}


// ---------------------------------------------------------------------------
//
//  main
//
// ---------------------------------------------------------------------------
int main(int argc, char **argv)
{
    int retval = 0;
    int res;
    options (argc, argv);
    if (needs_help)
    {
	    usage(argv[0]);
	    return 0;
    }
	vector<string> urlBuff;
	Split(':',server_name, &urlBuff);
	if ( urlBuff.size() != 3 )
	{
		return atoi(E_RUI_PARSE_ID);
	}
	string HlrHostname = urlBuff[0];
	int HlrPort = atoi((urlBuff[1]).c_str());
	string HlrContact = urlBuff[2];

	resource_info r_info_query = { 
		rid_buff,
                "",  
		cert_subject_buff, 
		"", 
		0};
	string xml;
	res = xml_query(&xml, r_info_query);

#ifdef DEBUG
	cerr << "XML message formatted:\n";
	cerr << xml;
	cerr << "END of XML message\n";
#endif

	try{//try1
#ifdef GSI
	GSISocketClient *theClient = new GSISocketClient(HlrHostname, HlrPort);
	theClient -> ServerContact(HlrContact);
#else
	SocketClient *theClient = new SocketClient(HlrHostname, HlrPort);
#endif

	
	if ( !(theClient -> Open()) )
	{
		cerr << "ERROR: can't open connection"<< endl;
		exit (1);
	}
#ifdef DEBUG
	cerr << "Connection opened: sending message ..." <<endl;
#endif
	if ( !(theClient -> Send(xml) ))
	{
		cerr << "Error sending message" << endl;
	}
	
	string xml_answer;
	if ( !(theClient -> Receive(xml_answer) ))
	{
		cerr << "Error receiving message" << endl;
	}
	else
	{
		theClient -> Close();
	}
	delete theClient;
	#ifdef DEBUG
	cerr << "Answer from the server:" <<endl;
	cerr << "##" << xml_answer << "##" << endl;
	cerr << "end of the answer" << endl;
	#endif
	vector<resource_info> resources;
	res = xml_parse( xml_answer, &resources);
	if ( res > 0 ) 
	{
		if (res == ACCESS_DENIED)
		{
			cerr << "ERROR: Code " << res << endl;
			cerr << "Server answered: Access Denied" << endl;
		}
	}
	if ( res == 0 )
	{
	#ifdef DEBUG
		cerr << "Received N. " << resources.size() << " resources. " << endl;
	#endif
	}

	//now print out resources information
	if (resources.size() != 0 )
	{
		vector < resource_info >::const_iterator r_it = resources.begin();
		while ( r_it != resources.end() )
		{
			print (*r_it );
			++r_it;
		}
	}
	
	
#ifdef DEBUG
	cerr << "Done." <<endl;
#endif
	}//try1
	catch ( std::exception& e)
	{
		std::cerr << e.what() << std::endl;
		retval = atoi(E_NO_CONNECTION);
	}
    return retval;
}
