// DGAS (DataGrid Accounting System) 
// Server Daemon and protocol engines.
// 
// $Id: hlr_remote_trans_info.cpp,v 1.5 2003/12/05 13:29:37 guarise Exp $
// -------------------------------------------------------------------------
// Copyright (c) 2001-2002, The DataGrid project, INFN, 
// All rights reserved. See LICENSE file for details.
// -------------------------------------------------------------------------
// Authors: Stefano Barale <stefano.barale@to.infn.it>
//          Andrea Guarise <andrea.guarise@to.infn.it>
 /***************************************************************************
 * Code borrowed from:
 *  authors   :
 *  copyright : 
 ***************************************************************************/
//
// 

// ---------------------------------------------------------------------------
//  Includes
// ---------------------------------------------------------------------------
#include <getopt.h>
#include <vector>
#include <string>
#include <iostream>
#include <sstream>
#include <hlr_prot_errcode.h>
#include <xmlUtil.h>
#include <stringSplit.h>
// Socket ++ includes
#include "SocketClient.h"
#include "SocketAgent.h"
#ifdef GSI
#include "GSISocketClient.h"
#include "GSISocketAgent.h"
#endif
#include <dgasVersion.h>


// global definitions

#define OPTION_STRING "hu:r:s:f:t:T:m:"

using namespace std;
#ifndef EDG_SOCKETPP
using namespace edg::workload::common::socket_pp;
#endif

struct job_info{
        string id;
        string fromId;
        string toId;
        string remoteHlrUrl;
        int jobCost;
        string timeStamp;
        string tType;
        string log;
};

bool needs_help = 0;
string serverName = "";
string idBuff = "";
string toIdBuff = "";
string fromIdBuff = "";
string tTypeBuff = "ND";
string maxNBuff = "50";


// ---------------------------------------------------------------------------
//
//  Usage()
//
// ---------------------------------------------------------------------------
void usage(const char *progname)
{
  cerr << endl << " " << progname << " " << VERSION
       << endl << " Author: Andrea Guarise" << endl
       << "\n Usage:\n " 
       << progname << " [OPTIONS]\n"
       << "       -h --help (display this help and exit)\n"
       << "       -s --server HLR ACCT_BANK_ID\n"
       << "       -u --user_cert user certificate header\n"
       << "       -r --resource_cert resource certificate header\n"      
       << "       -f --fromid  Grid ID of the account originating the transaction\n"
       << "       -t --toid    Grid Id of the account receiving the transaction\n"
       << "       -T --t_type  Type of transaction, can be in or out\n"
       << "       -m --maxn  Max number of transaction to show\n"
       <<  endl;
}


int options ( int argc, char **argv )
{
	int option_char;
	int option_index = 0;
	static struct option long_options[] =
	{
		{"user_cert",1,0,'u'},
                {"resource_cert",1,0,'r'},
		{"fromid",1,0,'f'},
                {"toid",1,0,'t'},
                {"t_type",1,0,'T'},
                {"maxn",1,0,'m'},
		{"server",1,0,'s'},
		{"port",1,0,'p'},
		{"help",0,0,'h'},
		{0,0,0,0}
	};
	while (( option_char = getopt_long( argc, argv, OPTION_STRING,
					long_options, &option_index)) != EOF)
	 
		switch (option_char)
		{
			case 'r': toIdBuff = optarg; break;
		        case 'u': fromIdBuff = optarg; break;
			case 'f': fromIdBuff = optarg; break;
		        case 't': toIdBuff = optarg; break;
		        case 'T': tTypeBuff = optarg; break;
		        case 'm': maxNBuff = optarg; break;
			case 's': serverName = optarg; break;
			case 'h': needs_help =1; break;		  
			default : break;
		}
	
	return 0;
}


int print ( job_info j_info)
{
	cout << "|";
	cout << j_info.id << "|";
	cout << j_info.fromId << "|";
	cout << j_info.toId << "|";
	cout << j_info.remoteHlrUrl << "|";
	cout << j_info.jobCost << "|";
	cout << j_info.timeStamp << "|";
	cout << j_info.tType << "|";
	cout << j_info.log << "|";
	cout << endl;

	return 0;
}

int xml_query (string *xml)
{
	stringstream x_buf;
	x_buf << "<HLR type=\"UI_query\">\n" 
		"<HEAD>\n"
	       	"<VER>1.0</VER>\n"
		"</HEAD>\n"
		"<BODY>\n"
		"<TRS>\n"
		"<TO_GRID_ID>" << toIdBuff << "</TO_GRID_ID>\n"
	        "<FROM_GRID_ID>" << fromIdBuff << "</FROM_GRID_ID>\n"
	        "<TYPE>" << tTypeBuff << "</TYPE>\n"
	        "<MAX_N>" << maxNBuff << "</MAX_N>\n"
		"</TRS>\n"
		"</BODY>\n"
		"</HLR>\n";
	*xml = x_buf.str();
	return 0;
}//int xml_query()


int xml_parse( string &doc, vector<job_info> *jobs)
{
	node nodeBuff;
	while ( nodeBuff.status == 0 )
	{
		string tag = "TRS";
		nodeBuff = parse (&doc, tag);
		if ( nodeBuff.status != 0 )
			break;
		job_info jobBuffer;
		node tagBuff;
		tagBuff = parse ( &nodeBuff.text, "ERROR");
		if ( tagBuff.status == 0 )
		{
			return atoi((tagBuff.text).c_str());//report the error
		}
		tagBuff = parse ( &nodeBuff.text, "FROM_HLR_URL" );
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			 cerr << "Remote UI, got :"<<  tagBuff.text << endl;
			#endif
			 jobBuffer.remoteHlrUrl = tagBuff.text;
		}
		tagBuff = parse ( &nodeBuff.text, "TO_GRID_ID" );
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			 cerr << "Remote UI, got :"<<  tagBuff.text << endl;
			#endif
			 jobBuffer.toId = tagBuff.text;
		}
		tagBuff = parse ( &nodeBuff.text, "FROM_GRID_ID" );
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			 cerr << "Remote UI, got :"<<  tagBuff.text << endl;
			#endif
			 jobBuffer.fromId = tagBuff.text;
		}
		tagBuff = parse ( &nodeBuff.text, "T_STAMP" );
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			 cerr << "Remote UI, got :"<<  tagBuff.text << endl;
			#endif
			 jobBuffer.timeStamp = tagBuff.text;
		}
		tagBuff = parse ( &nodeBuff.text, "COST" );
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			 cerr << "Remote UI, got :"<<  tagBuff.text << endl;
			#endif
			 jobBuffer.jobCost = atoi((tagBuff.text).c_str());
		}
		tagBuff = parse ( &nodeBuff.text, "JID" );
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			 cerr << "Remote UI, got :"<<  tagBuff.text << endl;
			#endif
			 jobBuffer.id = tagBuff.text;
		}
		tagBuff = parse ( &nodeBuff.text, "LOG" );
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			 cerr << "Remote UI, got :"<<  tagBuff.text << endl;
			#endif
			 jobBuffer.log = tagBuff.text;
		}
		tagBuff = parse ( &nodeBuff.text, "TYPE" );
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			 cerr << "Remote UI, got :"<<  tagBuff.text << endl;
			#endif
			 jobBuffer.tType = tagBuff.text;
		}
		nodeBuff.release();
		jobs->push_back(jobBuffer);
	}
	return 0;	
	 
}


// ---------------------------------------------------------------------------
//
//  main
//
// ---------------------------------------------------------------------------
int main(int argc, char **argv)
{
    int retval = 0;
    int res = 0;
    options (argc, argv);
    if (needs_help)
    {
	    usage(argv[0]);
	    return 0;
    }
        
	vector<string> urlBuff;
	Split(':',serverName, &urlBuff);
	if ( urlBuff.size() != 3 )	
	{
		return atoi(E_RUI_PARSE_ID);
	}
	string HlrHostname = urlBuff[0];
	int HlrPort = atoi((urlBuff[1]).c_str());
	string HlrContact = urlBuff[2];
	string xml;
	res = xml_query(&xml);
         
#ifdef DEBUG
	cerr << "XML message formatted:\n";
	cerr << xml;
	cerr << "END of XML message\n";
#endif
	try{//try1
#ifdef GSI
	GSISocketClient *theClient = new GSISocketClient(HlrHostname, HlrPort);
	theClient -> ServerContact(HlrContact);
#else
	SocketClient *theClient = new SocketClient(HlrHostname, HlrPort);
#endif

	if ( !(theClient -> Open()) )
	{
		cerr << "ERROR: can't open conenction"<< endl;
		exit (1);
	}
#ifdef DEBUG
	cerr << "Connection opened: sending message ..." <<endl;
#endif
        if ( !(theClient -> Send(xml) ))
	{
		cerr << "Error sending message" << endl;
	}
	
	string xml_answer;
	if ( !(theClient -> Receive(xml_answer) ))
	{
		cerr << "Error receiving message" << endl;
	}
	else
	{
		theClient -> Close();
	}
	delete theClient;
	
	#ifdef DEBUG
	cerr << "Answer from the server:" <<endl;
	cerr << "##" << xml_answer << "##" << endl;
	cerr << "end of the answer" << endl;
	#endif
	vector<job_info> jobs;
	res = xml_parse( xml_answer, &jobs);
	if ( res > 0 ) 
	{
		if (res == ACCESS_DENIED)
		{
			cerr << "ERROR: Code " << res << endl;
			cerr << "Server answered: Access Denied" << endl;
		}
	}
	if ( res == 0 )
	{
	#ifdef DEBUG
		cerr << "Received N. " << jobs.size() << " jobs. " << endl;
	#endif
	}

	//now print out jobs information
	if (jobs.size() != 0)
	{
		vector < job_info >::const_iterator j_it = jobs.begin();
		while ( j_it != jobs.end() )
		{
			print (*j_it );
			++j_it;
		}
	}

#ifdef DEBUG
	cerr << "Done." <<endl;
#endif
    	}//try1
	catch ( std::exception& e)
	{
		std::cerr << e.what() << std::endl;
		retval = atoi(E_NO_CONNECTION);
	}
    return retval;
}
