
// ---------------------------------------------------------------------------
//  Includes
// ---------------------------------------------------------------------------
#include <getopt.h>
#include <vector>
#include <string>
#include <iostream>
#include <sstream>
#include <fstream>
#include "hlr_prot_errcode.h"
#include "xmlUtil.h"
#include "stringSplit.h"
//Socket++ includes
#include "SocketClient.h"
#include "SocketAgent.h"
#ifdef GSI
#include "GSISocketClient.h"
#include "GSISocketAgent.h"
#endif
#include "dgasVersion.h"
	
// global definitions

#define OPTION_STRING "hu:e:c:g:s:"

using namespace std;
#ifndef EDG_SOCKETPP
using namespace edg::workload::common::socket_pp;
#endif

struct user_info{
	string uid;
	string email;
	string descr;
	string cert_subject;
	string gid;
	int assigned;
	int booked;
	int spent;
};

bool needs_help = 0;
string server_name = "NULL";
string uid_buff = "";
string gid_buff = "";
string email_buff = "";
string cert_subject_buff = "";


// ---------------------------------------------------------------------------
//
//  Usage()
//
// ---------------------------------------------------------------------------
void usage(const char *progname)
{
  cerr << endl << " " << progname << " " << VERSION
       << endl << " Author: Andrea Guarise" << endl
       << "\n Usage:\n " 
       << progname << " [OPTIONS]\n"
       << "       -h --help (display this help and exit\n"
       << "       -s --server servername\n"
       << "       -c --cert user certificate header\n"
       << "       -e --email user e-mail address\n"
       << "       -u --uid user id\n"
       << "       -g --gid group id\n"
       <<  endl;
}


int options ( int argc, char **argv )
{
	int option_char;
	int option_index = 0;
	static struct option long_options[] =
	{
		{"uid",1,0,'u'},
		{"gid",1,0,'g'},
		{"email",1,0,'e'},
		{"cert",1,0,'c'},
		{"server",1,0,'s'},
		{"help",0,0,'h'},
		{0,0,0,0}
	};
	while (( option_char = getopt_long( argc, argv, OPTION_STRING,
					long_options, &option_index)) != EOF)
		switch (option_char)
		{
			case 'u': uid_buff = optarg; break;
			case 'g': gid_buff = optarg; break;
			case 'e': email_buff = optarg; break;
			case 'c': cert_subject_buff = optarg; break;
			case 's': server_name = optarg; break;
			case 'h': needs_help =1; break;		  
			default : break;
		}
	return 0;
}


int print ( user_info u_info)
{
	cout << "|";
	cout << u_info.uid << "|";
	cout << u_info.email << "|";
	cout << u_info.descr << "|";
	cout << u_info.cert_subject << "|";
	cout << u_info.gid << "|";
	cout << u_info.assigned << "|";
	cout << u_info.booked << "|";
	cout << u_info.spent << "|";
	cout << endl;

	return 0;
}

int xml_query (string *xml, user_info &u_info)
{
	stringstream x_buf;
	x_buf << "<HLR type=\"UI_query\">\n" 
		"<HEAD>\n"
	       	"<VER>1.0</VER>\n"
		"</HEAD>\n"
		"<BODY>\n"
		"<USER_INFO>\n"
		"<EMAIL>" << u_info.email << "</EMAIL>\n"
		"<CERT_H>" << u_info.cert_subject << "</CERT_H>\n"
		"<GID>" << u_info.gid << "</GID>\n"
		<< "</USER_INFO>\n"
		"</BODY>\n"
		"</HLR>\n";


	*xml = x_buf.str();
	return 0;
	
}


int xml_parse( string &doc, vector<user_info> *users)
{
	node nodeBuff;
	while ( nodeBuff.status == 0 )
	{
		string tag = "USER_INFO";
		nodeBuff = parse (&doc, tag);
		if ( nodeBuff.status != 0 )
			break;
		user_info usrBuff;
		node tagBuff;
		tagBuff = parse ( &nodeBuff.text, "ERROR");
		if ( tagBuff.status == 0 )
		{
			return atoi((tagBuff.text).c_str());
		}
		tagBuff = parse ( &nodeBuff.text, "HLR_UID" );
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			cerr << "Remote UI, got :"<<  tagBuff.text << endl;
			#endif
			usrBuff.uid = tagBuff.text;
		}
		tagBuff = parse ( &nodeBuff.text, "EMAIL" );
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			cerr << "Remote UI, got :"<<  tagBuff.text << endl;
			#endif
			usrBuff.email = tagBuff.text;
		}
		tagBuff = parse ( &nodeBuff.text, "DESC" );
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			cerr << "Remote UI, got :"<<  tagBuff.text << endl;
			#endif
			usrBuff.descr = tagBuff.text;
		}
		tagBuff = parse ( &nodeBuff.text, "CERT_H" );
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			cerr << "Remote UI, got :"<<  tagBuff.text << endl;
			#endif
			usrBuff.cert_subject = tagBuff.text;
		}
		tagBuff = parse ( &nodeBuff.text, "GID" );
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			cerr << "Remote UI, got :"<<  tagBuff.text << endl;
			#endif
			usrBuff.gid = tagBuff.text;
		}
		tagBuff = parse ( &nodeBuff.text, "FUND_ASS" );
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			cerr << "Remote UI, got :"<<  tagBuff.text << endl;
			#endif
			usrBuff.assigned = atoi((tagBuff.text).c_str());
		}
		tagBuff = parse ( &nodeBuff.text, "FUND_BKD" );
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			cerr << "Remote UI, got :"<<  tagBuff.text << endl;
			#endif
			usrBuff.booked = atoi((tagBuff.text).c_str());
		}
		tagBuff = parse ( &nodeBuff.text, "FUND_SPT" );
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			cerr << "Remote UI, got :"<<  tagBuff.text << endl;
			#endif
			usrBuff.spent = atoi((tagBuff.text).c_str());
		}
		nodeBuff.release();
		users->push_back(usrBuff);

	}
	return 0;	
}
	


// ---------------------------------------------------------------------------
//
//  main
//
// ---------------------------------------------------------------------------
int main(int argc, char **argv)
{
	int retval = 0;
	int res;
	options (argc, argv);
	if (needs_help)
	{
		usage(argv[0]);
		return 0;
	}

	vector<string> urlBuff;
	Split(':',server_name, &urlBuff);
	if ( urlBuff.size() != 3 )
	{
		return atoi(E_RUI_PARSE_ID);
	}
	string HlrHostname = urlBuff[0];
	int HlrPort = atoi((urlBuff[1]).c_str());
	string HlrContact = urlBuff[2];
   
	user_info u_info_query = { 
		uid_buff, 
		email_buff, 
		"", 
		cert_subject_buff, 
		gid_buff, 
		0,
		0,
		0};
	string xml;
	res = xml_query(&xml, u_info_query);

#ifdef DEBUG
	cerr << "XML message formatted:\n";
	cerr << xml;
	cerr << "END of XML message\n";
#endif
	try{//try1
#ifdef GSI
	GSISocketClient *theClient = new GSISocketClient(HlrHostname, HlrPort);
	theClient -> ServerContact(HlrContact);
#else
	SocketClient *theClient = new SocketClient(HlrHostname, HlrPort);
#endif
	theClient->SetTimeout(0);
	if ( !(theClient -> Open()) )
	{
		cerr << "ERROR: can't open connection"<< endl;
		exit (1);
	}
#ifdef DEBUG
	cerr << "Connection opened: sending message ..." <<endl;
#endif
	if ( !(theClient -> Send(xml) ))
	{
		cerr << "Error sending message" << endl;
	}
	string xml_answer;
	if ( !(theClient -> Receive(xml_answer) ))
	{
		cerr << "Error receiving message" << endl;
		theClient -> Close();
	}
	else
	{
		theClient -> Close();
	}
	delete theClient;
	
	#ifdef DEBUG
	cerr << "Answer from the server:" <<endl;
	cerr << "##" << xml_answer << "##" << endl;
	cerr << "end of the answer" << endl;
	#endif
	vector<user_info> users;
	res = xml_parse( xml_answer, &users);
	if ( res > 0 ) 
	{
		if (res == ACCESS_DENIED)
		{
			cerr << "ERROR: Code " << res << endl;
			cerr << "Server answered: Access Denied" << endl;
		}
	}
	if ( res == 0 )
	{
	#ifdef DEBUG
		cerr << "Received N. " << users.size() << " users. " << endl;
	#endif
	}

	//now print out users information
	if ( users.size() != 0)
	{	
		vector < user_info >::const_iterator u_it = users.begin();
		while ( u_it != users.end() )
		{
			print (*u_it );
			++u_it;
		}
	}

	
	
#ifdef DEBUG
	cerr << "Done." <<endl;
#endif
	}//try1
	catch ( std::exception& e)
	{
		std::cerr << e.what() << std::endl;
		retval=atoi(E_NO_CONNECTION);
	}
    return retval;
}
