// DGAS (DataGrid Accounting System) 
// Client APIs.
// 
// $Id: edg-wl-dgas-ATMClient.cpp,v 1.4 2004/01/12 15:32:09 guarise Exp $
// -------------------------------------------------------------------------
// Copyright (c) 2001-2002, The DataGrid project, INFN, 
// All rights reserved. See LICENSE file for details.
// -------------------------------------------------------------------------
// Author: Andrea Guarise <andrea.guarise@to.infn.it>
 /***************************************************************************
 * Code borrowed from:
 *  authors   :
 *  copyright : 
 ***************************************************************************/
//
//    

// ---------------------------------------------------------------------------
//  Includes
// ---------------------------------------------------------------------------
#include <vector>
#include <string>
#include <sstream>
#include <fstream>
#include <map>
#include "edg-wl-dgas-ATMClient.h"
#include <dgas_config.h>

//The main code of the client that is the public method too.
//Exit 0 on success, >0 on failure
int ATM_client(string user_acct_bank_id, ATM_job_data &job_data, ATM_usage_info &usage_info, string *server_answer, string confFileName)
{
	int res;
	int returncode = 0;
	string output_message;
	string input_message;
	vector<string> urlBuff;
	Split(':',user_acct_bank_id, &urlBuff);
	if ( urlBuff.size() != 3 )
		return atoi(E_JA_PARSE_ID);
	string userBankHostname = urlBuff[0];
	int userBankPort = atoi((urlBuff[1]).c_str());
	string userBankContact = urlBuff[2];
	//easter egg workaround
	//if the acct_PA_res_id and the acct_bank_res_id are
	//not specified, get them form a configuration file.
	map <string,string> confMap;
	if ( dgas_conf_read ( confFileName, &confMap ) == 0 )
	{
		if ( job_data.res_acct_PA_id == "" )
		{
			job_data.res_acct_PA_id =(confMap["res_acct_PA_id"]).c_str();
		#ifdef DEBUG
			cerr << "got: res_acct_PA_id=" << job_data.res_acct_PA_id << endl;
		#endif
		}	
		if ( job_data.res_acct_bank_id == "" )
		{
			job_data.res_acct_bank_id =(confMap["res_acct_bank_id"]).c_str();
		#ifdef DEBUG
			cerr << "got: res_acct_bank_id=" << job_data.res_acct_bank_id << endl;
		#endif
		}	
	}
	else
	{
		cerr << "WARNING: Error reading conf file: " << confFileName << endl;
		cerr << "There can be problems processing the transaction" << endl;
	}
	
	//Now compose the XML ATM_request
	ATMc_xml_compose(job_data, usage_info, &output_message);
	//contact the user HLR server
	try{//try1
	#ifdef GSI
	GSISocketClient *theClient = new GSISocketClient(userBankHostname, userBankPort);
	theClient-> ServerContact(userBankContact);
	#else
	SocketClient *theClient = new SocketClient(userBankHostname, userBankPort );
	#endif
	if ( !(theClient->Open()))
	{
		#ifdef DEBUG
		 cerr << "ERROR: can't open connection" << endl;
		#endif
		 returncode = atoi(E_NO_CONNECTION); 
	}
	else
	{
		#ifdef DEBUG
		 cerr << "Connection opened, sending message..." << endl;
		#endif
		if ( !(theClient->Send(output_message)))
		{
			#ifdef DEBUG
			 cerr << "ERROR: can't send message!" << endl;
			#endif
			 returncode = atoi(E_SEND_MESSAGE);
		}
		#ifdef DEBUG
		 cerr << "Message sent, waiting for answer..." << endl;
		#endif
		if ( !(theClient->Receive(input_message)) )
		{
			#ifdef DEBUG
			 cerr << "ERROR: can't receive message!" << endl;
			#endif
			 returncode = atoi(E_RECEIVE_MESSAGE);
		}
		
		#ifdef DEBUG
		 cerr << input_message << endl;
		#endif
		 theClient->Close();
		 if (returncode == 0)
		 {
			res = ATMc_parse_xml(input_message);
			returncode = res;
		 }
		 *server_answer = input_message;

	}
	delete theClient;
	}//try1
	catch( std::exception& e)
	{
		std::cerr << e.what() << std::endl;
		returncode = atoi(E_NO_CONNECTION);
	}
	return returncode;
	
}//ATM_client(string HLR_url_string, string HLR_contact_string ,ATM_job_data &job_data, ATM_usage_info &usage_info, string *server_answer)


//Parse the XML input, returns 0 if success >1 otherwise, differerntly from
//the other client code, here we will not use Xerces since we want the code
//to be very light and easily portable.
int ATMc_parse_xml(string input_message)
{
	//we only need the return code, the whole message can
	//be seen as a receipt
	string buffer;
	unsigned int pos; 
	pos = input_message.find( "<CODE>" );
	if ( pos == string::npos )
	{
		return atoi(E_PARSE_ERROR);
	}
	int start_pos = pos +6;
	pos = input_message.find( "</CODE>" );
	if ( pos == string::npos ) 
	{
		return atoi(E_PARSE_ERROR);
	}
	int end_pos = pos -1;
	buffer = input_message.substr( start_pos, end_pos-start_pos);
	//now isolate the code
	pos = buffer.find_first_of( "1234567890" );
	if ( pos == string::npos )
	{
		return atoi(E_PARSE_ERROR);
        }
	start_pos = pos;
	pos = buffer.find_first_of( " \n", start_pos );
	if ( pos == string::npos )
	{
		return atoi((buffer.substr(start_pos)).c_str());
        }
	else
	{
		end_pos = pos;
		return atoi((buffer.substr(start_pos, end_pos-start_pos)).c_str());
	}
	
}

//Compose the XML ATM_request message used to send the job info to the user
//DGAS server.
void ATMc_xml_compose(ATM_job_data &job_data, ATM_usage_info &usage_info, string *xml)
{
	*xml = "<HLR type=\"ATM_request\">\n<HEAD>\n<VER>1.0</VER>\n</HEAD>\n";
	*xml += "<BODY>\n";
	*xml += "<JOB_PAYMENT>\n";
	*xml += "<DG_JOBID>\n";
	*xml += job_data.dgJobId;
	*xml += "\n</DG_JOBID>\n";
	*xml += "<SUBMISSION_TIME>\n";
	*xml += int2string(job_data.time);
	*xml += "\n</SUBMISSION_TIME>\n";
	*xml += "<RES_ACCT_PA_ID>\n";
	*xml += job_data.res_acct_PA_id;
	*xml += "\n</RES_ACCT_PA_ID>\n";
	*xml += "<RES_ACCT_BANK_ID>\n";
	*xml += job_data.res_acct_bank_id;
	*xml += "\n</RES_ACCT_BANK_ID>\n";
	*xml += "<USER_CERT_SUBJECT>\n";
	*xml += job_data.user_CertSubject;
	*xml += "\n</USER_CERT_SUBJECT>\n";
	*xml += "<RES_GRID_ID>\n";
	*xml += job_data.res_grid_id;
	*xml += "\n</RES_GRID_ID>\n";
	*xml += "<JOB_INFO>\n";
	*xml += "<CPU_TIME>\n";
	*xml += int2string(usage_info.cpu_time);
	*xml += "\n</CPU_TIME>\n";
	*xml += "<WALL_TIME>\n";
	*xml += int2string(usage_info.wall_time);
	*xml += "\n</WALL_TIME>\n";
	*xml += "<MEM>\n";
	*xml += int2string(usage_info.mem);
	*xml += "\n</MEM>\n";
	*xml += "<VMEM>\n";
	*xml += int2string(usage_info.vmem);
	*xml += "\n</VMEM>\n";
	*xml += "</JOB_INFO>\n";
	*xml += "</JOB_PAYMENT>\n";
	*xml += "</BODY>\n";
	*xml += "</HLR>\n";
	return;
}


