#include <string>
#include <iostream>
#include <getopt.h>
#include <vector>
#include "edg-wl-dgas-ATMClient.h"

#define OPTION_STRING "hj:t:p:l:C:U:r:"

using namespace std;

bool needs_help = 0;

string dgJobId_buff = "NULL"; // DgJobId of the current job
string timeStamp_buff = "0"; // Submission time of the current job
string res_acct_PA_id_buff = ""; // Price authority for this CE (URL)
string res_acct_bank_id_buff = ""; // HLR for this CE (URL)
string resX509cert_buff = "NULL"; // X509 subject of this CE
string usrX509cert_buff = ""; // X509 subject of the user owning the job
string usr_acct_bank_id_buff = "NULL"; // HLR of the user owning the job (URL)
vector<string> info_v;

string cpu_time = "-1";
string wall_time = "-1";
string mem = "-1";
string vmem = "-1";

void help()
{
	cerr << "DGAS ATM client" << endl;
	cerr << "Author: Andrea Guarise <andrea.guarise@to.infn.it>" << endl << endl;
	cerr << "Usage:" << endl;
	cerr << "ATM_client <OPTIONS>" << endl;
	cerr << "Where options are:" << endl;
	cerr << "-h  --help         :this help" << endl;
	cerr << "-j  --jobid        :DgJobId of the current job" << endl;
	cerr << "-t  --time         :Submission time of the current job" << endl;
	cerr << "-p  --paid        :Price authority for this CE (URL)" << endl;
	cerr << "-l  --localbankid  :Local (CE) ACCT_BANK_ID, i.e. HLR for this CE" << endl;
	cerr << "-C  --resgridid    :Grid ID of this CE" << endl;
	cerr << "-U  --usrcert      :X509 subject of the user owning the job" << endl;
	cerr << "-r  --remhlrid    :Remote (user) ACCT_BANK_ID, HLR of the user." << endl;
	cerr << endl;
	
		
}

int options ( int argc, char **argv )
{
	int option_char;
	int option_index = 0;
	static struct option long_options[] =
	{
		{"jobid",1,0,'j'},
		{"time",1,0,'t'},
		{"paid",1,0,'p'},
		{"localbankid",1,0,'l'},
		{"resgridid",1,0,'C'},
		{"usrcert",1,0,'U'},
		{"remhlrid",1,0,'r'},
		{"help",0,0,'h'},
		{0,0,0,0}
	};
	while (( option_char = getopt_long( argc, argv, OPTION_STRING,
					long_options, &option_index)) != EOF)
		switch (option_char)
		{
			case 'j': dgJobId_buff=optarg; break;
			case 't': timeStamp_buff = optarg; break;
			case 'p': res_acct_PA_id_buff = optarg; break;
			case 'l': res_acct_bank_id_buff = optarg; break;
			case 'C': resX509cert_buff = optarg; break;
			case 'U': usrX509cert_buff = optarg; break;
			case 'r': usr_acct_bank_id_buff = optarg; break;
			case 'h': needs_help =1; break;		  
			default : break;
		}
		if (optind < argc) 
		{
                      while (optind < argc)
                         info_v.push_back(argv[optind++]);
                }
	return 0;
}
int main (int argc, char *argv[])
{
	options(argc, argv);
	if (needs_help)
	{
		help();
		return 0;
	}
	ATM_job_data job_data ={
		dgJobId_buff, //dgJobId
		atoi(timeStamp_buff.c_str()), //time
		res_acct_PA_id_buff, //res_PA_url
		res_acct_bank_id_buff, //res_HLR_url
		usrX509cert_buff, // usr_cert_subj
		resX509cert_buff //res_cert_subj
	};
	vector <string>::const_iterator info_v_it = info_v.begin();
	while (info_v_it != info_v.end())
	{
		size_t pos = (*info_v_it).find("CPU_TIME=");
		if ( pos != string::npos)
		{
			cpu_time = (*info_v_it).substr(pos+9);
		}
		pos = (*info_v_it).find("WALL_TIME=");
		if ( pos != string::npos)
		{
			wall_time = (*info_v_it).substr(pos+10);
		}
		pos = (*info_v_it).find("MEM=");
		if ( pos != string::npos)
		{
			mem = (*info_v_it).substr(pos+4);
		}
		pos = (*info_v_it).find("VMEM=");
		if ( pos != string::npos)
		{
			vmem = (*info_v_it).substr(pos+5);
		}
		info_v_it++;
	}
	if (cpu_time == "-1")
	{
		cerr << "Must specify at least the job cpu time!" << endl;
		return 1;
	}
	if (wall_time == "-1")
	{
		wall_time = "0";
	}
	if (mem == "-1")
	{
		mem = "0";
	}
	if (vmem == "-1")
	{
		vmem = "0";
	}
	ATM_usage_info usage_info = { atoi(cpu_time.c_str()), 
		atoi(wall_time.c_str()) 
			};
	#ifdef DEBUG
	cerr << "Usage Records: " << endl;
	cerr << "CPU_TIME = " << usage_info.cpu_time << endl;
	cerr << "WALL_TIME = " << usage_info.wall_time << endl;
	cerr << "MEM = " << usage_info.wall_time << endl;
	cerr << "VMEM = " << usage_info.wall_time << endl;
	#endif
	int res;
	string output;
	res = ATM_client( 
			usr_acct_bank_id_buff, 
			job_data, 
			usage_info, 
			&output  );
	cerr << "Return code:" << res << endl;
	cout << output << endl;
	return res;
}

