#include <string>
#include <iostream>
#include <getopt.h>
#include <vector>
#include "edg-wl-dgas-jobAuthClient.h"

#define OPTION_STRING "hj:t:U:u:"

using namespace std;

bool needs_help = 0;

string dgJobId_buff = ""; // DgJobId of the current job
string timeStamp_buff = "0"; // Submission time of the current job
string usrX509cert_buff = ""; // X509 subject of the user owning the job
string usr_acct_bank_id_buff = ""; // HLR of the user owning the job (URL)
vector<string> info_v;

void help()
{
	cerr << "DGAS jobAuth client" << endl;
	cerr << "Author: Andrea Guarise <andrea.guarise@to.infn.it>" << endl << endl;
	cerr << "Usage:" << endl;
	cerr << "jobAuth_client <OPTIONS>" << endl;
	cerr << "Where options are:" << endl;
	cerr << "-h  --help         :this help" << endl;
	cerr << "-j  --jobid        :DgJobId of the current job" << endl;
	cerr << "-t  --time         :Submission time of the current job" << endl;
	cerr << "-U  --usrcert      :X509 subject of the user owning the job" << endl;
	cerr << "-u  --usrhlrid    :Remote (user) ACCT_BANK_ID, HLR of the user." << endl;
	cerr << endl;
	
		
}

int options ( int argc, char **argv )
{
	int option_char;
	int option_index = 0;
	static struct option long_options[] =
	{
		{"jobid",1,0,'j'},
		{"time",1,0,'t'},
		{"usrcert",1,0,'U'},
		{"usrhlrid",1,0,'u'},
		{"help",0,0,'h'},
		{0,0,0,0}
	};
	while (( option_char = getopt_long( argc, argv, OPTION_STRING,
					long_options, &option_index)) != EOF)
		switch (option_char)
		{
			case 'j': dgJobId_buff=optarg; break;
			case 't': timeStamp_buff = optarg; break;
			case 'U': usrX509cert_buff = optarg; break;
			case 'u': usr_acct_bank_id_buff = optarg; break;
			case 'h': needs_help =1; break;		  
			default : break;
		}
		if (optind < argc) 
		{
                      while (optind < argc)
                         info_v.push_back(argv[optind++]);
                }
	return 0;
}
int main (int argc, char *argv[])
{
	options(argc, argv);
	if (needs_help)
	{
		help();
		return 0;
	}
	jobAuth_data auth_data ={
		dgJobId_buff, //dgJobId
		usrX509cert_buff, // usr_cert_subj
		atoi(timeStamp_buff.c_str()), //time
	};
	int res;
	string output;
	res = dgas_jobAuth_client( 
			usr_acct_bank_id_buff, 
			auth_data, 
			&output  );
	#ifdef DEBUG
	cerr << "Return code:" << res << endl;
	cout << output << endl;
	#endif
	return res;
}

