// DGAS (DataGrid Accounting System) 
// Client APIs.
// 
// $Id: edg-wl-dgas-pingClient.cpp,v 1.2 2003/12/02 13:50:06 guarise Exp $
// -------------------------------------------------------------------------
// Copyright (c) 2001-2002, The DataGrid project, INFN, 
// All rights reserved. See LICENSE file for details.
// -------------------------------------------------------------------------
// Author: Andrea Guarise <andrea.guarise@to.infn.it>
 /***************************************************************************
 * Code borrowed from:
 *  authors   :
 *  copyright : 
 ***************************************************************************/
//
//    

// ---------------------------------------------------------------------------
//  Includes
// ---------------------------------------------------------------------------
#include <string>
#include <vector>
#include <iostream>
#include <edg-wl-dgas-pingClient.h>


void ping_xml_compose(int pingType, string *xml)
{
	//initialize the message
	//the message will be of type PA_query
	*xml = "<HLR type=\"ping_request\">\n<HEAD>\n<VER>1.0</VER>\n</HEAD>\n<BODY>\n";
	*xml += "<PING>\n";
	*xml += "<TYPE>\n";
	*xml += int2string(pingType);
	*xml += "\n</TYPE>\n";
	*xml += "</PING>\n";
	*xml += "</BODY>\n</HLR>\n";
	return;
}//pa_xml_compose


int ping_parse_xml ( string &xmlInput, statusInfo *status, errorInfo *errors)
{
	string buffer;
	node Node;
	string tag = "STATUS_INFO";
	Node = parse (&xmlInput, tag); //isolate JOB_AUTH_INFO node
	if ( Node.status == 0 )
	{
		node tagBuff;
		tagBuff = parse ( &Node.text, "ENGINES");
		if ( tagBuff.status == 0 )
		{
			status->engines = tagBuff.text;
			#ifdef DEBUG
			cerr << "got:" << "ENGINES" << "=" << tagBuff.text << endl;
			#endif
		}
		tagBuff = parse ( &Node.text, "UI_REQUESTS");
		if ( tagBuff.status == 0 )
		{
			status->uiRequests = atoi((tagBuff.text).c_str());
			#ifdef DEBUG
			cerr << "got:" << "UI_REQUESTS" << "=" << tagBuff.text << endl;
			#endif
		}
		tagBuff = parse ( &Node.text, "ATM_REQUESTS");
		if ( tagBuff.status == 0 )
		{
			status->ATMRequests = atoi((tagBuff.text).c_str());
			#ifdef DEBUG
			cerr << "got:" << "ATM_REQUESTS" << "=" << tagBuff.text << endl;
			#endif
		}
		tagBuff = parse ( &Node.text, "BANK_REQUESTS");
		if ( tagBuff.status == 0 )
		{
			status->bankRequests = atoi((tagBuff.text).c_str());
			#ifdef DEBUG
			cerr << "got:" << "BANK_REQUESTS" << "=" << tagBuff.text << endl;
			#endif
		}
		tagBuff = parse ( &Node.text, "JOBAUTH_REQUESTS");
		if ( tagBuff.status == 0 )
		{
			status->jobAuthRequests = atoi((tagBuff.text).c_str());
			#ifdef DEBUG
			cerr << "got:" << "JOBAUTH_REQUESTS" << "=" << tagBuff.text << endl;
			#endif
		}
		tagBuff = parse ( &Node.text, "USERAUTH_REQUESTS");
		if ( tagBuff.status == 0 )
		{
			status->userAuthRequests = atoi((tagBuff.text).c_str());
			#ifdef DEBUG
			cerr << "got:" << "USERAUTH_REQUESTS" << "=" << tagBuff.text << endl;
			#endif
		}
		tagBuff = parse ( &Node.text, "PA_REQUESTS");
		if ( tagBuff.status == 0 )
		{
			status->paRequests = atoi((tagBuff.text).c_str());
			#ifdef DEBUG
			cerr << "got:" << "PA_REQUESTS" << "=" << tagBuff.text << endl;
			#endif
		}
		tagBuff = parse ( &Node.text, "PING_REQUESTS");
		if ( tagBuff.status == 0 )
		{
			status->pingRequests = atoi((tagBuff.text).c_str());
			#ifdef DEBUG
			cerr << "got:" << "PING_REQUESTS" << "=" << tagBuff.text << endl;
			#endif
		}
		tagBuff = parse ( &Node.text, "ATM_ERRORS");
		if ( tagBuff.status == 0 )
		{
			errors->ATMErrors = atoi((tagBuff.text).c_str());
			#ifdef DEBUG
			cerr << "got:" << "ATM_ERRORS" << "=" << tagBuff.text << endl;
			#endif
		}
		tagBuff = parse ( &Node.text, "BANK_ERRORS");
		if ( tagBuff.status == 0 )
		{
			errors->bankErrors = atoi((tagBuff.text).c_str());
			#ifdef DEBUG
			cerr << "got:" << "BANK_ERRORS" << "=" << tagBuff.text << endl;
			#endif
		}
		tagBuff = parse ( &Node.text, "JOBAUTH_ERRORS");
		if ( tagBuff.status == 0 )
		{
			errors->jobAuthErrors = atoi((tagBuff.text).c_str());
			#ifdef DEBUG
			cerr << "got:" << "JOBAUTH_ERRORS" << "=" << tagBuff.text << endl;
			#endif
		}
		tagBuff = parse ( &Node.text, "USERAUTH_ERRORS");
		if ( tagBuff.status == 0 )
		{
			errors->userAuthErrors = atoi((tagBuff.text).c_str());
			#ifdef DEBUG
			cerr << "got:" << "USERAUTH_ERRORS" << "=" << tagBuff.text << endl;
			#endif
		}
		tagBuff = parse ( &Node.text, "PA_ERRORS");
		if ( tagBuff.status == 0 )
		{
			errors->paErrors = atoi((tagBuff.text).c_str());
			#ifdef DEBUG
			cerr << "got:" << "PA_ERRORS" << "=" << tagBuff.text << endl;
			#endif
		}


	}
	return 0;
}


int dgas_ping_client(string &acct_id, int pingType, statusInfo *status,errorInfo *errors , string *server_answer)
{
	int returnCode = 0;
	string output_message; 
	string input_message;

	char delim = ':';
	//decompose remote HLR id into server_name and port.
	vector<string> urlBuff;
	Split(delim, acct_id, &urlBuff);
	if (urlBuff.size() != 3)
	       return atoi(E_PING_PARSE_ID);	
	string hlrHostname = urlBuff[0];
	int hlrPort = atoi((urlBuff[1]).c_str());
	string hlrContact = urlBuff[2];
	#ifdef DEBUG
	cerr << "HLR url: Hostname: #" << hlrHostname 
		<< "# , Port : #" << hlrPort << "#" << endl;
	#endif
	//compose the xml message.
	ping_xml_compose(pingType, &output_message);
	//contact the PA server
	try{//try1
	#ifdef GSI
	GSISocketClient *theClient = new GSISocketClient(hlrHostname,hlrPort);
	theClient-> ServerContact(hlrContact);
	#else
	SocketClient *theClient = new SocketClient(hlrHostname,hlrPort);
	#endif
	if (!(theClient -> Open()))
	{
		#ifdef DEBUG
		cerr << "ERROR: can't open connection\n" << endl; 
		#endif
		returnCode = atoi(E_NO_CONNECTION) ;
        }
	else
	{	
		#ifdef DEBUG
		cerr << "Connection opened, sending message..." << endl;
		#endif
		if ( !(theClient->Send(output_message)) )
		{
			#ifdef DEBUG
			cerr << "ERROR: can't send message\n" << endl; 
			#endif
			returnCode =  atoi(E_SEND_MESSAGE);
		}
		#ifdef DEBUG
		cerr << "Sending ..." << endl;
		cerr << output_message << endl;
		cerr << "Message sent. Waiting for answer..." << endl;
		#endif
		if ( !(theClient->Receive(input_message)) )
		{
			#ifdef DEBUG
			cerr << "ERROR: can't receive message\n" << endl; 
			#endif
			returnCode =  atoi(E_RECEIVE_MESSAGE);
		}
		#ifdef DEBUG
		cerr << input_message << endl;
		#endif
		theClient->Close();
		delete theClient;
		*server_answer = input_message;
		if (returnCode == 0)
		{
			returnCode = ping_parse_xml(input_message, status, errors);
		}
	}
	}//try1
	catch( std::exception& e)
	{
		std::cerr << e.what() << std::endl;
		returnCode = atoi(E_NO_CONNECTION);
	}
	return returnCode;
}//pa_client


