#include <string>
#include <iostream>
#include <getopt.h>
#include <vector>
#include "edg-wl-dgas-pingClient.h"

#define OPTION_STRING "hs:t:"

using namespace std;

bool needs_help = 0;

string acct_id_buff = ""; 
int type_buff = 0;
vector<string> info_v;

void help()
{
	cerr << "DGAS ping client" << endl;
	cerr << "Author: Andrea Guarise <andrea.guarise@to.infn.it>" << endl << endl;
	cerr << "Usage:" << endl;
	cerr << "jobAuth_client <OPTIONS>" << endl;
	cerr << "Where options are:" << endl;
	cerr << "-h  --help         :this help" << endl;
	cerr << "-s  --server       :remote server ACCT_ID" << endl;
	cerr << "-t  --type         :Ping Type, 0 Normal >1 Status info" << endl;
	cerr << endl;
	
		
}

int options ( int argc, char **argv )
{
	int option_char;
	int option_index = 0;
	static struct option long_options[] =
	{
		{"server",1,0,'s'},
		{"type",1,0,'t'},
		{"help",0,0,'h'},
		{0,0,0,0}
	};
	while (( option_char = getopt_long( argc, argv, OPTION_STRING,
					long_options, &option_index)) != EOF)
		switch (option_char)
		{
			case 's': acct_id_buff=optarg; break;
			case 't': type_buff = atoi(optarg); break;
			case 'h': needs_help =1; break;		  
			default : break;
		}
		if (optind < argc) 
		{
                      while (optind < argc)
                         info_v.push_back(argv[optind++]);
                }
	return 0;
}
int main (int argc, char *argv[])
{
	options(argc, argv);
	if (needs_help)
	{
		help();
		return 0;
	}
	statusInfo status;
	status.engines="";
	status.uiRequests=0;
	status.ATMRequests=0;
	status.bankRequests=0;
	status.jobAuthRequests=0;
	status.userAuthRequests=0;
	status.paRequests=0;
	status.pingRequests=0;
	
	errorInfo errors;
	errors.engines="";
	errors.uiErrors=0;
	errors.ATMErrors=0;
	errors.bankErrors=0;
	errors.jobAuthErrors=0;
	errors.userAuthErrors=0;
	errors.paErrors=0;
	errors.pingErrors=0;
	
	int res;
	string output;
	res = dgas_ping_client( 
			acct_id_buff,
		        type_buff,	
			&status, 
			&errors, 
			&output);
	if ( res != 0 )
	{
		cout << "Server: " << acct_id_buff << "Unreachable." << endl;
	}
	else
	{
		if ( type_buff > 0 )
		{
			cout << "Server status:" << endl << endl;
			cout << "Available Engines: " << status.engines << endl;
			if ( status.uiRequests != 0 )
			cout << "User Interface requests: " << status.uiRequests << endl;
			if ( status.ATMRequests != 0 )
			{
				cout << "ATM requests: " << status.ATMRequests << 
					"/" << errors.ATMErrors << endl;
			}
			if ( status.bankRequests != 0 )
			{
				cout << "bank requests: " << status.bankRequests 
					<< "/" << errors.bankErrors << endl;
			}
			if ( status.jobAuthRequests != 0 )
			{
				cout << "jobAuth requests: " << status.jobAuthRequests 
					<< "/" << errors.jobAuthErrors << endl;
			}
			if ( status.userAuthRequests != 0 )
			{
				cout << "userAuth requests: " << 
					status.userAuthRequests 
					<< "/" << errors.userAuthErrors << endl;
			}
			if ( status.paRequests != 0 )
			{
				cout << "Price Authority Requests: " << 
					status.paRequests 
					<< "/" << errors.paErrors << endl;
			}
			if ( status.pingRequests != 0 )
			cout << "Ping requests: " << status.pingRequests << endl;
		}
		else
		{
			cout << "Server alive." << endl;
		}
	}
	#ifdef DEBUG
	cout << output << endl;
	#endif
	return res;
}

