// DGAS (DataGrid Accounting System) 
// Client APIs.
// 
// $Id: edg-wl-dgas-userAuthClient.cpp,v 1.2 2003/12/02 13:50:06 guarise Exp $
// -------------------------------------------------------------------------
// Copyright (c) 2001-2002, The DataGrid project, INFN, 
// All rights reserved. See LICENSE file for details.
// -------------------------------------------------------------------------
// Author: Andrea Guarise <andrea.guarise@to.infn.it>
 /***************************************************************************
 * Code borrowed from:
 *  authors   :
 *  copyright : 
 ***************************************************************************/
//
//    

// ---------------------------------------------------------------------------
//  Includes
// ---------------------------------------------------------------------------
#include <string>
#include <iostream>
#include <vector>
#include <edg-wl-dgas-userAuthClient.h>


void ua_xml_compose(string &dgJobId, string *xml)
{
	//initialize the message
	*xml = "<HLR type=\"UserAuth_request\">\n<HEAD>\n<VER>1.0</VER>\n</HEAD>\n<BODY>\n";
	*xml += "<USER_AUTH_INFO>\n";
	*xml += "<DG_JOBID>\n";
	*xml += dgJobId;
	*xml += "\n</DG_JOBID>\n";
	*xml += "</USER_AUTH_INFO>\n";
	*xml += "</BODY>\n</HLR>\n";
	return;
}//pa_xml_compose


int uac_parse_xml ( string &xmlInput, userAuth_data *auth_data)
{
	string buffer;
	node Node;
	string tag = "USER_AUTH_INFO";
	Node = parse (&xmlInput, tag); //isolate USER_AUTH_INFO node
	if ( Node.status == 0 )
	{
		node tagBuff;
		tagBuff = parse ( &Node.text, "DG_JOBID");
		if ( tagBuff.status == 0 )
		{
			buffer = tagBuff.text;
			auth_data->dgJobId = buffer;
			#ifdef DEBUG
			cerr << "got:" << "DG_JOBID" << "=" << tagBuff.text << endl;
			#endif
		}
		tagBuff = parse ( &Node.text, "jobIsRegistered");
		if ( tagBuff.status == 0 )
		{
			buffer = tagBuff.text;
			if ( buffer == "true" )
				auth_data->jobIsRegistered = true;
			if ( buffer == "false" )
				auth_data->jobIsRegistered = false;
			#ifdef DEBUG
			cerr << "got:" << "jobIsRegistered" << "=" << tagBuff.text << endl;
			#endif
		}
		tagBuff = parse ( &Node.text, "userAccountIsValid");
		if ( tagBuff.status == 0 )
		{
			buffer = tagBuff.text;
			if ( buffer == "true" )
				auth_data->userHasValidAccount = true;
			if ( buffer == "false" )
				auth_data->userHasValidAccount = false;
			#ifdef DEBUG
			cerr << "got:" << "userAccountIsValid" << "=" << tagBuff.text << endl;
			#endif
		}
		tagBuff = parse ( &Node.text, "jobRegTime");
		if ( tagBuff.status == 0 )
		{
			buffer = tagBuff.text;
				auth_data->jobRegTime = atoi(buffer.c_str());
			#ifdef DEBUG
			cerr << "got:" << "jobRegTime" << "=" << tagBuff.text << endl;
			#endif
		}
		tagBuff = parse ( &Node.text, "userAvailableCredits");
		if ( tagBuff.status == 0 )
		{
			buffer = tagBuff.text;
				auth_data->availableCredits = atoi(buffer.c_str());
			#ifdef DEBUG
			cerr << "got:" << "userAvailableCredits" << "=" << tagBuff.text << endl;
			#endif
		}

		tagBuff = parse ( &Node.text, "STATUS");
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			cerr << "got:" << "STATUS" << "=" << tagBuff.text << endl;
			#endif
			return atoi((tagBuff.text).c_str());//report the error
		}
		
	}
	return 0;
}


int dgas_userAuth_client(string &user_acct_bank_id , userAuth_data &auth_data, string *server_answer)
{
	int returnCode = 0;
	string output_message; 
	string input_message;

	char delim = ':';
	//decompose remote HLR id into server_name and port.
	vector<string> urlBuff;
	Split(delim, user_acct_bank_id, &urlBuff);
	if (urlBuff.size() != 3)
	       return atoi(E_JA_PARSE_ID);	
	string hlrHostname = urlBuff[0];
	int hlrPort = atoi((urlBuff[1]).c_str());
	string hlrContact = urlBuff[2];
	#ifdef DEBUG
	cerr << "User HLR url: Hostname: #" << hlrHostname 
		<< "# , Port : #" << hlrPort << "#" << endl;
	#endif
	//compose the xml message.
	ua_xml_compose(auth_data.dgJobId, &output_message);
	//contact the PA server
	try{//try1
	#ifdef GSI
	GSISocketClient *theClient = new GSISocketClient(hlrHostname,hlrPort);
	theClient-> ServerContact(hlrContact);
	#else
	SocketClient *theClient = new SocketClient(hlrHostname,hlrPort);
	#endif
	if (!(theClient -> Open()))
	{
		#ifdef DEBUG
		cerr << "ERROR: can't open connection\n" << endl; 
		#endif
		returnCode = atoi(E_NO_CONNECTION) ;
        }
	else
	{	
		#ifdef DEBUG
		cerr << "Connection opened, sending message..." << endl;
		#endif
		if ( !(theClient->Send(output_message)) )
		{
			#ifdef DEBUG
			cerr << "ERROR: can't send message\n" << endl; 
			#endif
			returnCode =  atoi(E_SEND_MESSAGE);
		}
		#ifdef DEBUG
		cerr << "Message sent. Waiting for answer..." << endl;
		#endif
		if ( !(theClient->Receive(input_message)) )
		{
			#ifdef DEBUG
			cerr << "ERROR: can't receive message\n" << endl; 
			#endif
			returnCode =  atoi(E_RECEIVE_MESSAGE);
		}
		#ifdef DEBUG
		cerr << input_message << endl;
		#endif
		theClient->Close();
		delete theClient;
		*server_answer = input_message;
		if (returnCode == 0)
		{
			returnCode = uac_parse_xml(input_message, &auth_data);
		}
	}
	}//try1
	catch( std::exception& e)
	{
		std::cerr << e.what() << std::endl;
		returnCode = atoi(E_NO_CONNECTION);
	}
	return returnCode;
}//dgas_jobauth_client


