// DGAS (DataGrid Accounting System) 
// Server Daemon and protocol engines.
// 
// $Id: dgas_hlr_qtransMgrd.cpp,v 1.5 2003/12/02 13:50:07 guarise Exp $
// -------------------------------------------------------------------------
// Copyright (c) 2001-2002, The DataGrid project, INFN, 
// All rights reserved. See LICENSE file for details.
// -------------------------------------------------------------------------
// Author: Andrea Guarise <andrea.guarise@to.infn.it>
 /***************************************************************************
 * Code borrowed from:
 *  authors   : 
 *  copyright : 
 ***************************************************************************/
//
//    

#include <cstdlib>
#include <getopt.h>
#include <unistd.h>
#include <csignal>
//c++ includes
#include <string>
#include <iostream>
#include <fstream>
#include <sstream>
#include <vector>
//hlr api includes
#include <dgas_config.h>
#include <libdgas_log.h>
#include <int2string.h>
#include <dgas_lock.h>
#include <qTransaction.h>
#include <hlrTransaction.h>
#include <ATM_bank_client.h>
#include <hlr_prot_errcode.h>
#include <dgasVersion.h>

#define OPTION_STRING "hl:L:c:e"
//global variables
//

using namespace std;
#ifndef EDG_SOCKETPP
using namespace edg::workload::common::socket_pp;
#endif
const char * hlr_sql_server;
const char * hlr_sql_user;
const char * hlr_sql_password;
const char * hlr_sql_dbname;
const char * hlr_tmp_sql_dbname;
ofstream logStream;


volatile sig_atomic_t keep_going = 1;
int help_flag = 0;
bool error_flag = false;
string configFileName = "";
string logFileName = "";
string lockFileName = "";
int hlr_qmgr_tPerIter = 10;
int hlr_qmgr_qDepth = 10;
int hlr_qmgr_expPeriod = 600;
int hlr_qmgr_pollPeriod = 20;
string logBuff;

int
print_help()
{
	cerr<< endl;
	cerr<< "DGAS hlr_qtransMgrd" <<endl;
	cerr<< "Version :" << VERSION << endl ;
	cerr<< "Author: A.Guarise <andrea.guarise@to.infn.it>"<< endl;
	cerr<< endl << "Usage: " << endl << endl;;
	cerr<< "dgas_hlr_qtransMgrd OPTIONS" << endl << endl;;
	cerr<< "OPTIONS:" <<endl << endl;
	cerr<< "-l  --log\t:  Set the log file name." << endl;
	cerr<< "-L  --Lock\t:  Set the Lock file name." << endl;
	cerr<< "-c  --config\t:  Set the configuration file name." << endl;
	cerr<< "-e  --error\t:  Process the transaction in fatal error state." << endl;
	cerr<< "-h  --help\t:  Print this help message." << endl;
	return 0;
}//print_help()


int 
get_options (int argc, char **argv)
{
	int	option_char;
	int 	option_index;
	static	struct option long_options[] =
	{
		{"help",0,0,'h'},
		{"log",1,0,'l'},
		{"error",0,0,'e'},
		{"Lock",1,0,'L'},
		{"config",1,0,'c'},
		{0,0,0,0}
	};
	while (( option_char = getopt_long (argc,
			argv,
			OPTION_STRING,
			long_options,
			&option_index)) != EOF)
		switch (option_char)
		{
			case 'h': help_flag = 1; break;
			case 'l': logFileName = optarg; break;
			case 'e': error_flag = true; break;
			case 'L': lockFileName = optarg; break;
			case 'c': configFileName = optarg; break;
			default : break;
		}
	return 0;
}//get_options()

void mainLoop( int hlr_qmgr_tPerIter )
{
	int i = 0;
	for (int counter = 0; counter < hlr_qmgr_tPerIter; counter++)
	{
		qTransaction trans;
		if ( trans.get(i) == 0 )
		{
			hlrTransaction t(
					 0,
					 nd,
					 trans.transactionId,
					 trans.fromCertSubject,
					 trans.toCertSubject,
					 trans.fromHlrId,
					 trans.toHlrId,
					 trans.amount,
					 trans.timestamp,
					 trans.logData
					);
			if ( trans.tType == "out" )
			{
				t.tType = out;
			}
			else
			{
				if ( trans.tType == "in" )
				{
					t.tType = in;
				}
			}
			logBuff = "hlr_qMgr::mainLoop(): Processing transaction: " + t.id + " with priority:" + int2string(trans.priority);
			hlr_log( logBuff, &logStream, 1);
			if ( dgas_bankClient(t) != 0 )
			{
				logBuff = "hlr_qMgr::mainLoop(): Can't process transaction:" + t.id;
				hlr_log( logBuff, &logStream, 1);
				if ((time(NULL) - trans.statusTime) > hlr_qmgr_expPeriod )
				{
					trans.priority++;
					trans.statusTime = time(NULL);
					trans.update();
				}
			}
			else
			{
				logBuff = "hlr_qMgr::mainLoop(): Transaction:" + t.id + " has been processed";
				hlr_log( logBuff, &logStream, 1);
				
				if ( trans.remove() != 0 )
				{
					logBuff = "hlr_qMgr::mainLoop(): Error removing processed transaction from queue";
				hlr_log( logBuff, &logStream, 1);	
				}
				else
				{
				logBuff = "hlr_qMgr::mainLoop(): Transation removed from the queue";
				hlr_log( logBuff, &logStream, 1);
				//FIXME log the transaction
				}
			}
		}
		else
		{
			i++;
			if ( i == hlr_qmgr_qDepth )
				i = 0;
		}
	}
	return;
}

int processErrorTransactions()
{
	//FIXME 
	return 0;
}

int putLock(string lockFile)
{
        dgasLock Lock(lockFile);
        if ( Lock.exists() )
        {
                //the lock file already exists, return an error.
                return 1;
        }
        else
        {
                //the lock file doesn't exists, therefore creates it
                //and insert some important information inside the file.
                if ( Lock.put() != 0 )
                {
                        //there was an error creating the lock file.
                        //exit with an error.
                        return 2;
                }
                else
                {
                        return 0;
                }
        }

}

int removeLock( string lockFile)
{
        dgasLock Lock(lockFile);
        if ( Lock.exists() )
        {
                //the lock file exits, remove it.
                if ( Lock.remove() != 0 )
                {
			hlr_log("Error removing the lock file", &logStream, 1);
                        //error removing the lock
                        return 2;
                }
                else
                {
			hlr_log("lock file removed", &logStream, 1);
                        return 0;
                }
        }
        else
        {
		hlr_log("lock file doesn't exists", &logStream, 1);
                return 1;
        }
}


void fatal_error_signal (int sig)
{
	keep_going = 0;
	cerr << "Exiting..." << endl;
	signal (sig, fatal_error_signal);
}

void sighup_signal (int sig)
{
	keep_going = 1;
	error_flag = true;
	cerr << "Set error_flag to true" << endl;
	signal (sig, sighup_signal);
}

int main ( int argc, char * argv[] )
{
	get_options(argc, argv);
	if ( help_flag == 1 )
	{
		print_help();
		exit(1);
	}
        map <string,string> confMap;
        dgas_conf_read ( configFileName, &confMap );
        hlr_sql_server = (confMap["hlr_sql_server"]).c_str();
        hlr_sql_user = (confMap["hlr_sql_user"]).c_str();
        hlr_sql_password = (confMap["hlr_sql_password"]).c_str();
        hlr_sql_dbname = (confMap["hlr_sql_dbname"]).c_str();
        hlr_tmp_sql_dbname = (confMap["hlr_tmp_sql_dbname"]).c_str();
	hlr_qmgr_expPeriod = atoi((confMap["hlr_qmgr_expPeriod"]).c_str());
	hlr_qmgr_qDepth = atoi((confMap["hlr_qmgr_qDepth"]).c_str());
	hlr_qmgr_tPerIter = atoi((confMap["hlr_qmgr_tPerIter"]).c_str());
	hlr_qmgr_pollPeriod = atoi((confMap["hlr_qmgr_pollPeriod"]).c_str());
	if (logFileName == "" )
	{
		#ifdef DEBUG
		cerr << "Using default log file: ";
		#endif
		logFileName = confMap["hlr_qmgr_def_log"];
		#ifdef DEBUG
		cerr << logFileName << endl;
		#endif
	}
	logStream.open( logFileName.c_str() , ios::app);
	if (lockFileName == "" )
	{
		lockFileName = confMap["hlr_qmgr_def_lock"];
		logBuff = "hlr_qMgr: Using default lock file: " +
			lockFileName;
		hlr_log(logBuff, &logStream, 1);
	}
	if ( putLock(lockFileName) != 0 )
        {
                //for some reasons it was impossible to create the lock
                //threfore the server won't be started.
                hlr_log("hlr_qMgr: Error: Startup failed, Error creating the lock file.",
                                &logStream, 1);
		logStream.close();
                exit(atoi(E_LOCK_OPEN));
        }
							
	signal (SIGTERM, fatal_error_signal);
	signal (SIGINT, fatal_error_signal);
	signal (SIGHUP, sighup_signal);
	while (keep_going)
	{
		if ( error_flag )
		{
			processErrorTransactions();
			error_flag= false;
		}
		hlr_log("hlr_qMgr: Entering main loop", &logStream, 1);
		mainLoop(hlr_qmgr_tPerIter);		
		sleep(hlr_qmgr_pollPeriod);
	}
	
hlr_log ( "hlr_qMgr: Received termination signal:  exiting...",
			&logStream, 1);
removeLock(lockFileName);
#ifdef DEBUG	
cerr << "Exit." << endl;
#endif	
logStream.close();
return 0;
}
