
// ---------------------------------------------------------------------------
//  Includes
// ---------------------------------------------------------------------------
#include <getopt.h>
#include <vector>
#include <string>
#include <iostream>
#include <sstream>
#include <fstream>
#include "hlr_prot_errcode.h"
#include "xmlUtil.h"
#include "stringSplit.h"
//Socket++ includes
#include "SocketClient.h"
#include "SocketAgent.h"
#ifdef GSI
#include "GSISocketClient.h"
#include "GSISocketAgent.h"
#endif
#include <dgasVersion.h>

// global definitions

#define OPTION_STRING "hs:f:"

using namespace std;
#ifndef EDG_SOCKETPP
using namespace edg::workload::common::socket_pp;
#endif

bool needs_help = 0;
string hlrUrlBuff = "";
string fileName ="";


// ---------------------------------------------------------------------------
//
//  Usage()
//
// ---------------------------------------------------------------------------
void usage()
{
    cerr << "\nUsage:\n" 
	    "hlr_ugetinfo (remote client)\n"
	    "Ver "
	    VERSION
            "\n-h --help (display this help and exit\n"
	    "-s --server servername\n"
            "-f --file file containing the DGASML\n"
          <<  endl;
}


int options ( int argc, char **argv )
{
	int option_char;
	int option_index = 0;
	static struct option long_options[] =
	{
		{"file",1,0,'f'},
		{"server",1,0,'s'},
		{"help",0,0,'h'},
		{0,0,0,0}
	};
	while (( option_char = getopt_long( argc, argv, OPTION_STRING,
					long_options, &option_index)) != EOF)
		switch (option_char)
		{
			case 'f': fileName = optarg; break;
			case 's': hlrUrlBuff = optarg; break;
			case 'h': needs_help =1; break;		  
			default : break;
		}
	return 0;
}

int fileRead( string fileBuff, string *xmlBuff )
{
        ifstream inFile(fileBuff.c_str());
        if ( !inFile )
        {
                #ifdef DEBUG
                cerr << "DEBUG:" << __FILE__ << "," << __LINE__;
                cerr << ",Error openig the file:" << fileBuff << endl;
                #endif
                return 1;
        }
        else
        {
                ostringstream buf;
                char ch;
                while ( buf && inFile.get(ch) )
                buf.put(ch);
                *xmlBuff = buf.str();
                return 0;
        }
}


// ---------------------------------------------------------------------------
//
//  main
//
// ---------------------------------------------------------------------------
int main(int argc, char **argv)
{
	int retval = 0;
	options (argc, argv);
	if (needs_help)
	{
		usage();
		return 0;
	}
	vector<string> urlBuff;
	Split(':',hlrUrlBuff, &urlBuff);
	if ( urlBuff.size() != 3 )
	{
		return atoi(E_RUI_PARSE_ID);
	}
	string HlrHostname = urlBuff[0];
	int HlrPort = atoi((urlBuff[1]).c_str());
	string HlrContact = urlBuff[2];
	string xmlBuff;
	if ( fileRead( fileName, &xmlBuff ) != 0 )
	{
		cerr << "Error reading the input file" << endl;
	}
	
#ifdef DEBUG
	cerr << "XML message:\n";
	cerr << xmlBuff;
	cerr << "END of XML message\n";
#endif

#ifdef GSI
	GSISocketClient *theClient = new GSISocketClient(HlrHostname, HlrPort);
	theClient -> ServerContact(HlrContact);
#else
	SocketClient *theClient = new SocketClient(HlrHostname, HlrPort);
#endif
	theClient->SetTimeout(0);
	if ( !(theClient -> Open()) )
	{
		cerr << "ERROR: can't open conenction"<< endl;
		exit (1);
	}
#ifdef DEBUG
	cerr << "Connection opened: sending message ..." <<endl;
#endif
	if ( !(theClient -> Send(xmlBuff) ))
	{
		cerr << "Error sending message" << endl;
	}
	string xml_answer;
	if ( !(theClient -> Receive(xml_answer) ))
	{
		cerr << "Error receiving message" << endl;
	}
	else
	{
		theClient -> Close();
	}
	delete theClient;
	#ifdef DEBUG
	cerr << "Answer from the server:" <<endl;
	#endif
	cout << xml_answer  << endl;

	#ifdef DEBUG
	cerr << "Done." <<endl;
	#endif
	
    	return retval;
}
