// DGAS (DataGrid Accounting System) 
// Server Daeomn and protocol engines.
// 
// $Id: pa_serverd.cpp,v 1.5 2003/12/02 13:50:07 guarise Exp $
// -------------------------------------------------------------------------
// Copyright (c) 2001-2002, The DataGrid project, INFN, 
// All rights reserved. See LICENSE file for details.
// -------------------------------------------------------------------------
// Author: Andrea Guarise <andrea.guarise@to.infn.it>
 /***************************************************************************
 * Code borrowed from:
 *  authors   : Salvatore Monforte <salvatore.monforte@ct.infn.it>
 *  copyright : (C) 2001-2002 INFN
 ***************************************************************************/
//
//    

#define DEBUG

#include <cstdlib>
#include <getopt.h>
#include <csignal>
//c++ includes
#include <string>
#include <iostream>
#include <fstream>
//local includes
//hlr api includes
#include "comm_struct.h"
#include <dgas_config.h>
#include <dgas_lock.h>
#include <libdgas_log.h>
#include <int2string.h>
#include "xml_pa_helper.h"
//socketpp includes
#ifdef GSI
  #include "GSISocketServer.h"
  #include "GSISocketAgent.h"
#else
  #include "SocketServer.h"
  #include "SocketAgent.h"
#endif
#include <ping_engine.hpp>
#include <dgasVersion.h>
//global defines
//
#define DEFAULT_CONFIG "./hlr_serverd.conf"

#define OPTION_STRING "hp:l:L:c:"
//global variables
//

using namespace std;
#ifndef EDG_SOCKETPP
using namespace edg::workload::common::socket_pp;
#endif

const char * hlr_sql_server;   
const char * hlr_sql_user;
const char * hlr_sql_password;
const char * hlr_sql_dbname;

const char * pa_sql_server;  
const char * pa_sql_user;
const char * pa_sql_password;
const char * pa_sql_dbname;

const char * pa_pricing_scheme;
const char * pa_def_price_ttl;

ofstream logStream;

volatile sig_atomic_t keep_going = 1;
int help_flag = 0;
int server_port = -1;
int has_finished = 0;
string logFileName = "";
string lockFileName = "";
string configFileName = DEFAULT_CONFIG;
string server_contact = "";

statusInfo serverStatus;
errorInfo errorStatus;

int
print_help()
{
	cerr<< endl;
        cerr<< "DGAS pa_serverd" <<endl;
        cerr<< "Version :" << VERSION << endl ;
        cerr<< "Author: A.Guarise <andrea.guarise@to.infn.it>"<< endl;
        cerr<< endl << "Usage: " << endl << endl;;
        cerr<< "pa_serverd OPTIONS" << endl << endl;;
        cerr<< "OPTIONS:" <<endl << endl;
        cerr<< "-p  --port\t:  Set the listening port." << endl;
        cerr<< "-l  --log\t:  Set the log file name." << endl;
        cerr<< "-L  --Lock\t:  Set the Lock file name." << endl;
        cerr<< "-c  --config\t:  Set the configuration file name." << endl;
        cerr<< "-h  --help\t:  Print this help message." << endl;
	return 0;
}//print_help()


int 
get_options (int argc, char **argv)
{
	int	option_char;
	int 	option_index;
	static	struct option long_options[] =
	{
		{"help",0,0,'h'},
		{"port",1,0,'p'},
		{"log",1,0,'l'},
		{"Lock",1,0,'L'},
		{"config",1,0,'c'},
		{0,0,0,0}
	};
	while (( option_char = getopt_long (argc,
			argv,
			OPTION_STRING,
			long_options,
			&option_index)) != EOF)
		switch (option_char)
		{
			case 'h': help_flag = 1; break;
			case 'p': server_port = atoi(optarg); break;
			case 'l': logFileName = optarg; break;
			case 'L': lockFileName = optarg; break;
			case 'c': configFileName = optarg; break;
			default : break;
		}
	return 0;
}//get_options()

int putLock(string lockFile)
{
        dgasLock Lock(lockFile);
        if ( Lock.exists() )
        {
                //the lock file already exists, return an error.
                return 1;
        }
        else
        {
                //the lock file doesn't exists, therefore creates it
                //and insert some important information inside the file.
                if ( Lock.put() != 0 )
                {
                        //there was an error creating the lock file.
                        //exit with an error.
                        return 2;
                }
                else
                {
                        string portBuffer= "ServerPort=\""+int2string(server_port)+"\"";
                        string contactBuffer = "ServerContact=\""+server_contact+"\"";
                        Lock.appendString(portBuffer);
                        Lock.appendString(contactBuffer);
                        return 0;
                }
        }

}

int removeLock( string lockFile)
{
        dgasLock Lock(lockFile);
        if ( Lock.exists() )
        {
                //the lock file exits, remove it.
                if ( Lock.remove() != 0 )
                {
                        //error removing the lock
                        return 2;
                }
                else
                {
                        return 0;
                }
        }
        else
        {
                return 1;
        }
}

void fatal_error_signal (int sig)
{
	keep_going = 0;
	cerr << "Exiting..." << endl;
	signal (sig, fatal_error_signal);
}


int main ( int argc, char * argv[] )
{
	int res;
	get_options(argc, argv);
	if ( help_flag )
	{
		print_help();
		return 0;
	}
	map <string,string> confMap;
        dgas_conf_read ( configFileName, &confMap );
        pa_sql_server = (confMap["pa_sql_server"]).c_str();
        pa_sql_user = (confMap["pa_sql_user"]).c_str();
        pa_sql_password = (confMap["pa_sql_password"]).c_str();
        pa_sql_dbname = (confMap["pa_sql_dbname"]).c_str();
	server_contact = (confMap["pa_server_contact"]).c_str();
	pa_pricing_scheme = (confMap["pricing_scheme"]).c_str();
	pa_def_price_ttl = (confMap["def_price_minTTL"]).c_str();

	serverStatus.engines="PA:PING";
	serverStatus.uiRequests=0;
	serverStatus.ATMRequests=0;
	serverStatus.bankRequests=0;
	serverStatus.jobAuthRequests=0;
	serverStatus.paRequests=0;
	serverStatus.pingRequests=0;

	if (server_port == -1)
	{
		#ifdef DEBUG
		cerr << "Using default port number: ";
		#endif
		server_port = atoi((confMap["pa_def_port"]).c_str());
		if ( server_port == 0 )
		{
			cerr << "Please check that in "<< configFileName << endl;
			cerr << "The entry hlr_def_port is set up correctly" << endl;
			exit(1);
		}
		#ifdef DEBUG
		cerr << server_port << endl;
		#endif
	}
	if (logFileName == "")
	{
		#ifdef DEBUG
		cerr << "Using default log file: ";
		#endif
		logFileName = confMap["pa_def_log"];
		#ifdef DEBUG
		cerr << logFileName << endl;
		#endif
	}
	#ifdef DEBUG
         cerr << "Log file:" << logFileName << endl;
        #endif

	res = bootstrapLog(logFileName, &logStream);
	if ( res != 0 )
	{
		cerr << "Error bootstrapping the Log file:" << endl;
		cerr << logFileName<< endl;
		exit(1);
	}
	hlr_log ( 
			"Server listening on port : " + int2string(server_port)
			, &logStream, 1 );
	
	if (lockFileName == "" )
        {
                #ifdef DEBUG
                cerr << "Using default lock file: ";
                #endif
                lockFileName = confMap["pa_def_lock"];
                #ifdef DEBUG
                cerr << lockFileName << endl;
                #endif
        }
	// put the lock file
	if ( putLock(lockFileName) != 0 )
	{
		//for some reasons it was impossible to create the lock
		//threfore the server won't be started.
		hlr_log("Server startup failed: Error creating the lock file.",
                                &logStream, 1);
		exit(atoi(E_LOCK_OPEN));
	}
#ifdef WITH_SOCKET_EXCEPTIONS
try //try #1
{
#endif	
	// instantiate the Socket server
	#ifdef GSI
	GSISocketServer *theServer = new GSISocketServer(server_port);
	#else
	SocketServer *theServer = new SocketServer(server_port);
	#endif
	if ( !(theServer -> Open()) )	
	{
		hlr_log(
			  	"Error opening the server! Exiting.",
				&logStream, 1 );
		removeLock(lockFileName);
		exit(atoi(E_SERVER_START));
	}
	else
	{
		signal (SIGTERM, fatal_error_signal);
		signal (SIGINT, fatal_error_signal);
		while (keep_going)
		{
		#ifdef WITH_SOCKET_EXCEPTIONS
		try//try #2
		{
		#endif
			//thread stuff here!
			#ifdef DEBUG
			cerr << "Starting new Agent" << endl;
			#endif
			#ifdef GSI
			GSISocketAgent *theAgent =  theServer ->Listen();
			#else
			SocketAgent *theAgent =  theServer ->Listen();
			#endif
				string xml_input ="";
				string xml_output ="";
				string logString;
			if (theAgent != NULL)
			{
				#ifdef GSI
				string remCertSubject = theAgent->CertificateSubject();
				#else
				
				string remCertSubject = "Unknown";
				#endif
				connInfo connectionInfo;
				#ifdef DGAS_SOCKETPP
				theAgent -> GetHostName(connectionInfo.hostName);
				#else
				connectionInfo.hostName = theAgent->HostName();
				#endif
				
				connectionInfo.contactString = remCertSubject;
			  	logString = "Connection received from : " +connectionInfo.hostName + "\n";
				#ifdef GSI
				logString += "With cert subject: " + remCertSubject + "\n";
				#endif 
				#ifdef DEBUG	
				cerr << "Begin I/O" <<endl;
				cerr << " receiving message..." << endl;
				#ifdef GSI
				cerr << "remCertSubject = " << remCertSubject << endl;
				#endif
				#endif
				if ( !(theAgent -> Receive( xml_input )))
				{
			  		logString += "Error receiving message from client.";
				}
				else
				{
					#ifdef DEBUG
					cerr << xml_input;
					#endif 
					logString += "message received :\n";
					logString += xml_input + "\n";
					res = parse_xml (xml_input, connectionInfo ,&xml_output);
					if (res == 0)
					{
						#ifdef DEBUG
						cerr << "Sending message..." << endl;
						cerr << xml_output;
						#endif 
						if ( !(theAgent -> Send( xml_output) ))
						{
							logString += "Error sending message";
						}
						logString += "sent answer :\n";
						logString += xml_output + "\n";
					}
				}

				#ifdef DEBUG	
				cerr << "Message sent. Agent end" << endl;
				#endif
				logString += "end of transaction.\n";
				theServer -> KillAgent( (SocketAgent*) theAgent );
				hlr_log (logString, &logStream, 1);
				//end of Agent stuff	
			}
		#ifdef WITH_SOCKET_EXCEPTIONS
	        }//try #2
	        catch( std::exception& e)
		{
		        std::cout << e.what() << std::endl;
		}
	        #endif
		}
					
	}
	
theServer->Close();
delete theServer;
#ifdef WITH_SOCKET_EXCEPTIONS
}//try #1
catch( std::exception& e)
{
	         std::cout << e.what() << std::endl;
}
#endif
hlr_log ("Server closed, removing lock file.", &logStream, 1);
removeLock(lockFileName);
hlr_log ("Exiting.", &logStream, 1);
cerr << "Exit." << endl;
return 0;
}
