// DGAS (DataGrid Accounting System) 
// Server Daeomn and protocol engines.
// 
// $Id: ATM_engine.cpp,v 1.4 2004/01/12 15:53:29 guarise Exp $
// -------------------------------------------------------------------------
// Copyright (c) 2001-2002, The DataGrid project, INFN, 
// All rights reserved. See LICENSE file for details.
// -------------------------------------------------------------------------
// Author: Andrea Guarise <andrea.guarise@to.infn.it>
 /***************************************************************************
 * Code borrowed from:
 *  authors   :
 *  copyright : 
 ***************************************************************************/
//
//    

// ---------------------------------------------------------------------------
//  Includes
// ---------------------------------------------------------------------------
#include "ATM_engine.h"

extern ofstream logStream;

inline int urlSplit(char delim, string url_string , url_type *url_buff)
{
        size_t pos = 0;
        pos = url_string.find_first_of( delim, 0);
        url_buff->hostname=url_string.substr(0,pos);
        url_buff->port=atoi((url_string.substr(pos+1,url_string.size()).c_str()));
				                
        return 0;
}


//retrieve the resource contact string (X509 subject) if the RES_CERT_SUBJECT
//field in the XML contains th CE URL instead of the X509 subject.
//i.e. CE_URL->CE_X509_subject from the resource GRIS
//It is used only if the sensor didn't furnish The CEId directly.
//And should however be updated to retrieve the CEId and not the ContactString.
int
atm_gris_get_res_contact(string res_gris_url_string, string *res_contact)
{
	url_type res_gris_url;
	urlSplit(':',res_gris_url_string, &res_gris_url);
	LDAPSynchConnection connection( "o=Grid",
			res_gris_url.hostname,
			res_gris_url.port,
			30);
	vector<string> attributes;
	attributes.push_back( "GlobusResourceContactString" );
	string filter = "ObjectClass=computingelement" ;
	LDAPQuery query(&connection, filter, attributes);
	LDAPForwardIterator ldap_it( &query );
	if ( connection.Open() )
	{
		if ( ldap_it.rewind())
		{
			do
			{
				for (vector<string>::const_iterator
						it = attributes.begin();
						it != attributes.end();
						it++)
				{
					string value;
					(*ldap_it).EvaluateAttribute( (*it), value);
					#ifdef DEBUG
					 cerr << (*it) << " = " << value << endl;
					#endif
					 if ( (*it) == "GlobusResourceContactString" )
					 {
						 vector<string> output;
						 Split ( ':', value , &output);
						 
						 *res_contact = output[output.size()-1];
						 continue;
					 }
				}
			} while ( ++ldap_it );
		}
		else
		{
			return 1;
		}
	}
	else
	{
		return 1;
	}
	return 0;
	
}
/*
 * This shoul be modified to use CEId end not contact strings!!!
 * */
void
atm_retrieve_res_cert(string *certSubject)
{
	// This function is used only if the Sensor didn't give the 
	// CEId directly.
	//check if certSubject contains a
	//certificate contact string or an url
	size_t pos = 0;
	pos = (certSubject)->find("hostname:");
	if ( pos != string::npos)
	{
		//client gave the hostname
		#ifdef DEBUG
		cerr << "Sensor furnished the CE hostname:" << certSubject << endl;
		#endif
		*certSubject = (certSubject)->substr(pos+9);
		string CE_url_string = *certSubject + ":" + GRIS_PORT;
		string contact_string;
		atm_gris_get_res_contact(CE_url_string, &contact_string );
		*certSubject = contact_string;	
		#ifdef DEBUG
		 cerr << "Found the contact string:" << *certSubject  << endl;
		#endif
	}
	pos = (certSubject)->find("contact:");
	if ( pos != string::npos)
	{
		//client gave the contact string
		//strip the prefix
		#ifdef DEBUG
		 cerr << "Sensor furnished the CE contact_string:" << *certSubject << endl;
		#endif
		*certSubject = (certSubject)->substr(pos+8);
		#ifdef DEBUG
		 cerr << "Found the contact string:" << *certSubject  << endl;
		#endif
	}
	pos = (certSubject)->find("https://");
	if ( pos != string::npos)
	{
		//client probably gave GLOBUS_GRAM_JOB_CONTACT
		#ifdef DEBUG
		 cerr << "Sensor furnished the CE contact_string:" << *certSubject << endl;
		#endif
		*certSubject = (certSubject)->substr(pos+8);
		vector<string> v_buffer;
		Split ( ':', *certSubject, &v_buffer);
		string CE_url_string; 
		CE_url_string = v_buffer[0] + ":" + GRIS_PORT;
		string contact_string;
		atm_gris_get_res_contact(CE_url_string, &contact_string );
		*certSubject = contact_string;	
		#ifdef DEBUG
		 cerr << "Found the contact string:" << *certSubject  << endl;
		#endif
			
	}
	return;	
}

void
atm_retrieve_user_cert(string *certSubject)
{
	//eventually strip "subject= " and proxy info from the
	//user x509 subject
	//
	size_t pos;
	pos = (certSubject)->find("subject= ");
	if ( pos != string::npos)
	{
		//strip the prefix
		#ifdef DEBUG
		 cerr << "Sensor furnished the User x509 subject:" << *certSubject << endl;
		#endif
		*certSubject = (certSubject)->substr(pos+9);
		#ifdef DEBUG
		 cerr << "Now the User x509 subject is:" << *certSubject  << endl;
		#endif
	}
	pos = (certSubject)->find("proxy");
	if ( pos != string::npos)
	{
		string buffer = "";
		vector<string> cert_v_buffer;
		Split('/',*certSubject, &cert_v_buffer);
		vector<string>::const_iterator iter = cert_v_buffer.begin();
		while (iter != cert_v_buffer.end())
		{
			if ((*iter).find("proxy") == string::npos)
			{
				buffer += "/" + *iter;
			}
			++iter;
		}
		*certSubject = buffer;
		
	}
	return;
	
}


int ATM_parse_xml (string &doc, ATM_job_data *job_data, ATM_usage_info *usage_info)
{
	node nodeBuff;
	while ( nodeBuff.status == 0 )
	{
		string tag = "JOB_PAYMENT";
		nodeBuff = parse(&doc, tag);
		if ( nodeBuff.status != 0 )
			break;
		node tagBuff;
		tagBuff = parse(&doc, "DG_JOBID");
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			 cerr << "ATM_engine, got: #" << tagBuff.text << "#" << endl;
			#endif
			job_data->dgJobId = tagBuff.text;
		}
		tagBuff = parse(&doc, "SUBMISSION_TIME");
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			 cerr << "ATM_engine, got: #" << tagBuff.text << "#" << endl;
			#endif
			job_data->time = atoi((tagBuff.text).c_str());
		}
		tagBuff = parse(&doc, "RES_ACCT_PA_ID");
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			 cerr << "ATM_engine, got: #" << tagBuff.text << "#" << endl;
			#endif
			job_data->res_acct_PA_id = tagBuff.text;
		}
		tagBuff = parse(&doc, "RES_ACCT_BANK_ID");
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			 cerr << "ATM_engine, got: #" << tagBuff.text << "#" << endl;
			#endif
			job_data->res_acct_bank_id = tagBuff.text;
		}
		tagBuff = parse(&doc, "USER_CERT_SUBJECT");
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			 cerr << "ATM_engine, got: #" << tagBuff.text << "#" << endl;
			#endif
			job_data->user_CertSubject = tagBuff.text;
		}
		tagBuff = parse(&doc, "RES_GRID_ID");
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			 cerr << "ATM_engine, got: #" << tagBuff.text << "#" << endl;
			#endif
			job_data->res_grid_id = tagBuff.text;
		}
		tagBuff = parse(&doc, "JOB_INFO");
		if ( tagBuff.status == 0 )
		{
			node jobInfoNode;  
			jobInfoNode = parse(&tagBuff.text, "CPU_TIME" );
			if ( jobInfoNode.status == 0)
			{
				#ifdef DEBUG
				 cerr << "ATM_engine, got: #" << jobInfoNode.text << "#" << endl;
				#endif
				usage_info->cpu_time = atoi((jobInfoNode.text).c_str());
			}
			jobInfoNode = parse(&tagBuff.text, "WALL_TIME" );
			if ( jobInfoNode.status == 0)
			{
				#ifdef DEBUG
				 cerr << "ATM_engine, got: #" << jobInfoNode.text << "#" << endl;
				#endif
				usage_info->wall_time = atoi((jobInfoNode.text).c_str());
			}
			jobInfoNode = parse(&tagBuff.text, "MEM" );
			if ( jobInfoNode.status == 0)
			{
				#ifdef DEBUG
				 cerr << "ATM_engine, got: #" << jobInfoNode.text << "#" << endl;
				#endif
				usage_info->mem = atoi((jobInfoNode.text).c_str());
			}
			jobInfoNode = parse(&tagBuff.text, "VMEM" );
			if ( jobInfoNode.status == 0)
			{
				#ifdef DEBUG
				 cerr << "ATM_engine, got: #" << jobInfoNode.text << "#" << endl;
				#endif
				usage_info->vmem = atoi((jobInfoNode.text).c_str());
			}


		}
		nodeBuff.release();
	}
	return 0;
	
}//ATM_parse_xml (string &doc, ATM_job_data *job_data, ATM_usage_info *usage_info)

bool request_is_authorized (ATM_job_data *job_data)
{
	int res;
	jobAuth auth;
	res = auth.get(job_data->dgJobId);
	if ( res != 0 )
	{
		//the job is unknown or there where error in the 
		//process.
		return false;
	}
	else
	{
		//the job is correctly authorized
		//check if we need to retrieve the submission time
		if (job_data->time == 0)
		{
			job_data->time = auth.submTime;
		}
		//check if we need to retrieve the userCert
		if (job_data->user_CertSubject == "")
		{
			job_data->user_CertSubject = auth.usrCert;
		}
		return true;
	}
	//This authorization will be sent to the HLR after 
	//the job submission to the Broker directly by the DATAMAT dg_job_submit
	return false; 
}//request_is_authorized (ATM_job_data &job_data)


int job_cost_computation(int *cost, price &res_price, ATM_usage_info &info)
{
	*cost = (res_price.priceValue)*(info.cpu_time)/100; //demo
	return 0;
}//job_cost_computation(int *cost, price &res_price, ATM_usage_info &info)

int ATM_compose_xml(ATM_job_data &job_data ,string status_msg, string *output)
{
	*output = "<HLR type=\"ATM_answer\">\n<HEAD>\n<VER>1.0</VER>\n</HEAD>\n";
	*output += "<BODY>\n";
	*output += "<JOB_PAYMENT>\n";
	*output += "<DG_JOBID>\n";
	*output += job_data.dgJobId;
	*output += "\n</DG_JOBID>\n";
	*output += "<SUBMISSION_TIME>\n";
	*output += int2string(job_data.time);
	*output += "\n</SUBMISSION_TIME>\n";
	*output += "<RES_ACCT_PA_ID>\n";
	*output += job_data.res_acct_PA_id;
	*output += "\n</RES_ACCT_PA_ID>\n";
	*output += "<RES_ACCT_BANK_ID>\n";
	*output += job_data.res_acct_bank_id;
	*output += "\n</RES_ACCT_BANK_ID>\n";
	*output += "<USER_CERT_SUBJECT>\n";
	*output += job_data.user_CertSubject;
	*output += "\n</USER_CERT_SUBJECT>\n";
	*output += "<RES_GRID_ID>\n";
	*output += job_data.res_grid_id;
	*output += "\n</RES_GRID_ID>\n";
	*output += status_msg;
	*output += "</JOB_PAYMENT>\n";
	*output += "</BODY>\n";
	*output += "</HLR>\n";
	return 0;
}//ATM_compose_xml()

//get the xml object from the daemon, parse the information 
int ATMEngine( string &input, connInfo &connectionInfo, string *output )
{
	hlr_log ("ATM Engine: Entering.", &logStream);
	int job_cost=0;
	bool success = true;
	int code = 0;
	//Get info concerning the job from the incoming request originating 
	//from the sensor
	ATM_job_data job_data;
	ATM_usage_info usage_info;
	if ( ATM_parse_xml(input, &job_data, &usage_info) != 0 )
	{
		//something went wrong parsing the input DGASML
		#ifdef DEBUG
		 cerr << "DEBUG:ATMEngine():"<<__FILE__<<":" <<__LINE__;
	         cerr << ",ERROR parsing the xml request!" << endl;
		#endif	
		 hlr_log ("ATM Engine: Error parsing the XML, reporting error.", &logStream);
		 code = atoi(E_PARSE_ERROR);
		 success = false;
	}
	//now check if the user gave authorization to process the transaction.
	if ( success && !request_is_authorized(&job_data) )
	{
		#ifdef DEBUG
		cerr << "DEBUG:ATMEngine(): Authorization error!" << endl;
		#endif
		 hlr_log ("ATM Engine: Error: the job hasn't been registered.", &logStream);
		 code = atoi(ATM_E_AUTH);
		 success = false;
	}
	else
	{
		string logBuff ="ATM Engine: edg_jobId: " + job_data.dgJobId + "is authorized for payment by user: " + job_data.user_CertSubject;
		hlr_log ( logBuff, &logStream);
		//check for duplicated transaction
		hlrTransaction t;
		t.id = job_data.dgJobId;
		t.tType = out; //we are on the "from" side of the transaction.
		if ( t.exists() )
		{
			#ifdef DEBUG
			 cerr << "DEBUG:ATMEngine(): Job duplicated" << endl;
			#endif
			 hlr_log ("ATM Engine: Error: the transaction is already present.", &logStream);
			 code = atoi(ATM_E_DUPLICATED);
			 success = false;
		}
		else
		{
			//retrieve missing information
			atm_retrieve_res_cert(&job_data.res_grid_id);//useful only if the client (sensor) doesn't give the CEId directly.
			atm_retrieve_user_cert(&job_data.user_CertSubject);
			//FIXME migrate to new PA class
			price res_price;
			res_price.resID = job_data.res_grid_id;
			res_price.time = job_data.time;
			if ( dgas_pa_client(job_data.res_acct_PA_id,&res_price) != 0 )
			{
				#ifdef DEBUG
		        	 cerr << "DEBUG:ATMEngine(): Error retrieving the resource price!" <<endl;
			        #endif
				 hlr_log ("ATM Engine: Error retrieving the resource price!", &logStream);
				 code = atoi(ATM_E_PRICE);
				 success=false;
			}
			else
			{
				hlr_log ("ATM Engine: Computing job cost.", &logStream);
				//compute the job cost
				if ( job_cost_computation(&job_cost, res_price, usage_info) != 0 )
				{
					#ifdef DEBUG
			        	 cerr << "ATM_engine: Error computing the job cost!" <<endl;
			        	#endif
					hlr_log ("ATM Engine: Error computing job cost", &logStream);
					code = atoi(ATM_E_COST);
			 		success = false;
				}
				else
				{
					//put the transactuion in the queue;
					string urBuff;
					urBuff = "WALL_TIME=" + int2string(usage_info.wall_time);
					urBuff += ",CPU_TIME=" + int2string(usage_info.cpu_time);
					urBuff += ",CE_ID=" + res_price.resID; 
					urBuff += ",CE_PRICE_TIME=" + int2string(res_price.time); 
					urBuff += ",MEM=" + int2string(usage_info.mem); 
					urBuff += ",VMEM=" + int2string(usage_info.vmem); 
					qTransaction qt( t.id,
							job_data.user_CertSubject,
							job_data.res_grid_id,
							"local",
							job_data.res_acct_bank_id,
							job_cost,
							int2string(time(NULL)),
							urBuff,
							0,
							time(NULL),
							"out"
						       );
					if ( qt.put() != 0 )
					{
						#ifdef DEBUG
						cerr << "DEBUG:ATMEngine(): Error queuing the transaction!"<< endl;
						#endif
						success = false;
						code = atoi(ATM_E_TRANS);
					}

				}
			}
			
		}
	}
	string message;
        if (success)
        {
	 message = "<PAYMENT_INFO>\n";
	 message += "<COST>\n";
	 message += int2string(job_cost);
	 message += "\n</COST>\n";
	 message += "<STATUS>\n";
	 message += "OK";
	 message += "\n</STATUS>\n";
	 message += "<CODE>\n";
	 message += "0";
	 message += "\n</CODE>\n";
	 message += "</PAYMENT_INFO>\n";
        }
	else
	{
	 message = "<PAYMENT_INFO>\n";
	 message += "<STATUS>\n";
	 message += "FAILED";
	 message += "\n</STATUS>\n";
	 message += "<CODE>\n";
	 message += int2string(code);
	 message += "\n</CODE>\n";
	 message += "</PAYMENT_INFO>\n";
	}
        if ( ATM_compose_xml(job_data, message, output) != 0 )
        {
		#ifdef DEBUG
	         cerr << "ATM_engine: Error composing the XML answer!" <<endl;
	        #endif
        }
	hlr_log ("ATM Engine: Exiting.", &logStream);	
	return code;
}//ATMEngine( string doc, connInfo &connectionInfo, string *output )


