// DGAS (DataGrid Accounting System) 
// Server Daeomn and protocol engines.
// 
// $Id: bank_engine.cpp,v 1.3 2003/12/09 14:46:43 guarise Exp $
// -------------------------------------------------------------------------
// Copyright (c) 2001-2002, The DataGrid project, INFN, 
// All rights reserved. See LICENSE file for details.
// -------------------------------------------------------------------------
// Author: Andrea Guarise <andrea.guarise@to.infn.it>
 /***************************************************************************
 * Code borrowed from:
 *  authors   :
 *  copyright : 
 ***************************************************************************/
//
//    

// ---------------------------------------------------------------------------
//  Includes
// ---------------------------------------------------------------------------
#include <bank_engine.h>

extern int server_port;
extern string server_contact;

extern ofstream logStream;

int bankEngine( string &input, connInfo &connectionInfo, string *output )
{
	hlr_log("Bank Engine: Entering.", &logStream);
	string stringLogBuff = "Bank Engine: Serving: " + connectionInfo.hostName +
		"(" + connectionInfo.contactString + ")";
	hlr_log( stringLogBuff, &logStream);
	int returnCode = 0;
 	hlrTransaction t;	 
	node nodeBuff;
	while ( nodeBuff.status == 0 )
	{
		//parse the input XML to retrieve the needed INFO
		string tag = "TR_INFO";
		nodeBuff = parse (&input, tag);
		if ( nodeBuff.status != 0 )
			break;
		node tagBuff;
		tagBuff = parse (&input, "TO_GRID_ID");
		if ( tagBuff.status == 0)
		{
			#ifdef DEBUG
				cerr<< "DEBUG:Bank_engine, found:" << tagBuff.text << endl;
			#endif
			t.toId = tagBuff.text;
		}
		tagBuff = parse (&input, "ID");
		if ( tagBuff.status == 0)
		{
			#ifdef DEBUG
				cerr << "DEBUG:Bank_engine, found:" << tagBuff.text << endl;
			#endif
			t.id = tagBuff.text;
		}
		tagBuff = parse (&input, "FROM_GRID_ID");
		if ( tagBuff.status == 0)
		{
			#ifdef DEBUG
				cerr << "DEBUG:Bank_engine, found:" << tagBuff.text << endl;
			#endif
			t.fromId = tagBuff.text;
		}
		tagBuff = parse (&input, "AMOUNT");
		if ( tagBuff.status == 0)
		{
			#ifdef DEBUG
				cerr << "DEBUG:Bank_engine, found:" << tagBuff.text << endl;
			#endif
			t.amount = atoi((tagBuff.text).c_str());
		}
		tagBuff = parse (&input, "LOGDATA");
		if ( tagBuff.status == 0)
		{
			#ifdef DEBUG
				cerr << "DEBUG:Bank_engine, found:" << tagBuff.text << endl;
			#endif
			t.logData = tagBuff.text;
		}
		tagBuff = parse (&input, "FROM_HLR_URL");
		if ( tagBuff.status == 0)
		{
			#ifdef DEBUG
				cerr << "DEBUG:Bank_engine, found:" << tagBuff.text << endl;
			#endif
			t.fromHlrUrl = tagBuff.text;
		}
		//Now set the timestamp of the transaction
		t.timeStamp=int2string(time(NULL));
		//Now use connInfo to fill missing records in t
		//if fromHlrUrl is missing fill it
		if ( t.fromHlrUrl == "" ) 
		{
			t.fromHlrUrl = connectionInfo.hostName +
				":" + int2string(connectionInfo.port) +
				":" + connectionInfo.contactString;
		}
		//if toHlrUrl is missing fill it, since we are
		//on the server side and the amount is incoming
		//toHlrUrl is our bankID.
		if ( t.toHlrUrl == "" )
		{
			t.toHlrUrl == "localhost.localdomain:" +
				int2string(server_port) + ":" + server_contact;
		}
		//since this is an incoming transaction we must specify
		//it in the t object
		t.tType = in;
		//Now we can release the node already parsed and go on
		nodeBuff.release();
	}
	// Once we have those information we can compose the XML answer and put
	// it into the string output.
	

		
	
	//Check if the job dg_jobid has been already credited, if so
	//return an error.
	if ( t.exists() )
	{
		stringLogBuff="Bank Engine: Error: transaction for job: "+
			t.id + "has been processed already";
	        hlr_log( stringLogBuff, &logStream );	
		*output = "<HLR type=\"HLR_error\">\n<HEAD>\n<VER>1.0</VER>\n</HEAD>\n<BODY>\n";
		*output += "<TR_INFO>\n";
		*output += "<TO_GRID_ID>";
		*output += t.toId;
		*output += "</TO_GRID_ID>\n";
		*output += "<ID>";
		*output += t.id;
		*output += "</ID>";
		*output += "<FROM_GRID_ID>";
		*output += t.fromId;
		*output += "</FROM_GRID_ID>";
		*output += "<AMOUNT>";
		*output += int2string(t.amount);
		*output += "<AMOUNT>";
		*output += "</TR_INFO>\n";
		*output += "<STATUS>\n";
		*output += CREDIT_DUPL;
		*output += "\n</STATUS>\n";
		returnCode = atoi(CREDIT_DUPL);
	}
	else
	{
		// the tranaction hasn't been credited already.
		// so we can credit the local account and
		// send a succesfull receipt back.
		if ( t.process() != 0 )
		{
			stringLogBuff="Bank Engine: Error: transaction for job: "+
				t.id + "hasn't been processed, account not being credited.";
	        	hlr_log( stringLogBuff, &logStream );	
			#ifdef DEBUG
			cerr << "DEBUG: Error crediting the local account!" << endl;
			#endif
			*output = "<HLR type =\"HLR_error\">\n<HEAD>\n<VER>1.0</VER>\n</HEAD>\n";
			*output += "<BODY>\n";
			*output += "<TR_INFO>\n";
			*output += "<STATUS>\n";
			*output += CREDIT_ERROR;
			*output += "\n</STATUS>\n";
			*output += "</TR_INFO>\n";
			returnCode = atoi(CREDIT_ERROR);
		}
		else
		{
			*output = "<HLR type =\"bank_receipt\">\n<HEAD>\n<VER>1.0</VER>\n</HEAD>\n";
			*output += "<BODY>\n";
			*output += "<TR_INFO>\n";
			*output += "<TO_GRID_ID>\n";
			*output += t.toId;
			*output += "\n</TO_GRID_ID>\n";
			*output += "<ID>\n";
			*output += t.id;
			*output += "\n</ID>\n";
			*output += "<FROM_GRID_ID>\n";
			*output += t.fromId;
			*output += "\n</FROM_GRID_ID>\n";
			*output += "<AMOUNT>\n";
			*output += int2string(t.amount);
			*output += "\n</AMOUNT>\n";
			*output += "<STATUS>\n";
			*output += "0";
			*output += "\n</STATUS>\n";
			*output += "</TR_INFO>\n";
			//log the information.
			string log = "SENDER=";
			log += connectionInfo.hostName;
			log +="," + t.logData;
			
			transInLog logBuff(t.id, log);
			logBuff.put();

			
		}
	}
		
	*output +="</BODY>\n</HLR>\n";
	// end of the job credit.
	
	#ifdef DEBUG
		cerr << "DEBUG:Formatted xml message:" << endl;
		cerr << *output << endl;
	#endif
	hlr_log("BankEngine: Exiting.",&logStream);
	return returnCode;
}


