// DGAS (DataGrid Accounting System) 
// Server Daeomn and protocol engines.
// 
// $Id: pa_engine.cpp,v 1.4 2003/12/02 13:50:09 guarise Exp $
// -------------------------------------------------------------------------
// Copyright (c) 2001-2002, The DataGrid project, INFN, 
// All rights reserved. See LICENSE file for details.
// -------------------------------------------------------------------------
// Author: Andrea Guarise <andrea.guarise@to.infn.it>
//         Rosario M. Piro <piro@to.infn.it>
 /***************************************************************************
 * Code borrowed from:
 *  authors   :
 *  copyright : 
 ***************************************************************************/
//
//    

// ---------------------------------------------------------------------------
//  Includes
// ---------------------------------------------------------------------------
#include "pa_engine.hpp"

#define GRIS_PORT "2135"

extern const char * pa_sql_server;
extern const char * pa_sql_user;
extern const char * pa_sql_password;
extern const char * pa_sql_dbname;
extern const char * pa_pricing_scheme;
extern const char * pa_def_price_ttl;

extern ofstream logStream;

#define DL_PRICELIB_DYN_NAME "libedg-wl-dgas-pAlgDyn.so" // name of lib with dynamic
                                                  // pricing alg.
#define DL_PRICELIB_MAN_NAME "libedg-wl-dgas-pAlgMan.so" // library with manual
                                                  // pricing alg.

inline string &composeErrMess(string &message, string resID, string errnum) {
  message += "<PRICE_INFO>\n";
  message += "<RES_ID>\n";
  message += resID;
  message += "\n</RES_ID>\n";
  message += "<ERROR>\n";
  message += errnum;
  message += "\n</ERROR>\n";
  message += "</PRICE_INFO>\n";
  return message;
}



inline int get_last_price(string resID, paPrice *priceBuff)
{
	time_t currTime;
	time (&currTime);
	priceBuff->resID = resID;
	priceBuff->time = currTime;
	return priceBuff->get(); 
}


bool needs_historical (paPrice reqPrice)
{
	#ifdef DEBUG
	 cerr << "needs_historical." << endl;
	#endif
	paPrice lastPrice;
	if ( get_last_price(reqPrice.resID, &lastPrice) != 0 )
	{
		hlr_log("PA Engine: needs_historical(), Can't find a valid price, needs new one", &logStream);
		return false;
	}
	if ( reqPrice.time < lastPrice.time  )
	{
		hlr_log("PA Engine: needs_historical(), reqPrice.time < lastPrice.time: Old price needed.", &logStream);
		//old price, needs historical
		return true;
	}
	else
	{
		//new price
		 hlr_log("PA Engine: needs_historical(), A new price is needed", &logStream);
		if ( (reqPrice.time-lastPrice.time) < lastPrice.minTTL )
		{
			#ifdef DEBUG
		 	cerr << "case B: New price, but last price not expired" << endl;
			#endif
			// last price hasn't expired -> needs historical
			return true;
		}
		else
		{
			#ifdef DEBUG
		 	cerr << "case C: New price, last price expired" << endl;
			#endif
			// last price has expired ->needs new price
			return false;
		}
	}
}//needs_historical (paPrice reqPrice)


int mds_res_get_info (string resID, string *queryKeys, int num_keys, vector<string> &priceInfo)
{
	//get GRIS URL from certificate
	string command;
	string gris_hostname;
	string gris_port = GRIS_PORT;
	int retCode = 0;
	size_t pos_start = resID.find("/CN=");
	size_t pos_end;
	if ( pos_start == string::npos )
	{
		#ifdef DEBUG
		cerr << "Can't parse the certificate subject to retrieve Hostname" << endl;
		#endif
		retCode = 1;
	}
	else
	{
		pos_end = resID.find_first_of("/",pos_start+1);
		if ( pos_end == string::npos )
		{
			#ifdef DEBUG
			cerr << "Can't parse the certificate subject to retrieve Hostname" << endl;
			#endif
			retCode = 1;
		}
		else
		{
			gris_hostname = resID.substr(pos_start+4,pos_end-pos_start-4);
		}
	}
	if ( retCode == 1 )
	{
		#ifdef DEBUG 
		 cerr << "May be it's CEid" << endl;
		#endif
		pos_start = 0;
	        pos_end = resID.find_first_of(":",pos_start+1);	
		if ( pos_end == string::npos )
		{
			#ifdef DEBUG
			 cerr << "Can't parse the ID to retrieve Hostname" << endl;
			#endif
			 retCode = 2;
		}
		else
		{
			gris_hostname = resID.substr(0, pos_end-pos_start );
		}
		
		 
	}
	#ifdef DEBUG
	cerr << "host_name: " << gris_hostname << endl;
        cerr << "GRIS port: " << gris_port << endl;	
	#endif
	LDAPSynchConnection connection( "mds-vo-name=local,o=Grid", 
			gris_hostname, 
			atoi(gris_port.c_str()), 
			30);	
	vector<string> attributes;
	for(int count = 0; count < num_keys; count++) {
	  attributes.push_back(queryKeys[count]); // copy query keys
	  priceInfo.push_back("0"); // initialize server return values
	}

	string filter = "(&(CEId="+resID+")(objectClass=computingelement))";
	#ifdef DEBUG
	cerr << "filter for LDAPsearch: " << filter << endl;
	cerr << "attributes:";
	for (int count=0; count<attributes.size(); count++)
	  cerr << " " << attributes[count];
	cerr << endl;
	#endif

	LDAPQuery query(&connection, filter, attributes);
	LDAPForwardIterator ldap_it( &query );

	if ( connection.Open() )
	{
		if ( ldap_it.rewind())
		{
		  do {
		    vector<string>::iterator res_it =
		      priceInfo.begin();
		    vector<string>::const_iterator it =
		      attributes.begin();
		    for ( ; it != attributes.end(); it++, res_it++)
		      {
			string value;
			(*ldap_it).EvaluateAttribute( (*it), value);
                        #ifdef DEBUG
			cerr << (*it) << " = " << value << endl;
                        #endif
			//populating the info structure
			(*res_it) = value;
			// the info structure has the same
			// order as the query keys!
		      }
		  } while (++ldap_it);
		}
		else
		{
			return atoi(PA_ERR_MDS_INFO);
		}
	}
	else
	{
		return atoi(PA_ERR_MDS_INFO);
	}

	return 0;
	
}
		

int getPrice(vector<paPrice>::const_iterator priceIt, int &price_value) {
  //get the resource price

  string priceLib_name; // name of library to link dynamically
  void *priceLib_handle; // handle for linking the price library dynamically
  int (*getResPrice)(void*,string&); // pointer to funct. in dyn. lib
  int *num_keys;
  string *queryKeys;
  char *symerror;
  vector<string> priceInfo; // when querying the mds server
  map<string,string> dbInfo; // when looking up the price in the PA database
  string price_string;
  void *p_lib_helper; // this is what we pass to the dynamically linked pricing
                      // library. It may contain different information needed
                      // needed by a specific pricing lib. and will be casted
                      // there ...
  int Error=0; /* no error */

  // first determine which library to link dynamically
  if (!strcmp(pa_pricing_scheme, "manual")) { 
    priceLib_name = DL_PRICELIB_MAN_NAME;
    #ifdef DEBUG
    cerr << "DEBUG: in getPrice: name of library to link dynamically = '"
	 << priceLib_name << "'" << endl << flush; 
    #endif
    // populate dbInfo, that will be passed to the dyn. linked lib.
    dbInfo["pa_sql_server"] = pa_sql_server;
    dbInfo["pa_sql_user"] = pa_sql_user;
    dbInfo["pa_sql_password"] = pa_sql_password;
    dbInfo["pa_sql_dbname"] = pa_sql_dbname;
    dbInfo["record_resID"] = (*priceIt).resID;

    p_lib_helper = &dbInfo; // information to pass to pricing algorithm
  }
  else if (!strcmp(pa_pricing_scheme, "dynamic")) {
    priceLib_name = DL_PRICELIB_DYN_NAME;
    #ifdef DEBUG
    cerr << "DEBUG: in getPrice: name of library to link dynamically = '"
	 << priceLib_name << "'" << endl << flush; 
    #endif
  }
  else { // no library specified in configuration file, use default!
    priceLib_name = DL_PRICELIB_DYN_NAME;    
    #ifdef DEBUG
    cerr << "DEBUG: in getPrice: no library specified in config. file"
	 << ", default:" << endl
	 << "   name of library to link dynamically = '" << priceLib_name
	 << "'" << endl << flush; 
    #endif
  }

  // link dynamic library!
  priceLib_handle = dlopen (priceLib_name.c_str(), RTLD_NOW);
  if(!priceLib_handle) { // couldn't open library
    #ifdef DEBUG
    cerr << "Error in file " << __FILE__ << ", line" << __LINE__ << ": "
	 << dlerror() << endl;
    #endif
    return atoi(PA_ERR_DL_OPEN);
  }
  #ifdef DEBUG
  else
    cerr << "Library " << priceLib_name.c_str() << " dynamically linked!"
	 << endl;
  #endif


  // get number of query keys
  num_keys = (int*) dlsym (priceLib_handle, "num_keys");
  if ( (symerror=dlerror()) !=NULL ){ // couldn't find symbol num_keys
    #ifdef DEBUG
    cerr << "Error in file " << __FILE__ << ", line " << __LINE__ << ": "
	 << symerror << endl;
    #endif
    return atoi(PA_ERR_DL_SYMBOL);
  }

  // get query keys from price lib
  if(*num_keys) { // if num_keys > 0 !!!
    queryKeys = (string *) dlsym (priceLib_handle, "q_keys");
    if ( (symerror=dlerror()) !=NULL ){ // couldn't find symbol q_keys
      #ifdef DEBUG
      cerr << "Error in file " << __FILE__ << ", line " << __LINE__ << ": "
	   << symerror << endl;
      #endif
      return atoi(PA_ERR_DL_SYMBOL);
    }
  }
  else
    queryKeys = NULL; // no query keys


  // find price algorithm in dyn. lib.
  getResPrice =
    (int(*)(void*,string&)) dlsym (priceLib_handle, "getResPrice");

  if ( (symerror=dlerror()) !=NULL ){ // couldn't find symbol getResPrice
    #ifdef DEBUG
    cerr << "Error in file " << __FILE__ << ", line " << __LINE__ << ": "
	 << symerror << endl;
    #endif
    return atoi(PA_ERR_DL_SYMBOL);
  }


  // the following is done only, if we have to query the GRIS
  if(*num_keys) { // if num_keys > 0 !!!
  
    // we found all symbols, now query the server!
    Error=mds_res_get_info ((*priceIt).resID, queryKeys, *num_keys, priceInfo);
    p_lib_helper = &priceInfo; // we'll pass the price info to the dyn. linked
                               // library, the pointer will be casted there!
  }


  if (!Error) { // no error occured in mds_res_get_info()
    // we now can compute the price.
    // the returned strings in priceInfo have the same order as the query 
    // keys defined in the dynamically linked library!

    //compute the price
    // use algorithm in dyn. lib.
    getResPrice(p_lib_helper, price_string);
    // convert price to integer
    price_value=atoi(price_string.c_str());

  } // if(!queryError) ...


  // whether queryError is 0 (no error) or not (error in mds_res_get_info())
  // we have to close the dynamic library!
  queryKeys = NULL;
  num_keys = NULL;
  getResPrice = NULL;
  if (dlclose(priceLib_handle)) {
    #ifdef DEBUG
    cerr<<"Error in file "<<__FILE__<<", line "<<__LINE__
	<<": couldn't close dynamically linked price library!" << endl;
    #endif
  }
  #ifdef DEBUG
  else
    cerr << "Dynamically linked library " << priceLib_name.c_str()
	 << " was closed!" << endl;
  #endif


  return Error; // 0 if no error occured in mds_res_get_info()

} // getPrice ...

		

int pa_query ( string &doc, connInfo &connectionInfo, string *output )
{
	hlr_log ("PA Engine: Entering.", &logStream);
	string logBuff = "PA Engine: Serving: " + connectionInfo.hostName +
		                "(" + connectionInfo.contactString + ")";
	hlr_log( logBuff, &logStream );
	int res;
	int returnCode;
	string buffer;
	if ( true )
	{
		vector<paPrice> prices;	
		node nodeBuff;
		while ( nodeBuff.status == 0 )
		{
			string tag = "PRICE_INFO";
			nodeBuff = parse(&doc, tag);
			if ( nodeBuff.status != 0)
				break;
			paPrice priceBuff;
			node tagBuff;
			tagBuff = parse(&doc, "RES_ID");
			if ( tagBuff.status == 0 )
			{
				#ifdef DEBUG
					cerr << tagBuff.text << endl;
				#endif
					priceBuff.resID = tagBuff.text;
			}
			tagBuff = parse(&doc, "TIME");
			if ( tagBuff.status == 0 )
			{
				#ifdef DEBUG
					cerr << tagBuff.text << endl;
				#endif
					priceBuff.time = atoi((tagBuff.text).c_str());
			}
			if ( nodeBuff.status == 0)
			{
				prices.push_back(priceBuff);
				nodeBuff.release();
			}
		}
		#ifdef DEBUG
		cerr << "DEBUG: stored n: " << prices.size()
		     << " price requests" << endl;
		#endif
		//now we have a vector<price> prices that contains the 
		//elements needed for the query, 
		//for every price request we have to check if there is 
		//unexpired price in the history and return it. 
		//Otherwise we calculate a new price, put it into the 
		//history and return it to the caller.
		string message;
		vector<paPrice>::const_iterator priceIt = prices.begin();
		vector<paPrice> priceAnswers;
		while ( priceIt != prices.end() )
		{
			//check if the resource exists in the database
			paResource resBuffer((*priceIt).resID);
			if ( resBuffer.exists() ) //the resource exists in the pa database
			{
				//the resource is already present
				logBuff = "PA Engine: Resource: " + 
					resBuffer.resID + 
					" found in the Database";
				hlr_log( logBuff, &logStream);
				
				//check if we need an hisorical price
				logBuff = "PA Engine: checking for resource: " + 
					(*priceIt).resID + 
					" at time: " + int2string((*priceIt).time);
				hlr_log( logBuff, &logStream);
				paPrice priceBuff((*priceIt).resID, (*priceIt).time);
				if (needs_historical(priceBuff))
				{
					//needs historical
					// get the price from the db
					hlr_log( "PA Engine: trying to get the price from the DB", &logStream);
					if ( priceBuff.get() != 0)
					{
						hlr_log( "PA Engine: Couldn't find price in the DB, notifing error to client.", &logStream);
					        //compose an error
					        message = composeErrMess(message, priceBuff.resID, PA_ERR_NO_PRICE);
						returnCode = atoi(PA_ERR_NO_PRICE);
					}
					else
					{
						logBuff = "PA Engine: found price: resID=" + priceBuff.resID + ",price=" + int2string(priceBuff.priceValue) + ",startTime=" + int2string(priceBuff.time) + ",preiceTTL=" + int2string(priceBuff.minTTL);
						hlr_log( logBuff, &logStream);
						//everything ok, put price in the answer vector
						priceAnswers.push_back(priceBuff);
					}
				}
				else
				{
					//get res info from the IS
					
					hlr_log( "PA Engine: computing a new price", &logStream);

				        int price_value;
 				        res = getPrice(priceIt, price_value);

					// library errors or no res info
					if (res != 0)
					{	// some error!
					  hlr_log("PA Engine: Error computing the new price", &logStream);
					  message = composeErrMess(message, priceBuff.resID, int2string(res));
					  returnCode = res;
					}

					else
					{
						// compose price buffer
						paResource resBuff;
						resBuff.get((*priceIt).resID);

						priceBuff.resID = (*priceIt).resID;
						priceBuff.time = (*priceIt).time;
						priceBuff.priceValue = price_value;
						priceBuff.minTTL= resBuff.defMinTTL;
						
						
						//add price to the database
						if ( priceBuff.put() != 0)
						{
						  message = composeErrMess(message, priceBuff.resID, PA_ERR_PUT_PRICE);
						  returnCode = atoi(PA_ERR_PUT_PRICE);
						}
						else
						{
						  priceAnswers.push_back(priceBuff);
						}
					}	
				}
					
			}
			else
			{
				//the resource doesn't exists -> initialize
				//the resource record, then compute the price
				//and add it to the database.

			        // verify that the TTL taken from the config
			        // file, isn't below the minimum value
			        int minPriceTTL = atoi(pa_def_price_ttl);

				paResource resBuff((*priceIt).resID,minPriceTTL);
				if ( resBuff.put() != 0)
				{
				  message = composeErrMess(message, (*priceIt).resID, PA_ERR_RES_INIT);
				  returnCode = atoi(PA_ERR_RES_INIT);
				}
				else
				{
				        int price_value;
					res = getPrice(priceIt, price_value);

					// library errors or no res info
					if (res != 0) // some error occured
					{
					  message = composeErrMess(message, (*priceIt).resID, int2string(res));
					  returnCode = res;
					}
					else
					{
						// compose price buffer
						paPrice priceBuff((*priceIt).resID, (*priceIt).time, price_value);
						priceBuff.minTTL = resBuff.defMinTTL;
						//add price to the database
						if ( priceBuff.put() != 0)
						{
						  message = composeErrMess(message, priceBuff.resID, PA_ERR_PUT_PRICE);
						  returnCode = atoi(PA_ERR_PUT_PRICE);
						}
						else
						{
						  priceAnswers.push_back(priceBuff);
						}
							
					}
							
				}
			}
			priceIt++;
		}// end of while iterator over price_query
	
		//Now we have a vector price_answer containing all
		//the prices that were correctly computed and a string:
		//message that contains the error messages for those
		//resource whose price couldn't be computed.
	       	
		//initialize the answer message;
		*output = "<HLR type=\"PA_answer\">\n";
		*output += "<HEAD>\n";
		*output += "<VER>1.0</VER>\n";
		*output += "</HEAD>\n";
		*output += "<BODY>\n";
		*output += message;
		//iterate over the vector and compose the answers
		vector<paPrice>::const_iterator answersIt = priceAnswers.begin();
		while ( answersIt != priceAnswers.end() )
		{
			*output += "<PRICE_INFO>\n";
			*output += "<RES_ID>\n";
			*output += (*answersIt).resID;
			*output += "\n</RES_ID>\n";
			*output += "<TIME>\n";
			*output += int2string((*answersIt).time);
			*output += "\n</TIME>\n";
			*output += "<PRICE>\n";
			*output += int2string((*answersIt).priceValue);
			*output += "\n</PRICE>\n";
			*output += "<MIN_TTL>\n";
			*output += int2string((*answersIt).minTTL);
			*output += "\n</MIN_TTL>\n";
			*output += "</PRICE_INFO>\n";
			answersIt++;
		}
		//close the answer 
		*output += "</BODY>\n</HLR>\n";
	}
	#ifdef DEBUG
	 cerr << "Formatted xml message:" << endl;
	 cerr << *output << endl;
	#endif
	 hlr_log( "PA Engine: Exiting" , &logStream );
	return returnCode;
	 
}
