// DGAS (DataGrid Accounting System) 
// Server Daemon and protocol engines.
// 
// $Id: ui_engine.cpp,v 1.2 2003/04/28 14:44:49 guarise Exp $
// -------------------------------------------------------------------------
// Copyright (c) 2001-2002, The DataGrid project, INFN, 
// All rights reserved. See LICENSE file for details.
// -------------------------------------------------------------------------
// Authors: Andrea Guarise <andrea.guarise@to.infn.it>
//          Stefano Barale <stefano.barale@to.infn.it>
 /***************************************************************************
 * Code borrowed from:
 *  authors   :
 *  copyright : 
 ***************************************************************************/
//
//    

// ---------------------------------------------------------------------------
//  Includes
// ---------------------------------------------------------------------------
#include "ui_engine.h"


void uiEngine( string &doc, connInfo &connectionInfo, string *output )
{
	bool userFlag = false;
	bool resourceFlag = false;
	bool groupFlag = false;
	bool fundFlag = false;
	bool trsFlag = false;
	#ifdef DEBUG
		cerr << "DEBUG: ENTERING  uiEngine()" << endl;
	#endif
	// query to find all the nodes corresponding to a user query.
	vector<hlrUser> uv;
	node nodeBuff;
	while ( nodeBuff.status == 0 )
	{
		string tag = "USER_INFO";
		nodeBuff = parse(&doc, tag);
		if ( nodeBuff.status != 0 )
			break;
		userFlag = true;
		hlrUser u;
		node tagBuff;
		tagBuff = parse ( &nodeBuff.text, "EMAIL");
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
				cerr << "DEBUG:uiEngine(),got :"<<  tagBuff.text << endl;
			#endif
			u.email = tagBuff.text;
		}
		tagBuff = parse ( &nodeBuff.text, "CERT_H");
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
				cerr << "DEBUG:uiEngine(),got :"<<  tagBuff.text << endl;
			#endif
			u.certSubject = tagBuff.text;
		}
		tagBuff = parse ( &nodeBuff.text, "GID");
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
				cerr << "DEBUG:uiEngine(),got :"<<  tagBuff.text << endl;
			#endif
			u.gid = tagBuff.text;
		}
		tagBuff = parse ( &nodeBuff.text, "FID");
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
				cerr << "DEBUG:uiEngine(),got :"<<  tagBuff.text << endl;
			#endif
			u.fid = tagBuff.text;
		}
		nodeBuff.release();
		uv.push_back(u);
	}
	nodeBuff.status = 0;
	#ifdef DEBUG
	cerr << "DEBUG:uiEngine(),stored " << uv.size() << " users query" <<endl;
	#endif
	// now we have a vector<user_info> users that contains the elements
	// needed for the query. 
	// Every element in the vector has to be passed to the HLR API, that
	// will return the information about every user.
	// Once we have those information we can compose the XML answer and put
	// it into the string output.
	
	//initialize buffer string
	
	//user_info vector iterator
	if ( userFlag )
	{
		*output = "<HLR type=\"UI_reply\">\n<HEAD>\n<VER>1.0</VER>\n</HEAD>\n<BODY>\n";
		vector < hlrUser >::iterator uv_it = uv.begin();
		while ( uv_it != uv.end() )
		{
			if ( (*uv_it).get() != 0)
			{
				//trow an error!
				#ifdef DEBUG
				cerr << "error retrieving user information!" << endl;
				#endif
				*output = "<HLR type=\"HLR_error\">\n<HEAD>\n<VER>1.0</VER>\n</HEAD>\n<BODY>\n";
				*output +="<ERROR>";
				*output +=INFO_NOT_FOUND;
				*output +="</ERROR>\n";
			} 
			else
			{
				*output += "<USER_INFO>\n";
				*output += "<HLR_UID>" + (*uv_it).uid + "</HLR_UID>\n";
				*output += "<EMAIL>" + (*uv_it).email + "</EMAIL>\n";
				*output += "<DESC>" + (*uv_it).descr + "</DESC>\n";
				*output += "<CERT_H>" + (*uv_it).certSubject + "</CERT_H>\n";
				*output += "<GID>" + (*uv_it).gid + "</GID>\n";
				*output += "<FID>" + (*uv_it).fid + "</FID>\n";
				*output += "<FUND_ASS>"; 
				*output	+= int2string((*uv_it).assigned);
				*output += "\n</FUND_ASS>\n";
				*output += "<FUND_BKD>"; 
				*output += int2string((*uv_it).booked); 
				*output += "\n</FUND_BKD>\n";
				*output += "<FUND_SPT>"; 
       			 	*output += int2string((*uv_it).spent);
		       		*output += "\n</FUND_SPT>\n";
				*output += "</USER_INFO>\n";
			}
			uv_it++;
		
		}
		*output +="</BODY>\n</HLR>\n";
	}
	// end of the user query.
	
	//
	// query to find all the nodes corresponding to a resource query.
	vector<hlrResource> rv;
	while ( nodeBuff.status == 0 )
	{
		string tag = "RESOURCE_INFO";
		nodeBuff = parse (&doc , tag);
		if ( nodeBuff.status != 0 )
			break;
		hlrResource r;
		resourceFlag = true;
		node tagBuff;
		tagBuff = parse ( &nodeBuff.text, "RID");
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
				cerr << "DEBUG:uiEngine(),got :"<<  tagBuff.text << endl;
			#endif
			r.rid = tagBuff.text;
		}
		tagBuff = parse ( &nodeBuff.text, "CERT_H");
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
				cerr << "DEBUG:uiEngine(),got :"<<  tagBuff.text << endl;
			#endif
			r.certSubject = tagBuff.text;
		}
		nodeBuff.release();
		rv.push_back(r);
		
	}
	nodeBuff.status = 0;
	#ifdef DEBUG
	cerr << "stored " << rv.size() << " resources query" <<endl;
	#endif
	// now we have a vector<hlrResource> rv that contains the elements
	// needed for the query. 
	// Every element in the vector has to be passed to the HLR API, that
	// will return the information about every resource.
	// Once we have those information we can compose the XML answer and put
	// it into the string output.
	
	//initialize buffer string
	if ( resourceFlag)
	{
	
		*output = "<HLR type=\"UI_reply\">\n<HEAD>\n<VER>1.0</VER>\n</HEAD>\n<BODY>\n";
		//resource_info vector iterator
		vector < hlrResource >::iterator rv_it = rv.begin();
		while ( rv_it != rv.end() )
		{
			if ( (*rv_it).get() != 0)
			{
			//trow an error!
			#ifdef DEBUG
			cerr << "error retrieving user information!" << endl;
			#endif
			*output = "<HLR type=\"HLR_error\">\n<HEAD>\n<VER>1.0</VER>\n</HEAD>\n<BODY>\n";
			*output +="<ERROR>";
			*output +=INFO_NOT_FOUND;
			*output +="</ERROR>\n";
			} 
			else
			{
				*output += "<RESOURCE_INFO>\n";
				*output += "<HLR_RID>" + (*rv_it).rid + "</HLR_RID>\n";
				*output += "<DESC>" + (*rv_it).descr + "</DESC>\n";
				*output += "<CERT_H>" + (*rv_it).certSubject + "</CERT_H>\n";
				*output += "<GID>" + (*rv_it).gid + "</GID>\n";
				*output += "<FUND_TOT>"; 
				*output	+= int2string((*rv_it).total); 
				*output += "</FUND_TOT>\n";
				*output += "</RESOURCE_INFO>\n";
			}
			rv_it++;
		}
		*output +="</BODY>\n</HLR>\n";
	}
	//
	// query to find all the nodes corresponding to a group query.
	
        vector<hlrGroup> gv;
	while ( nodeBuff.status == 0 )
	{
		string tag = "GROUP_INFO";
		nodeBuff = parse (&doc , tag);
		if ( nodeBuff.status != 0 )
			break;
		groupFlag = true;
		hlrGroup g;
		node tagBuff;
		tagBuff = parse ( &nodeBuff.text, "HLR_GID");
		if ( tagBuff.status == 0 )
		{
			#ifdef DEBUG
			cerr << "DEBUG:uiEngine(),got :"<< tagBuff.text << endl;
			#endif
			g.gid = tagBuff.text;
		}
		nodeBuff.release();
		gv.push_back(g);
		
	}
	nodeBuff.status=0;
	#ifdef DEBUG
	cerr << "stored " << gv.size() << " groups query" <<endl;
	#endif
	// now we have a vector<hlrGroup> gv that contains the elements
	// needed for the query. 
	// Every element in the vector has to be passed to the HLR API, that
	// will return the information about every group.
	// Once we have those information we can compose the XML answer and put
	// it into the string output.
	
	//initialize buffer string
	
	if ( groupFlag )
	{
	
		*output = "<HLR type=\"UI_reply\">\n<HEAD>\n<VER>1.0</VER>\n</HEAD>\n<BODY>\n";
		//group_info vector iterator
		vector < hlrGroup >::iterator gv_it = gv.begin();
		while ( gv_it != gv.end() )
		{
			if ( (*gv_it).get() != 0)
			{
				//trow an error!
			#ifdef DEBUG
			cerr << "error retrieving group information!" << endl;
			#endif
			*output = "<HLR type=\"HLR_error\">\n<HEAD>\n<VER>1.0</VER>\n</HEAD>\n<BODY>\n";
			*output +="<ERROR>";
			*output +=INFO_NOT_FOUND;
			*output +="</ERROR>\n";
				
			} 
			else
			{
		
				*output += "<GROUP_INFO>\n";
				*output += "<HLR_GID>" + (*gv_it).gid + "</HLR_GID>\n";
				*output += "<DESC>" + (*gv_it).descr + "</DESC>\n";
				*output += "<FUND_ASS>"; 
				*output	+= int2string((*gv_it).total); 
				*output += "</FUND_ASS>\n";
				*output += "<FUND_BKD>"; 
				*output	+= int2string((*gv_it).booked); 
				*output += "</FUND_BKD>\n";
				*output += "<FUND_SPT>"; 
				*output	+= int2string((*gv_it).spent); 
				*output += "</FUND_SPT>\n";
				*output += "</GROUP_INFO>\n";
			}
			gv_it++;
		
		}
		*output +="</BODY>\n</HLR>\n";

	}
	//
	// query to find all the nodes corresponding to a fund query.

	vector<hlrFund> fv;
	while ( nodeBuff.status == 0 )
	{
		string tag = "FUND_INFO";
		nodeBuff = parse (&doc , tag);
		if ( nodeBuff.status != 0 )
			break;
		fundFlag = true;
		hlrFund f;
		node tagBuff;
		tagBuff = parse ( &nodeBuff.text, "HLR_FID");
		if ( tagBuff.status == 0)
		{
			#ifdef DEBUG
			cerr << "DEBUG:uiEngine(),got :"<< tagBuff.text << endl;
			#endif
			f.fid = tagBuff.text;
		}
		nodeBuff.release();
		fv.push_back(f);
			
	}
	nodeBuff.status = 0;
	#ifdef DEBUG
	cerr << "stored " << fv.size() << "funds query" <<endl;
	#endif
	// now we have a vector<hlrFund> fv that contains the elements
	// needed for the query. 
	// Every element in the vector has to be passed to the HLR API, that
	// will return the information about every group.
	// Once we have those information we can compose the XML answer and put
	// it into the string output.
	
	//initialize buffer string
	
	if ( fundFlag )
	{
	
		*output = "<HLR type=\"UI_reply\">\n<HEAD>\n<VER>1.0</VER>\n</HEAD>\n<BODY>\n";
		//group_info vector iterator
		vector < hlrFund >::iterator fv_it = fv.begin();
		while ( fv_it != fv.end() )
		{
			if ( (*fv_it).get() != 0)
			{
			//trow an error!
			#ifdef DEBUG
			cerr << "error retrieving fund information!" << endl;
			#endif
			*output = "<HLR type=\"HLR_error\">\n<HEAD>\n<VER>1.0</VER>\n</HEAD>\n<BODY>\n";
			*output +="<ERROR>";
			*output +=INFO_NOT_FOUND;
			*output +="</ERROR>\n";
				
			} 
			else
			{
				
				*output += "<FUND_INFO>\n";
				*output += "<HLR_FID>" + (*fv_it).fid + "</HLR_FID>\n";
				*output += "<DESC>" + (*fv_it).descr + "</DESC>\n";
				*output += "<FUND_ASS>"; 
				*output	+= int2string((*fv_it).total); 
				*output += "</FUND_ASS>\n";
				*output += "<FUND_SPT>"; 
				*output	+= int2string((*fv_it).spent);
				*output += "</FUND_SPT>\n";
				*output += "</FUND_INFO>\n";
			}
			fv_it++;
		
		}
		*output +="</BODY>\n</HLR>\n";

	}

	//
	// query to find all the nodes corresponding to a transaction query.

	vector<hlrTransaction> tv;
	vector<int> numV;
	while ( nodeBuff.status == 0 )
	{
		string tag = "TRS";
		nodeBuff = parse (&doc , tag);
		if ( nodeBuff.status != 0 )
			break;
		trsFlag = true;
		hlrTransaction t;
		int numBuff = 50;
		node tagBuff;
		tagBuff = parse ( &nodeBuff.text, "TO_GRID_ID");
		if ( tagBuff.status == 0)
		{
			#ifdef DEBUG
				cerr << "UI_engine, got: " <<tagBuff.text << endl;
			#endif
			t.toId = tagBuff.text; 
		}
		tagBuff = parse ( &nodeBuff.text, "FROM_GRID_ID");
		if ( tagBuff.status == 0)
		{
			#ifdef DEBUG
				cerr << "UI_engine, got: " <<tagBuff.text << endl;
			#endif
			t.fromId = tagBuff.text;	
		}
		tagBuff = parse ( &nodeBuff.text, "MAX_N");
		if ( tagBuff.status == 0)
		{
			#ifdef DEBUG
				cerr << "UI_engine, got: " <<tagBuff.text << endl;
			#endif
			numBuff = atoi((tagBuff.text).c_str());
		}
		tagBuff = parse ( &nodeBuff.text, "TYPE");
		if ( tagBuff.status == 0)
		{
			#ifdef DEBUG
				cerr << "UI_engine, got: " <<tagBuff.text << endl;
			#endif
			if ( tagBuff.text == "INCOMING" )
			{
				t.tType = in;
			}
			if ( tagBuff.text == "OUTCOMING" )
			{
				t.tType = out;
			}
			if ( tagBuff.text == "ND" )
			{
				t.tType = nd;
			}
		}
		
		numV.push_back(numBuff);
    		tv.push_back(t);
		nodeBuff.release();
	}
	nodeBuff.status = 0;
	#ifdef DEBUG
	cerr << "stored " << tv.size() << " transactions queries" <<endl;
	cerr << "stored " << numV.size() << " transaction limits" <<endl;
	#endif
	// now we have a vector<hlrTransaction> tv that contains the elements
	// needed for the query. 
	// Every element in the vector has to be passed to the HLR API, that
	// will return the information about every transaction_in.
	// The vector numV helds the info about the max N of transaction to 
	// report per each query.
	// Once we have those information we can compose the XML answer and put
	// it into the string output.
	
	//initialize buffer string
	if ( trsFlag )
	{
	
		*output = "<HLR type=\"UI_reply\">\n<HEAD>\n<VER>1.0</VER>\n</HEAD>\n<BODY>\n";
		//tr_info vector iterator
		int i = 0;
		vector < hlrTransaction >::iterator tv_it = (tv.begin());//!!
		while ( tv_it != tv.end() )
		{
			vector <hlrTransaction > resultV;
			if ( (*tv_it).get(resultV) != 0)
			{
				//trow an error!
			*output = "<HLR type=\"HLR_error\">\n<HEAD>\n<VER>1.0</VER>\n</HEAD>\n<BODY>\n";
			*output +="<ERROR>";
			*output +=INFO_NOT_FOUND;
			*output +="</ERROR>\n";
				
			} 			
			else
			{
			  vector <hlrTransaction>::iterator resultV_it;
		          if ( resultV.size() > (size_t)numV[i])
			  {
			  resultV_it = (resultV.end())-(size_t)numV[i];
			  }
			  else
			  {
			  resultV_it = (resultV.begin());
			  }
			  while ( resultV_it != resultV.end() )   
			  {
				*output += "<TRS>\n";
				*output += "<FROM_HLR_URL>" + (*resultV_it).fromHlrUrl + "</FROM_HLR_URL>\n";
				*output += "<TO_GRID_ID>" + (*resultV_it).toId + "</TO_GRID_ID>\n";
				*output += "<FROM_GRID_ID>" + (*resultV_it).fromId + "</FROM_GRID_ID>\n";
				if ( (*resultV_it).tType == out )
				{
					*output += "<TYPE>OUTCOMING</TYPE>\n";
				}
				if ( (*resultV_it).tType == in )
				{
					*output += "<TYPE>INCOMING</TYPE>\n";
				}
				*output += "<T_STMP>" + (*resultV_it).timeStamp + "</T_STMP>\n";
				*output += "<JID>" + (*resultV_it).id +"</JID>\n";
				*output += "<LOG>" + (*resultV_it).logData +"</LOG>\n";
				*output += "<COST>"; 
				*output	+= int2string((*resultV_it).amount); 
				*output += "</COST>\n";
				*output += "</TRS>\n";
				resultV_it++;
			  }
			}
			i++;
			tv_it++;
		
		}
		*output +="</BODY>\n</HLR>\n";

	}

	#ifdef DEBUG
		cerr << "Formatted xml message:" << endl;
		cerr << *output << endl;
	#endif 
	return;
}


