/* DGAS (DataGrid Accounting System) 
// Dynamic pricing algorithm for the PA.
// 
// $Id: edg-wl-dgas-pa-pAlgDyn.h,v 1.1 2003/04/11 16:21:12 guarise Exp $
// -------------------------------------------------------------------------
// Copyright (c) 2001-2002, The DataGrid project, INFN, 
// All rights reserved. See LICENSE file for details.
// -------------------------------------------------------------------------
// Author: Andrea Guarise <andrea.guarise@to.infn.it>
//         Rosario M. Piro <piro@to.infn.it>
*/
 /***************************************************************************
 * Code borrowed from:
 *  authors   :
 *  copyright : 
 ***************************************************************************/
/*
      Note: If dynamically linked, this library will be searched for
            three symbols (num_keys, q_keys and getResPrice). The names
	    of these symbols MUST NOT BE CHANGED, otherwise the still running
	    program won't find them in a recompiled newer library version!
//
//
//    

// ---------------------------------------------------------------------------
//  Includes
// ---------------------------------------------------------------------------
*/
#ifndef P_ALG_DYN
#define P_ALG_DYN

#include <cstdio>
#include <vector>
#include <hlr_prot_errcode.h>
#include "edg-wl-dgas-paPriceType.h"

#define NUM_KEYS 9        /* we have 9 query keys! */

using namespace std;


/* definition of query keys */
/* these are the symbols we search for, when linking dynamically!  */
/* DO NOT CHANGE THE NAMES OF THESE SYMBOLS (num_keys and q_keys), */
/* otherwise you will have to change the dlsym() calls of the main */
/* program as well!                                                */
int num_keys = NUM_KEYS;
string q_keys[NUM_KEYS] = { "TotalCPUs",    /* search keys for */
			    "FreeCPUs",     /* query to server */
			    "TotalJobs",
			    "RunningJobs",
			    "IdleJobs",
			    "MaxTotalJobs",
			    "MaxRunningJobs",
			    "WorstTraversalTime",
			    "EstimatedTraversalTime" };

/* this structure will hold the query results (price info) from the server */
/* the results, however, are passed to getResPrice() by means of a */
/* string vector! */
#define pr_info_type int        /* structure stores integers  */
#define strToInfo(a) atoi(a)    /* convert info string to int */

struct pr_info {
  pr_info_type tot_cpus;
  pr_info_type free_cpus;
  pr_info_type total_jobs;
  pr_info_type running_jobs;
  pr_info_type idle_jobs;
  pr_info_type max_total_jobs;
  pr_info_type max_running_jobs;
  pr_info_type worst_qtt;
  pr_info_type estimated_qtt;
};


#ifdef __cplusplus
extern "C" { /* no name mangling, header usable for C and C++ */
#endif
/* the resulting price is returned as a string, in order to
   allow the change of pricing from int to double without necessarily
   changing the interface to this library; p_lib_helper in this case
   is a pointer to a vector<string> object and will be casted in getResPrice.
*/

int getResPrice (void *p_lib_helper, string &price_value);

/* DO NOT CHANGE THE NAME OF THIS FUNCTION (getResPrice), since
   it is a symbol that we have to search for (with dlsym()) when linking
   this library dynamically! */

#ifdef __cplusplus
}
#endif


#endif /* #ifndef P_ALG */
