/* DGAS (DataGrid Accounting System) 
// Server Daeomn and protocol engines.
// 
// $Id: edg-wl-dgas-pa-pAlgMan.cpp,v 1.2 2003/10/13 13:30:13 guarise Exp $
// -------------------------------------------------------------------------
// Copyright (c) 2001-2002, The DataGrid project, INFN, 
// All rights reserved. See LICENSE file for details.
// -------------------------------------------------------------------------
// Author: Rosario M. Piro <piro@to.infn.it>
*/
 /***************************************************************************
 * Code borrowed from:
 *  authors   :
 *  copyright : 
 ***************************************************************************/
/*
      Note: If dynamically linked, this library will be searched for
            three symbols (num_keys, q_keys and getResPrice). The names
	    of these symbols MUST NOT BE CHANGED, otherwise the still running
	    program won't find them in a recompiled newer library version!

// -------------------------------------------------------------------------
//  Includes
// -------------------------------------------------------------------------*/
#include <iostream>
#include "edg-wl-dgas-paSuggPrice.h"
#include <map>
#include "edg-wl-dgas-pa-pAlgMan.h"

const char * _pa_sql_server;  
const char * _pa_sql_user;
const char * _pa_sql_password;
const char * _pa_sql_dbname;
const char * _record_resID;

/* the resulting price is returned as a string, in order to
   allow the change of pricing from int to double without necessarily
   changing the interface to this library; the p_lib_helper in this case
   is a pointer to a map<string,string> object containing the necessary
   information needed to identify the database and the specific record in which
   to search for the suggested price. The pointer will be cast in getResPrice.
*/
/* DO NOT CHANGE THE NAME OF THIS FUNCTION (getResPrice), since
   it is a symbol that we have to search for (with dlsym()) when linking
   this library dynamically! */
#ifdef __cplusplus
extern "C" {
#endif

  int getResPrice (void *p_lib_helper, string &price_value) {

    map<string,string> *db_info = (map<string,string> *) p_lib_helper;
    price_t price;
    int errCode;

    // first find out in which database to look for the suggested price:
    _pa_sql_server = ((*db_info)["pa_sql_server"]).c_str();
    _pa_sql_user = ((*db_info)["pa_sql_user"]).c_str();
    _pa_sql_password = ((*db_info)["pa_sql_password"]).c_str();
    _pa_sql_dbname = ((*db_info)["pa_sql_dbname"]).c_str();
    // which is the record to look for?
    _record_resID = ((*db_info)["record_resID"]).c_str();
    string resID(_record_resID);

    // now read the database record:
    paSuggPrice resSuggPrice;
    errCode = resSuggPrice.get(resID);

    if(errCode == 0) {
      #ifdef DEBUG
      cerr << "getResPrice: suggested price read from database is ";
      #endif



      /* very simple price algorithm, for demonstrative purpose only
	 the real one will be build up as a dynamic object. */
      price = resSuggPrice.suggestedPrice;

      /* write price to price_value string */
      price_value = priceToString(price);

      #ifdef DEBUG
      cerr << price_value << endl << flush;
      #endif

      return 0; /* no error */
    }
    else
      return errCode; // error while reading from database
  }

#ifdef __cplusplus
} // extern "C"
#endif
