#include <boost/filesystem/path.hpp>
#include <boost/filesystem/operations.hpp>
#include <boost/filesystem/exception.hpp>

#include "edg/workload/common/common_namespace.h"
#include "edg/workload/common/configuration/Configuration.h"
#include "edg/workload/common/configuration/JCConfiguration.h"
#include "edg/workload/common/configuration/LMConfiguration.h"
#include "edg/workload/common/logger/manipulators.h"
#include "edg/workload/common/jobid/manipulation.h"
#include "edg/workload/common/jobid/JobId.h"
#include "edg/workload/common/utilities/boost_fs_add.h"
#ifdef ENABLE_PURGER
#include "edg/workload/purger/purger.h"
#endif
#include "edg/workload/jobcontrol/jobcontrol_namespace.h"
#include "edg/workload/jobcontrol/common/files.h"

#include "JobFilePurger.h"

USING_COMMON_NAMESPACE;
using namespace std;

JOBCONTROL_NAMESPACE_BEGIN {

namespace jccommon {

JobFilePurger::JobFilePurger( const jobid::JobId &id ) : jfp_id( id ) {}

JobFilePurger::~JobFilePurger( void ) {}

void JobFilePurger::do_purge( bool everything )
{
  const configuration::LMConfiguration    *lmconfig = configuration::Configuration::instance()->lm();
  logger::StatePusher                      pusher( clog, "JobFilePurger::do_purge(...)" );
#ifdef ENABLE_PURGER
  bool     purge;
#endif

  if( lmconfig->remove_job_files() ) {
    unsigned long int            removed;
    Files                        files( this->jfp_id );

    try {
      clog << logger::setlevel( logger::info ) << "Removing job directory: " << files.output_directory().file_path() << endl;
      removed = boost::filesystem::remove_all( files.output_directory() );
      clog << logger::setlevel( logger::ugly ) << "Removed " << removed << " files." << endl;
    }
    catch( boost::filesystem::filesystem_error &err ) {
      clog << logger::setlevel( logger::null ) << "Failed to remove job directory." << endl
	   << "Reason: " << err.what() << endl;
    }

    try {
      clog << logger::setlevel( logger::info ) << "Removing submit file: " << files.submit_file().file_path() << endl;
      boost::filesystem::remove( files.submit_file() );
      clog << logger::setlevel( logger::ugly ) << "Removed..." << endl;
    }
    catch( boost::filesystem::filesystem_error &err ) {
      clog << logger::setlevel( logger::null ) << "Failed to remove submit file." << endl
	   << "Reason: " << err.what() << endl;
    }

    try {
      clog << logger::setlevel( logger::info ) << "Removing classad file: " << files.classad_file().file_path() << endl;
      boost::filesystem::remove( files.classad_file() );
      clog << logger::setlevel( logger::ugly ) << "Removed..." << endl;
    }
    catch( boost::filesystem::filesystem_error &err ) {
      clog << logger::setlevel( logger::null ) << "Failed to remove classad file." << endl
	   << "Reason: " << err.what() << endl;
    }

    try {
      clog << logger::setlevel( logger::info ) << "Removing wrapper file: " << files.jobwrapper_file().file_path() << endl;
      boost::filesystem::remove( files.jobwrapper_file() );
      clog << logger::setlevel( logger::ugly ) << "Removed..." << endl;
    }
    catch( boost::filesystem::filesystem_error &err ) {
      clog << logger::setlevel( logger::null ) << "Failed to remove wrapper file." << endl
	   << "Reason: " << err.what() << endl;
    }
  }
  else
    clog << logger::setlevel( logger::info )
	 << "Job files not removed." << endl;

  if( everything ) {
    clog << logger::setlevel( logger::ugly ) << "Going to purge job storage..." << endl;

#ifdef ENABLE_PURGER
    purge = purger::purgeStorage( this->jfp_id );

    clog << logger::setlevel( logger::verylow ) << "Purging command returned " << (purge ? "ok" : "an error") << endl;
#else
    clog << logger::setlevel( logger::null ) << "Job purging support not compiled." << endl;
#endif
  }

  return;
}

}; // jccommon namespace

} JOBCONTROL_NAMESPACE_END;
