#ifndef EDG_WORKLOAD_JOBCONTROL_CONTROLLER_JOBCONTROLLERFACTORY_H
#define EDG_WORKLOAD_JOBCONTROL_CONTROLLER_JOBCONTROLLERFACTORY_H

// File: JobControllerFactory.h
// Author: Francesco Giacomini <Francesco.Giacomini@cnaf.infn.it>
// Copyright (c) 2001 EU DataGrid.
// For license conditions see http://www.eu-datagrid.org/license.html

// $Id: JobControllerFactory.h,v 1.6 2004/02/02 16:53:12 peluso Exp $

#include <memory>

#include <classad_distribution.h>

#include "edg/workload/jobcontrol/jobcontrol_namespace.h"
#include "edg/workload/common/utilities/FileList.h"
#include "edg/workload/common/utilities/FileListLock.h"

typedef  struct _edg_wll_Context  *edg_wll_Context;

JOBCONTROL_NAMESPACE_BEGIN {

namespace controller {

class JobControllerImpl;
class JobControllerClientImpl;

class Empty {};

class JobControllerFactory {
  friend class Empty;

public:
  ~JobControllerFactory( void );

  JobControllerImpl *create_server( edg_wll_Context *cont );
  JobControllerClientImpl *create_client( void );

  static JobControllerFactory *instance( void );

private:
  typedef common::utilities::FileList<classad::ClassAd>    queue_type;
  typedef common::utilities::FileListMutex                 mutex_type;

  JobControllerFactory( const JobControllerFactory &rhs ); // Not implemented
  JobControllerFactory &operator=( const JobControllerFactory &rhs ); // Not implemented

  JobControllerFactory( void );

  void createQueue( void );

  std::auto_ptr<mutex_type>     jcf_mutex;
  std::auto_ptr<queue_type>     jcf_queue;

  static JobControllerFactory *jcf_s_instance;
};

} // namespace controller

} JOBCONTROL_NAMESPACE_END;

#endif /* EDG_WORKLOAD_JOBCONTROL_CONTROLLER_JOBCONTROLLERFACTORY_H */

// Local Variables:
// mode: c++
// End:
