#include <iostream>
#include <fstream>

#include <boost/filesystem/path.hpp>
#include <boost/filesystem/operations.hpp>
#include <boost/filesystem/exception.hpp>
#include <classad_distribution.h>

#include "edg/workload/common/requestad/convert.h"
#include "edg/workload/common/requestad/JobAdManipulation.h"
#include "edg/workload/common/requestad/PrivateAdManipulation.h"
#include "edg/workload/common/logger/manipulators.h"
#include "edg/workload/common/jobid/JobId.h"

#include "JobControllerFake.h"
#include "JobControllerExceptions.h"
#include "SubmitAd.h"
#include "SubmitAdExceptions.h"

using namespace std;
USING_COMMON_NAMESPACE;

JOBCONTROL_NAMESPACE_BEGIN {

namespace controller {

JobControllerFake::JobControllerFake( void ) {}

JobControllerFake::~JobControllerFake( void ) {}

int JobControllerFake::submit( const classad::ClassAd *pad )
try {
  ofstream              ofs;
  SubmitAd              ad( pad );
  logger::StatePusher   pusher( clog, "JobControllerFake::submit(...)" );

  clog << logger::setlevel( logger::null )
       << "Got request for submission of job " << ad.job_id() << endl
       << "Original classad: " << *pad << endl
       << "Modified classad: " << ad.classad() << endl
       << "Writing condor submit file: " << ad.submit_file() << endl;

  ofs.open( ad.submit_file().c_str() );

  if( ofs.good() ) {
    requestad::to_submit_stream( ofs, ad );

    ofs.close();

    clog << "File successfully created." << endl;
  }
  else
    clog << "Cannot open condor submit file." << endl;

  return 0;
}
catch( SubmitAdException &error ) { throw CannotExecute( error.error() ); }

bool JobControllerFake::cancel( const jobid::JobId &id, const char *condorid, const char *logfile, bool force )
{
  logger::StatePusher     pusher( clog, "JobControllerFake::cancel(...)" );

  clog << logger::setlevel( logger::null )
       << "Got request for cancellation of job." << endl
       << "EDG Id = " << id.toString() << endl;

  if( force )
    clog << "Must force job removal..." << endl;

  return true;
}

bool JobControllerFake::cancel( int condorid, bool force )
{
  logger::StatePusher     pusher( clog, "JobControllerFake::cancel(...)" );

  clog << logger::setlevel( logger::null )
       << "Got request for cancellation of job." << endl
       << "condor Id = " << condorid << endl;

  if( force )
    clog << "Must force job removal..." << endl;

  return true;
  
}

size_t JobControllerFake::queue_size( void )
{
  return 0;
}

}; // namespace controller

} JOBCONTROL_NAMESPACE_END;
