#include <string>

#include <boost/filesystem/path.hpp>
#include <boost/filesystem/operations.hpp>
#include <boost/filesystem/exception.hpp>
#include <boost/lexical_cast.hpp>

#include "edg/workload/common/common_namespace.h"
#include "edg/workload/common/configuration/Configuration.h"
#include "edg/workload/common/configuration/LMConfiguration.h"
#include "edg/workload/common/utilities/LineParser.h"
#include "edg/workload/jobcontrol/jobcontrol_namespace.h"
#include "edg/workload/jobcontrol/common/IdContainer.h"

#include "exceptions.h"
#include "JobStatusExtractor.h"
#include "JobWrapperOutputParser.h"

USING_COMMON_NAMESPACE;
using namespace std;

JOBCONTROL_NAMESPACE_BEGIN {

namespace logmonitor {

JobStatusExtractor::JobStatusExtractor( const utilities::LineParser &options ) : jse_parser()
{
  const configuration::LMConfiguration    *config = configuration::Configuration::instance()->lm();
  string                                   cid, edgid;
  auto_ptr<jccommon::IdContainer>          container;

  if( options.is_present('c') ) {
    boost::filesystem::path                  idrep( config->monitor_internal_dir(), boost::filesystem::system_specific );

    idrep <<= config->id_repository_name();
    container.reset( new jccommon::IdContainer(idrep.file_path()) );

    cid.assign( options['c'].getStringValue() );
    edgid.assign( container->condor_id(cid) );

    if( edgid.size() == 0 ) throw InvalidJobId( cid );
  }
  else edgid.assign( options['e'].getStringValue() );

  this->jse_parser.reset( new JobWrapperOutputParser(edgid) );
}

JobStatusExtractor::~JobStatusExtractor( void ) {}

int JobStatusExtractor::get_job_status( string &errors )
{
  int                                   retcode;
  JobWrapperOutputParser::status_type   status;

  status = this->jse_parser->parse_file( retcode, errors );

  return retcode;
}

}; // Namespace logmonitor

} JOBCONTROL_NAMESPACE_END;
