#include <cstring>

#include "exceptions.h"

using namespace std;

JOBCONTROL_NAMESPACE_BEGIN {

namespace logmonitor {

MonitorException::MonitorException( void ) : exception() {}

MonitorException::~MonitorException( void ) throw() {}

const char *MonitorException::what( void ) const throw()
{
  this->me_what.assign( this->reason() );

  return this->me_what.c_str();
}

CannotOpenFile::CannotOpenFile( const std::string &filename, int error ) : MonitorException(),
									   cof_errno( error ), cof_name( filename )
{}

CannotOpenFile::~CannotOpenFile( void ) throw() {}

string CannotOpenFile::reason( void ) const
{
  string     reason( "Cannot open file: \"" );

  reason.append( this->cof_name );

  if( this->cof_errno != 0 ) {
    reason.append( "\", reason: \"" );
    reason.append( strerror(this->cof_errno) );
  }

  reason.append( "\"." );

  return reason;
}

FileSystemError::FileSystemError( const string &error ) : MonitorException(), fse_error( error ) {}

FileSystemError::~FileSystemError( void ) throw() {}

string FileSystemError::reason( void ) const
{
  string     reason( "Error during file access \"" );

  reason.append( this->fse_error ); reason.append( "\"." );

  return reason;
}

CannotExecute::CannotExecute( const string &reason ) : ce_reason( reason ) {}

CannotExecute::~CannotExecute( void ) throw() {}

string CannotExecute::reason( void ) const
{
  string    reason( "Error while running logmonitor: \"" );

  reason.append( this->ce_reason ); reason.append( "\"." );

  return reason;
}

InvalidJobId::InvalidJobId( const string &id ) : iji_id( id ) {}

InvalidJobId::~InvalidJobId( void ) throw() {}

string InvalidJobId::reason( void ) const
{
  string    reason( "Passed condor cluster \"" );

  reason.append( this->iji_id ); reason.append( "\" not valid for LogMonitor." );

  return reason;
}

}; // Namespace logmonitor

} JOBCONTROL_NAMESPACE_END;
