#ident "$Header: /local/reps/lcgware/workload/logging/common/events.c.T,v 1.22 2003/11/12 11:20:21 ljocha Exp $"
/*

  !! Automatically generated file
  !! Do not edit, your changes will be discarded upon build
  !! Change the corresponding template file events.c.T

*/

#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <errno.h>

#include "edg/workload/common/jobid/jobid.h"
#include "edg/workload/logging/client/producer.h"
#include "edg/workload/logging/common/ulm_parse.h"

static const struct timeval null_timeval = {0,0};

/**
 * Predefined event types names
 */
static const char *eventNames[] = {
	"Undefined",
	"Transfer",
	"Accepted",
	"Refused",
	"EnQueued",
	"DeQueued",
	"HelperCall",
	"HelperReturn",
	"Running",
	"Resubmission",
	"Done",
	"Cancel",
	"Abort",
	"Clear",
	"Purge",
	"Match",
	"Pending",
	"RegJob",
	"Chkpt",
	"Listener",
	"CurDescr",
	"UserTag",
	"ChangeACL",
	"SysCmpStat",
	"SysClStat",
};

/**
 * \fn edg_wll_EventCode edg_wll_StringToEvent(char *name)
 * \param name          a string event name (e.g. "JobTransfer")
 * \return corresponding numeric code (edg_wll_EventCode)
 * \brief convert a string event name to the corresponding numeric code  
 * Calls: strcasecmp
 * Algorithm: array lookup
 */
edg_wll_EventCode edg_wll_StringToEvent(char *name)
{
	unsigned int	i;

	for (i=1; i<sizeof(eventNames)/sizeof(eventNames[0]); i++)
		if (strcasecmp(eventNames[i],name) == 0) return (edg_wll_EventCode) i;
	return EDG_WLL_EVENT_UNDEF;
}

/**
 * \fn char *edg_wll_EventToString(edg_wll_EventCode event)
 * \param event         an event numeric code (edg_wll_EventCode)
 * \return corresponding string (e.g. "JobTransfer")
 * \brief convert an event numeric code to the corresponding string
 * Calls: strdup
 */
char *edg_wll_EventToString(edg_wll_EventCode event)
{
	if ((int)event < 0 || event >= sizeof(eventNames)/sizeof(eventNames[0])) return NULL;
	return strdup(eventNames[event]);
}


/**
 * Predefined ULM key types names
 */
static const char *keyNames[] = {
	"Undefined",
	"DG.EVNT",
	"DATE",
	"HOST",
	"LVL",
	"DG.PRIORITY",
	"DG.JOBID",
	"DG.SEQCODE",
	"DG.USER",
	"DG.SOURCE",
	"DG.SRC_INSTANCE",
	"DG.TRANSFER.DESTINATION",
	"DG.TRANSFER.DEST_HOST",
	"DG.TRANSFER.DEST_INSTANCE",
	"DG.TRANSFER.JOB",
	"DG.TRANSFER.RESULT",
	"DG.TRANSFER.REASON",
	"DG.TRANSFER.DEST_JOBID",
	"DG.ACCEPTED.FROM",
	"DG.ACCEPTED.FROM_HOST",
	"DG.ACCEPTED.FROM_INSTANCE",
	"DG.ACCEPTED.LOCAL_JOBID",
	"DG.REFUSED.FROM",
	"DG.REFUSED.FROM_HOST",
	"DG.REFUSED.FROM_INSTANCE",
	"DG.REFUSED.REASON",
	"DG.ENQUEUED.QUEUE",
	"DG.ENQUEUED.JOB",
	"DG.ENQUEUED.RESULT",
	"DG.ENQUEUED.REASON",
	"DG.DEQUEUED.QUEUE",
	"DG.DEQUEUED.LOCAL_JOBID",
	"DG.HELPERCALL.HELPER_NAME",
	"DG.HELPERCALL.HELPER_PARAMS",
	"DG.HELPERCALL.SRC_ROLE",
	"DG.HELPERRETURN.HELPER_NAME",
	"DG.HELPERRETURN.RETVAL",
	"DG.HELPERRETURN.SRC_ROLE",
	"DG.RUNNING.NODE",
	"DG.RESUBMISSION.RESULT",
	"DG.RESUBMISSION.REASON",
	"DG.RESUBMISSION.TAG",
	"DG.DONE.STATUS_CODE",
	"DG.DONE.REASON",
	"DG.DONE.EXIT_CODE",
	"DG.CANCEL.STATUS_CODE",
	"DG.CANCEL.REASON",
	"DG.ABORT.REASON",
	"DG.CLEAR.REASON",
	"DG.MATCH.DEST_ID",
	"DG.PENDING.REASON",
	"DG.REGJOB.JDL",
	"DG.REGJOB.NS",
	"DG.REGJOB.PARENT",
	"DG.REGJOB.JOBTYPE",
	"DG.REGJOB.NSUBJOBS",
	"DG.REGJOB.SEED",
	"DG.CHKPT.TAG",
	"DG.CHKPT.CLASSAD",
	"DG.LISTENER.SVC_NAME",
	"DG.LISTENER.SVC_HOST",
	"DG.LISTENER.SVC_PORT",
	"DG.CURDESCR.DESCR",
	"DG.USERTAG.NAME",
	"DG.USERTAG.VALUE",
	"DG.CHANGEACL.USER_ID",
	"DG.CHANGEACL.USER_ID_TYPE",
	"DG.CHANGEACL.PERMISSION",
	"DG.CHANGEACL.PERMISSION_TYPE",
	"DG.CHANGEACL.OPERATION",
};

/*
 * \fn edg_wll_KeyNameCode edg_wll_StringToKeyName(char *name)
 * \param name		a string ULM key name (e.g. "DG.JOB.TRANSFER.DEST")
 * \return corresponding numeric code (edg_wll_KeyNameCode)
 * \brief convert a string ULM key name to the corresponding numeric code
 * Calls: strcasecmp
 * Algorithm: array lookup
 */
edg_wll_KeyNameCode edg_wll_StringToKeyName(char *name)
{
	unsigned int	i;

	for (i=1; i<sizeof(keyNames)/sizeof(keyNames[0]); i++)
		if (strcasecmp(keyNames[i],name) == 0) return (edg_wll_KeyNameCode) i;
	return UNDEFINED;
}

/*
 * \fn char *edg_wll_KeyNameToString(edg_wll_KeyNameCode key)
 * \param key		a ULM key name numeric code (edg_wll_KeyNameCode)
 * \return corresponding string (e.g. "DG.JOB.TRANSFER.DEST")
 * \brief convert a ULM key name numeric code to the corresponding string
 * Calls: strdup
 */
char *edg_wll_KeyNameToString(edg_wll_KeyNameCode key)
{
	if ((int)key < 0 || key >= sizeof(keyNames)/sizeof(keyNames[0])) return NULL;
	return strdup(keyNames[key]);
}


/**
 * Predefined _code_ types names and
 * related StringTo_code and _code_ToString function implementations
 */

/**
 * Predefined code names for Level
 */
static const char *edg_wll_LevelNames[] = {
	"UNDEFINED",
	"EMERGENCY",
	"ALERT",
	"ERROR",
	"WARNING",
	"AUTH",
	"SECURITY",
	"USAGE",
	"SYSTEM",
	"IMPORTANT",
	"DEBUG",
}; 

/**
 * \fn enum edg_wll_Level edg_wll_StringToLevel(char *name)
 * Calls: strcasecmp
 * Algorithm: array lookup
 */
enum edg_wll_Level edg_wll_StringToLevel(char *name) 
{
	unsigned int	i;

	for (i=1; i<sizeof(edg_wll_LevelNames)/sizeof(edg_wll_LevelNames[0]); i++) 
		if (strcasecmp(edg_wll_LevelNames[i],name) == 0) return (enum edg_wll_Level) i;
	return (enum edg_wll_Level) EDG_WLL_LEVEL_UNDEFINED;
}


/**
 * \fn *edg_wll_LevelToString(enum edg_wll_Level code)
 * Calls: strdup
 */
char *edg_wll_LevelToString(enum edg_wll_Level code) 
{
	if ((int)code < 0 || code >= sizeof(edg_wll_LevelNames)/sizeof(edg_wll_LevelNames[0])) return NULL;
	return strdup(edg_wll_LevelNames[code]);

}



/**
 * Predefined code names for TransferResult
 */
static const char *edg_wll_TransferResultNames[] = {
	"UNDEFINED",
	"START",
	"OK",
	"REFUSED",
	"FAIL",
}; 

/**
 * \fn enum edg_wll_TransferResult edg_wll_StringToTransferResult(char *name)
 * Calls: strcasecmp
 * Algorithm: array lookup
 */
enum edg_wll_TransferResult edg_wll_StringToTransferResult(char *name) 
{
	unsigned int	i;

	for (i=1; i<sizeof(edg_wll_TransferResultNames)/sizeof(edg_wll_TransferResultNames[0]); i++) 
		if (strcasecmp(edg_wll_TransferResultNames[i],name) == 0) return (enum edg_wll_TransferResult) i;
	return (enum edg_wll_TransferResult) EDG_WLL_TRANSFER_UNDEFINED;
}


/**
 * \fn *edg_wll_TransferResultToString(enum edg_wll_TransferResult code)
 * Calls: strdup
 */
char *edg_wll_TransferResultToString(enum edg_wll_TransferResult code) 
{
	if ((int)code < 0 || code >= sizeof(edg_wll_TransferResultNames)/sizeof(edg_wll_TransferResultNames[0])) return NULL;
	return strdup(edg_wll_TransferResultNames[code]);

}



/**
 * Predefined code names for EnQueuedResult
 */
static const char *edg_wll_EnQueuedResultNames[] = {
	"UNDEFINED",
	"START",
	"OK",
	"REFUSED",
	"FAIL",
}; 

/**
 * \fn enum edg_wll_EnQueuedResult edg_wll_StringToEnQueuedResult(char *name)
 * Calls: strcasecmp
 * Algorithm: array lookup
 */
enum edg_wll_EnQueuedResult edg_wll_StringToEnQueuedResult(char *name) 
{
	unsigned int	i;

	for (i=1; i<sizeof(edg_wll_EnQueuedResultNames)/sizeof(edg_wll_EnQueuedResultNames[0]); i++) 
		if (strcasecmp(edg_wll_EnQueuedResultNames[i],name) == 0) return (enum edg_wll_EnQueuedResult) i;
	return (enum edg_wll_EnQueuedResult) EDG_WLL_ENQUEUED_UNDEFINED;
}


/**
 * \fn *edg_wll_EnQueuedResultToString(enum edg_wll_EnQueuedResult code)
 * Calls: strdup
 */
char *edg_wll_EnQueuedResultToString(enum edg_wll_EnQueuedResult code) 
{
	if ((int)code < 0 || code >= sizeof(edg_wll_EnQueuedResultNames)/sizeof(edg_wll_EnQueuedResultNames[0])) return NULL;
	return strdup(edg_wll_EnQueuedResultNames[code]);

}



/**
 * Predefined code names for HelperCallSrc_role
 */
static const char *edg_wll_HelperCallSrc_roleNames[] = {
	"UNDEFINED",
	"CALLING",
	"CALLED",
}; 

/**
 * \fn enum edg_wll_HelperCallSrc_role edg_wll_StringToHelperCallSrc_role(char *name)
 * Calls: strcasecmp
 * Algorithm: array lookup
 */
enum edg_wll_HelperCallSrc_role edg_wll_StringToHelperCallSrc_role(char *name) 
{
	unsigned int	i;

	for (i=1; i<sizeof(edg_wll_HelperCallSrc_roleNames)/sizeof(edg_wll_HelperCallSrc_roleNames[0]); i++) 
		if (strcasecmp(edg_wll_HelperCallSrc_roleNames[i],name) == 0) return (enum edg_wll_HelperCallSrc_role) i;
	return (enum edg_wll_HelperCallSrc_role) EDG_WLL_HELPERCALL_UNDEFINED;
}


/**
 * \fn *edg_wll_HelperCallSrc_roleToString(enum edg_wll_HelperCallSrc_role code)
 * Calls: strdup
 */
char *edg_wll_HelperCallSrc_roleToString(enum edg_wll_HelperCallSrc_role code) 
{
	if ((int)code < 0 || code >= sizeof(edg_wll_HelperCallSrc_roleNames)/sizeof(edg_wll_HelperCallSrc_roleNames[0])) return NULL;
	return strdup(edg_wll_HelperCallSrc_roleNames[code]);

}



/**
 * Predefined code names for HelperReturnSrc_role
 */
static const char *edg_wll_HelperReturnSrc_roleNames[] = {
	"UNDEFINED",
	"CALLING",
	"CALLED",
}; 

/**
 * \fn enum edg_wll_HelperReturnSrc_role edg_wll_StringToHelperReturnSrc_role(char *name)
 * Calls: strcasecmp
 * Algorithm: array lookup
 */
enum edg_wll_HelperReturnSrc_role edg_wll_StringToHelperReturnSrc_role(char *name) 
{
	unsigned int	i;

	for (i=1; i<sizeof(edg_wll_HelperReturnSrc_roleNames)/sizeof(edg_wll_HelperReturnSrc_roleNames[0]); i++) 
		if (strcasecmp(edg_wll_HelperReturnSrc_roleNames[i],name) == 0) return (enum edg_wll_HelperReturnSrc_role) i;
	return (enum edg_wll_HelperReturnSrc_role) EDG_WLL_HELPERRETURN_UNDEFINED;
}


/**
 * \fn *edg_wll_HelperReturnSrc_roleToString(enum edg_wll_HelperReturnSrc_role code)
 * Calls: strdup
 */
char *edg_wll_HelperReturnSrc_roleToString(enum edg_wll_HelperReturnSrc_role code) 
{
	if ((int)code < 0 || code >= sizeof(edg_wll_HelperReturnSrc_roleNames)/sizeof(edg_wll_HelperReturnSrc_roleNames[0])) return NULL;
	return strdup(edg_wll_HelperReturnSrc_roleNames[code]);

}



/**
 * Predefined code names for ResubmissionResult
 */
static const char *edg_wll_ResubmissionResultNames[] = {
	"UNDEFINED",
	"WILLRESUB",
	"WONTRESUB",
}; 

/**
 * \fn enum edg_wll_ResubmissionResult edg_wll_StringToResubmissionResult(char *name)
 * Calls: strcasecmp
 * Algorithm: array lookup
 */
enum edg_wll_ResubmissionResult edg_wll_StringToResubmissionResult(char *name) 
{
	unsigned int	i;

	for (i=1; i<sizeof(edg_wll_ResubmissionResultNames)/sizeof(edg_wll_ResubmissionResultNames[0]); i++) 
		if (strcasecmp(edg_wll_ResubmissionResultNames[i],name) == 0) return (enum edg_wll_ResubmissionResult) i;
	return (enum edg_wll_ResubmissionResult) EDG_WLL_RESUBMISSION_UNDEFINED;
}


/**
 * \fn *edg_wll_ResubmissionResultToString(enum edg_wll_ResubmissionResult code)
 * Calls: strdup
 */
char *edg_wll_ResubmissionResultToString(enum edg_wll_ResubmissionResult code) 
{
	if ((int)code < 0 || code >= sizeof(edg_wll_ResubmissionResultNames)/sizeof(edg_wll_ResubmissionResultNames[0])) return NULL;
	return strdup(edg_wll_ResubmissionResultNames[code]);

}



/**
 * Predefined code names for DoneStatus_code
 */
static const char *edg_wll_DoneStatus_codeNames[] = {
	"UNDEFINED",
	"OK",
	"FAILED",
	"CANCELLED",
}; 

/**
 * \fn enum edg_wll_DoneStatus_code edg_wll_StringToDoneStatus_code(char *name)
 * Calls: strcasecmp
 * Algorithm: array lookup
 */
enum edg_wll_DoneStatus_code edg_wll_StringToDoneStatus_code(char *name) 
{
	unsigned int	i;

	for (i=1; i<sizeof(edg_wll_DoneStatus_codeNames)/sizeof(edg_wll_DoneStatus_codeNames[0]); i++) 
		if (strcasecmp(edg_wll_DoneStatus_codeNames[i],name) == 0) return (enum edg_wll_DoneStatus_code) i;
	return (enum edg_wll_DoneStatus_code) EDG_WLL_DONE_UNDEFINED;
}


/**
 * \fn *edg_wll_DoneStatus_codeToString(enum edg_wll_DoneStatus_code code)
 * Calls: strdup
 */
char *edg_wll_DoneStatus_codeToString(enum edg_wll_DoneStatus_code code) 
{
	if ((int)code < 0 || code >= sizeof(edg_wll_DoneStatus_codeNames)/sizeof(edg_wll_DoneStatus_codeNames[0])) return NULL;
	return strdup(edg_wll_DoneStatus_codeNames[code]);

}



/**
 * Predefined code names for CancelStatus_code
 */
static const char *edg_wll_CancelStatus_codeNames[] = {
	"UNDEFINED",
	"REQ",
	"REFUSE",
	"DONE",
	"ABORT",
}; 

/**
 * \fn enum edg_wll_CancelStatus_code edg_wll_StringToCancelStatus_code(char *name)
 * Calls: strcasecmp
 * Algorithm: array lookup
 */
enum edg_wll_CancelStatus_code edg_wll_StringToCancelStatus_code(char *name) 
{
	unsigned int	i;

	for (i=1; i<sizeof(edg_wll_CancelStatus_codeNames)/sizeof(edg_wll_CancelStatus_codeNames[0]); i++) 
		if (strcasecmp(edg_wll_CancelStatus_codeNames[i],name) == 0) return (enum edg_wll_CancelStatus_code) i;
	return (enum edg_wll_CancelStatus_code) EDG_WLL_CANCEL_UNDEFINED;
}


/**
 * \fn *edg_wll_CancelStatus_codeToString(enum edg_wll_CancelStatus_code code)
 * Calls: strdup
 */
char *edg_wll_CancelStatus_codeToString(enum edg_wll_CancelStatus_code code) 
{
	if ((int)code < 0 || code >= sizeof(edg_wll_CancelStatus_codeNames)/sizeof(edg_wll_CancelStatus_codeNames[0])) return NULL;
	return strdup(edg_wll_CancelStatus_codeNames[code]);

}



/**
 * Predefined code names for ClearReason
 */
static const char *edg_wll_ClearReasonNames[] = {
	"UNDEFINED",
	"USER",
	"TIMEOUT",
	"NOOUTPUT",
}; 

/**
 * \fn enum edg_wll_ClearReason edg_wll_StringToClearReason(char *name)
 * Calls: strcasecmp
 * Algorithm: array lookup
 */
enum edg_wll_ClearReason edg_wll_StringToClearReason(char *name) 
{
	unsigned int	i;

	for (i=1; i<sizeof(edg_wll_ClearReasonNames)/sizeof(edg_wll_ClearReasonNames[0]); i++) 
		if (strcasecmp(edg_wll_ClearReasonNames[i],name) == 0) return (enum edg_wll_ClearReason) i;
	return (enum edg_wll_ClearReason) EDG_WLL_CLEAR_UNDEFINED;
}


/**
 * \fn *edg_wll_ClearReasonToString(enum edg_wll_ClearReason code)
 * Calls: strdup
 */
char *edg_wll_ClearReasonToString(enum edg_wll_ClearReason code) 
{
	if ((int)code < 0 || code >= sizeof(edg_wll_ClearReasonNames)/sizeof(edg_wll_ClearReasonNames[0])) return NULL;
	return strdup(edg_wll_ClearReasonNames[code]);

}



/**
 * Predefined code names for RegJobJobtype
 */
static const char *edg_wll_RegJobJobtypeNames[] = {
	"UNDEFINED",
	"SIMPLE",
	"DAG",
	"PARTITIONABLE",
	"PARTITIONED",
}; 

/**
 * \fn enum edg_wll_RegJobJobtype edg_wll_StringToRegJobJobtype(char *name)
 * Calls: strcasecmp
 * Algorithm: array lookup
 */
enum edg_wll_RegJobJobtype edg_wll_StringToRegJobJobtype(char *name) 
{
	unsigned int	i;

	for (i=1; i<sizeof(edg_wll_RegJobJobtypeNames)/sizeof(edg_wll_RegJobJobtypeNames[0]); i++) 
		if (strcasecmp(edg_wll_RegJobJobtypeNames[i],name) == 0) return (enum edg_wll_RegJobJobtype) i;
	return (enum edg_wll_RegJobJobtype) EDG_WLL_REGJOB_UNDEFINED;
}


/**
 * \fn *edg_wll_RegJobJobtypeToString(enum edg_wll_RegJobJobtype code)
 * Calls: strdup
 */
char *edg_wll_RegJobJobtypeToString(enum edg_wll_RegJobJobtype code) 
{
	if ((int)code < 0 || code >= sizeof(edg_wll_RegJobJobtypeNames)/sizeof(edg_wll_RegJobJobtypeNames[0])) return NULL;
	return strdup(edg_wll_RegJobJobtypeNames[code]);

}



/**
 * Initialise an event structure
 * \return pointer to initialised event structure
 */
edg_wll_Event *edg_wll_InitEvent(
		edg_wll_EventCode eventcode	/* eventcode IN */
) 
{
edg_wll_Event *event;

/* allocate memory for 'event' (edg_wll_Event); use calloc to also clean it */
/* FIXME: what if calloc fails? should be checked somehow -> nomem etc. */
	event = (edg_wll_Event *) calloc(1,sizeof(edg_wll_Event));
/* initialize common fields */
	event->any.timestamp = null_timeval;
	event->any.host = NULL;
	event->any.level = 0;
	event->any.priority = -1;
	event->any.jobId = NULL;
	event->any.seqcode = NULL;
	event->any.user = NULL;
	event->any.source = EDG_WLL_SOURCE_NONE;
	event->any.src_instance = NULL;
/* initialize dependent fields */
switch (eventcode) {
   case EDG_WLL_EVENT_TRANSFER : 
   	event->transfer.destination = EDG_WLL_SOURCE_NONE;
   	event->transfer.dest_host = NULL;
   	event->transfer.dest_instance = NULL;
   	event->transfer.job = NULL;
   	event->transfer.result = 0;
   	event->transfer.reason = NULL;
   	event->transfer.dest_jobid = NULL;
      break;
   case EDG_WLL_EVENT_ACCEPTED : 
   	event->accepted.from = EDG_WLL_SOURCE_NONE;
   	event->accepted.from_host = NULL;
   	event->accepted.from_instance = NULL;
   	event->accepted.local_jobid = NULL;
      break;
   case EDG_WLL_EVENT_REFUSED : 
   	event->refused.from = EDG_WLL_SOURCE_NONE;
   	event->refused.from_host = NULL;
   	event->refused.from_instance = NULL;
   	event->refused.reason = NULL;
      break;
   case EDG_WLL_EVENT_ENQUEUED : 
   	event->enQueued.queue = NULL;
   	event->enQueued.job = NULL;
   	event->enQueued.result = 0;
   	event->enQueued.reason = NULL;
      break;
   case EDG_WLL_EVENT_DEQUEUED : 
   	event->deQueued.queue = NULL;
   	event->deQueued.local_jobid = NULL;
      break;
   case EDG_WLL_EVENT_HELPERCALL : 
   	event->helperCall.helper_name = NULL;
   	event->helperCall.helper_params = NULL;
   	event->helperCall.src_role = 0;
      break;
   case EDG_WLL_EVENT_HELPERRETURN : 
   	event->helperReturn.helper_name = NULL;
   	event->helperReturn.retval = NULL;
   	event->helperReturn.src_role = 0;
      break;
   case EDG_WLL_EVENT_RUNNING : 
   	event->running.node = NULL;
      break;
   case EDG_WLL_EVENT_RESUBMISSION : 
   	event->resubmission.result = 0;
   	event->resubmission.reason = NULL;
   	event->resubmission.tag = NULL;
      break;
   case EDG_WLL_EVENT_DONE : 
   	event->done.status_code = 0;
   	event->done.reason = NULL;
   	event->done.exit_code = -1;
      break;
   case EDG_WLL_EVENT_CANCEL : 
   	event->cancel.status_code = 0;
   	event->cancel.reason = NULL;
      break;
   case EDG_WLL_EVENT_ABORT : 
   	event->abort.reason = NULL;
      break;
   case EDG_WLL_EVENT_CLEAR : 
   	event->clear.reason = 0;
      break;
   case EDG_WLL_EVENT_PURGE : 
      break;
   case EDG_WLL_EVENT_MATCH : 
   	event->match.dest_id = NULL;
      break;
   case EDG_WLL_EVENT_PENDING : 
   	event->pending.reason = NULL;
      break;
   case EDG_WLL_EVENT_REGJOB : 
   	event->regJob.jdl = NULL;
   	event->regJob.ns = NULL;
   	event->regJob.parent = NULL;
   	event->regJob.jobtype = 0;
   	event->regJob.nsubjobs = 0;
   	event->regJob.seed = NULL;
      break;
   case EDG_WLL_EVENT_CHKPT : 
   	event->chkpt.tag = NULL;
   	event->chkpt.classad = NULL;
      break;
   case EDG_WLL_EVENT_LISTENER : 
   	event->listener.svc_name = NULL;
   	event->listener.svc_host = NULL;
   	event->listener.svc_port = 0;
      break;
   case EDG_WLL_EVENT_CURDESCR : 
   	event->curDescr.descr = NULL;
      break;
   case EDG_WLL_EVENT_USERTAG : 
   	event->userTag.name = NULL;
   	event->userTag.value = NULL;
      break;
   case EDG_WLL_EVENT_CHANGEACL : 
   	event->changeACL.user_id = NULL;
   	event->changeACL.user_id_type = -1;
   	event->changeACL.permission = -1;
   	event->changeACL.permission_type = -1;
   	event->changeACL.operation = -1;
      break;
   case EDG_WLL_EVENT_UNDEF :
   default :
      break;
}
return event;
}

/**
 * edg_wll_FreeEvent 
 * \brief free memory allocated for edg_wll_Event internal fields
 * Calls: free
 */
void edg_wll_FreeEvent(
          edg_wll_Event *event       /* event IN */
)
{
edg_wll_EventCode eventcode=event->type;

/* free the common fields */
        if (event->any.jobId) edg_wlc_JobIdFree(event->any.jobId);
	if (event->any.host) free(event->any.host);
	if (event->any.seqcode) free(event->any.seqcode);
	if (event->any.user) free(event->any.user);
	if (event->any.src_instance) free(event->any.src_instance);

/* free the rest */
switch (eventcode) {
   case EDG_WLL_EVENT_TRANSFER :
   	if (event->transfer.dest_host) free(event->transfer.dest_host);
   	if (event->transfer.dest_instance) free(event->transfer.dest_instance);
   	if (event->transfer.job) free(event->transfer.job);
   	if (event->transfer.reason) free(event->transfer.reason);
   	if (event->transfer.dest_jobid) free(event->transfer.dest_jobid);
      break;
   case EDG_WLL_EVENT_ACCEPTED :
   	if (event->accepted.from_host) free(event->accepted.from_host);
   	if (event->accepted.from_instance) free(event->accepted.from_instance);
   	if (event->accepted.local_jobid) free(event->accepted.local_jobid);
      break;
   case EDG_WLL_EVENT_REFUSED :
   	if (event->refused.from_host) free(event->refused.from_host);
   	if (event->refused.from_instance) free(event->refused.from_instance);
   	if (event->refused.reason) free(event->refused.reason);
      break;
   case EDG_WLL_EVENT_ENQUEUED :
   	if (event->enQueued.queue) free(event->enQueued.queue);
   	if (event->enQueued.job) free(event->enQueued.job);
   	if (event->enQueued.reason) free(event->enQueued.reason);
      break;
   case EDG_WLL_EVENT_DEQUEUED :
   	if (event->deQueued.queue) free(event->deQueued.queue);
   	if (event->deQueued.local_jobid) free(event->deQueued.local_jobid);
      break;
   case EDG_WLL_EVENT_HELPERCALL :
   	if (event->helperCall.helper_name) free(event->helperCall.helper_name);
   	if (event->helperCall.helper_params) free(event->helperCall.helper_params);
      break;
   case EDG_WLL_EVENT_HELPERRETURN :
   	if (event->helperReturn.helper_name) free(event->helperReturn.helper_name);
   	if (event->helperReturn.retval) free(event->helperReturn.retval);
      break;
   case EDG_WLL_EVENT_RUNNING :
   	if (event->running.node) free(event->running.node);
      break;
   case EDG_WLL_EVENT_RESUBMISSION :
   	if (event->resubmission.reason) free(event->resubmission.reason);
   	if (event->resubmission.tag) free(event->resubmission.tag);
      break;
   case EDG_WLL_EVENT_DONE :
   	if (event->done.reason) free(event->done.reason);
      break;
   case EDG_WLL_EVENT_CANCEL :
   	if (event->cancel.reason) free(event->cancel.reason);
      break;
   case EDG_WLL_EVENT_ABORT :
   	if (event->abort.reason) free(event->abort.reason);
      break;
   case EDG_WLL_EVENT_CLEAR :
      break;
   case EDG_WLL_EVENT_PURGE :
      break;
   case EDG_WLL_EVENT_MATCH :
   	if (event->match.dest_id) free(event->match.dest_id);
      break;
   case EDG_WLL_EVENT_PENDING :
   	if (event->pending.reason) free(event->pending.reason);
      break;
   case EDG_WLL_EVENT_REGJOB :
   	if (event->regJob.jdl) free(event->regJob.jdl);
   	if (event->regJob.ns) free(event->regJob.ns);
   	if (event->regJob.seed) free(event->regJob.seed);
      break;
   case EDG_WLL_EVENT_CHKPT :
   	if (event->chkpt.tag) free(event->chkpt.tag);
   	if (event->chkpt.classad) free(event->chkpt.classad);
      break;
   case EDG_WLL_EVENT_LISTENER :
   	if (event->listener.svc_name) free(event->listener.svc_name);
   	if (event->listener.svc_host) free(event->listener.svc_host);
      break;
   case EDG_WLL_EVENT_CURDESCR :
   	if (event->curDescr.descr) free(event->curDescr.descr);
      break;
   case EDG_WLL_EVENT_USERTAG :
   	if (event->userTag.name) free(event->userTag.name);
   	if (event->userTag.value) free(event->userTag.value);
      break;
   case EDG_WLL_EVENT_CHANGEACL :
   	if (event->changeACL.user_id) free(event->changeACL.user_id);
      break;
   case EDG_WLL_EVENT_UNDEF :
   default:
      break;
}

}
