#ident "$Header: /local/reps/lcgware/workload/logging/common/mini_http.c,v 1.24 2003/05/23 13:27:39 salvet Exp $"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <signal.h>
#include <assert.h>
#include <fcntl.h>
#include <syslog.h>

#include "globus_config.h"
#include "sslutils.h"

#include "edg/workload/logging/common/mini_http.h"
#include "edg/workload/logging/common/dgssl.h"
#include "edg/workload/logging/common/context.h"

#define min(x,y)	((x) < (y) ? (x) : (y))
#define CONTENT_LENGTH	"Content-Length:"

edg_wll_ErrorCode edg_wll_http_recv(edg_wll_Context ctx,char **firstOut,char ***hdrOut,char **bodyOut)
{
	char	**hdr = NULL,*first = NULL,*body = NULL;
	enum	{ FIRST, HEAD, BODY, DONE }	pstat = FIRST;
	int	len, nhdr = 0,rdmore = 0,clen = 0,blen = 0;
	int	sock;

#define bshift(shift) {\
	memmove(ctx->buf,ctx->buf+(shift),ctx->bufUse-(shift));\
	ctx->bufUse -= (shift);\
}
	edg_wll_ResetError(ctx);


	if (ctx->ssl) sock = SSL_get_fd(ctx->ssl);
	else { 
		edg_wll_SetError(ctx,ENOTCONN,NULL);
		goto error;
	}

	if (!ctx->buf) ctx->buf = malloc(ctx->bufSize = BUFSIZ);

	do {
		len = edg_wll_ssl_read(ctx->ssl,
			ctx->buf+ctx->bufUse,ctx->bufSize-ctx->bufUse,&ctx->p_tmp_timeout);

		switch (len) {
			case EDG_WLL_SSL_OK:
			case EDG_WLL_SSL_ERROR_SSL:
				edg_wll_SetError(ctx,EDG_WLL_ERROR_SSL,
					ERR_error_string(ERR_get_error(), NULL));
				goto error;
			case EDG_WLL_SSL_ERROR_ERRNO:
				edg_wll_SetError(ctx,errno,"edg_wll_ssl_read()");
				goto error;
			case EDG_WLL_SSL_ERROR_TIMEOUT:
				edg_wll_SetError(ctx,ETIMEDOUT,NULL);
				goto error; 
			case EDG_WLL_SSL_ERROR_EOF:
				edg_wll_SetError(ctx,ENOTCONN,NULL);
				goto error; 
			/* default: fallthrough */
		}


		ctx->bufUse += len;
		rdmore = 0;

		while (!rdmore && pstat != DONE) switch (pstat) {
			char	*cr; 

			case FIRST:
				if ((cr = memchr(ctx->buf,'\r',ctx->bufUse)) &&
					ctx->bufUse >= cr-ctx->buf+2 && cr[1] == '\n')
				{
					*cr = 0;
					first = strdup(ctx->buf);
					bshift(cr-ctx->buf+2);
					pstat = HEAD;
				} else rdmore = 1;
				break;
			case HEAD:
				if ((cr = memchr(ctx->buf,'\r',ctx->bufUse)) &&
					ctx->bufUse >= cr-ctx->buf+2 && cr[1] == '\n')
				{
					if (cr == ctx->buf) {
						bshift(2);
						pstat = clen ? BODY : DONE;
						if (clen) body = malloc(clen+1);
						break;
					}

					*cr = 0;
					hdr = realloc(hdr,(nhdr+2) * sizeof(*hdr));
					hdr[nhdr] = strdup(ctx->buf);
					hdr[++nhdr] = NULL;

					if (!strncasecmp(ctx->buf,CONTENT_LENGTH,sizeof(CONTENT_LENGTH)-1))
						clen = atoi(ctx->buf+sizeof(CONTENT_LENGTH)-1);
	
					bshift(cr-ctx->buf+2);
				} else rdmore = 1;
				break;
			case BODY:
				if (ctx->bufUse) {
					int	m = min(ctx->bufUse,clen-blen);
					memcpy(body+blen,ctx->buf,m);
					blen += m;
					bshift(m);
				}
				rdmore = 1;
				if (blen == clen) {
					pstat = DONE;
					body[blen] = 0;
				}
				break;
			default:
				break;
		}
	} while (pstat != DONE);

error:
	if (edg_wll_Error(ctx,NULL,NULL)) {
		if (hdr) {
			char 	**h;
			for (h = hdr; *h; h++) free(*h);
			free(hdr);
		}
		free(first);
		free(body);
	} else {
		if (firstOut) *firstOut = first; else free(first);
		if (hdrOut) *hdrOut = hdr; 
		else if (hdr) {
			char 	**h;
			for (h = hdr; *h; h++) free(*h);
			free(hdr);
		}
		if (bodyOut) *bodyOut = body; else free(body);
	}

	return edg_wll_Error(ctx,NULL,NULL);
}

static int real_write(edg_wll_Context ctx, SSL *ssl,int fd,const char *data,int len)
{
	int	once,total = 0;
	struct sigaction	sa,osa;

	memset(&sa,0,sizeof(sa)); assert(sa.sa_handler == NULL);
	sa.sa_handler = SIG_IGN;
	sigaction(SIGPIPE,&sa,&osa);

	while (total < len) {
		int	sslerr;

		switch (once = edg_wll_ssl_write(ssl,(void*)(data+total),len-total,&ctx->p_tmp_timeout)) {
			case EDG_WLL_SSL_ERROR_EOF:
				errno = ENOTCONN;
				total = -1;
				goto end;
			case EDG_WLL_SSL_ERROR_TIMEOUT:
				errno = ETIMEDOUT;
				total = -1;
				goto end;
			case EDG_WLL_SSL_ERROR_ERRNO:
				if (errno == EPIPE) errno = ENOTCONN;
				total = -1;
				goto end;
			case EDG_WLL_SSL_OK:
			case EDG_WLL_SSL_ERROR_SSL:
				sslerr = SSL_get_error(ssl,once);
				if (sslerr == SSL_ERROR_SYSCALL) {
				       	if (errno == EPIPE) errno = ENOTCONN;
				}
				else errno = EDG_WLL_ERROR_SSL;
				total = -1;
				goto end;
			default:
				total += once;
		}
	}
end:
	sigaction(SIGPIPE,&osa,NULL);
	return total;
}

edg_wll_ErrorCode edg_wll_http_send(edg_wll_Context ctx,const char *first,const char * const *head,const char *body)
{
	const char* const *h;
	int	len = 0, blen, sock;

	edg_wll_ResetError(ctx);

        if (ctx->ssl) sock = SSL_get_fd(ctx->ssl);
        else return edg_wll_SetError(ctx,ENOTCONN,NULL);

	if (real_write(ctx,ctx->ssl,sock,first,strlen(first)) < 0 ||
		real_write(ctx,ctx->ssl,sock,"\r\n",2) < 0) 
		return edg_wll_SetError(ctx,errno,"edg_wll_http_send()");

	if (head) for (h=head; *h; h++) 
		if (real_write(ctx,ctx->ssl,sock,*h,strlen(*h)) < 0 ||
			real_write(ctx,ctx->ssl,sock,"\r\n",2) < 0) 
			return edg_wll_SetError(ctx,errno,"edg_wll_http_send()");

	if (body) {
		char	buf[100];

		len = strlen(body);
		blen = sprintf(buf,CONTENT_LENGTH " %d\r\n",len);
		if (real_write(ctx,ctx->ssl,sock,buf,blen) < 0) 
			return edg_wll_SetError(ctx,errno,"edg_wll_http_send()");
	}

	if (real_write(ctx,ctx->ssl,sock,"\r\n",2) < 0) 
		return edg_wll_SetError(ctx,errno,"edg_wll_http_send()");
	if (body && real_write(ctx,ctx->ssl,sock,body,len) < 0)  
		return edg_wll_SetError(ctx,errno,"edg_wll_http_send()");

	return edg_wll_Error(ctx,NULL,NULL);
}
