#ident "$Header: /local/reps/lcgware/workload/logging/common/xml_conversions.c,v 1.36 2003/08/01 07:46:30 honik Exp $"

#include <string.h>
#include <expat.h>

#include "edg/workload/thirdparty/trio/trio.h"

#include "edg/workload/logging/common/xml_conversions.h"



static const struct timeval null_timeval = {0,0};


/************************************************************************/
/* Context manipulation functions 					*/	


void edg_wll_initXMLCtx(edg_wll_XML_ctx *c) {
	c->ctx = NULL;
        c->p = NULL;
	c->message_body = NULL;
        c->eventCode = EDG_WLL_EVENT_UNDEF;
        c->position = 0;
        c->position2 = 0;
	c->row = 0;
	c->row2 = 0;
        c->level = 0;
        memset(&(c->element), 0, sizeof(c->element));
        c->char_buf = NULL;
        c->char_buf_len = 0;
        c->XML_tag = NULL;
        c->XML_tag2 = NULL;
	c->job_conditions = NULL;
	c->event_conditions = NULL;
	c->type = EDG_WLL_QUERY_TYPE_UNDEF;
	c->conditions = NULL;
	c->flags = 0;
        c->jobsOutGlobal = NULL;
        c->eventsOutGlobal = NULL;
        c->jobStatGlobal = NULL;
	memset(&(c->jobStatSingleGlobal),0,sizeof(c->jobStatSingleGlobal));
	c->strListGlobal = NULL;
	c->intListGlobal = NULL;
	c->tagListGlobal = NULL;
	c->stsListGlobal = NULL;
	memset(&(c->purgeRequestGlobal),0,sizeof(c->purgeRequestGlobal));
	memset(&(c->purgeResultGlobal),0,sizeof(c->purgeResultGlobal));
	memset(&(c->dumpRequestGlobal),0,sizeof(c->dumpRequestGlobal));
	memset(&(c->dumpResultGlobal),0,sizeof(c->dumpResultGlobal));
	c->errCode = 0;
	c->bound = 0;
	c->errDesc = NULL;
	c->stat_begin = 0;
        c->errtxt = NULL;
	c->warntxt = NULL;
}


void edg_wll_freeXMLCtx(edg_wll_XML_ctx *c) {
        if (c->char_buf) free(c->char_buf);
        if (c->errtxt) free(c->errtxt);
        if (c->warntxt) free(c->warntxt);
	if (c->XML_tag) free(c->XML_tag);
	if (c->XML_tag2) free(c->XML_tag2);
}


void edg_wll_freeBuf(edg_wll_XML_ctx *c) {
        free(c->char_buf);
        c->char_buf = NULL;
        c->char_buf_len = 0;
}



/************************************************************************/
/* type to XML conversion functions					*/


/* edg_wll_add_string_to_XMLBody(&body, eventsOut[i].jobMatch.destination, "destination", NULL) */

void edg_wll_add_string_to_XMLBody(char **body, const char *toAdd, const char *tag, const char *null)
{
	if ( toAdd != null ) {
		char *newBody;
		
		trio_asprintf(&newBody,"%s\t\t\t<%s>%|Xs</%s>\r\n", *body, tag, toAdd, tag);

		free(*body);
        	*body = newBody;
	}
}

void edg_wll_add_tagged_string_to_XMLBody(char **body, const char *toAdd, const char *tag, 
					  const char *name, const char *tag2, const char *null)
{
	if ( toAdd != null ) {
		char *newBody;
		
		trio_asprintf(&newBody,"%s\t\t\t<%s %s=\"%|Xs\">%|Xs</%s>\r\n", 
			*body, tag, tag2, name, toAdd, tag);

		free(*body);
        	*body = newBody;
	}
}

/* edg_wll_add_int_to_XMLBody(&body, eventsOut[i].jobMatch.code, "code", 0) */

void edg_wll_add_int_to_XMLBody(char **body, const int toAdd, const char *tag, const int null)
{
	if (toAdd != null) {
                char *newBody;

                trio_asprintf(&newBody,"%s\t\t\t<%s>%|Xd</%s>\r\n", *body, tag, toAdd, tag);

                free(*body);
                *body = newBody;
	}
}


/* edg_wll_add_timeval_to_XMLBody(&body, eventsOut[i].any.tv, "timestamp", -1) */

void edg_wll_add_timeval_to_XMLBody(char **body, struct timeval toAdd, const char *tag, const struct timeval null)
{

	if (toAdd.tv_sec != null.tv_sec || toAdd.tv_usec != null.tv_usec) {
                char *newBody;

                trio_asprintf(&newBody,"%s\t\t\t<%s>%ld.%06ld</%s>\r\n", 
		*body, tag, toAdd.tv_sec, toAdd.tv_usec, tag);

                free(*body);
                *body = newBody;
	}
}


/* edg_wll_add_jobid_to_XMLBody(&body, eventsOut[i].any.jobId, "jobId", NULL) */

void edg_wll_add_jobid_to_XMLBody(char **body, edg_wlc_JobId toAdd, const char *tag, const void *null)
{
	if (toAdd != (edg_wlc_JobId) null) {
                char *newBody, *pom;

                trio_asprintf(&newBody,"%s\t\t\t<%s>%|Xs</%s>\r\n",
                *body, tag, pom = edg_wlc_JobIdUnparse(toAdd), tag);

                free(*body);
		free(pom);
                *body = newBody;
	}
}


/* edg_wll_add_edg_wll_JobStatCode_to_XMLBody(&body, eventsOut[i].any.jobId, "jobId", EDG_WLL_JOB_UNDEF) */

void edg_wll_add_edg_wll_JobStatCode_to_XMLBody(char **body, edg_wll_JobStatCode toAdd, const char *tag, const edg_wll_JobStatCode null)
{
	if (toAdd != null) {
                char *newBody, *pom;

                trio_asprintf(&newBody,"%s\t\t\t<%s>%|Xs</%s>\r\n",
                	*body, tag, pom = edg_wll_StatToString(toAdd), tag);

                free(*body);
		free(pom);
                *body = newBody;
	}
}

void edg_wll_add_edg_wll_EventCode_to_XMLBody(char **body, edg_wll_EventCode toAdd, const char *tag, const edg_wll_EventCode null)
{
	char *newBody, *pom;
	
	if (toAdd != null) {
		trio_asprintf(&newBody,"%s\t\t\t<%s>%|Xs</%s>\r\n",
			*body, tag, pom = edg_wll_EventToString(toAdd), tag);	

		free(*body);
		free(pom);
		*body = newBody;
	}
}


void edg_wll_add_time_t_to_XMLBody(char **body, const time_t toAdd, const char *tag, const time_t null)
{
	if (toAdd != null) {
                char *newBody;

                trio_asprintf(&newBody,"%s\t\t\t<%s>%|Xld</%s>\r\n", *body, tag, toAdd, tag);

                free(*body);
                *body = newBody;
	}
}


void edg_wll_add_tagged_time_t_to_XMLBody(char **body, const time_t toAdd, const char *tag,
                                          const char *name, const char *tag2, const time_t null)
{
        if ( toAdd != null ) {
                char *newBody;

                trio_asprintf(&newBody,"%s\t\t\t<%s %s=\"%|Xs\">%|Xld</%s>\r\n",
                        *body, tag, tag2, name, toAdd, tag);

                free(*body);
                *body = newBody;
        }
}


void edg_wll_add_uint16_t_to_XMLBody(char **body, const uint16_t toAdd, const char *tag, const uint16_t null)
{
	if (toAdd != null) {
                char *newBody;

                trio_asprintf(&newBody,"%s\t\t\t<%s>%|Xu</%s>\r\n", *body, tag, toAdd, tag);

                free(*body);
                *body = newBody;
	}
}


void edg_wll_add_logsrc_to_XMLBody(char **body, const edg_wll_Source toAdd, const char *tag, const edg_wll_Source null)
{
	 if (toAdd != null) {
		char *newBody, *pom;

		trio_asprintf(&newBody,"%s\t\t\t<%s>%|Xs</%s>\r\n",
                        *body, tag, pom = edg_wll_SourceToString(toAdd), tag);
		
		free(*body);
		free(pom);
		*body = newBody;
	}
}

void edg_wll_add_strlist_to_XMLBody(char **body, char * const *toAdd, const char *tag,  const char *subTag, const char *indent,  const  char *null)
{
        char *pomA, *pomB, *newBody;
        char **list = NULL;
        int i = 0, len, tot_len = 0;
        int *len_list = NULL;

	
	if (!toAdd) return;
		
        while (toAdd[i] != null) {
                len = trio_asprintf(&pomA,"%s\t<%s>%|Xs</%s>\r\n",
                        indent,subTag,toAdd[i],subTag);

                i++;
                tot_len += len;

                list      = (char **) realloc(list, i * sizeof(*list));
                list[i-1] = pomA;
                pomA = NULL;
                len_list      = (int *) realloc(len_list, i * sizeof(*len_list));
                len_list[i-1] = len;
        }

        /* list termination */
        list = (char **) realloc(list, (i+1) * sizeof(*list));
        list[i] = NULL;

        /* glueing all list fields together & freeing the list */
        pomA = (char *) malloc(tot_len  * sizeof(char) + 1);
	pomB = pomA;

        i = 0;
        while (list[i]) {
                memcpy(pomB, list[i], len_list[i] );
                pomB += len_list[i];

                /* freeing the list */
                free(list[i]);

                i++;
        }
	*pomB = '\0';
        free(list);
        free(len_list);

	asprintf(&newBody,"%s%s<%s>\r\n%s%s</%s>\r\n", *body, indent, tag, pomA, indent, tag);
	free(*body);
	free(pomA);
	*body = newBody;	
}

void edg_wll_add_intlist_to_XMLBody(char **body, const int *toAdd, const char *tag, const char *subTag, const char *indent, const int from, const int to)
{
        char *pomA, *pomB, *newBody;
        char **list = NULL;
        int i, len, tot_len = 0;
        int *len_list = NULL;

	
	i = from;
        while (i < to) {
                len = trio_asprintf(&pomA,"%s\t<%s>%|Xd</%s>\r\n",
                        indent, subTag,toAdd[i],subTag);

		i++;
                tot_len += len;

                list      = (char **) realloc(list, i * sizeof(*list));
                list[i-1] = pomA;
                pomA = NULL;
                len_list      = (int *) realloc(len_list, i * sizeof(*len_list));
                len_list[i-1] = len;
        }

        /* list termination */
        list = (char **) realloc(list, (i+1) * sizeof(*list));
        list[i] = NULL;

        /* glueing all list fields together & freeing the list */
        pomA = (char *) malloc(tot_len  * sizeof(char) + 1);
	pomB = pomA;

        i = from;
        while (list[i]) {
                memcpy(pomB, list[i], len_list[i] );
                pomB += len_list[i];

                /* freeing the list */
                free(list[i]);

                i++;
        }
	*pomB = '\0';
        free(list);
        free(len_list);

	asprintf(&newBody,"%s%s<%s>\r\n%s%s</%s>\r\n", *body, indent, tag, pomA, indent, tag);
        free(*body);
        free(pomA);
        *body = newBody;
}


void edg_wll_add_taglist_to_XMLBody(char **body,  const edg_wll_TagValue *toAdd, const char *tag,  const char *subTag, const char *subTag2, const char *indent, const  char *null)
{
        char *pomA, *pomB, *newBody;
        char **list = NULL;
        int i = 0, len, tot_len = 0;
        int *len_list = NULL;


        while (toAdd && (toAdd[i].tag != null) ) {
                len = trio_asprintf(&pomA,"%s\t<%s %s=\"%|Xs\">%|Xs</%s>\r\n",
                        indent,subTag,subTag2,toAdd[i].tag,toAdd[i].value,subTag);

                i++;
                tot_len += len;

                list      = (char **) realloc(list, i * sizeof(*list));
                list[i-1] = pomA;
                pomA = NULL;
                len_list      = (int *) realloc(len_list, i * sizeof(*len_list));
                len_list[i-1] = len;
        }

        /* list termination */
        list = (char **) realloc(list, (i+1) * sizeof(*list));
        list[i] = NULL;

        /* glueing all list fields together & freeing the list */
        pomA = (char *) malloc(tot_len  * sizeof(char) + 1);
	pomB = pomA;

        i = 0;
        while (list[i]) {
                memcpy(pomB, list[i], len_list[i] );
                pomB += len_list[i];

                /* freeing the list */
                free(list[i]);

                i++;
        }
	*pomB = '\0';
        free(list);
        free(len_list);

	asprintf(&newBody,"%s%s<%s>\r\n%s%s</%s>\r\n", *body, indent, tag, pomA, indent, tag);
	free(*body);
	free(pomA);
	*body = newBody;	
}


void edg_wll_add_time_t_list_to_XMLBody(char **body, const time_t *toAdd, const char *tag, const char *subTag, const char *indent, const int from, const int to)
{
        char *pomA, *pomB, *newBody;
        char **list = NULL;
        int i, len, tot_len = 0;
        int *len_list = NULL;

	
	i = from;
        while (i < to) {
                len = trio_asprintf(&pomA,"%s\t<%s>%|Xld</%s>\r\n",
                        indent, subTag,toAdd[i],subTag);

		i++;
                tot_len += len;

                list      = (char **) realloc(list, i * sizeof(*list));
                list[i-1] = pomA;
                pomA = NULL;
                len_list      = (int *) realloc(len_list, i * sizeof(*len_list));
                len_list[i-1] = len;
        }

        /* list termination */
        list = (char **) realloc(list, (i+1) * sizeof(*list));
        list[i] = NULL;

        /* glueing all list fields together & freeing the list */
        pomA = (char *) malloc(tot_len  * sizeof(char) + 1);
	pomB = pomA;

        i = 0;
        while (list[i]) {
                memcpy(pomB, list[i], len_list[i] );
                pomB += len_list[i];

                /* freeing the list */
                free(list[i]);

                i++;
        }
	*pomB = '\0';
        free(list);
        free(len_list);

	asprintf(&newBody,"%s%s<%s>\r\n%s%s</%s>\r\n", *body, indent, tag, pomA, indent, tag);
        free(*body);
        free(pomA);
        *body = newBody;
}


// void edg_wll_add_stslist_to_XMLBody(char **body, const edg_wll_JobStat *toAdd, const char *tag, const char *UNUSED_subTag, const int null)
// in lbserver/lb_xml_parse.c.T

/************************************************************************/
/* string to type conversion functions					*/


/* XMLCtx->eventsOutGlobal[XMLCtx->position].any.prog = edg_wll_from_string_to_string(XMLCtx); */
char *edg_wll_from_string_to_string(edg_wll_XML_ctx *XMLCtx)
{
	return(XMLCtx->char_buf);
} 


/* XMLCtx->eventsOutGlobal[XMLCtx->position].any.jobId = edg_wll_from_string_to_dgJobId(XMLCtx); */
edg_wlc_JobId edg_wll_from_string_to_jobid(edg_wll_XML_ctx *XMLCtx)
{
	edg_wlc_JobId out;

	edg_wlc_JobIdParse(XMLCtx->char_buf, &out);
	edg_wll_freeBuf(XMLCtx); 

	return(out);
}


/* XMLCtx->eventsOutGlobal[XMLCtx->position].jobClear.clearReason = 
	edg_wll_from_string_to_int(XMLCtx); 				*/
int edg_wll_from_string_to_int(edg_wll_XML_ctx *XMLCtx)
{
        int out;

        out = atoi(XMLCtx->char_buf);
        edg_wll_freeBuf(XMLCtx);

        return(out);
}


long edg_wll_from_string_to_long(edg_wll_XML_ctx *XMLCtx)
{
        long out;

        out = atol(XMLCtx->char_buf);
        edg_wll_freeBuf(XMLCtx);

        return(out);
}


uint16_t edg_wll_from_string_to_uint16_t(edg_wll_XML_ctx *XMLCtx)
{
        return( (uint16_t) edg_wll_from_string_to_int(XMLCtx) );
}


struct timeval edg_wll_from_string_to_timeval(edg_wll_XML_ctx *XMLCtx)
{
	struct timeval out;
	char *needle, *nil;

	out.tv_sec  = strtol(XMLCtx->char_buf, &needle, 10);
	out.tv_usec = strtol(needle+1, &nil, 10);
	edg_wll_freeBuf(XMLCtx);

	return(out);
}


time_t edg_wll_from_string_to_time_t(edg_wll_XML_ctx *XMLCtx)
{
        return( (time_t) edg_wll_from_string_to_long(XMLCtx) );
}


edg_wll_Source edg_wll_from_string_to_logsrc(edg_wll_XML_ctx *XMLCtx)
{
	edg_wll_Source	out = edg_wll_StringToSource(XMLCtx->char_buf);
	
	edg_wll_freeBuf(XMLCtx);
	return(out);
}



/************************************************************************/
/* various conversion functions					*/


char *edg_wll_flags_to_string(int flags)
{
        char *cflags = NULL, *temp_cflags = NULL;


        if (flags & EDG_WLL_STAT_CLASSADS) asprintf(&cflags,"%s","classadd");
        if (flags & EDG_WLL_STAT_CHILDREN) {
                if (cflags) {
                        asprintf(&temp_cflags,"%s+%s",cflags,"children");
                        free(cflags);
                        cflags=temp_cflags;
                }
                else asprintf(&cflags,"%s","children");
        }
        if (flags & EDG_WLL_STAT_CHILDSTAT) {
                if (cflags) {
                        asprintf(&temp_cflags,"%s+%s",cflags,"childstat");
                        free(cflags);
                        cflags=temp_cflags;
                }
                else asprintf(&cflags,"%s","childstat");
        }
        if (flags & EDG_WLL_STAT_NO_JOBS) {
                if (cflags) {
                        asprintf(&temp_cflags,"%s+%s",cflags,"no_jobs");
                        free(cflags);
                        cflags=temp_cflags;
                }
                else asprintf(&cflags,"%s","no_jobs");
        }
        if (flags & EDG_WLL_STAT_NO_STATES) {
                if (cflags) {
                        asprintf(&temp_cflags,"%s+%s",cflags,"no_states");
                        free(cflags);
                        cflags=temp_cflags;
                }
                else asprintf(&cflags,"%s","no_states");
        }

        if (!cflags) cflags = strdup("");

        return(cflags);
}


int edg_wll_string_to_flags(char *cflags)
{
	int flags = 0;
	char *sflag, *last;

	if (cflags == NULL) return 0;

	sflag = strtok_r(cflags, "+", &last);
	while (sflag != NULL) {
		if (!strcmp(sflag,"classadd")) flags = flags | EDG_WLL_STAT_CLASSADS;
		if (!strcmp(sflag,"children")) flags = flags | EDG_WLL_STAT_CHILDREN;
		if (!strcmp(sflag,"childstat")) flags = flags | EDG_WLL_STAT_CHILDSTAT;
		if (!strcmp(sflag,"no_jobs")) flags = flags | EDG_WLL_STAT_NO_JOBS;
		if (!strcmp(sflag,"no_states")) flags = flags | EDG_WLL_STAT_NO_STATES;
		sflag = strtok_r(NULL, "+", &last);
	} 

	return(flags);
}
