#ident "$Header: /local/reps/lcgware/workload/logging/lbserver/dump.c,v 1.4 2003/11/12 11:20:21 ljocha Exp $"

#include <time.h>
#include <stdio.h>
#include <string.h>
#include <syslog.h>
#include <assert.h>
#include <unistd.h>

#include "edg/workload/thirdparty/trio/trio.h"
#include "edg/workload/common/jobid/jobid.h"

#include "edg/workload/logging/client/context.h"
#include "edg/workload/logging/common/context.h"
#include "edg/workload/logging/common/events_parse.h"
#include "edg/workload/logging/client/dump.h"
#include "edg/workload/logging/lbserver/lbs_db.h"
#include "edg/workload/logging/lbserver/query.h"
#include "edg/workload/logging/lbserver/get_events.h"
#include "edg/workload/logging/lbserver/server_state.h"

static int handle_specials(edg_wll_Context,time_t *);

#define sizofa(a) (sizeof(a)/sizeof((a)[0]))

int edg_wll_DumpEvents(edg_wll_Context ctx,const edg_wll_DumpRequest *req,edg_wll_DumpResult *result)
{
	char	*from_s, *to_s, *stmt;
	time_t	start,end;
	edg_wll_Stmt	q = NULL;
	char		*res[9];
	int	event;
	edg_wll_Event	e;
	int	ret,dump = 2;	/* TODO: manage dump file */
	time_t	from = req->from,to = req->to;

	from_s = to_s = stmt = NULL;
	memset(res,0,sizeof res);
	memset(&e,0,sizeof e);

	time(&start);
	edg_wll_ResetError(ctx);
	if (handle_specials(ctx,&from) || handle_specials(ctx,&to))
		return edg_wll_Error(ctx,NULL,NULL);

	from_s = strdup(edg_wll_TimeToDB(from));
	to_s = strdup(edg_wll_TimeToDB(to));

	trio_asprintf(&stmt,"select event,dg_jobid,code,prog,host,u.cert_subj,time_stamp,usec,level "
			"from events e,users u,jobs j "
			"where u.userid=e.userid "
			"and j.jobid = e.jobid "
			"and j.dg_jobid like 'https://%|Ss:%d%%' "
			"and arrived > '%|Ss' and arrived <= '%|Ss'",
			ctx->srvName,ctx->srvPort,
			from_s,to_s);

	if (edg_wll_ExecStmt(ctx,stmt,&q) < 0) goto clean;

	while ((ret = edg_wll_FetchRow(q,res)) > 0) {
		assert(ret == sizofa(res));
		event = atoi(res[0]); free(res[0]); res[0] = NULL;

		if (convert_event_head(ctx,res+1,&e)
			|| edg_wll_get_event_flesh(ctx,event,&e))
		{
			char	*et,*ed;
			int	i;

		/* Most likely sort of internal inconsistency. 
		 * Must not be fatal -- just complain
		 */
			edg_wll_Error(ctx,&et,&ed);
			fprintf(stderr,"%s event %d: %s (%s)\n",res[1],event,et,ed);
			syslog(LOG_WARNING,"%s event %d: %s (%s)",res[1],event,et,ed);
			free(et); free(ed);
			for (i=0; i<sizofa(res); i++) free(res[i]);
			edg_wll_ResetError(ctx);
		}
		else {
			char	*event_s = edg_wll_UnparseEvent(ctx,&e);
			int	len = strlen(event_s), written, total = 0;
			while (total != len) {
				written = write(dump,event_s+total,len-total);
				if (written < 0 && errno != EAGAIN) {
					edg_wll_SetError(ctx,errno,"writing dump file");
					free(event_s);
					goto clean;
				}
				total += written;
			}
			write(dump,"\n",1);
			free(event_s);
		}
		edg_wll_FreeEvent(&e); memset(&e,0,sizeof e);
	}

	time(&end);
	edg_wll_SetServerState(ctx,EDG_WLL_STATE_DUMP_START,edg_wll_TimeToDB(start));
	edg_wll_SetServerState(ctx,EDG_WLL_STATE_DUMP_END,edg_wll_TimeToDB(end));
	result->from = from;
	result->to = to;

clean:
	edg_wll_FreeEvent(&e);
	edg_wll_FreeStmt(&q);

	free(stmt);
	free(from_s);
	free(to_s);
	return edg_wll_Error(ctx,NULL,NULL);
}

static int handle_specials(edg_wll_Context ctx,time_t *t)
{
	char	*time_s;

	edg_wll_ResetError(ctx);
	switch (*t) {
		case EDG_WLL_DUMP_NOW:
			time(t);
			return 0;
		case EDG_WLL_DUMP_LAST_START:
		case EDG_WLL_DUMP_LAST_END:
			switch (edg_wll_GetServerState(ctx,
					*t == EDG_WLL_DUMP_LAST_START ? 
						EDG_WLL_STATE_DUMP_START:
						EDG_WLL_STATE_DUMP_END,
					&time_s))
			{
				case ENOENT: *t = 0; break;
				case 0: *t = edg_wll_DBToTime(time_s); break;
				default: break;
			}
			break;
		default: if (*t < 0) return edg_wll_SetError(ctx,EINVAL,"special time limit unrecognized");
	}
	return edg_wll_Error(ctx,NULL,NULL);
}


