#ident "$Header: /local/reps/lcgware/workload/logging/lbserver/srv_purge.c,v 1.4 2003/09/19 12:19:16 ljocha Exp $"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <syslog.h>
#include <assert.h>
#include <string.h>

#include "edg/workload/thirdparty/trio/trio.h"

#include "edg/workload/common/jobid/jobid.h"
#include "edg/workload/logging/common/context.h"
#include "edg/workload/logging/common/events_parse.h"
#include "edg/workload/logging/common/mini_http.h"
#include "edg/workload/logging/lbserver/lb_html.h"
#include "edg/workload/logging/lbserver/lb_proto.h"
#include "edg/workload/logging/lbserver/store.h"
#include "edg/workload/logging/lbserver/lock.h"
#include "edg/workload/logging/lbserver/lbs_db.h"
#include "edg/workload/logging/lbserver/query.h"
#include "edg/workload/logging/lbserver/get_events.h"
#include "edg/workload/logging/client/purge.h"
#include "edg/workload/logging/client/dump.h"
#include "edg/workload/logging/common/purge.h"
#include "edg/workload/logging/lbserver/lb_xml_parse.h"


#define DUMP_FILE_STORAGE					"/tmp/"

#define sizofa(a) (sizeof(a)/sizeof((a)[0]))

static const char* const resp_headers[] = {
	"Cache-Control: no-cache",
	"Accept: application/x-dglb",
	"User-Agent: edg_wll_Server/" PROTO_VERSION "/" COMP_PROTO,
	"Content-Type: application/x-dglb",
	NULL
};

static int purge_one(edg_wll_Context ctx,const edg_wlc_JobId,int,int);


int edg_wll_PurgeServer(edg_wll_Context ctx,const edg_wll_PurgeRequest *request)
{
	int	i,parse = 0,dumpfile = -1;
	edg_wlc_JobId	job;
	char	*message = NULL, *response = NULL;
	int	naffected_jobs = 0;
	edg_wll_PurgeResult result;
	int		ret = HTTP_OK;


	if (!ctx->noAuth) {
		edg_wll_SetError(ctx,EPERM,"only superusers may purge");
		goto abort;
	}

	edg_wll_ResetError(ctx);
	memset(&result, 0, sizeof(edg_wll_PurgeResult));


	if (request->flags & EDG_WLL_PURGE_SERVER_DUMP) 
	{
		unsigned int seq = 0;
		char fname[255];
		struct timeval tv;

		gettimeofday(&tv, NULL);
		while ( 1 )
		{
			sprintf(fname, "%s%s_%ld_%d", ctx->purgeStorage, "dump", (long int)tv.tv_sec, seq);
			if ( (dumpfile = open(fname, O_WRONLY|O_CREAT|O_EXCL|O_APPEND,0600)) == -1 )
			{
				if ( errno != EEXIST )
					return edg_wll_SetError(ctx, errno, "Purge couldn't create server dump file");
			}
			else
				break;
			seq++;
		}

		result.server_file = strdup(fname);
	}

	if (request->jobs) for (i=0; request->jobs[i]; i++) {
		if (edg_wlc_JobIdParse(request->jobs[i],&job)) {
			fprintf(stderr,"%s: parse error\n",request->jobs[i]);
			parse = 1;
		}
		else {
			if (check_strict_jobid(ctx,job)) {
				fprintf(stderr,"%s: not my job\n",request->jobs[i]);
				parse = 1;
			}
			else {
				if (purge_one(ctx,job,dumpfile,request->flags&EDG_WLL_PURGE_REALLY_PURGE))
					goto abort;

				if (request->flags & EDG_WLL_PURGE_LIST_JOBS) {
					result.jobs = realloc(result.jobs,(naffected_jobs+2) * sizeof(*result.jobs));
					result.jobs[naffected_jobs] = request->jobs[i];
					result.jobs[naffected_jobs+1] = NULL;
					naffected_jobs++;
				}
			}
			edg_wlc_JobIdFree(job);
		}
	}
	else {
		edg_wll_Stmt	s;
		char		*job_s;
		int		res;
		time_t		timeout[EDG_WLL_NUMBER_OF_STATCODES],
				now = time(NULL);

		for (i=0; i<EDG_WLL_NUMBER_OF_STATCODES; i++)
			timeout[i] = request->timeout[i] < 0 ? ctx->purge_timeout[i] : request->timeout[i];

		if (edg_wll_ExecStmt(ctx,"select dg_jobid from jobs",&s) < 0) goto abort;
		while ((res = edg_wll_FetchRow(s,&job_s)) > 0) {
			if (edg_wlc_JobIdParse(job_s,&job)) {
				fprintf(stderr,"%s: parse error (internal inconsistency !)\n",job_s);
				parse = 1;
			}
			else {
				edg_wll_JobStat	stat;

				memset(&stat,0,sizeof stat);
				if (edg_wll_JobStatus(ctx,job,0,&stat)) goto abort; /* XXX: memory leak */

				switch (stat.state) {
					case EDG_WLL_JOB_CLEARED:
					case EDG_WLL_JOB_ABORTED:
					case EDG_WLL_JOB_CANCELLED:
						i = stat.state;
						break;
					default:
						i = EDG_WLL_PURGE_JOBSTAT_OTHER;
				}

				if (now-stat.lastUpdateTime.tv_sec > timeout[i]
					&& !check_strict_jobid(ctx,job))
				{
					if (purge_one(ctx,job,dumpfile,request->flags&EDG_WLL_PURGE_REALLY_PURGE))
						goto abort;

				/* XXX: change with the streaming interface */
					if (request->flags & EDG_WLL_PURGE_LIST_JOBS) {
						result.jobs = realloc(result.jobs,(naffected_jobs+2) * sizeof(*result.jobs));
						result.jobs[naffected_jobs] = job_s;
						result.jobs[++naffected_jobs] = NULL;
						job_s = NULL;
					}
				}

				edg_wlc_JobIdFree(job);
				edg_wll_FreeStatus(&stat);
				free(job_s);
			}
		}
		edg_wll_FreeStmt(&s);
abort:
	}

	if (parse && !edg_wll_Error(ctx,NULL,NULL))
		edg_wll_SetError(ctx,EINVAL,"Invalid JobId(s) but other jobs purged");

	switch ( edg_wll_Error(ctx,NULL,NULL) )
	{
		case 0:
			ret = HTTP_OK;
			break;
		case EINVAL:
			ret = HTTP_INVALID;
			break;
		case EPERM:
			ret = HTTP_UNAUTH;
			break;
		
		/* fatal errors */
		case ENOMEM:
			/* fall through */
		default:
			ret = HTTP_INTERNAL;
			break;
	}
	
	if (ret != HTTP_INTERNAL) {
		if ( edg_wll_PurgeResultToXML(ctx, &result, &message) )
			ret = HTTP_INTERNAL;
		else 
			printf("%s", message);
	}
	
	asprintf(&response, "HTTP/1.1 %d %s", ret, edg_wll_HTTPErrorMessage(ret));

	edg_wll_http_send(ctx, response, resp_headers, message);

	return edg_wll_Error(ctx,NULL,NULL);
}

static void unlock_and_check(edg_wll_Context ctx,edg_wlc_JobId job)
{
	char	*job_s,*et,*ed;

	if (edg_wll_UnlockJob(ctx,job)) {
		job_s = edg_wlc_JobIdUnparse(job);

		edg_wll_Error(ctx,&et,&ed);
		fprintf(stderr,"%s: edg_wll_UnlockJob(): %s (%s) -- expect bogus things\n",
				job_s,et,ed);
		syslog(LOG_CRIT,"%s: edg_wll_UnlockJob(): %s (%s) -- expect bogus things",
				job_s,et,ed);
		free(et); free(ed); free(job_s);
	}
}


int purge_one(edg_wll_Context ctx,const edg_wlc_JobId job,int dump, int purge)
{
	char	*dbjob;
	char	*stmt = NULL;
	edg_wll_Stmt	q;
	int		ret;

	edg_wll_ResetError(ctx);
	if ( !purge && dump < 0 ) return 0;

	dbjob = edg_wlc_JobIdGetUnique(job);	/* XXX: strict jobid already checked */
	if (edg_wll_LockJob(ctx,job)) goto clean;

	if ( purge )
	{
		trio_asprintf(&stmt,"delete from jobs where jobid = '%|Ss'",dbjob);
		if (edg_wll_ExecStmt(ctx,stmt,NULL) < 0) {
			unlock_and_check(ctx,job);
			goto clean;
		}
		free(stmt); stmt = NULL;

		trio_asprintf(&stmt,"delete from states where jobid = '%|Ss'",dbjob);
		if (edg_wll_ExecStmt(ctx,stmt,NULL) < 0) {
			unlock_and_check(ctx,job);
			goto clean;
		}
		free(stmt); stmt = NULL;

/* Why on earth ?
		trio_asprintf(&stmt,"delete from states where jobid = '%|Ss'",dbjob);
		if (edg_wll_ExecStmt(ctx,stmt,NULL) < 0) {
			unlock_and_check(ctx,job);
			goto clean;
		}
		free(stmt); stmt = NULL;
*/

	}

	if (!ctx->strict_locking) unlock_and_check(ctx,job);

	if ( purge )
	{
		trio_asprintf(&stmt,"delete from status_tags where jobid = '%|Ss'",dbjob);
		if (edg_wll_ExecStmt(ctx,stmt,NULL) < 0) goto unlock;
		free(stmt); stmt = NULL;
	}

	if (dump >= 0) 
		trio_asprintf(&stmt,"select event,code,prog,host,u.cert_subj,time_stamp,usec,level "
			"from events e,users u "
			"where e.jobid='%|Ss' "
			"and u.userid=e.userid "
			"order by event", dbjob);
	else
		trio_asprintf(&stmt,"select event from events "
			"where jobid='%|Ss' "
			"order by event", dbjob);

/* check for events repeatedly -- new one may have arrived in the meantime */
	while ((ret = edg_wll_ExecStmt(ctx,stmt,&q)) > 0) {
		char	*res[8];

		while ((ret = edg_wll_FetchRow(q,res)) > 0) {
			int	event;

			event = atoi(res[0]);
			free(res[0]); res[0] = NULL;

			if (dump >= 0) {
				edg_wll_Event	e;

				assert(ret == 8);
				res[0] = edg_wlc_JobIdUnparse(job);
				if (convert_event_head(ctx,res,&e) || edg_wll_get_event_flesh(ctx,event,&e))
				{
					char	*et,*ed;
					int	i;

				/* Most likely sort of internal inconsistency. 
				 * Must not be fatal -- just complain
				 */
					edg_wll_Error(ctx,&et,&ed);
					fprintf(stderr,"%s event %d: %s (%s)\n",dbjob,event,et,ed);
					syslog(LOG_WARNING,"%s event %d: %s (%s)",dbjob,event,et,ed);
					free(et); free(ed);
					for (i=0; i<sizofa(res); i++) free(res[i]);
					edg_wll_ResetError(ctx);
				}
				else {
					char	*event_s = edg_wll_UnparseEvent(ctx,&e);
					int	len = strlen(event_s), written, total = 0;
					while (total != len) {
						written = write(dump,event_s+total,len-total);
						if (written < 0 && errno != EAGAIN) {
							perror("dump to file");
							syslog(LOG_ERR,"dump to file: %m");
							dump = -1; /* XXX: likely to be a permanent error
								    * give up writing but do purge */
							break;
						}
						total += written;
					}
					/* write(dump,"\n",1); edg_wll_UnparseEvent does so */
					free(event_s);
				}
				edg_wll_FreeEvent(&e);
			}

			if ( purge ) {
				if (edg_wll_delete_event(ctx,dbjob,event)) {
					char	*et,*ed;

				/* XXX: just complain and carry on. Is it OK? */
					edg_wll_Error(ctx,&et,&ed);
					fprintf(stderr,"%s event %d: %s (%s)\n",dbjob,event,et,ed);
					syslog(LOG_WARNING,"%s event %d: %s (%s)",dbjob,event,et,ed);
					free(et); free(ed);
					edg_wll_ResetError(ctx);
				}
			}
		}
		edg_wll_FreeStmt(&q);
		if (ret < 0 || !purge) break;
	}
	edg_wll_FreeStmt(&q);

unlock:
	if (ctx->strict_locking) unlock_and_check(ctx,job);

clean:
	free(dbjob);
	free(stmt);
	return edg_wll_Error(ctx,NULL,NULL);
}
