#ident "$Header: /local/reps/lcgware/workload/logging/lbserver/write2rgma.cpp,v 1.4 2005/09/06 15:08:26 dhsmith Exp $"

#include <assert.h>
#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/file.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/un.h>
#include <sys/uio.h>

#include <string>
#include <fstream>
#include <memory>

#include <classad_distribution.h>

#include <boost/filesystem/path.hpp>
#include <boost/filesystem/operations.hpp>

#include "edg/workload/logging/lbserver/get_events.h"

#include "edg/workload/logging/client/producer.h"
#include "edg/workload/logging/common/context.h"
#include "edg/workload/logging/lbserver/store.h"
#include "edg/workload/logging/lbserver/lock.h"
#include "edg/workload/logging/lbserver/index.h"

#include "edg/workload/logging/client/jobstat.h"
#include "edg/workload/logging/lbserver/jobstat.h"
#include "edg/workload/thirdparty/trio/trio.h"


#include "edg/workload/common/logger/edglog.h"
#include "edg/workload/common/requestad/PrivateAdManipulation.h"
#include "edg/workload/common/logger/manipulators.h"
#include "edg/workload/common/requestad/JobAdManipulation.h"
#include "edg/workload/common/requestad/ManipulationExceptions.h"

using namespace std;

/* write2rgma_sql():
   old_style causes this func to write out the rgma statefile in the old
   format: ie. including length bytes, null termination and using flock()
   locking. The new style does not include length bytes, null terminator
   and uses fcntl() locking
*/
static void write2rgma_sql(char *sqlstat, int old_style)
{
	int slen;
	struct iovec iov[2];
        struct stat stat_buf,stat_fbuf;
        struct flock filelock;
        int filelock_status;
        char *rgma_sockname;

        static int	rgma_sock = -1;
        static int	rgma_fd = -1;
        static int      rgma_fd_locked = 0;
        static struct sockaddr_un	rgma_saddr;
        static char	*rgma_fname = NULL;

	assert(sqlstat != NULL);

        if (rgma_fd < -1) {
                goto leave;
        } else if (rgma_fd == -1) {
                if (rgma_fname == NULL) {
  		  rgma_fname = getenv("EDG_WL_RGMA_FILE");
                }
		if (rgma_fname == NULL) {
                  rgma_fd = -2;
                  goto leave;
                }

		rgma_fd = open(rgma_fname, O_WRONLY|O_CREAT|O_APPEND, 0777);
		if (rgma_fd == -1) goto leave;
	}

	if (rgma_sock == -1) {
		rgma_sockname = getenv("EDG_WL_RGMA_SOCK");
		if (rgma_sockname != NULL) {

 		  if ((strlen(rgma_sockname) + 1) > sizeof(rgma_saddr.sun_path)) {
                        rgma_sock = -2;
			goto leave;
                  }

  		  rgma_sock = socket(PF_UNIX, SOCK_DGRAM,0);
		  if (rgma_sock == -1) goto leave;
		
		  memset(&rgma_saddr, sizeof(rgma_saddr), 0);
		  rgma_saddr.sun_family = PF_UNIX;
		  strcpy(rgma_saddr.sun_path, rgma_sockname);
                } else {
                  rgma_sock = -2;
                }
	}

	slen = strlen(sqlstat);
        if (old_style) {
  	  iov[0].iov_base = &slen;
	  iov[0].iov_len = sizeof(slen);
	  iov[1].iov_base = sqlstat;
	  iov[1].iov_len = slen + 1;
        } else {
          iov[0].iov_base = sqlstat;
          iov[0].iov_len = slen;
        }

        if (!rgma_fd_locked) {
          if (old_style) {
            filelock_status = flock(rgma_fd, LOCK_SH);
          } else {
            do {
              filelock.l_type = F_WRLCK;
              filelock.l_whence = SEEK_SET;
              filelock.l_start = 0;
              filelock.l_len = 0;
            } while((filelock_status = fcntl(rgma_fd,F_SETLKW,&filelock))<0 && errno==EINTR);
          }
          
          if (filelock_status<0) goto leave;
        }

        rgma_fd_locked = 1;

        if (fstat(rgma_fd, &stat_fbuf)<0 || stat(rgma_fname, &stat_buf)<0) {
          goto leave;
        }

        if (stat_fbuf.st_dev != stat_buf.st_dev || stat_fbuf.st_ino != stat_buf.st_ino) {
          goto leave;
        }
	
        if (writev(rgma_fd, iov, old_style ? 2 : 1) < 0) {
          goto leave;
        }

        fsync(rgma_fd);

        if (old_style) {
          filelock_status = flock(rgma_fd, LOCK_UN);
        } else {
          do {
            filelock.l_type = F_UNLCK;
            filelock.l_whence = SEEK_SET;
            filelock.l_start = 0;
            filelock.l_len = 0;
          } while((filelock_status = fcntl(rgma_fd,F_SETLKW,&filelock))<0 && errno==EINTR);
        }

        if (filelock_status < 0) goto leave;

        rgma_fd_locked = 0;

        if (rgma_sock >= 0) {
  	  (void)sendto(rgma_sock, &slen, 1, old_style ? 0 : MSG_DONTWAIT,
		  (struct sockaddr*) &rgma_saddr, SUN_LEN(&rgma_saddr));
        }

leave:
        /* add explicit unlock, incase the later close should fail for some reason */
        if (rgma_fd>=0 && rgma_fd_locked) {
          if (old_style) {
            filelock_status = flock(rgma_fd, LOCK_UN);
          } else {
            do {
              filelock.l_type = F_UNLCK;
              filelock.l_whence = SEEK_SET;
              filelock.l_start = 0;
              filelock.l_len = 0;
            } while((filelock_status = fcntl(rgma_fd,F_SETLKW,&filelock))<0 && errno==EINTR);
          }

          if (filelock_status == 0) {
            rgma_fd_locked = 0;
          }
        }

        /* close the statefile */
        if (rgma_fd>=0 && close(rgma_fd)==0) {
          rgma_fd = -1;
          rgma_fd_locked = 0;
        }

	return;
}

void write2rgma_status(edg_wll_JobStat *stat)
{
	char *msg = NULL;
	char *rgma_fname;
        static int mode = 0;
	char *string_jobid, *string_stat, *string_server;
        int include_len;

	string_jobid = edg_wlc_JobIdUnparse(stat->jobId);
	string_stat = edg_wll_StatToString(stat->state);
        string_server = edg_wlc_JobIdGetServer(stat->jobId);

        if (mode == 0) {
  	  rgma_fname = getenv("EDG_WL_RGMA_FILE");
     	  if (rgma_fname != NULL && strlen(rgma_fname)>4 && !strcmp(&rgma_fname[strlen(rgma_fname)-4],".dat")) {
            mode = 1;
          } else {
            mode = 2;
          }
        }

        if (mode == 1) {
          include_len = 1;
          trio_asprintf(&msg, "INSERT INTO JobStatusRaw VALUES ('%|Ss','%s','%|Ss','%|Ss', '%d%03d')",
                        string_jobid, string_stat, stat->owner, string_server,
                        stat->stateEnterTime.tv_sec, stat->stateEnterTime.tv_usec/1000);
        } else {
          size_t msg_size = 0;
          classad::ClassAdParser parser;
          auto_ptr<classad::ClassAd> jobadp;
          char *stmt[13];
          int i;

          include_len = 0;
          jobadp.reset(parser.ParseClassAd(stat->jdl));

          std::string vo;
          if (jobadp.get() != NULL) {
            bool good;
            std::string s = edg::workload::common::requestad::get_virtual_organisation(*jobadp,good);
            if (good) vo = s;
          }

          trio_asprintf(&stmt[0], "JOBID=%|Ss ", string_jobid);
          trio_asprintf(&stmt[1], "OWNER=%|Ss ", stat->owner);
          trio_asprintf(&stmt[2], "BKSERVER=%|Ss ", string_server);
          trio_asprintf(&stmt[3], "NETWORKSERVER=%|Ss ", stat->network_server);
          trio_asprintf(&stmt[4], "VO=%|Ss ", vo.empty() ? "" : vo.c_str());
          trio_asprintf(&stmt[5], "LASTUPDATETIME=%d ", stat->lastUpdateTime.tv_sec);
          trio_asprintf(&stmt[6], "STATENAME=%s ", string_stat);
          trio_asprintf(&stmt[7], "STATEENTERTIME=%d ", stat->stateEnterTime.tv_sec);
          trio_asprintf(&stmt[8], "CONDORID=%|Ss ", (stat->condorId) ? (stat->condorId) : "");
          trio_asprintf(&stmt[9], "DESTINATION=%|Ss ", (stat->destination) ? (stat->destination) : "");
          trio_asprintf(&stmt[10], "EXITCODE=%d ", stat->exit_code);
          trio_asprintf(&stmt[11], "DONECODE=%d ", stat->done_code);
          trio_asprintf(&stmt[12], "STATUSREASON=%|Ss\n", (stat->reason) ? (stat->reason) : "");

          for(i=0;i<13;++i) {
            size_t l = msg_size;
            if (msg == NULL) {
              msg_size = strlen(stmt[i]) + 1;
              msg = (char *)malloc(msg_size);
            } else {
              char *p;
              msg_size += strlen(stmt[i]);
              p = (char *)realloc(msg, msg_size + strlen(stmt[i]));
              if (p == NULL) {
                free(msg);
                msg_size = 0;
              }
              msg = p;
            }
            sprintf(&msg[l ? (l-1) : 0],"%s",stmt[i]);
            free(stmt[i]);
          }
        }

	if (msg) write2rgma_sql(msg,include_len);

	free(string_jobid);
	free(string_stat);
        free(string_server);
	if (msg) free(msg);
}

#ifdef TEST
int main(int argc, char* argv[])
{
	setenv("EDG_WL_RGMA_FILE", "/tmp/rgma_statefile", 1);
	setenv("EDG_WL_RGMA_SOCK", "/tmp/rgma_statesock", 1);
 
	write2rgma_sql("INSERT INTO JobStatusRaw VALUES ('Job_id9','CLEARED','Owner','BKServer', '1050024158210')",1);
	return 0;
}
#endif
