#ident "$Header: /local/reps/lcgware/workload/logging/locallogger/il_master.c,v 1.33 2004/11/24 17:04:15 dhsmith Exp $"

#include <errno.h>
#include <string.h>

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#include "edg/workload/common/jobid/jobid.h"
#include "edg/workload/logging/locallogger/interlogd.h"
#include "edg/workload/logging/common/il_string.h"
#include "edg/workload/logging/client/context.h"

int 
enqueue_msg(struct event_queue *eq, struct server_msg *msg)
{
	/* fire thread to take care of this queue */
	if(event_queue_create_thread(eq) < 0) 
		return(-1);

	/* avoid losing signal to thread */
	if(event_queue_cond_lock(eq) < 0) 
		return(-1);

	/* insert new event */
	if(event_queue_insert(eq, msg) < 0)
		return(-1);
      
	/* signal thread that we have a new message */
	if(event_queue_signal(eq) < 0)
		return(-1);

	/* allow thread to continue */
	if(event_queue_cond_unlock(eq) < 0)
		return(-1);

	return(0);
}


pthread_mutex_t flush_lock = PTHREAD_MUTEX_INITIALIZER;
pthread_cond_t flush_cond = PTHREAD_COND_INITIALIZER;


static 
int
parse_cmd(char *event, char **job_id_s, long *receipt, int *timeout)
{
	char *token, *r;
	int ret;

	if(strstr(event, "DG.TYPE=\"command\"") == NULL)
		return(-1);

	*job_id_s = NULL;
	*timeout = 0;
	*receipt = 0;
	ret = 0;

	for(token = strtok(event, " "); token != NULL; token = strtok(NULL, " ")) {
		r = index(token, '=');
		if(r == NULL) {
			ret = -1;
			continue;
		}
		if(strncmp(token, "DG.COMMAND", r - token) == 0) {
			if(strcmp(++r, "\"flush\"")) {
				il_log(LOG_WARNING, "  command %s not implemented\n", r);
				ret = -1;
				continue;
			}
		} else if(strncmp(token, "DG.JOBID", r - token) == 0) {
			char  *p;
      
			r += 2; /* skip =" */
			p = index(r, '"');
			if(p == NULL) { ret = -1; continue; }
			*job_id_s = strndup(r, p-r);

		} else if(strncmp(token, "DG.TIMEOUT", r - token) == 0) {
			sscanf(++r, "\"%d\"", timeout);
		} else if(strncmp(token, "DG.LLLID", r - token) == 0) {
			sscanf(++r, "%ld", receipt);
		}
    
	}
	return(0);
}


/* return value:
 *   0 - not command
 *   1 - success
 *  -1 - failure
 */

static 
int 
handle_cmd(char *event, long offset)
{
	char *job_id_s;
	struct event_queue *eq;
	int  num_replies, num_threads = 0;
	int  timeout, result;
	long receipt;
	struct timespec endtime;
	struct timeval  tv;

	/* parse command */
	if(parse_cmd(event, &job_id_s, &receipt, &timeout) < 0) 
		return(0);

	il_log(LOG_DEBUG, "  received FLUSH command\n");

	/* catchup with all neccessary event files */
	if(job_id_s) {
		struct event_store *es = event_store_find(job_id_s);

		if(es == NULL) {
			goto cmd_error;
		}
		result = event_store_recover(es);
		/* NOTE: if flush had been stored in file, there would have been
		   no need to lock the event_store at all */
		event_store_release(es);
		if(result < 0) {
			il_log(LOG_ERR, "  error trying to catch up with event file: %s\n",
			       error_get_msg());
			clear_error();
		}
	} else 
	  /* this call does not fail :-) */
	  event_store_recover_all();

	il_log(LOG_DEBUG, "  alerting threads to report status\n");

	/* prevent threads from reporting too early */
	if(pthread_mutex_lock(&flush_lock) < 0) {
		/*** this error is considered too serious to allow the program run anymore!
		     set_error(IL_SYS, errno, "pthread_mutex_lock: error locking flush lock");
		     goto cmd_error;
		*/
		abort();
	}

	/* wake up all threads */
	if(job_id_s) {
		/* find appropriate queue */
		eq = queue_list_get(job_id_s);
		if(eq == NULL) goto cmd_error;
		if(!event_queue_empty(eq) && !queue_list_is_log(eq)) {
			num_threads++;
			if(event_queue_cond_lock(eq) < 0) goto cmd_error;
			eq->flushing = 1;
			if(event_queue_wakeup(eq) < 0) goto cmd_error;
			if(event_queue_cond_unlock(eq) < 0) goto cmd_error;
		}
	} else {
		/* iterate over event queues */
		for(eq=queue_list_first(); eq != NULL; eq=queue_list_next()) {
			if(!event_queue_empty(eq) && !queue_list_is_log(eq)) {
				num_threads++;
				if(event_queue_cond_lock(eq) < 0) goto cmd_error;
				eq->flushing = 1;
				if(event_queue_wakeup(eq) < 0) goto cmd_error;
				if(event_queue_cond_unlock(eq) < 0) goto cmd_error;
			}
		}
	}
	if(!bs_only) {
		eq = queue_list_get(NULL);
		if(eq == NULL) goto cmd_error;
		if(!event_queue_empty(eq)) {
			num_threads++;
			if(event_queue_cond_lock(eq) < 0) goto cmd_error;
			eq->flushing = 1;
			if(event_queue_wakeup(eq) < 0) goto cmd_error;
			if(event_queue_cond_unlock(eq) < 0) goto cmd_error;
		}
	}

	/* wait for thread replies */
	num_replies = 0;
	result = 1;
	gettimeofday(&tv, NULL);
	endtime.tv_sec = tv.tv_sec + timeout;
	endtime.tv_nsec = 1000 * tv.tv_usec;
	while(num_replies < num_threads) {
		int ret;
		if((ret=pthread_cond_timedwait(&flush_cond, &flush_lock, &endtime)) < 0) {
			il_log(LOG_ERR, "    error waiting for thread reply: %s\n", strerror(errno));
			result = (ret == ETIMEDOUT) ? 0 : -1;
			break;
		}
		
		/* collect results from reporting threads */
		if(job_id_s) {
			/* find appropriate queue */
			eq = queue_list_get(job_id_s);
			if(eq == NULL) goto cmd_error;
			if(!queue_list_is_log(eq)) {
				if(event_queue_cond_lock(eq) < 0) goto cmd_error;
				if(eq->flushing == 2) {
					eq->flushing = 0;
					num_replies++;
					result = ((result == 1) || (eq->flush_result < 0))  ? 
						eq->flush_result : result;
				}
				if(event_queue_cond_unlock(eq) < 0) goto cmd_error;
			}
		} else {
			/* iterate over event queues */
			for(eq=queue_list_first(); eq != NULL; eq=queue_list_next()) {
				if(!queue_list_is_log(eq)) {
					if(event_queue_cond_lock(eq) < 0) goto cmd_error;
					if(eq->flushing == 2) {
						eq->flushing = 0;
						num_replies++;
						il_log(LOG_DEBUG, "    thread reply: %d\n", eq->flush_result);
						result = ((result == 1) || (eq->flush_result < 0))  ? 
							eq->flush_result : result;
					}
					if(event_queue_cond_unlock(eq) < 0) goto cmd_error;
				}
			}
		}
		if(!bs_only) {
			eq = queue_list_get(NULL);
			if(eq == NULL) goto cmd_error;
			if(event_queue_cond_lock(eq) < 0) goto cmd_error;
			if(eq->flushing == 2) {
				eq->flushing = 0;
				num_replies++;
				result = ((result == 1) || (eq->flush_result < 0))  ? 
					eq->flush_result : result;
			}
			if(event_queue_cond_unlock(eq) < 0) goto cmd_error;
		}
	}

	/* prevent deadlock in next flush */
	if(pthread_mutex_unlock(&flush_lock) < 0) 
		abort();


	/* report back to local logger */
	switch(result) {
	case 1:
		result = 0; break;
	case 0:
		result = EDG_WLL_IL_EVENTS_WAITING; break;
	default:
		result = EDG_WLL_IL_SYS; break;
	}
	if(job_id_s) free(job_id_s);
	result = send_confirmation(receipt, result);
	if(result <= 0) 
		il_log(LOG_ERR, "handle_cmd: error sending status: %s\n", error_get_msg());
	return(1);


cmd_error:
	if(job_id_s) free(job_id_s);
	return(-1);
}


static 
int
handle_msg(char *event, long offset)
{ 
	struct server_msg *msg = NULL;
	struct event_store *es;

	int ret;

	/* convert event to message for server */
	if((msg = server_msg_create(event)) == NULL) {
		il_log(LOG_ERR, "    handle_msg: error parsing event '%s':\n      %s\n", event, error_get_msg());
		return(0);
	}
  
	/* sync event store with IPC (if neccessary)
	 * This MUST be called before inserting event into output queue! */
	if((es = event_store_find(msg->job_id_s)) == NULL) 
		return(0);
	msg->es = es;
        msg->offset = offset;

	ret = event_store_sync(es, offset, event, msg);
	il_log(LOG_DEBUG, "  syncing event store at %d with event at %d, result %d\n", es->offset, offset, ret);
	if(ret == 0) {
		il_log(LOG_ERR, "    handle_msg: error syncing event store:\n      %s\n", error_get_msg());
	} else if(ret >= 1) {
		/* event either queued OK, or seen before */
                ret = 1;
	}
        
        server_msg_free(msg);
	event_store_release(es);

        return(ret);
}



int 
loop()
{
	/* receive events */
	while(1) {
		char *msg;
		long offset;
		int ret;
    
		clear_error();
		if((ret = input_queue_get(&msg, &offset, INPUT_TIMEOUT)) < 0) 
		{
			if(error_get_maj() == IL_PROTO) {
				il_log(LOG_DEBUG, "  premature EOF while receiving event\n");
				/* problems with socket input, try to catch up from files */
				event_store_recover_all();
				continue;
			} else 
				return(-1);
		}
		else if(ret == 0) {
			continue;
		}

		ret = handle_cmd(msg, offset);
		if(ret == 0)
			ret = handle_msg(msg, offset);
		free(msg);
		if(ret < 0)
			switch (error_get_maj()) {
				case IL_SYS:
				case IL_NOMEM:
					return (ret);
					break;
				default: 
    					il_log(LOG_ERR, "Error: %s\n", error_get_msg());
					break;
			}
	} /* while */
}
