#ident "$Header: /local/reps/lcgware/workload/logging/locallogger/queue_mgr.c,v 1.15 2003/10/13 10:24:34 michal Exp $"

#include <string.h>
#include <errno.h>
#include <assert.h>

#include "edg/workload/logging/locallogger/interlogd.h"
#include "edg/workload/logging/client/consumer.h"

struct queue_list {
  struct event_queue *queue;
  char   *dest;
  struct queue_list *next;
};

static struct event_queue *log_queue;
static struct queue_list  *queues;


static 
int
queue_list_create()
{
  queues = NULL;

  return(0);
}


static
int
queue_list_find(char *dest, struct queue_list **el, struct queue_list **prev)
{
  struct queue_list *q, *p;

  assert(el != NULL);

  *el = NULL;
  if(prev)
    *prev = NULL;

  if(queues == NULL) 
    return(0);

  q = NULL;
  p = queues;

  while(p) {
    if(strcmp(p->dest, dest) == 0) {
      *el = p;
      if(prev)
	*prev = q;
      return(1);
    }

    q = p;
    p = p->next;
  };

  return(0);
}


static
int
queue_list_add(char *dest, struct event_queue *eq)
{
  struct queue_list *el;
  
  assert(dest != NULL);
  assert(eq != NULL);

  el = malloc(sizeof(*el));
  if(el == NULL) {
    set_error(IL_NOMEM, ENOMEM, "queue_list_add: not enough room for new queue");
    return(-1);
  }

  el->dest = strdup(dest);
  if(el->dest == NULL) {
    free(el);
    set_error(IL_NOMEM, ENOMEM, "queue_list_add: not enough memory for new queue");
    return(-1);
  }
  el->queue = eq;
  el->next = queues;
  queues = el;
  return 0;
}


/*
static
int
queue_list_remove(struct queue_list *el, struct queue_list *prev)
{
  assert(el != NULL);

  if(prev) 
    prev->next = el->next;
  else
    queues = el->next;

  free(el);
  return(1);
}
*/


static
char *
jobid2dest(edg_wlc_JobId jobid)
{
  char *server_name,*out;
  unsigned int	server_port;

  if (!jobid) {
    set_error(IL_PROTO, EDG_WLL_ERROR_PARSE_BROKEN_ULM, "jobid2dest: invalid job id");
    return(NULL);
  }
  edg_wlc_JobIdGetServerParts(jobid,&server_name,&server_port);

  asprintf(&out,"%s:%d",server_name,server_port);
  free(server_name);
  if(!out)
    set_error(IL_SYS, ENOMEM, "jobid2dest: error creating server name");
  return(out);
}


struct event_queue *
queue_list_get(char *job_id_s)
{
  char *dest;
  struct queue_list *q;
  struct event_queue *eq;
  edg_wlc_JobId job_id;

  if(job_id_s == NULL || strcmp(job_id_s, "default") == 0)
    return(log_queue);

  if(edg_wlc_JobIdParse(job_id_s, &job_id)) {
    set_error(IL_LBAPI, EDG_WLL_ERROR_PARSE_BROKEN_ULM, "queue_list_get: invalid job id");
    return(NULL);
  }

  dest = jobid2dest(job_id);
  edg_wlc_JobIdFree(job_id);
  if(dest == NULL) 
    return(NULL);
  
  if(queue_list_find(dest, &q, NULL)) {
    free(dest);
    return(q->queue);
  } else {
    eq = event_queue_create(dest);
    if(eq)
      queue_list_add(dest, eq);
    free(dest);
    return(eq);
  }
}


int
queue_list_is_log(struct event_queue *eq)
{
  return(eq == queue_list_get(NULL));
}


int
queue_list_init(char *ls)
{
  /* create queue for log server */
  log_queue = event_queue_create(ls);
  if(log_queue == NULL)
    return(-1);

  return(queue_list_create());
}


static struct queue_list *current;


struct event_queue *
queue_list_first()
{
  current = queues;
  return(current ? current->queue : NULL);
}


struct event_queue *
queue_list_next()
{
  current = current ? current->next : NULL;
  return(current ? current->queue : NULL);
}
