#ident "$Header: /local/reps/lcgware/workload/logging/locallogger/server_msg.c,v 1.15 2003/10/20 14:27:05 salvet Exp $"

#include <errno.h>
#include <assert.h>
#include <string.h>

#include "edg/workload/logging/locallogger/interlogd.h"
#include "edg/workload/logging/common/il_string.h"
#include "edg/workload/logging/common/events_parse.h"
#include "edg/workload/logging/client/consumer.h"

static
int 
create_msg(char *event, char **buffer, long *receipt)
{
  char *p;  int  len;
  char *ucs = "michal";
  
  *receipt = 0;

  /* find DG.LLLID */
  if(strncmp(event, "DG.LLLID",8) == 0 ||
	strncmp(event, "DG.LLPID",8) == 0) { /* 8 == strlen("DG.LLLID") */

    /* skip the key */
    event += 9;  /* 9 = strlen("DG.LLLID=") */
    *receipt = atol(event);
    p = strchr(event, ' ');
    if(!p) {
      set_error(IL_LBAPI, EDG_WLL_ERROR_PARSE_BROKEN_ULM, 
		"create_msg: error parsing locallogger PID");
      return(-1);
    }
    /* skip the value */
    event = p + 1;

    /* find DG.PRIORITY */
    p = strstr(event, "DG.PRIORITY");
    if(p) {
      int n;
      
      p += 12; /* skip the key and = */
      if((n = atoi(p)) == 0) {
	/* normal asynchronous message */
	*receipt = 0;
      }
    } else {
      /* could not find priority key */
      *receipt = 0;
    }
    
  } else {
    /* could not find local logger PID, confirmation can not be sent */
    *receipt = 0;
  }

  /* allocate enough room to hold the message */
  len = 17 + len_string(ucs) + len_string(event);
  if((*buffer = malloc(len)) == NULL) {
    set_error(IL_NOMEM, ENOMEM, "create_msg: out of memory allocating message");
    return(-1);
  }

  p = *buffer;

  /* write header */
  sprintf(p, "%16d\n", len - 17);
  p += 17;

  /* write rest of the message */
  p = put_string(p, ucs);
  p = put_string(p, event);

  return(p - *buffer);

}


struct server_msg *
server_msg_create(char *event)
{
  struct server_msg *msg;

  msg = malloc(sizeof(*msg));
  if(msg == NULL) {
    set_error(IL_NOMEM, ENOMEM, "server_msg_create: out of memory allocating message");
    return(NULL);
  }

  if(server_msg_init(msg, event) < 0) {
    server_msg_free(msg);
    return(NULL);
  }

  return(msg);
}


struct server_msg *
server_msg_copy(struct server_msg *src)
{
  struct server_msg *msg;

  msg = malloc(sizeof(*msg));
  if(msg == NULL) {
    set_error(IL_NOMEM, ENOMEM, "server_msg_copy: out of memory allocating message");
    return(NULL);
  }
  
  msg->msg = malloc(src->len);
  if(msg->msg == NULL) {
    set_error(IL_NOMEM, ENOMEM, "server_msg_copy: out of memory allocating server message");
    server_msg_free(msg);
    return(NULL);
  }
  msg->len = src->len;
  memcpy(msg->msg, src->msg, src->len);

  msg->job_id_s = strdup(src->job_id_s);
  msg->ev_len = src->ev_len;
  msg->es = src->es;
  msg->receipt_to = src->receipt_to;
  return(msg);
}


int
server_msg_init(struct server_msg *msg, char *event)
{
  assert(msg != NULL);

  memset(msg, 0, sizeof(*msg));

  /* remember to add event separator to the length */
  msg->ev_len = strlen(event) + 1;
  msg->len = create_msg(event, &msg->msg, &msg->receipt_to);
  if(msg->len < 0) {
    return(-1);
  }
  msg->job_id_s = edg_wll_GetJobId(event);
  if(msg->job_id_s == NULL) {
    set_error(IL_LBAPI, EDG_WLL_ERROR_PARSE_BROKEN_ULM, "server_msg_init: error getting event's jobid");
    return(-1);
  }

  return(0);
}


int
server_msg_is_priority(struct server_msg *msg)
{
  assert(msg != NULL);

  return(msg->receipt_to != 0);
}


int
server_msg_free(struct server_msg *msg)
{
  assert(msg != NULL);

  if(msg->msg) free(msg->msg);
  if(msg->job_id_s) free(msg->job_id_s);
  free(msg);
  return 0;
}
