#include <assert.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/file.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/un.h>
#include <unistd.h>

static int	rgma_fd = -1;
static off_t	rgma_fd_off = 0;
static int	rgma_sock = -1;

static int read4rgma(int wait, char **out)
{
	char *rgma_fname;
	int slen;
	int sysret;
	fd_set fdset;
	char dummy;
	struct sockaddr_un rgma_saddr;
	struct timeval tv;
	struct stat statbuf;

	if (rgma_fd == -1) {
		rgma_fname = getenv("EDG_WL_RGMA_FILE");
		if (rgma_fname == NULL) return -1;

		rgma_fd = open(rgma_fname, O_RDWR|O_CREAT, 0777);
		if (rgma_fd == -1) return -1;
	}

	if (rgma_sock == -1) {
		rgma_fname = getenv("EDG_WL_RGMA_SOCK");
		if (rgma_fname == NULL) return -1;

		if ((strlen(rgma_fname) + 1) > sizeof(rgma_saddr.sun_path))
			return -1;

		unlink(rgma_fname);
		rgma_sock = socket(PF_UNIX, SOCK_DGRAM,0);
		if (rgma_sock == -1) return -1;
		
		memset(&rgma_saddr, sizeof(rgma_saddr), 0);
		rgma_saddr.sun_family = PF_UNIX;
		strcpy(rgma_saddr.sun_path, rgma_fname);

		sysret = bind(rgma_sock, (struct sockaddr*) &rgma_saddr,
				SUN_LEN(&rgma_saddr));
		if (sysret == -1) return -1;
	}
	
	if (wait) {
		FD_ZERO(&fdset);
		FD_SET(rgma_sock, &fdset);
		tv.tv_sec = 10; tv.tv_usec = 0;
		sysret = select(rgma_sock + 1, &fdset, NULL, NULL, &tv);
	} else sysret = 1;

	if (sysret > 0) {
		if (wait) {
			sysret = recvfrom(rgma_sock, &dummy, 1, 0, NULL, 0);
		}
		if ((sysret = flock(rgma_fd, LOCK_EX)) != -1) {
			sysret = read(rgma_fd, &slen, sizeof(slen));
			if (sysret != sizeof(slen)) goto errout;

			*out = malloc(slen+1);
			if (*out == NULL) goto errout;
			sysret = read(rgma_fd, *out, slen+1);
			if (sysret != (slen+1)) {
				free(*out);
				*out = NULL;
				goto errout;
			}
			rgma_fd_off = lseek(rgma_fd, 0, SEEK_CUR);
			if (fstat(rgma_fd, &statbuf) == 0 && statbuf.st_size == rgma_fd_off)
				if (ftruncate(rgma_fd, (off_t) 0) == 0)
					lseek(rgma_fd, (off_t) 0, SEEK_SET);
			flock(rgma_fd, LOCK_UN);
			return 1;
		}
	} else if (sysret == -1) {
		return -1;
	} else { /* timeout */
		return 0;
	}

errout:
	flock(rgma_fd, LOCK_UN);
	return -1;
}

#ifdef TEST
int main(int argc, char* argv[])
{
	char *out;

	setenv("EDG_WL_RGMA_FILE", "/tmp/rgma_statefile", 1);
	setenv("EDG_WL_RGMA_SOCK", "/tmp/rgma_statesock", 1);
	
	/* Retrieve backlogged records */
	while (read4rgma(0 /* no wait */, &out) > 0) {
		printf("%s\n",out);
		free(out);
	}

	/* Receive new records in realtime */
	while (1)
	while (read4rgma(1 /* wait */, &out) > 0) {
		printf("%s\n",out);
		free(out);
	}
 
	return 0;
}
#else
/* !defined(TEST) */

#include <jni.h>
#include "LBProducerSource.h"

JNIEXPORT jstring JNICALL Java_LBProducerSource_insertString
  (JNIEnv *jenv, jobject jobj) {
	
	char *out = NULL;
	int ret;
	
	setenv("EDG_WL_RGMA_FILE", "/tmp/rgma_statefile", 1);
	setenv("EDG_WL_RGMA_SOCK", "/tmp/rgma_statesock", 1);
	for (;;) {
		/* Try to read backlog first */
		ret = read4rgma(0, &out);
		if (ret == 1)
			return (*jenv)->NewStringUTF(jenv, out);
		else {
			/* then wait for new updates */
			ret = read4rgma(1, &out);
			if (ret == 1) 
				return (*jenv)->NewStringUTF(jenv, out);
		}
	}
};

#endif
