/* 
 * Command.cpp
 * 
 * Copyright (c) 2002 EU DataGrid.
 * For license conditions see http://www.eu-datagrid.org/license.html.
 */

// $Id: Command.cpp,v 1.26 2005/10/06 15:29:05 dhsmith Exp $
 
#include <string>
#include <classad_distribution.h>
#include "edg/workload/networkserver/commands/logging.h"
#include "edg/workload/networkserver/commands/Command.h"
#include "edg/workload/networkserver/commands/fsm/CommandState.h"
#include "edg/workload/common/socket++/GSISocketAgent.h"
#include "edg/workload/common/utilities/classad_utils.h"

#include "edg/workload/common/logger/edglog.h"
#include "edg/workload/common/logger/manipulators.h"

#include "edg/workload/networkserver/commands/jni_deploy_macros.h"
 
namespace logger = edg::workload::common::logger;
namespace socket_pp = edg::workload::common::socket_pp;
namespace utilities = edg::workload::common::utilities;

namespace edg {
namespace workload {
namespace networkserver {
namespace commands {

Command::Command()
{
  ctx.reset( new edg_wll_Context() );
  jobId.reset( new edg_wlc_JobId() );
  ad = NULL;
  fsm = NULL;
}

Command::~Command()
{
 edg_wll_FreeContext(*ctx);
 edg_wlc_JobIdFree(*jobId);
 if (fsm != NULL) delete fsm;
 if (ad != NULL) delete ad;
}

classad::ClassAd& Command::asClassAd() const 
{ 
  assert( ad != NULL );
  return (*ad);
}

bool Command::serialize(socket_pp::SocketAgent *sck)
{
  this -> sck = sck;
  return serializeImpl(sck, this);
}

bool Command::execute()
{  
  edglog_fn(Inside Command);

  if( this -> fsm -> empty() ) { return false; }

  fsm::CommandState::shared_ptr state(fsm -> front());
  this -> fsm -> pop();
#ifdef DEBUG
  edglog(veryugly) << utilities::asString(*ad) << std::endl; 
#endif
  return state -> execute(this);
}

bool Command::isDone() const
{
  return this -> fsm -> empty();
}

const fsm::CommandState& Command::state()
{
  assert( !this -> fsm -> empty() );
  return *this -> fsm -> front();
}

socket_pp::SocketAgent& Command::agent() const 
{
  assert( sck );
  return *sck;
}

std::string Command::name()
{
  std::string nm;
  assert(ad && ad -> EvaluateAttrString("Command",nm));
  return nm;
}

std::string Command::version()
{
  std::string ver;
  assert(ad && ad -> EvaluateAttrString("Version",ver));
  return ver;
}

bool Command::getParam(const std::string& name, bool& b)
{
  classad::ClassAd* a = DYNAMIC_CAST(classad::ClassAd*, ad -> Lookup("Arguments"));
  return(a && a -> EvaluateAttrBool(name, b) );
}

bool Command::getParam(const std::string& name, int& i)
{
  classad::ClassAd* a = DYNAMIC_CAST(classad::ClassAd*, ad -> Lookup("Arguments"));
  return(a && a -> EvaluateAttrInt(name, i) );
}

bool Command::getParam(const std::string& name, double& d)
{
  classad::ClassAd* a = DYNAMIC_CAST(classad::ClassAd*, ad -> Lookup("Arguments"));
  return(a && a -> EvaluateAttrReal(name, d) );
}

bool Command::getParam(const std::string& name, std::string& s)
{ 
  classad::ClassAd* a = DYNAMIC_CAST(classad::ClassAd*, ad -> Lookup("Arguments"));
  return(a && a -> EvaluateAttrString(name, s) );
}

bool Command::getParam(const std::string& name, std::vector<std::string>& v)
{
  classad::ClassAd* a = DYNAMIC_CAST(classad::ClassAd*, ad -> Lookup("Arguments"));
  return(a && utilities::EvaluateAttrList(*a,name, v) );
}

bool Command::setParam( const std::string &name, bool b )
{ 
  classad::ClassAd* a = DYNAMIC_CAST(classad::ClassAd*, ad -> Lookup("Arguments"));
  return(a && a -> InsertAttr(name, b) );
}

bool Command::setParam( const std::string &name, int i )
{ 
  classad::ClassAd* a = DYNAMIC_CAST(classad::ClassAd*, ad -> Lookup("Arguments"));
  return(a && a -> InsertAttr(name, i) ); 
}

bool Command::setParam( const std::string &name, double d )
{ 
  classad::ClassAd* a = DYNAMIC_CAST(classad::ClassAd*, ad -> Lookup("Arguments"));
  return(a && a -> InsertAttr(name, d) ); 
}

bool Command::setParam( const std::string &name, const std::string &s )
{ 

  classad::ClassAd* a = DYNAMIC_CAST(classad::ClassAd*, ad -> Lookup("Arguments"));
  return(a && a -> InsertAttr(name, s) ); 
}

bool Command::setParam( const std::string &name, classad::ClassAd *pad )
{
  classad::ClassAd* a = DYNAMIC_CAST(classad::ClassAd*, ad -> Lookup("Arguments"));
  return( a && (a -> Insert(name, pad->Copy())) );
}

bool Command::setParam( const std::string &name, const std::vector<std::string> &v )
{ 
  classad::ClassAd* a = DYNAMIC_CAST(classad::ClassAd*, ad -> Lookup("Arguments"));
  return( a && utilities::InsertAttrList<std::string>(*a, name, v) );
}

edg_wll_Context* Command::getLogContext() {
  return ctx.get();
}

edg_wlc_JobId* Command::getLogJobId() {
  return jobId.get();
}

} // namespace commands
} // namespace networkserver
} // namespace workload
} // namespace edg
