/*
 * File: submit.cpp
 *
 * @author: Marco Pappalardo <marco.pappalardo@ct.infn.it>
 * Copyright (c) 2002 EU DataGrid.
 * For license conditions see http://www.eu-datagrid.org/license.html
 */

#include <iostream>
#include <fstream>
#include <string>
#include <algorithm>
#include <boost/scoped_ptr.hpp>

#include "edg/workload/networkserver/client/NSClient.h"
#include "edg/workload/common/utilities/Exceptions.h"
#include "edg/workload/common/utilities/edgstrstream.h"

using namespace edg::workload::networkserver::client;
using namespace edg::workload::common::socket_pp;
using namespace edg::workload::common::utilities;

void usage()
{
  std::cout << "submit: -h ns_host -p ns_port -f jdl_file" << std::endl;
}

int main( int argc, char* argv[])
{
  if( argc < 7 ) {
    std::cout << "submit: insufficient number of arguments" << std::endl;
    usage();
  }
  else {
    bool errors = false;
    int port;
    std::string host;
    std::string jdl;
    std::string jdl_file;
    typedef enum { _param_, _port_, _host_, _jdl_ } arg_t;
    arg_t expected = _param_;
    for( int i = 1; !errors && i < argc;  i++) {  
      
      boost::scoped_ptr<std::istream> in(new iedgstrstream(argv[i]));
      
      switch( expected ) {
      case _port_: (*in) >> port; expected = _param_; break;
      case _host_: (*in) >> host; expected = _param_; break;
      case _jdl_ : (*in) >> jdl_file; expected = _param_; break;
      default:
	std::string token;
	(*in) >> token;
	if( token == "-p" ) expected = _port_;
	else if ( token == "-h" ) expected = _host_;
	else if ( token == "-f" ) expected = _jdl_;
	else {
	  std::cout << "submit: invalid parameter " << token << std::endl;
	  usage();
	  errors = true;
	}
	
      }
    }
 
    if( !errors ) {		
      try {
        std::ifstream f;
        f.open( jdl_file.c_str() );
        if( f.is_open() ) {
	  while(!f.eof()) {
            std::string line;
            // f >> line;
	    getline(f, line);
	    std::cout << "- " << line << std::endl;
            jdl += line;
          }
	  std::cout << "questa e' la jdl letta dal file: " << jdl << std::endl;
          f.close(); 

	  NSClient theNSClient(host, port);
	  std::cout << theNSClient.jobSubmit(jdl) << std::endl;
	} else {
	  std::cout << "Jdl file: " << jdl_file << "not found." << std::endl; 
	}
      } catch (edg::workload::common::utilities::Exception& e) {
	std::cout << "L'exception e' catchata!" << std::endl;
        std::cout << e.what() << std::endl;
        std::cout << e.getExceptionName() << std::endl;
	return 5;
      }
    }
  }
} 


















