/***************************************************************************
 *  filename  : URL.cpp
 *  authors   : Elisabetta Ronchieri <elisabetta.ronchieri@cnaf.infn.it>
 *              Francesco Giacomini <francesco.giacomini@cnaf.infn.it>
 *  copyright : (C) 2001 by INFN
 ***************************************************************************/

#include <iostream>
#include <string>

#include <cctype>
#include <cstdlib>

#include "edg/workload/planning/common/URL.h"

using namespace std;

namespace edg {
namespace workload {
namespace planning {
namespace common {

ExInvalidURL::ExInvalidURL(const char *par)
  : m_invalidurl_parameter(par)
{
}
	
const string& ExInvalidURL::parameter(void) const
{ 
  return m_invalidurl_parameter; 
}
    
URL::URL(void)
  : m_is_empty(true)
{
}

URL::URL(string url)
  : m_is_empty(false)
{
  parse(url);
}

URL::~URL(void)
{
}

void
URL::parse(string url)
{
  string::size_type pos = url.find("://");

  if (pos == string::npos) {
    throw ExInvalidURL("there is not ://");
  }
  string protocol = url.substr(0, pos);
  url      = url.substr(pos+3);

  pos      = url.find("/");
  if (pos == string::npos) {
    throw ExInvalidURL("there is not /");
  }
  string hostpart = url.substr(0, pos);

  string host;
  string port;
  string::size_type host_end = hostpart.find(":");
  if (host_end == string::npos) {  // no port
    host = hostpart;
    port = "";
  } else {                             // port
    host = hostpart.substr(0, host_end);
    port     = hostpart.substr(host_end + 1, pos);

    // -*- Check port value -*-
    for(string::const_iterator it = port.begin();
	it != port.end(); it++){
      if(!isdigit(*it)){
	throw ExInvalidURL("the port is wrong");
      }
    }
  }

  string path = url.substr(pos);

  m_protocol = protocol;
  m_host     = host;
  m_port     = port;
  m_path     = path;
}

bool
URL::is_empty(void) const
{
  return m_is_empty;
}

string
URL::protocol(void) const
{
  return m_protocol;
}

string
URL::host(void) const
{
  return m_host;
}

string
URL::port(void) const
{
  return m_port;
}

string
URL::path(void) const
{
  return m_path;
}

string
URL::as_string(void) const
{
  return m_protocol + "://" + m_host + ((m_port != "") ? ":" + m_port : "")+ m_path;
}

} // namespace common
} // namespace planning
} // namespace workload
} // namespace edg
