/***************************************************************************
 *  filename  : InteractiveJobWrapper.cpp
 *  authors   : Elisabetta Ronchieri elisabetta.ronchieri@cnaf.infn.it
 *  Copyright (c) 2002 CERN and INFN on behalf of the EU DataGrid.
 *  For license conditions see LICENSE file or
 *  http://www.edg.org/license.html
 ***************************************************************************/

#include <algorithm>
#include <cassert>

#include "edg/workload/planning/jobadapter/JobWrapper.h"
#include "edg/workload/planning/jobadapter/InteractiveJobWrapper.h"

using namespace std;

namespace url = edg::workload::planning::common;

namespace edg {
namespace workload {
namespace planning {
namespace jobadapter {

InteractiveJobWrapper::InteractiveJobWrapper(const string& job)
  : JobWrapper(job)
{  
}

InteractiveJobWrapper::~InteractiveJobWrapper(void)
{
}

ostream& 
InteractiveJobWrapper::make_bypass_transfer(ostream& os,
					    const string& prefix) const
{
  vector<string> infiles;
  infiles.push_back("edg-wl-pipe-input");
  infiles.push_back("edg-wl-pipe-output");
  infiles.push_back("edg-wl-job-agent");
  
  os << "for f in ";
  for (vector<string>:: const_iterator it = infiles.begin();
       it != infiles.end(); ++it) {
    os << " \"" << *it << "\"";
  }
  os << " ; do" << endl;
      
  os << "   globus-url-copy " << prefix << "opt/edg/bin/${f} file://${workdir}/${f}" << endl
     << "   chmod +x ${workdir}/${f}" << endl
     << "done" << endl	  
     << endl;

  string libfile("/opt/edg/lib/libedg-wl-grid-console-agent.so");
  
  os << "globus-url-copy " << prefix << libfile << " file://${workdir}/libedg-wl-grid-console-agent.so" << endl
     << endl;
  
  return os << endl;  
}

ostream&
InteractiveJobWrapper::execute_job(ostream& os,
		                   const string& arguments,
		                   const string& job,
		                   const string& stdi,
		                   const string& stdo,
		                   const string& stde,
		                   int           node) const
{
  os << "./edg-wl-job-agent ${BYPASS_SHADOW_HOST} ${BYPASS_SHADOW_PORT}"; 
	  
  if (arguments != "") {
    os << " \"" << job << " " << arguments << " $*\"";
  }
  else {
    os << " \"" << job << " $*\"";
  }

  return os << endl
            << endl;
}

} // namespace jobadapter
} // namespace planning
} // namespace workload
} // namespace edg
