// File: Dispatcher.cpp
// Author: Francesco Giacomini <Francesco.Giacomini@cnaf.infn.it>
// Copyright (c) 2001 EU DataGrid.
// For license conditions see http://www.eu-datagrid.org/license.html

// $Id: Dispatcher.cpp,v 1.5 2003/09/09 06:46:32 giaco Exp $

#include "edg/workload/planning/manager/Dispatcher.h"
#include <algorithm>
#include <cctype>
#include "edg/workload/planning/manager/DispatcherImpl.h"
#include "edg/workload/planning/manager/DispatcherFactory.h"
#include "edg/workload/planning/common/logger_utils.h"
#include "edg/workload/common/configuration/Configuration.h"
#include "edg/workload/common/configuration/WMConfiguration.h"

namespace configuration = edg::workload::common::configuration;

namespace edg {
namespace workload {
namespace planning {
namespace manager {

namespace {

dispatcher_type normalize(dispatcher_type const& id)
{
  dispatcher_type result(id);
  std::transform(result.begin(), result.end(), result.begin(), ::tolower);
  return result;
}

std::string
get_dispatcher_type()
{
  configuration::Configuration const* const config
    = configuration::Configuration::instance();
  if (!config || config->get_module() != configuration::ModuleType::workload_manager) {
    Fatal("empty or invalid configuration");
  }
  configuration::WMConfiguration const* const wm_config = config->wm();
  if (!wm_config) {
    Fatal("empty WM configuration");
  }

  return normalize(wm_config->dispatcher_type());
}

} // {anonymous}

Dispatcher::Dispatcher()
  : m_impl(DispatcherFactory::instance()->create_dispatcher(get_dispatcher_type()))
{
}

Dispatcher::~Dispatcher()
{
}

void
Dispatcher::run()
{
  m_impl->run(write_end());     // the pipe write end is not available at
                                // construction time
}

} // namespace manager
} // namespace planning
} // namespace workload
} // namespace edg
