// File: DispatcherFactory.cpp
// Author: Francesco Giacomini <Francesco.Giacomini@cnaf.infn.it>
// Copyright (c) 2002 EU DataGrid.
// For license conditions see http://www.eu-datagrid.org/license.html

// $Id: DispatcherFactory.cpp,v 1.1 2003/03/27 14:27:29 giaco Exp $

#include "edg/workload/planning/manager/DispatcherFactory.h"
#include "edg/workload/thirdparty/loki/Factory.h"

namespace edg {
namespace workload {
namespace planning {
namespace manager {

DispatcherFactory* DispatcherFactory::s_instance = 0;

class DispatcherFactory::Impl: boost::noncopyable
{
  typedef Loki::Factory<product_type, dispatcher_type, product_creator_type> factory_type;
  factory_type m_factory;

public:

  bool register_dispatcher(dispatcher_type const& id, product_creator_type creator);
  bool unregister_dispatcher(dispatcher_type const& id);
  product_type* create_dispatcher(dispatcher_type const& id);
};

bool
DispatcherFactory::Impl::register_dispatcher(dispatcher_type const& id, product_creator_type creator)
{
  return m_factory.Register(id, creator);
}

bool
DispatcherFactory::Impl::unregister_dispatcher(dispatcher_type const& id)
{
  return m_factory.Unregister(id);
}

DispatcherFactory::product_type*
DispatcherFactory::Impl::create_dispatcher(dispatcher_type const& id)
{
  return m_factory.CreateObject(id);
}

DispatcherFactory*
DispatcherFactory::instance()
{
  if (s_instance == 0) {
    s_instance = new DispatcherFactory;
  }

  return s_instance;
}

DispatcherFactory::DispatcherFactory()
  : m_impl(new Impl)
{
}

bool
DispatcherFactory::register_dispatcher(dispatcher_type const& id, product_creator_type creator)
{
  return m_impl->register_dispatcher(id, creator);
}

bool
DispatcherFactory::unregister_dispatcher(dispatcher_type const& id)
{
  return m_impl->unregister_dispatcher(id);
}

DispatcherFactory::product_type*
DispatcherFactory::create_dispatcher(dispatcher_type const& id)
{
  return m_impl->create_dispatcher(id);
}

}}}} // edg::workload::planning::manager

