// File: WorkloadManager.cpp
// Author: Francesco Giacomini <Francesco.Giacomini@cnaf.infn.it>
// Copyright (c) 2001 EU DataGrid.
// For license conditions see http://www.eu-datagrid.org/license.html

// $Id: WorkloadManager.cpp,v 1.11 2003/09/09 06:46:33 giaco Exp $

#include "edg/workload/planning/manager/WorkloadManager.h"
#include <algorithm>
#include <cctype>
#include "edg/workload/planning/manager/WMFactory.h"
#include "edg/workload/planning/manager/WMImpl.h"
#include "edg/workload/planning/common/logger_utils.h"
#include "edg/workload/common/configuration/Configuration.h"
#include "edg/workload/common/configuration/WMConfiguration.h"

namespace configuration = edg::workload::common::configuration;

namespace edg {
namespace workload {
namespace planning {
namespace manager {

namespace {

WMFactory::wm_type
normalize(
  WMFactory::wm_type const& id)
{
  WMFactory::wm_type result(id);
  std::transform(result.begin(), result.end(), result.begin(), ::tolower);
  return result;
}

WMFactory::wm_type default_wm_id("Real");

WMFactory::wm_type
get_wm_type()
{
  WMFactory::wm_type result(default_wm_id);

  configuration::Configuration const* const config
    = configuration::Configuration::instance();
  if (!config) {
    Fatal("empty or invalid configuration");
  }
  configuration::WMConfiguration const* const wm_config = config->wm();
  if (!wm_config) {
    Fatal("empty or invalid WM configuration");
  }

  if (config->get_module() != configuration::ModuleType::workload_manager) {
    Fatal("WM Proxy not implemented yet");
    //    result = wm_config->proxy_type();
  }

  return normalize(result);
}

}

WorkloadManager::WorkloadManager()
  : m_impl(WMFactory::instance()->create_wm(get_wm_type()))
{
}

WorkloadManager::~WorkloadManager()
{
}

void
WorkloadManager::submit(classad::ClassAd const* request_ad)
{
  m_impl->submit(request_ad);
}

void
WorkloadManager::resubmit(common::jobid::JobId const& request_id)
{
  m_impl->resubmit(request_id);
}

void
WorkloadManager::cancel(common::jobid::JobId const& request_id)
{
  m_impl->cancel(request_id);
}

}}}} // edg::workload::planning::manager
