/*
 * File: retrieveCloseSEInfo.cpp
 * Author: Monforte Salvatore <Salvatore.Monforte@ct.infn.it>
 * Copyright (c) 2001 EU DataGrid.
 * For license conditions see http://www.eu-datagrid.org/license.html
 */

// $Id: retrieveCloseSEsInfo.cpp,v 1.6 2005/10/06 09:40:32 dhsmith Exp $

#include <classad_distribution.h>
#include <fnCall.h>
#include <vector>
#include <string>
#include <numeric>
#include <functional>
#include <regex.h>
#include <dlfcn.h>

#include <boost/thread/mutex.hpp>
#include <boost/scoped_ptr.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/thread/tss.hpp>

#include "edg/workload/planning/broker/matchmaking/brokerinfo.h"
#include "edg/workload/planning/broker/matchmaking/brokerinfoGlueImpl.h"

using namespace std;
#ifdef WANT_NAMESPACES
using namespace classad;
#endif

namespace edg {
namespace workload {
namespace planning {
namespace plugin {

namespace broker = edg::workload::planning::broker;
namespace matchmaking = broker::matchmaking;

typedef matchmaking::brokerinfo::BrokerInfo<matchmaking::brokerinfo::brokerinfoGlueImpl> BrokerInfo;

static boost::thread_specific_ptr<boost::shared_ptr<BrokerInfo *> > *tsp_glue_ptr=NULL;
static boost::mutex tsp_mtx;

static void fill_tsp() {
  boost::mutex::scoped_lock lk(tsp_mtx);
  if (tsp_glue_ptr != NULL) return;

  void *p=dlopen(NULL,RTLD_GLOBAL|RTLD_LAZY);
  if (p == NULL) return;

  void *s=dlsym(p,"get_BIp");
  if (s == NULL) return;

  boost::thread_specific_ptr<boost::shared_ptr<BrokerInfo *> > *(*func)() =
    (boost::thread_specific_ptr<boost::shared_ptr<BrokerInfo *> > *(*)())s;

  tsp_glue_ptr = (*func)();
}

namespace {

struct EvaluateExprInVector : public std::binary_function<vector<string>*, 
	classad::ExprTree*, vector<string> * >
{
	vector<string>* operator()(vector<string>* v, classad::ExprTree* e) 
	{
		classad::Value value;
		dynamic_cast<classad::Literal*>(e) -> GetValue(value);
		string s;
		if( value.IsStringValue(s) ) v -> push_back( s );
			return v;
	}
};

}

bool retrieveCloseSEsInfo(const char         *name,
		const ArgumentList &arguments,
		EvalState          &state,
		Value              &result)
{
	bool  eval_successful = false;
	result.SetErrorValue();
	// We check to make sure that we passed exactly three arguments...
	if (arguments.size() == 3) {
		
		// ...the first and second arguments should evaluate to a string...
		Value       arg1, arg2;
		std::string ceid, vo;
		if (arguments[0] -> Evaluate(state, arg1) &&
				arguments[1] -> Evaluate(state, arg2) &&
				arg1.IsStringValue(ceid) &&
				arg2.IsStringValue(vo)) {
			
			Value    arg3;
			const ExprList *attrList;
			// ...the third argument should evaluate to a list of strings...
			if (arguments[2] -> Evaluate(state, arg3) &&
					arg3.IsListValue(attrList)) {
				
				eval_successful = true;
				
				vector<string> attrs;
				accumulate(attrList->begin(), attrList->end(), &attrs, EvaluateExprInVector());

                                boost::shared_ptr<BrokerInfo *> *BI_h = 0;
                                fill_tsp();
                                {
                                  boost::mutex::scoped_lock lk(tsp_mtx);
                                  if (tsp_glue_ptr != 0) BI_h = tsp_glue_ptr->get();
                                }
                                BrokerInfo *BI;
                                boost::scoped_ptr<BrokerInfo> bi;
                                if (BI_h == 0 || BI_h->use_count() != 2) {
                                  bi.reset(new BrokerInfo);
                                  BI = bi.get();
                                } else {
                                  BI = *BI_h->get();
                                }
                                 
				BI->retrieveCloseSEsInfo(ceid, &attrs);
				BI->retrieveCloseSAsInfo(vo, &attrs);

				classad::ExprList* CloseSEsExprList ((*BI)->CloseStorageElements());
				result.SetListValue(CloseSEsExprList);
                                delete attrList;
			}

		}
	}
				
	
	return eval_successful;
}

} // namespace plugin
} // namespace planning
} // namespace workload
} // namespace edg
