// File: helperTest.cpp
// Author: Francesco Giacomini <Francesco.Giacomini@cnaf.infn.it>
// Copyright (c) 2002 EU DataGrid.
// For license conditions see http://www.eu-datagrid.org/license.html

// $Id: helperTest.cpp,v 1.3 2003/09/23 15:59:39 guarise Exp $

#include <iostream>
#include <iterator>
#include <string>
#include "edg/workload/planning/helper/Helper.h"
#include "edg/workload/planning/helper/HelperFactory.h"
#include "edg/workload/planning/broker/exceptions.h"
#include "edg/workload/common/configuration/Configuration.h"

namespace helper = edg::workload::planning::helper;
namespace broker = edg::workload::planning::broker;
namespace configuration = edg::workload::common::configuration;

namespace {

int const E_WRONG_USAGE       = 1;
int const E_KNOWN_EXCEPTION   = 2;
int const E_GENERIC_EXCEPTION = 3;

std::string program_name;

void usage(std::ostream& os)
{
  os
    << "Usage: " << program_name << " --help\n"
    << "   or: " << program_name << " --list\n"
    << "   or: " << program_name << " helper requestad\n";
}

} // {anonymous}

int main(int argc, char* argv[])
{
  program_name = argv[0];

  if (argc == 2 && std::string(argv[1]) == "--help") {

    usage(std::cout);

  } else if (argc == 2 && std::string(argv[1]) == "--list") {

    std::vector<std::string> helpers(helper::HelperFactory::instance()->list());
    std::copy(
      helpers.begin(),
      helpers.end(),
      std::ostream_iterator<std::string>(std::cout, "\n")
    );

  } else if (argc == 3) {

    try {
      configuration::Configuration config("edg_wl.conf",
                                          configuration::ModuleType::workload_manager);

      std::string helper(argv[1]);
      std::string input_file(argv[2]);

      std::cout << helper::Helper(helper).resolve(input_file) << '\n';
    } 
    catch ( broker::NoCompatibleCEs& e) 
    {
      std::cerr << "Caught exception: " << e.what() << '\n';
      return 5;
    } 
    catch (std::exception& e) 
    {
      std::cerr << "Caught exception: " << e.what() << '\n';
      return E_KNOWN_EXCEPTION;
    } 
    catch (...) 
    {
      std::cerr << "Caught generic exception\n";
      return E_GENERIC_EXCEPTION;
    }

  } else {

    usage(std::cerr);
    return E_WRONG_USAGE;

  }
}
