// File: plannerTest.cpp
// Author: Francesco Giacomini <Francesco.Giacomini@cnaf.infn.it>
// Copyright (c) 2003 EU DataGrid.
// For license conditions see http://www.eu-datagrid.org/license.html

// $Id: plannerTest.cpp,v 1.1 2003/03/27 14:49:58 giaco Exp $

#include <iostream>
#include <fstream>
#include <cstring>
#include <boost/scoped_ptr.hpp>
#include <classad_distribution.h>
#include "edg/workload/planning/manager/RequestPlanningPolicy.h"
#include "edg/workload/common/utilities/classad_utils.h"
#include "edg/workload/common/utilities/Exceptions.h"
#include "edg/workload/common/configuration/Configuration.h"
#include "edg/workload/common/requestad/convert.h"

namespace manager = edg::workload::planning::manager;
namespace utilities = edg::workload::common::utilities;
namespace requestad = edg::workload::common::requestad;
namespace configuration = edg::workload::common::configuration;

namespace {

struct Planner: manager::RequestPlanningPolicy
{
};

int const E_WRONG_USAGE       = 1;
int const E_KNOWN_EXCEPTION   = 2;
int const E_GENERIC_EXCEPTION = 3;
int const E_OTHER_ERROR       = 4;

std::string program_name;

void usage(std::ostream& os)
{
  os
    << "Usage: " << program_name << " --help\n"
    << "   or: " << program_name << " SOURCE DEST\n";
}

} // {anonymous}

int
main(int argc, char* argv[])
{
  program_name = argv[0];

  if (argc == 2 && std::string(argv[1]) == "--help") {

    usage(std::cout);
    return 0;

  } else if (argc != 3) {

    usage(std::cerr);
    return E_WRONG_USAGE;

  }

  char const* const input_file = argv[1];
  std::ifstream is(input_file);
  if (!is) {
    std::cerr << "Cannot open input file " << input_file << "\n";
    return E_OTHER_ERROR;
  }
  char const *const output_file = argv[2];
  std::ofstream os(output_file);
  if (!os) {
    std::cerr << "Cannot open output file" << output_file << "\n";
    return E_OTHER_ERROR;
  }

  try {
    configuration::Configuration config("edg_wl.conf",
                                        configuration::ModuleType::workload_manager);

    boost::scoped_ptr<classad::ClassAd> input_ad(utilities::parse_classad(is));
    boost::scoped_ptr<classad::ClassAd> output_ad(Planner().Plan(*input_ad));
    if (!output_ad) {
      std::cerr << "Planning failed\n";
      return E_OTHER_ERROR;
    }
    os << *output_ad;
    //   requestad::to_submit_stream(os, *output_ad);
  } catch (std::exception& e) {
    std::cerr << e.what() << "\n";
    return E_KNOWN_EXCEPTION;
  } catch (...) {
    std::cerr << "Caught generic exception\n";
    return E_GENERIC_EXCEPTION;
  }
}
