#include "grid_console.h"
/*
Bypass
Copyright (C) 1999-2001 Douglas Thain
http://www.cs.wisc.edu/condor/bypass
This program is released under the GNU General Public License.
See the file COPYING for details.
*/

extern "C" {
#include <stdarg.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <dlfcn.h>

#include <fcntl.h>
#include <unistd.h>
#include <dirent.h>
#include <signal.h>

#include <sys/stat.h>
#include <sys/poll.h>
#include <sys/socket.h>
}

#include "hashtable.h"
#include "layer.h"
#include "network.h"
#include "packet.h"
#include "external.h"
#include "external_unix.h"
#include "auth.h"
#include "private_malloc.h"

#ifdef linux
	extern "C" long syscall( long num, ... );
#else
	extern "C" int syscall( int num, ... );
#endif

#ifndef sgi
	#include <sys/syscall.h>
#else
	#include <sys.s>
#endif

/* macros should be outlawed */
#undef rewinddir
extern "C" int isatty( int fd );
extern "C" void _exit( int status );
extern "C" ssize_t write( int fd, const void* data, size_t length );
extern "C" ssize_t read( int fd, void* data, size_t length );
extern "C" off_t lseek( int fd, off_t offset, int whence );
extern "C" int close( int fd );
extern "C" int open( const char* path, int flags, ... );
extern "C" int poll( struct pollfd* ufds, unsigned long nfds, int timeout );
extern "C" int bypass_shadow_isatty( int fd );
extern "C" void bypass_shadow__exit( int status );
extern "C" ssize_t bypass_shadow_write( int fd, void* data, size_t length );
extern "C" ssize_t bypass_shadow_read( int fd, void* data, size_t length );
extern "C" off_t bypass_shadow_lseek( int fd, off_t offset, int whence );
extern "C" int bypass_shadow_close( int fd );
extern "C" int bypass_shadow_open( char* path, int flags, ... );
extern "C" int bypass_shadow_poll( struct pollfd* ufds, unsigned long nfds, int timeout );
/*
Bypass
Copyright (C) 1999-2001 Douglas Thain
http://www.cs.wisc.edu/condor/bypass
This program is released under the GNU General Public License.
See the file COPYING for details.
*/

#define TRY(x) if(!(x)) goto fail;

#if defined(linux)
	#define BYPASS_PRELOAD_LIST "LD_FAKE_PRELOAD"
	#define BYPASS_PRELOAD_SEPARATORS " \t"
	#define BYPASS_LIBC "libc.so.6"
	#define BYPASS_LIBPTHREAD "libpthread.so"
#elif defined(sun)
	#define BYPASS_PRELOAD_LIST "LD_FAKE_PRELOAD"
	#define BYPASS_PRELOAD_SEPARATORS " \t"
	#define BYPASS_LIBC "libc.so"
	#define BYPASS_LIBSOCKET "libsocket.so"
	#define BYPASS_LIBPTHREAD "libpthread.so"
#elif defined(__osf__) || defined(sgi)
	#define BYPASS_PRELOAD_LIST "_RLD_LIST"
	#define BYPASS_PRELOAD_SEPARATORS ":"
	#define BYPASS_LIBC "libc.so"
	#define BYPASS_LIBPTHREAD "libpthread.so"
#else
	#error "Unsupported operating system!"
#endif

#define BYPASS_PRELOAD_LIST_MAX 1024

char * bypass_call_string( int call );

/*
Note that these values are non-static so that they are
shared between all agents loaded into one process.
*/

int bypass_debug_mode = 0;
int bypass_failure_passthrough = 0;

void bypass_die()
{
	kill(getpid(),SIGKILL);
	exit(-1);
}

void bypass_vprintf( char *fmt, va_list args )
{
	char buffer[1024];

	syscall( SYS_write, 2, "bypass: ", 8 );
	vsprintf( buffer, fmt, args);
	syscall( SYS_write, 2, buffer, strlen(buffer) );
}

void bypass_debug( char *fmt, ... )
{
	va_list args;
	va_start(args,fmt);

	if(!bypass_debug_mode) return;
	bypass_vprintf( fmt, args );

	va_end(args);
}

void bypass_error( char *fmt, ... )
{
	va_list args;
	va_start(args,fmt);

	bypass_vprintf( fmt, args );

	va_end(args);
}

void bypass_fatal( char *fmt, ... )
{
	va_list args;
	va_start(args,fmt);

	bypass_vprintf( fmt, args );
	
	bypass_die();
	va_end(args);
}

void bypass_call_error( int call, char *message )
{	
	bypass_fatal("error while performing '%s': %s\n",bypass_call_string(call),message);
}

/*
This function is called inititalize the layer data structures.
it is called at the beginning of every trapped function, including
pthread_create.  Because it comes before any thread creation, it
does not need to be thread-safe.
*/

extern "C" void bypass_layer_init()
{
	static int init_done=0;
	static char listcopy[BYPASS_PRELOAD_LIST_MAX];
	char *list;
	char *item;

	if(init_done) {
		return;
	}

	init_done=1;

	if(getenv("BYPASS_DEBUG")) {
		bypass_debug_mode = 1;
		bypass_debug("debugging mode on\n");
	}

	if(getenv("BYPASS_FAILURE_PASSTHROUGH")) {
		bypass_failure_passthrough=1;
		bypass_debug("failure passthrough mode on\n");
	}

	list = getenv(BYPASS_PRELOAD_LIST);
	if(!list) {
		bypass_error("%s is not set!\n",BYPASS_PRELOAD_LIST);
		bypass_die();
	}

	strcpy(listcopy,list);

	item = strtok(listcopy,BYPASS_PRELOAD_SEPARATORS);
	while(item) {
		if(strcmp(item,"DEFAULT")) {
			layer_register(item,1);
		}
		item = strtok(0,BYPASS_PRELOAD_SEPARATORS);
	}

	#ifdef BYPASS_LIBSOCKET
	layer_register(BYPASS_LIBSOCKET,0);
	#endif

	layer_register(BYPASS_LIBC,0);

	if(bypass_debug_mode) {
		bypass_debug("layers in effect:\n");
		layer_dump();
	}

	/*
	Important: When initialized, layers start with no current
	layer at all.  Once the layer stack is set up, this thread
	gets put at the top.  Other threads created through pthread_create
	work differently.
	*/

	layer_top();
}

#ifdef USE_PTHREADS

/*
Thread creation is a little tricky.  The problem is that the new thread
must begin executing in the same layer as the thread that created it.
Further complicating matters is that the thread system itself may fork
manager threads that must run in the lowest layer.  So, all threads are
created by default in the lowest layer.  (see layer.c)  The initial thread
is moved to the top layer after registration (see bypass_layer_init), and
children of pthread create are moved to the layer of their parent.
(see bypass_thread_start)
*/

#include <pthread.h>

typedef void * (*pthread_start_t) ( void *arg );
typedef int (*pthread_create_t) ( pthread_t *tid, const pthread_attr_t *attr, pthread_start_t start, void *arg );

/*
This is the data structure passed from parent to child, and is used to 
*/

struct thread_start_info {
	struct layer_entry *layer;
	pthread_start_t start;
	void *arg;
};

static void * bypass_thread_start( void *arg )
{
	struct thread_start_info *info = (struct thread_start_info *) arg;
	layer_set( info->layer );
	return info->start( info->arg );
}

extern "C" int pthread_create ( pthread_t *tid, const pthread_attr_t *attr, void *(*start) (void*), void *arg )
{
	void *handle;
	pthread_create_t fptr;
	struct thread_start_info *info;
	struct layer_entry *layer;

	bypass_layer_init();

	info = (struct thread_start_info *) private_malloc(sizeof(*info));

	layer = layer_get();

	info->layer = layer;
	info->start = start;
	info->arg = arg;

	/*
	We manually open libpthread here instead of using the layer
	library, because we do not want to be scanning through multiple
	layers to find *any* pthread_create -- we want the real one.
	*/
	
	handle = dlopen(BYPASS_LIBPTHREAD,RTLD_LAZY);
	if(!handle) bypass_fatal("couldn't open %s\n",BYPASS_LIBPTHREAD);

	fptr = (pthread_create_t) dlsym(handle,"pthread_create");
	if(!fptr) bypass_fatal("couldn't find pthread_create in %s\n",BYPASS_LIBPTHREAD);

	return fptr( tid, attr, bypass_thread_start, (void*) info );
}

/*
Note that it is ok for this mutex to be static,
because the functions that access it are public
and shared.  We get faster access to the mutex by making it private.
*/

static pthread_mutex_t bypass_layer_mutex = PTHREAD_MUTEX_INITIALIZER;

extern "C" void layer_pthread_mutex_lock()
{
	pthread_mutex_lock(&bypass_layer_mutex);
}

extern "C" void layer_pthread_mutex_unlock()
{
	pthread_mutex_unlock(&bypass_layer_mutex);
}

extern "C" int  layer_pthread_self()
{
	return (int) pthread_self();
}

#else

#include <pthread.h>

extern "C" int pthread_create ( pthread_t *tid, const pthread_attr_t *attr, void *(*start) (void*), void *arg )
{
	bypass_error("Oops, you called pthread_create, but this agent does not support threaded programs.\n");
	bypass_fatal("Try rebuilding it with -DUSE_PTHREADS\n");
	return 1;
}

extern "C" void layer_pthread_mutex_lock()
{
}

extern "C" void layer_pthread_mutex_unlock()
{
}

extern "C" int  layer_pthread_self()
{
	return 1;
}

#endif

/*
Bypass
Copyright (C) 1999-2001 Douglas Thain
http://www.cs.wisc.edu/condor/bypass
This program is released under the GNU General Public License.
See the file COPYING for details.
*/

#ifdef USE_PTHREADS
	pthread_key_t bypass_rpc_key;
#else
	int bypass_rpc_fd=0;
#endif

char *bypass_rpc_host=0;
int bypass_rpc_port;

long int bypass_rpc_fd_get()
{
	#ifdef USE_PTHREADS
		return (long int)pthread_getspecific(bypass_rpc_key);
	#else
		return bypass_rpc_fd;
	#endif
}

void bypass_rpc_fd_set(long int fd )
{
	#ifdef USE_PTHREADS
		pthread_setspecific(bypass_rpc_key,(void*)fd);
	#else
		bypass_rpc_fd = fd;
	#endif
}

void bypass_rpc_env_init()
{
	char *txt;

	if(bypass_rpc_host) return;

	bypass_rpc_host = getenv("BYPASS_SHADOW_HOST");
	if(!bypass_rpc_host) bypass_fatal("I expected a hostname in BYPASS_SHADOW_HOST!\n");

	txt = getenv("BYPASS_SHADOW_PORT");
	if(!txt) bypass_fatal("I expected a port number in BYPASS_SHADOW_PORT!\n");

	bypass_rpc_port = atoi(txt);

	#ifdef USE_PTHREADS
		pthread_key_create(&bypass_rpc_key,0);
	#endif
}

int bypass_rpc_init()
{
	int fd;

	bypass_rpc_env_init();

	if(bypass_rpc_fd_get()) return 1;

	bypass_debug("connecting to %s port %d\n",bypass_rpc_host,bypass_rpc_port);

	fd = network_connect(bypass_rpc_host,bypass_rpc_port);
	if(fd<0) return 0;

	/* Attempt to get a high-numbered fd */
	if(dup2(fd,126)==126) {
		fd = 126;
	}

	network_tune( fd, NETWORK_TUNE_INTERACTIVE );

	auth_trivial_register();

	#ifdef USE_GLOBUS_GSS
		auth_globus_register();
	#endif

	if(!auth_assert(fd,bypass_debug_mode)) {
		network_close(fd);
		return 0;
	}

	bypass_rpc_fd_set(fd);

	return 1;
}

void bypass_rpc_close()
{
	if(bypass_rpc_fd_get()) {
		network_close(bypass_rpc_fd_get());
		bypass_rpc_fd_set(0);
	}
}


	extern "C" {
		#include <sys/types.h>
		#include <sys/stat.h>
		#include <string.h>
		#include <sys/time.h>
	}

#if __GNUC__ == 2
		struct stat64
		{
			unsigned long long st_dev;
			unsigned short int __pad1;
			unsigned long int st_ino;
			unsigned long int  st_mode;
			unsigned long int  st_nlink;
			unsigned long int  st_uid;
			unsigned long int  st_gid;
			unsigned long long st_rdev;
			unsigned short int __pad2;
			unsigned long long st_size;
			unsigned long int  st_blksize;
			unsigned long long st_blocks;
			unsigned long int  st_atime;
			unsigned long int  __unused1;
			unsigned long int  st_mtime;
			unsigned long int  __unused2;
			unsigned long int  st_ctime;
			unsigned long int  __unused3;
			unsigned long int  __unused4;
			unsigned long int  __unused5;
		};
#endif

		#define HAS_STAT64
		#define STAT64_TO_XSTAT		

		#define st_atim st_atime
		#define st_mtim st_mtime
		#define st_ctim st_ctime

		#ifndef _STAT_VER
			#define _STAT_VER 3
		#endif
	

	

	#ifdef HAS_STAT64
		void bypass_convert_stat64( int version, struct stat *source, struct stat64 *target )
		{
			memset( target, 0, sizeof(*target) );
			target->st_dev = source->st_dev;
			target->st_ino = source->st_ino;
			target->st_mode = source->st_mode;
			target->st_nlink = source->st_nlink;
			target->st_uid = source->st_uid;
			target->st_gid = source->st_gid;
			target->st_rdev = source->st_rdev;
			target->st_size = source->st_size;
			target->st_atim = source->st_atim;
			target->st_mtim = source->st_mtim;
			target->st_ctim = source->st_ctim;
			target->st_blksize = source->st_blksize;
			target->st_blocks = source->st_blocks;
			
		}
	#endif

	void bypass_convert_stat( int version, struct stat *source, struct stat *dest )
	{
		
			memcpy( dest, source, sizeof(*dest) );
		

	}

	#define XSTAT_SWITCH( entry, name, qname, qaname, type_one, type_two, type_three, cast, convert ) 		extern "C" int entry ( type_one v, type_two n, type_three * s )		{			int result;			struct stat buf;			int (*fptr)(...);			bypass_layer_init();			fptr = (int(*)(...)) layer_lookup( qaname, qname, (int)entry );			if(!fptr) fptr = (int(*)(...)) entry;			if(!layer_is_agent() || fptr==(int(*)(...))entry ) {				layer_descend();				result = (*fptr) ( v, n, s );				layer_ascend();			} else {				layer_descend();				result = (*fptr) ( n, &buf );				convert ( v, &buf, s );				layer_ascend();			}			return result;		}

	#define PLAIN_SWITCH( name, qname, qaname, type_one, type_two, cast ) 		extern "C" int name ( type_one n, type_two *s ) 		{			int result;			int (*fptr)(...);			bypass_layer_init();			fptr = (int(*)(...)) layer_lookup( qaname, qname, (int)name );			if(!fptr) fptr = (int(*)(...)) name;			layer_descend();			result = (*fptr) ( n, s );			layer_ascend();			return result;		}

	#define PLAIN64_SWITCH( name, qname, qaname, type_one, type_two, cast ) 		extern "C" int name ( type_one n, type_two *s ) 		{			int result;			struct stat buf;			int (*fptr)(...);			bypass_layer_init();			fptr = (int(*)(...)) layer_lookup( qaname, qname, (int)name );			if(!fptr) fptr = (int(*)(...)) name;			if(!layer_is_agent() || fptr==(int(*)(...))name ) {				layer_descend();				result = (*fptr) ( n, s );				layer_ascend();			} else {				layer_descend();				result = (*fptr) ( n, &buf );				bypass_convert_stat64 ( 3, &buf, s );				layer_ascend();			}			return result;		}

	#define PLAIN_ENTRIES( entry, type_one, type_two ) 		extern "C" int _##entry ( type_one path, type_two * buf ) {			return entry ( path, buf );		}		extern "C" int __##entry ( type_one path, type_two * buf ) {			return entry ( path, buf );		}

	#define XSTAT_ENTRIES( entry, type_one, type_two, calls, version, cast ) 		extern "C" int entry ( type_one path, type_two * buf ) {			return calls ( version, path, (cast) buf );		}		extern "C" int _##entry ( type_one path, type_two * buf ) {			return calls ( version, path, (cast) buf );		}		extern "C" int __##entry ( type_one path, type_two * buf ) {			return calls ( version, path, (cast) buf );		}


	#include <stdio.h>
	#include <stdlib.h>
	#include <unistd.h>
	#include <limits.h>
	#include <ctype.h>
	#include <time.h>
	
	#define BUFFER_SIZE 131072
	static char buffer[BUFFER_SIZE];

	enum grid_console_mode {
		NONE,
		LOCAL,
		REMOTE
	};

	struct grid_console_state {
		int	fd;
		enum grid_console_mode mode;
		off_t	offset;
		off_t	chunk_start;
		char	*env;
		char	*remote_name;
		char	local_name[_POSIX_PATH_MAX];
		char	*common_name;
		int	open_flags;
		int	open_mode;
	};

	static struct grid_console_state grid_console_table[3] = {
		{ 0, NONE, 0, 0, "GRID_CONSOLE_STDIN", 0, "", "standard input",O_RDONLY,0666},
		{ 0, NONE, 0, 0, "GRID_CONSOLE_STDOUT", 0, "", "standard output",O_CREAT|O_WRONLY,0666},
		{ 0, NONE, 0, 0, "GRID_CONSOLE_STDERR", 0, "", "standard error",O_CREAT|O_WRONLY,0666},
	};

	static int grid_console_no_delay=0;
	static int grid_console_retry_limit=10;
	static int grid_console_retry_timeout=30;
	static int grid_console_debug_mode=0;
	static char *grid_console_temp_dir=0;

	static void grid_console_debug( char *fmt, ... )
	{
		va_list args;
		va_start(args,fmt);

		if(grid_console_debug_mode) {
			fprintf(stderr,"grid_console: ");
			vfprintf(stderr,fmt,args);
		}

		va_end(args);
	}

	static void grid_console_die()
	{
		bypass_die();
	}

	static void grid_console_init()
	{
		static int init_done=0;
		char *str;
		int i;

		if(init_done) return;

		bypass_failure_passthrough=1;

		for(i=0;i<3;i++) {
			grid_console_table[i].remote_name = getenv(grid_console_table[i].env);
		}

		str=getenv("GRID_CONSOLE_NO_DELAY");
		if(str) grid_console_no_delay=1;

		str=getenv("GRID_CONSOLE_RETRY_LIMIT");
		if(str) grid_console_retry_limit=atoi(str);

		str=getenv("GRID_CONSOLE_RETRY_TIMEOUT");
		if(str) grid_console_retry_timeout=atoi(str);

		str=getenv("GRID_CONSOLE_DEBUG");
		if(str) grid_console_debug_mode=1;

		str=getenv("GRID_CONSOLE_TEMP_DIR");
		if(str) {
			grid_console_temp_dir=str;
		} else {
			grid_console_temp_dir="/tmp";
		}

		grid_console_debug("init done\n");
	
		init_done=1;
	}

	static int grid_console_remote( int fd )
	{
		int result, actual;
		struct grid_console_state *f;
		int rfd;

		f = &grid_console_table[fd];

		if(f->mode==REMOTE) return 1;

		if(!f->remote_name) {
			f->fd = fd;
			f->mode = LOCAL;
			return 1;
		} else if(!strcmp(f->remote_name,"-")) {
			rfd = fd;
		} else {
			rfd = bypass_shadow_open(f->remote_name,f->open_flags,f->open_mode);
			if(rfd<0) return 0;
		}

		if(f->mode==LOCAL) {
			lseek(f->fd,0,SEEK_SET);
			bypass_shadow_lseek(rfd,f->chunk_start,SEEK_SET);

			while(1) {
				actual = read(f->fd,buffer,BUFFER_SIZE);
				if(actual==0) break;
				if(actual<0) return 0;

				result = bypass_shadow_write(rfd,buffer,actual);
				if(result!=actual) return 0;
			}
			close(f->fd);
		}

		f->fd=rfd;
		f->mode=REMOTE;

		return 1;
	}

	static int grid_console_local( int fd )
	{
		struct grid_console_state *f;
		f = &grid_console_table[fd];

		if(f->mode==LOCAL) return 1;
		f->mode = NONE;

		sprintf(f->local_name,"%s/grid_console-XXXXXX",grid_console_temp_dir);
		f->fd = mkstemp(f->local_name);
		if(f->fd<0) return 0;
		unlink(f->local_name);

		f->chunk_start = f->offset;
		f->mode = LOCAL;

		return 1;
	}

extern "C" int bypass_shadow_isatty( int fd )
{
	struct packet *bypass_packet=0;
	int bypass_errno=0;
	int bypass_number = BYPASS_CALL_isatty;
int result;
	if(!bypass_rpc_init()) goto fail;
	TRY(bypass_packet = packet_create(0));
	TRY(external(bypass_packet,EXTERNAL_OUT,&bypass_number));
		TRY(external(bypass_packet,EXTERNAL_OUT,&fd));
	TRY(packet_put(bypass_rpc_fd_get(),bypass_packet));
	packet_delete(bypass_packet);
	bypass_packet=0;

	TRY(bypass_packet = packet_get(bypass_rpc_fd_get()));
	TRY(external_errno_map(bypass_packet,EXTERNAL_IN,&bypass_errno));
		TRY(external(bypass_packet,EXTERNAL_IN,&result));
	packet_delete(bypass_packet);
	errno = bypass_errno;
	return result;

	fail:
	bypass_errno = errno;
	if(bypass_packet) packet_delete(bypass_packet);
	bypass_rpc_close();
	char message[1024];
	sprintf(message,"couldn't execute %s: %s\n",bypass_call_string(bypass_number),strerror(bypass_errno));
	if(bypass_failure_passthrough) {
		bypass_debug(message);
	} else {
		bypass_fatal(message);
	}
	errno = bypass_errno;
	return -1;
}

extern "C" int bypass_agent_action_isatty( int fd )
{
	
		grid_console_init();
		if(fd<=2) {
			return grid_console_no_delay;
		} else {
			return isatty(fd);
		}
	

}

extern "C" int isatty( int fd )
{
	int (*fptr)(int fd);
	int result;
	bypass_layer_init();
	fptr = (int(*)(int fd)) layer_lookup( "bypass_agent_action_isatty", "isatty", (long int) isatty );
	if(!fptr) fptr = isatty;
	layer_descend();
	result = (*fptr) ( fd );
	layer_ascend();
	return result;
}

extern "C" int _isatty( int fd )
{
	int (*fptr)(int fd);
	int result;
	bypass_layer_init();
	fptr = (int(*)(int fd)) layer_lookup( "bypass_agent_action_isatty", "_isatty", (long int) _isatty );
	if(!fptr) fptr = _isatty;
	layer_descend();
	result = (*fptr) ( fd );
	layer_ascend();
	return result;
}

extern "C" int __isatty( int fd )
{
	int (*fptr)(int fd);
	int result;
	bypass_layer_init();
	fptr = (int(*)(int fd)) layer_lookup( "bypass_agent_action_isatty", "__isatty", (long int) __isatty );
	if(!fptr) fptr = __isatty;
	layer_descend();
	result = (*fptr) ( fd );
	layer_ascend();
	return result;
}

extern "C" int __libc_isatty( int fd )
{
	int (*fptr)(int fd);
	int result;
	bypass_layer_init();
	fptr = (int(*)(int fd)) layer_lookup( "bypass_agent_action_isatty", "__libc_isatty", (long int) __libc_isatty );
	if(!fptr) fptr = __libc_isatty;
	layer_descend();
	result = (*fptr) ( fd );
	layer_ascend();
	return result;
}

extern "C" void bypass_shadow__exit( int status )
{
	struct packet *bypass_packet=0;
	int bypass_errno=0;
	int bypass_number = BYPASS_CALL__exit;
	if(!bypass_rpc_init()) goto fail;
	TRY(bypass_packet = packet_create(0));
	TRY(external(bypass_packet,EXTERNAL_OUT,&bypass_number));
		TRY(external(bypass_packet,EXTERNAL_OUT,&status));
	TRY(packet_put(bypass_rpc_fd_get(),bypass_packet));
	packet_delete(bypass_packet);
	bypass_packet=0;

	TRY(bypass_packet = packet_get(bypass_rpc_fd_get()));
	TRY(external_errno_map(bypass_packet,EXTERNAL_IN,&bypass_errno));
	packet_delete(bypass_packet);
	errno = bypass_errno;
	return;

	fail:
	bypass_errno = errno;
	if(bypass_packet) packet_delete(bypass_packet);
	bypass_rpc_close();
	char message[1024];
	sprintf(message,"couldn't execute %s: %s\n",bypass_call_string(bypass_number),strerror(bypass_errno));
	if(bypass_failure_passthrough) {
		bypass_debug(message);
	} else {
		bypass_fatal(message);
	}
	errno = bypass_errno;
	return ;
}

extern "C" void bypass_agent_action__exit( int status )
{
	
		int i;
		int fd;
		struct grid_console_state *f;

		grid_console_init();
		grid_console_debug("_exit(%d) in progress\n",status);

		for(fd=1;fd<3;fd++) {
			f = &grid_console_table[fd];
			for(i=0;!grid_console_retry_limit||i<grid_console_retry_limit;i++) {
				grid_console_debug("committing %s\n",f->common_name);
				if(grid_console_remote(fd)) {
					grid_console_debug("done\n");
					break;	
				} else {
					grid_console_debug("%s write attempt %d failed: %s\n",f->common_name,i,strerror(errno));
					grid_console_debug("sleeping %d seconds\n",grid_console_retry_timeout);
					sleep(grid_console_retry_timeout);
				}
			}
			if(i && i==grid_console_retry_limit) {
				grid_console_debug("giving up after %d attempts\n",i);
				grid_console_die();
			}	
		}
		grid_console_debug("all streams committed, continuing with _exit(%d)\n",status);
		_exit(status);
	

}

extern "C" void _exit( int status )
{
	void (*fptr)(int status);
	bypass_layer_init();
	fptr = (void(*)(int status)) layer_lookup( "bypass_agent_action__exit", "_exit", (long int) _exit );
	if(!fptr) fptr = _exit;
	layer_descend();
	(*fptr) ( status );
	layer_ascend();
	bypass_fatal("exit() returned without exiting!");
}

extern "C" ssize_t bypass_shadow_write( int fd, void* data, size_t length )
{
	struct packet *bypass_packet=0;
	int bypass_errno=0;
	int bypass_number = BYPASS_CALL_write;
ssize_t result;
	if(!bypass_rpc_init()) goto fail;
	TRY(bypass_packet = packet_create(0));
	TRY(external(bypass_packet,EXTERNAL_OUT,&bypass_number));
		TRY(external(bypass_packet,EXTERNAL_OUT,&fd));
		TRY(external_opaque(bypass_packet,EXTERNAL_OUT,(char**)&data,length));
		TRY(external(bypass_packet,EXTERNAL_OUT,&length));
	TRY(packet_put(bypass_rpc_fd_get(),bypass_packet));
	packet_delete(bypass_packet);
	bypass_packet=0;

	TRY(bypass_packet = packet_get(bypass_rpc_fd_get()));
	TRY(external_errno_map(bypass_packet,EXTERNAL_IN,&bypass_errno));
		TRY(external(bypass_packet,EXTERNAL_IN,&result));
	packet_delete(bypass_packet);
	errno = bypass_errno;
	return result;

	fail:
	bypass_errno = errno;
	if(bypass_packet) packet_delete(bypass_packet);
	bypass_rpc_close();
	char message[1024];
	sprintf(message,"couldn't execute %s: %s\n",bypass_call_string(bypass_number),strerror(bypass_errno));
	if(bypass_failure_passthrough) {
		bypass_debug(message);
	} else {
		bypass_fatal(message);
	}
	errno = bypass_errno;
	return -1;
}

extern "C" int bypass_grid_console_remote( int fd )
{
	int (*fptr)(int fd);
	bypass_layer_init();
	fptr = (int(*)(int fd)) layer_lookup( "bypass_agent_grid_console_remote", "bypass_agent_grid_console_remote", 0 );
	if (fptr) return( (*fptr) ( fd ));
	else return(1);
}

extern "C" int bypass_agent_grid_console_remote( int fd )
{
	/* Explicitely try to synchronize with remote end */
	return grid_console_remote(fd);
}

extern "C" ssize_t bypass_agent_action_write( int fd, const void* data, size_t length )
{
	
		int i,result;
		struct grid_console_state *f;
		static time_t last_check=0;
		time_t current;

		grid_console_init();

		if(fd>=3) return write(fd,data,length);

		for(i=0;!grid_console_retry_limit||i<grid_console_retry_limit;i++) {
			f = &grid_console_table[fd];

			if(f->mode!=REMOTE) {
				current = time(0);
				if(f->mode==NONE || (current-last_check)>grid_console_retry_timeout ) {
					grid_console_debug("testing %s on remote disk\n",f->common_name);
					if(grid_console_remote(fd)) {
						grid_console_debug("reconnected %s to remote disk\n",f->common_name);
						i=0;
					} else {
						grid_console_debug("still can't send %s to remote disk: %s\n",f->common_name,strerror(errno));
						grid_console_debug("continuing with local disk\n");
					}

					last_check = current;
				}
			}

			switch(f->mode) {
				case NONE:
					result=-1;
					break;
				case LOCAL:
					result = write(f->fd,data,length);
					break;
				case REMOTE:
					bypass_shadow_lseek(f->fd,f->offset,SEEK_SET);
					result = bypass_shadow_write(f->fd,(void*)data,length);
					break;
			}

			if(result>=0) {
				f->offset+=result;
				return result;
			} else {
				switch(f->mode) {
					case NONE:
						if(grid_console_remote(fd)) {
							grid_console_debug("sending %s to remote disk\n",f->common_name);
							i=0;
							continue;
						}
					case REMOTE:
						grid_console_debug("couldn't send %s to remote disk: %s\n",f->common_name,strerror(errno));
						if(grid_console_local(fd)) {
							grid_console_debug("sending %s to local disk\n",f->common_name);
							i=0;
							continue;
						}
					case LOCAL:
						grid_console_debug("%s write attempt %d failed: %s\n",f->common_name,i,strerror(errno));
						grid_console_debug("sleeping %d seconds\n",grid_console_retry_timeout);
						sleep(grid_console_retry_timeout);
						break;
				}
			}
		}
		grid_console_debug("giving up after %d attempts\n",i);
		grid_console_die();
		return -1;
	

}

extern "C" ssize_t write( int fd, const void* data, size_t length )
{
	ssize_t (*fptr)(int fd, const void* data, size_t length);
	ssize_t result;
	bypass_layer_init();
	fptr = (ssize_t(*)(int fd, const void* data, size_t length)) layer_lookup( "bypass_agent_action_write", "write", (long int) write );
	if(!fptr) fptr = write;
	layer_descend();
	result = (*fptr) ( fd, data, length );
	layer_ascend();
	return result;
}

extern "C" ssize_t _write( int fd, const void* data, size_t length )
{
	ssize_t (*fptr)(int fd, const void* data, size_t length);
	ssize_t result;
	bypass_layer_init();
	fptr = (ssize_t(*)(int fd, const void* data, size_t length)) layer_lookup( "bypass_agent_action_write", "_write", (long int) _write );
	if(!fptr) fptr = _write;
	layer_descend();
	result = (*fptr) ( fd, data, length );
	layer_ascend();
	return result;
}

extern "C" ssize_t __write( int fd, const void* data, size_t length )
{
	ssize_t (*fptr)(int fd, const void* data, size_t length);
	ssize_t result;
	bypass_layer_init();
	fptr = (ssize_t(*)(int fd, const void* data, size_t length)) layer_lookup( "bypass_agent_action_write", "__write", (long int) __write );
	if(!fptr) fptr = __write;
	layer_descend();
	result = (*fptr) ( fd, data, length );
	layer_ascend();
	return result;
}

extern "C" ssize_t __libc_write( int fd, const void* data, size_t length )
{
	ssize_t (*fptr)(int fd, const void* data, size_t length);
	ssize_t result;
	bypass_layer_init();
	fptr = (ssize_t(*)(int fd, const void* data, size_t length)) layer_lookup( "bypass_agent_action_write", "__libc_write", (long int) __libc_write );
	if(!fptr) fptr = __libc_write;
	layer_descend();
	result = (*fptr) ( fd, data, length );
	layer_ascend();
	return result;
}

extern "C" ssize_t bypass_shadow_read( int fd, void* data, size_t length )
{
	struct packet *bypass_packet=0;
	int bypass_errno=0;
	int bypass_number = BYPASS_CALL_read;
ssize_t result;
	if(!bypass_rpc_init()) goto fail;
	TRY(bypass_packet = packet_create(0));
	TRY(external(bypass_packet,EXTERNAL_OUT,&bypass_number));
		TRY(external(bypass_packet,EXTERNAL_OUT,&fd));
		TRY(external(bypass_packet,EXTERNAL_OUT,&length));
	TRY(packet_put(bypass_rpc_fd_get(),bypass_packet));
	packet_delete(bypass_packet);
	bypass_packet=0;

	TRY(bypass_packet = packet_get(bypass_rpc_fd_get()));
	TRY(external_errno_map(bypass_packet,EXTERNAL_IN,&bypass_errno));
		TRY(external(bypass_packet,EXTERNAL_IN,&result));
		TRY(external_opaque(bypass_packet,EXTERNAL_IN,(char**)&data,length));
	packet_delete(bypass_packet);
	errno = bypass_errno;
	return result;

	fail:
	bypass_errno = errno;
	if(bypass_packet) packet_delete(bypass_packet);
	bypass_rpc_close();
	char message[1024];
	sprintf(message,"couldn't execute %s: %s\n",bypass_call_string(bypass_number),strerror(bypass_errno));
	if(bypass_failure_passthrough) {
		bypass_debug(message);
	} else {
		bypass_fatal(message);
	}
	errno = bypass_errno;
	return -1;
}

extern "C" ssize_t bypass_agent_action_read( int fd, void* data, size_t length )
{
	
		int i;
		int result;
		struct grid_console_state *f;

		grid_console_init();

		f = &grid_console_table[fd];

		if(fd>=3) return read(fd,data,length);

		for( i=0; !grid_console_retry_limit || i<grid_console_retry_limit; i++ ) {
			if(grid_console_remote(fd)) {
				if(f->mode==REMOTE) {
					result = bypass_shadow_read(f->fd,data,length);
					if(result>=0) return result;
				} else {
					result = read(f->fd,data,length);
					if(result>=0) return result;
				}
			}
			grid_console_debug("%s read attempt %d failed: %s\n",f->common_name,i,strerror(errno));
			grid_console_debug("sleeping %d seconds\n",grid_console_retry_timeout);
			sleep(grid_console_retry_timeout);
		}
		grid_console_debug("giving up after %d attempts\n",i);
		grid_console_die();
		return -1;
	

}

extern "C" ssize_t read( int fd, void* data, size_t length )
{
	ssize_t (*fptr)(int fd, void* data, size_t length);
	ssize_t result;
	bypass_layer_init();
	fptr = (ssize_t(*)(int fd, void* data, size_t length)) layer_lookup( "bypass_agent_action_read", "read", (long int) read );
	if(!fptr) fptr = read;
	layer_descend();
	result = (*fptr) ( fd, data, length );
	layer_ascend();
	return result;
}

extern "C" ssize_t _read( int fd, void* data, size_t length )
{
	ssize_t (*fptr)(int fd, void* data, size_t length);
	ssize_t result;
	bypass_layer_init();
	fptr = (ssize_t(*)(int fd, void* data, size_t length)) layer_lookup( "bypass_agent_action_read", "_read", (long int) _read );
	if(!fptr) fptr = _read;
	layer_descend();
	result = (*fptr) ( fd, data, length );
	layer_ascend();
	return result;
}

extern "C" ssize_t __read( int fd, void* data, size_t length )
{
	ssize_t (*fptr)(int fd, void* data, size_t length);
	ssize_t result;
	bypass_layer_init();
	fptr = (ssize_t(*)(int fd, void* data, size_t length)) layer_lookup( "bypass_agent_action_read", "__read", (long int) __read );
	if(!fptr) fptr = __read;
	layer_descend();
	result = (*fptr) ( fd, data, length );
	layer_ascend();
	return result;
}

extern "C" ssize_t __libc_read( int fd, void* data, size_t length )
{
	ssize_t (*fptr)(int fd, void* data, size_t length);
	ssize_t result;
	bypass_layer_init();
	fptr = (ssize_t(*)(int fd, void* data, size_t length)) layer_lookup( "bypass_agent_action_read", "__libc_read", (long int) __libc_read );
	if(!fptr) fptr = __libc_read;
	layer_descend();
	result = (*fptr) ( fd, data, length );
	layer_ascend();
	return result;
}

extern "C" off_t bypass_shadow_lseek( int fd, off_t offset, int whence )
{
	struct packet *bypass_packet=0;
	int bypass_errno=0;
	int bypass_number = BYPASS_CALL_lseek;
off_t result;
	if(!bypass_rpc_init()) goto fail;
	TRY(bypass_packet = packet_create(0));
	TRY(external(bypass_packet,EXTERNAL_OUT,&bypass_number));
		TRY(external(bypass_packet,EXTERNAL_OUT,&fd));
		TRY(external(bypass_packet,EXTERNAL_OUT,&offset));
		TRY(external(bypass_packet,EXTERNAL_OUT,&whence));
	TRY(packet_put(bypass_rpc_fd_get(),bypass_packet));
	packet_delete(bypass_packet);
	bypass_packet=0;

	TRY(bypass_packet = packet_get(bypass_rpc_fd_get()));
	TRY(external_errno_map(bypass_packet,EXTERNAL_IN,&bypass_errno));
		TRY(external(bypass_packet,EXTERNAL_IN,&result));
	packet_delete(bypass_packet);
	errno = bypass_errno;
	return result;

	fail:
	bypass_errno = errno;
	if(bypass_packet) packet_delete(bypass_packet);
	bypass_rpc_close();
	char message[1024];
	sprintf(message,"couldn't execute %s: %s\n",bypass_call_string(bypass_number),strerror(bypass_errno));
	if(bypass_failure_passthrough) {
		bypass_debug(message);
	} else {
		bypass_fatal(message);
	}
	errno = bypass_errno;
	return -1;
}

extern "C" off_t bypass_agent_action_lseek( int fd, off_t offset, int whence )
{
	
		return lseek(fd,offset,whence);
	

}

/* lseek has additional code from 'also' statement */

		extern "C" long long llseek( int fd, long long offset, int whence ) {
			return lseek( fd, offset, whence );
		}
		extern "C" long long _llseek( int fd, long long offset, int whence ) {
			return lseek( fd, offset, whence );
		}
		extern "C" long long __llseek( int fd, long long offset, int whence ) {
			return lseek( fd, offset, whence );
		}
		extern "C" long long __libc_llseek( int fd, long long offset, int whence ) {
			return lseek( fd, offset, whence );
		}
		/*extern "C" long long lseek64( int fd, long long offset, int whence ) {
			return lseek( fd, offset, whence );
		}*/
		extern "C" long long _lseek64( int fd, long long offset, int whence ) {
			return lseek( fd, offset, whence );
		}
		extern "C" long long __lseek64( int fd, long long offset, int whence ) {
			return lseek( fd, offset, whence );
		}
		extern "C" long long __libc_lseek64( int fd, long long offset, int whence ) {
			return lseek( fd, offset, whence );
		}

		#ifdef __GLIBC__
		extern "C" _IO_off64_t _IO_file_seek ( _IO_FILE *fp, _IO_off64_t offset, int dir )
		{
		  return lseek (fp->_fileno, offset, dir);
		}
		#endif

extern "C" off_t lseek( int fd, off_t offset, int whence )
{
	off_t (*fptr)(int fd, off_t offset, int whence);
	off_t result;
	bypass_layer_init();
	fptr = (off_t(*)(int fd, off_t offset, int whence)) layer_lookup( "bypass_agent_action_lseek", "lseek", (long int) lseek );
	if(!fptr) fptr = lseek;
	layer_descend();
	result = (*fptr) ( fd, offset, whence );
	layer_ascend();
	return result;
}

extern "C" off_t _lseek( int fd, off_t offset, int whence )
{
	off_t (*fptr)(int fd, off_t offset, int whence);
	off_t result;
	bypass_layer_init();
	fptr = (off_t(*)(int fd, off_t offset, int whence)) layer_lookup( "bypass_agent_action_lseek", "_lseek", (long int) _lseek );
	if(!fptr) fptr = _lseek;
	layer_descend();
	result = (*fptr) ( fd, offset, whence );
	layer_ascend();
	return result;
}

extern "C" off_t __lseek( int fd, off_t offset, int whence )
{
	off_t (*fptr)(int fd, off_t offset, int whence);
	off_t result;
	bypass_layer_init();
	fptr = (off_t(*)(int fd, off_t offset, int whence)) layer_lookup( "bypass_agent_action_lseek", "__lseek", (long int) __lseek );
	if(!fptr) fptr = __lseek;
	layer_descend();
	result = (*fptr) ( fd, offset, whence );
	layer_ascend();
	return result;
}

extern "C" off_t __libc_lseek( int fd, off_t offset, int whence )
{
	off_t (*fptr)(int fd, off_t offset, int whence);
	off_t result;
	bypass_layer_init();
	fptr = (off_t(*)(int fd, off_t offset, int whence)) layer_lookup( "bypass_agent_action_lseek", "__libc_lseek", (long int) __libc_lseek );
	if(!fptr) fptr = __libc_lseek;
	layer_descend();
	result = (*fptr) ( fd, offset, whence );
	layer_ascend();
	return result;
}

extern "C" int bypass_shadow_close( int fd )
{
	struct packet *bypass_packet=0;
	int bypass_errno=0;
	int bypass_number = BYPASS_CALL_close;
int result;
	if(!bypass_rpc_init()) goto fail;
	TRY(bypass_packet = packet_create(0));
	TRY(external(bypass_packet,EXTERNAL_OUT,&bypass_number));
		TRY(external(bypass_packet,EXTERNAL_OUT,&fd));
	TRY(packet_put(bypass_rpc_fd_get(),bypass_packet));
	packet_delete(bypass_packet);
	bypass_packet=0;

	TRY(bypass_packet = packet_get(bypass_rpc_fd_get()));
	TRY(external_errno_map(bypass_packet,EXTERNAL_IN,&bypass_errno));
		TRY(external(bypass_packet,EXTERNAL_IN,&result));
	packet_delete(bypass_packet);
	errno = bypass_errno;
	return result;

	fail:
	bypass_errno = errno;
	if(bypass_packet) packet_delete(bypass_packet);
	bypass_rpc_close();
	char message[1024];
	sprintf(message,"couldn't execute %s: %s\n",bypass_call_string(bypass_number),strerror(bypass_errno));
	if(bypass_failure_passthrough) {
		bypass_debug(message);
	} else {
		bypass_fatal(message);
	}
	errno = bypass_errno;
	return -1;
}

extern "C" int bypass_agent_action_close( int fd )
{
	
		return close(fd);
	

}

extern "C" int close( int fd )
{
	int (*fptr)(int fd);
	int result;
	bypass_layer_init();
	fptr = (int(*)(int fd)) layer_lookup( "bypass_agent_action_close", "close", (long int) close );
	if(!fptr) fptr = close;
	layer_descend();
	result = (*fptr) ( fd );
	layer_ascend();
	return result;
}

extern "C" int _close( int fd )
{
	int (*fptr)(int fd);
	int result;
	bypass_layer_init();
	fptr = (int(*)(int fd)) layer_lookup( "bypass_agent_action_close", "_close", (long int) _close );
	if(!fptr) fptr = _close;
	layer_descend();
	result = (*fptr) ( fd );
	layer_ascend();
	return result;
}

extern "C" int __close( int fd )
{
	int (*fptr)(int fd);
	int result;
	bypass_layer_init();
	fptr = (int(*)(int fd)) layer_lookup( "bypass_agent_action_close", "__close", (long int) __close );
	if(!fptr) fptr = __close;
	layer_descend();
	result = (*fptr) ( fd );
	layer_ascend();
	return result;
}

extern "C" int __libc_close( int fd )
{
	int (*fptr)(int fd);
	int result;
	bypass_layer_init();
	fptr = (int(*)(int fd)) layer_lookup( "bypass_agent_action_close", "__libc_close", (long int) __libc_close );
	if(!fptr) fptr = __libc_close;
	layer_descend();
	result = (*fptr) ( fd );
	layer_ascend();
	return result;
}

extern "C" int bypass_shadow_open( char* path, int flags, ... )
{
	struct packet *bypass_packet=0;
	int bypass_errno=0;
	int bypass_number = BYPASS_CALL_open;
int result;
	va_list bypass_arglist;
	va_start(bypass_arglist,flags);
	int	mode;
	mode = va_arg(bypass_arglist,int);
	va_end(bypass_arglist);
	if(!bypass_rpc_init()) goto fail;
	TRY(bypass_packet = packet_create(0));
	TRY(external(bypass_packet,EXTERNAL_OUT,&bypass_number));
		TRY(external_string(bypass_packet,EXTERNAL_OUT,&path,0));
		TRY(external_open_map(bypass_packet,EXTERNAL_OUT,&flags));
		TRY(external(bypass_packet,EXTERNAL_OUT,&mode));
	TRY(packet_put(bypass_rpc_fd_get(),bypass_packet));
	packet_delete(bypass_packet);
	bypass_packet=0;

	TRY(bypass_packet = packet_get(bypass_rpc_fd_get()));
	TRY(external_errno_map(bypass_packet,EXTERNAL_IN,&bypass_errno));
		TRY(external(bypass_packet,EXTERNAL_IN,&result));
	packet_delete(bypass_packet);
	errno = bypass_errno;
	return result;

	fail:
	bypass_errno = errno;
	if(bypass_packet) packet_delete(bypass_packet);
	bypass_rpc_close();
	char message[1024];
	sprintf(message,"couldn't execute %s: %s\n",bypass_call_string(bypass_number),strerror(bypass_errno));
	if(bypass_failure_passthrough) {
		bypass_debug(message);
	} else {
		bypass_fatal(message);
	}
	errno = bypass_errno;
	return -1;
}

extern "C" int bypass_agent_action_open( const char* path, int flags, ... )
{
	va_list bypass_arglist;
	va_start(bypass_arglist,flags);
	int	mode;
	mode = va_arg(bypass_arglist,int);
	va_end(bypass_arglist);
	
		return open(path,flags,mode);
	

}

/* open has additional code from 'also' statement */

		extern "C" int creat( const char *path, mode_t mode ) {
			return open( path, O_CREAT|O_WRONLY|O_TRUNC, mode );
		}
		extern "C" int _creat( const char *path, mode_t mode ) {
			return creat( path, mode );
		}
		extern "C" int __creat( const char *path, mode_t mode ) {
			return creat( path, mode );
		}
		extern "C" int creat64( const char *path, mode_t mode ) {
			return creat( path, mode );
		}
		extern "C" int _creat64( const char *path, mode_t mode ) {
			return creat( path, mode );
		}
		extern "C" int __creat64( const char *path, mode_t mode ) {
			return creat( path, mode );
		}

		 

	
extern "C" int open( const char* path, int flags, ... )
{
	int (*fptr)(const char* path, int flags, ...);
	int result;
	va_list bypass_arglist;
	va_start(bypass_arglist,flags);
	int	mode;
	mode = va_arg(bypass_arglist,int);
	va_end(bypass_arglist);
	bypass_layer_init();
	fptr = (int(*)(const char* path, int flags, ...)) layer_lookup( "bypass_agent_action_open", "open", (long int) open );
	if(!fptr) fptr = open;
	layer_descend();
	result = (*fptr) ( path, flags, mode );
	layer_ascend();
	return result;
}

extern "C" int _open( const char* path, int flags, ... )
{
	int (*fptr)(const char* path, int flags, ...);
	int result;
	va_list bypass_arglist;
	va_start(bypass_arglist,flags);
	int	mode;
	mode = va_arg(bypass_arglist,int);
	va_end(bypass_arglist);
	bypass_layer_init();
	fptr = (int(*)(const char* path, int flags, ...)) layer_lookup( "bypass_agent_action_open", "_open", (long int) _open );
	if(!fptr) fptr = _open;
	layer_descend();
	result = (*fptr) ( path, flags, mode );
	layer_ascend();
	return result;
}

extern "C" int __open( const char* path, int flags, ... )
{
	int (*fptr)(const char* path, int flags, ...);
	int result;
	va_list bypass_arglist;
	va_start(bypass_arglist,flags);
	int	mode;
	mode = va_arg(bypass_arglist,int);
	va_end(bypass_arglist);
	bypass_layer_init();
	fptr = (int(*)(const char* path, int flags, ...)) layer_lookup( "bypass_agent_action_open", "__open", (long int) __open );
	if(!fptr) fptr = __open;
	layer_descend();
	result = (*fptr) ( path, flags, mode );
	layer_ascend();
	return result;
}

extern "C" int __libc_open( const char* path, int flags, ... )
{
	int (*fptr)(const char* path, int flags, ...);
	int result;
	va_list bypass_arglist;
	va_start(bypass_arglist,flags);
	int	mode;
	mode = va_arg(bypass_arglist,int);
	va_end(bypass_arglist);
	bypass_layer_init();
	fptr = (int(*)(const char* path, int flags, ...)) layer_lookup( "bypass_agent_action_open", "__libc_open", (long int) __libc_open );
	if(!fptr) fptr = __libc_open;
	layer_descend();
	result = (*fptr) ( path, flags, mode );
	layer_ascend();
	return result;
}

extern "C" int open64( const char* path, int flags, ... )
{
	int (*fptr)(const char* path, int flags, ...);
	int result;
	va_list bypass_arglist;
	va_start(bypass_arglist,flags);
	int	mode;
	mode = va_arg(bypass_arglist,int);
	va_end(bypass_arglist);
	bypass_layer_init();
	fptr = (int(*)(const char* path, int flags, ...)) layer_lookup( "bypass_agent_action_open", "open64", (long int) open64 );
	if(!fptr) fptr = open64;
	layer_descend();
	result = (*fptr) ( path, flags, mode );
	layer_ascend();
	return result;
}

extern "C" int _open64( const char* path, int flags, ... )
{
	int (*fptr)(const char* path, int flags, ...);
	int result;
	va_list bypass_arglist;
	va_start(bypass_arglist,flags);
	int	mode;
	mode = va_arg(bypass_arglist,int);
	va_end(bypass_arglist);
	bypass_layer_init();
	fptr = (int(*)(const char* path, int flags, ...)) layer_lookup( "bypass_agent_action_open", "_open64", (long int) _open64 );
	if(!fptr) fptr = _open64;
	layer_descend();
	result = (*fptr) ( path, flags, mode );
	layer_ascend();
	return result;
}

extern "C" int __open64( const char* path, int flags, ... )
{
	int (*fptr)(const char* path, int flags, ...);
	int result;
	va_list bypass_arglist;
	va_start(bypass_arglist,flags);
	int	mode;
	mode = va_arg(bypass_arglist,int);
	va_end(bypass_arglist);
	bypass_layer_init();
	fptr = (int(*)(const char* path, int flags, ...)) layer_lookup( "bypass_agent_action_open", "__open64", (long int) __open64 );
	if(!fptr) fptr = __open64;
	layer_descend();
	result = (*fptr) ( path, flags, mode );
	layer_ascend();
	return result;
}

extern "C" int __libc_open64( const char* path, int flags, ... )
{
	int (*fptr)(const char* path, int flags, ...);
	int result;
	va_list bypass_arglist;
	va_start(bypass_arglist,flags);
	int	mode;
	mode = va_arg(bypass_arglist,int);
	va_end(bypass_arglist);
	bypass_layer_init();
	fptr = (int(*)(const char* path, int flags, ...)) layer_lookup( "bypass_agent_action_open", "__libc_open64", (long int) __libc_open64 );
	if(!fptr) fptr = __libc_open64;
	layer_descend();
	result = (*fptr) ( path, flags, mode );
	layer_ascend();
	return result;
}

extern "C" int bypass_shadow_poll( struct pollfd* ufds, unsigned long nfds, int timeout )
{
	struct packet *bypass_packet=0;
	int bypass_errno=0;
	int bypass_number = BYPASS_CALL_poll;
int result;
	if(!bypass_rpc_init()) goto fail;
	TRY(bypass_packet = packet_create(0));
	TRY(external(bypass_packet,EXTERNAL_OUT,&bypass_number));
		TRY(external_array(bypass_packet,EXTERNAL_OUT,&ufds,nfds));
		TRY(external(bypass_packet,EXTERNAL_OUT,&nfds));
		TRY(external(bypass_packet,EXTERNAL_OUT,&timeout));
	TRY(packet_put(bypass_rpc_fd_get(),bypass_packet));
	packet_delete(bypass_packet);
	bypass_packet=0;

	TRY(bypass_packet = packet_get(bypass_rpc_fd_get()));
	TRY(external_errno_map(bypass_packet,EXTERNAL_IN,&bypass_errno));
		TRY(external(bypass_packet,EXTERNAL_IN,&result));
		TRY(external_array(bypass_packet,EXTERNAL_IN,&ufds,nfds));
	packet_delete(bypass_packet);
	errno = bypass_errno;
	return result;

	fail:
	bypass_errno = errno;
	if(bypass_packet) packet_delete(bypass_packet);
	bypass_rpc_close();
	char message[1024];
	sprintf(message,"couldn't execute %s: %s\n",bypass_call_string(bypass_number),strerror(bypass_errno));
	if(bypass_failure_passthrough) {
		bypass_debug(message);
	} else {
		bypass_fatal(message);
	}
	errno = bypass_errno;
	return -1;
}

extern "C" int bypass_agent_action_poll( struct pollfd* ufds, unsigned long nfds, int timeout )
{
	return bypass_shadow_poll( ufds, nfds, timeout );

}

extern "C" int poll( struct pollfd* ufds, unsigned long nfds, int timeout )
{
	int (*fptr)(struct pollfd* ufds, unsigned long nfds, int timeout);
	int result;
	bypass_layer_init();
	fptr = (int(*)(struct pollfd* ufds, unsigned long nfds, int timeout)) layer_lookup( "bypass_agent_action_poll", "poll", (long int) poll );
	if(!fptr) fptr = poll;
	layer_descend();
	result = (*fptr) ( ufds, nfds, timeout );
	layer_ascend();
	return result;
}

extern "C" int _poll( struct pollfd* ufds, unsigned long nfds, int timeout )
{
	int (*fptr)(struct pollfd* ufds, unsigned long nfds, int timeout);
	int result;
	bypass_layer_init();
	fptr = (int(*)(struct pollfd* ufds, unsigned long nfds, int timeout)) layer_lookup( "bypass_agent_action_poll", "_poll", (long int) _poll );
	if(!fptr) fptr = _poll;
	layer_descend();
	result = (*fptr) ( ufds, nfds, timeout );
	layer_ascend();
	return result;
}

extern "C" int __poll( struct pollfd* ufds, unsigned long nfds, int timeout )
{
	int (*fptr)(struct pollfd* ufds, unsigned long nfds, int timeout);
	int result;
	bypass_layer_init();
	fptr = (int(*)(struct pollfd* ufds, unsigned long nfds, int timeout)) layer_lookup( "bypass_agent_action_poll", "__poll", (long int) __poll );
	if(!fptr) fptr = __poll;
	layer_descend();
	result = (*fptr) ( ufds, nfds, timeout );
	layer_ascend();
	return result;
}

extern "C" int __libc_poll( struct pollfd* ufds, unsigned long nfds, int timeout )
{
	int (*fptr)(struct pollfd* ufds, unsigned long nfds, int timeout);
	int result;
	bypass_layer_init();
	fptr = (int(*)(struct pollfd* ufds, unsigned long nfds, int timeout)) layer_lookup( "bypass_agent_action_poll", "__libc_poll", (long int) __libc_poll );
	if(!fptr) fptr = __libc_poll;
	layer_descend();
	result = (*fptr) ( ufds, nfds, timeout );
	layer_ascend();
	return result;
}

/*
Bypass
Copyright (C) 1999-2001 Douglas Thain
http://www.cs.wisc.edu/condor/bypass
This program is released under the GNU General Public License.
See the file COPYING for details.
*/

static char * bypass_call_name_table[] = {

"isatty",
"_exit",
"write",
"read",
"lseek",
"close",
"open",
"poll",
};

char *bypass_call_string( int call_number )
{
	return bypass_call_name_table[call_number];
}
