/*
Bypass
Copyright (C) 1999-2001 Douglas Thain
http://www.cs.wisc.edu/condor/bypass
This program is released under the GNU General Public License.
See the file COPYING for details.
*/

#include "hashtable.h"
#include "private_malloc.h"
#include <stdlib.h>

struct entry {
	int is_valid;
	int key;
	void *value;
};

struct hashtable {
	int bucket_count;
	struct entry *buckets;
};

struct hashtable * hashtable_create( int bucket_count )
{
	struct hashtable *h;
	int i;

	h = private_malloc(sizeof(struct hashtable));
	h->bucket_count = bucket_count;
	h->buckets = private_malloc( sizeof(struct entry)*bucket_count );

	for( i=0; i<bucket_count; i++ ) {
		h->buckets[i].is_valid = 0;
	}

	return h;
}

void * hashtable_lookup( struct hashtable *h, int key )
{
	int start;
	int i;

	start = key % h->bucket_count;

	for(i=start;i<h->bucket_count;i++) {
		if( !h->buckets[i].is_valid ) {
			return 0;
		}
		if( h->buckets[i].key==key ) {
			return h->buckets[i].value;
		}
	}

	for(i=0;i<start;i++) {
		if( !h->buckets[i].is_valid ) {
			return 0;
		}
		if( h->buckets[i].key==key ) {
			return h->buckets[i].value;
		}
	}

	return 0;
}

int hashtable_insert( struct hashtable *h, int key, void *value )
{
	int i;
	int start;

	start = key % h->bucket_count;

	for(i=start;i<h->bucket_count;i++) {
		if( !h->buckets[i].is_valid || h->buckets[i].key==key ) {
			h->buckets[i].is_valid = 1;
			h->buckets[i].key = key;
			h->buckets[i].value = value;
			return 1;
		}
	}

	for(i=0;i<start;i++) {
		if( !h->buckets[i].is_valid || h->buckets[i].key==key ) {
			h->buckets[i].is_valid = 1;
			h->buckets[i].key = key;
			h->buckets[i].value = value;
			return 1;
		}
	}

	return 0;
}
