/**  LB Class implementation:  */
#include "edg/workload/common/jobid/JobId.h"
#include "edg/workload/common/utilities/Exceptions.h"
#include "edg/workload/logging/client/producer.h"
#include "edg/workload/userinterface/common/LbWrapper.h"
#include "edg/workload/logging/client/Job.h"


// Open SSL problem:
#include <globus_common.h>
#include "edg/workload/common/ssl_helpers/ssl_pthreads.h"
#include "edg/workload/common/ssl_helpers/ssl_inits.h"

#include <stdio.h>
#include <iostream>
#include <vector>

#define EDG_WL_UI_COMMON_TRY_ERRROR try{ error_code = false ;
#define EDG_WL_UI_COMMON_CATCH_ERRROR \
} catch (Exception &exc){  error_code= true; error = exc.what(); \
} catch (exception &exc){  error_code= true; error = exc.what(); \
} catch (...){  error_code= true; error = "Fatal Error: Unpredictalbe exception thrown by swig wrapper"; }

using namespace std ;
using namespace edg::workload::logging::client ;
using namespace edg::workload::common::utilities ;

edg::workload::logging::client::Job lbJob;
// Create a query. This method is used both for queryStatus and for queryEvents
void createQuery (
	vector<vector<QueryRecord> >&cond ,
	// User Tags parameters:
	const std::vector<std::string>& tagNames,
	const std::vector<std::string>& tagValues,
	// Include-Exclude States parameters
	const std::vector<int>& excludes,
	const std::vector<int>& includes,
	// Issuer value (if -all selected):
	std::string issuer,
	// --from and --to options:
	int from , int to )
{
	// EXCLUDES
	for (unsigned int i = 0 ; i < excludes.size() ; i++ ){
		cond.push_back(  vector<QueryRecord> (1, QueryRecord   ( QueryRecord::STATUS  , QueryRecord::UNEQUAL,  excludes[i] )   )  );
	}
	// INCLUDES
	if (  includes.size()  > 0 ){
		vector<QueryRecord>  inclVect ;
		for (unsigned int i = 0 ; i < includes.size() ; i++ )
			inclVect.push_back(   QueryRecord   ( QueryRecord::STATUS  , QueryRecord::EQUAL, includes[i] )   ) ;
		cond.push_back(  inclVect ) ;
	}
	// --ALL OPTION
	if ( issuer !="" ) {
		// cout << "Status::createQuery> adding query for Issuer:\n"<< issuer  << endl ;
		cond.push_back( vector<QueryRecord> (1,  QueryRecord   ( QueryRecord::OWNER , QueryRecord::EQUAL,  issuer ) ) );
	}
	// FROM - TO OPTIONS
	if ( from!=0 ) {
		// cout << "Status::createQuery> adding query for FROM: " << from << endl ;
		timeval   tvFrom=  { from   ,0 };
		cond.push_back(  vector<QueryRecord> (1, QueryRecord   ( QueryRecord::TIME , QueryRecord::GREATER, JobStatus::SUBMITTED, tvFrom ) ) );
	}
	if ( to!=0 ) {
		// cout << "Status::createQuery> adding query for TO: " << to<<endl ;
		timeval   toStruct = {  to   ,0 };
		cond.push_back(   vector<QueryRecord> (1, QueryRecord   ( QueryRecord::TIME , QueryRecord::LESS, JobStatus::SUBMITTED, toStruct )  )  );
	}
	// cout << "Status::createQuery> returning query size: "<< cond.size() << endl ;
	// return cond ;
}


Status::Status () {
   edg_wlc_SSLInitialization();
   if (edg_wlc_SSLLockingInit() != 0)
      log_error ("Unable to Initialise SSL context" ) ;
   else if (globus_module_activate(GLOBUS_COMMON_MODULE) != GLOBUS_SUCCESS)
      log_error ("Unable to Initialise SSL context") ;
} ;

Status::~Status () { };

Eve::Eve () {
   edg_wlc_SSLInitialization();
   if (edg_wlc_SSLLockingInit() != 0)
      log_error ("Unable to Initialise SSL context" ) ;
   else if (globus_module_activate(GLOBUS_COMMON_MODULE) != GLOBUS_SUCCESS)
      log_error ("Unable to Initialise SSL context") ;
} ;
Eve::~Eve() {} ;

/** Status::size(int status_number)*/
int Status::size(int status_number){
   error_code = false ;
   list<edg::workload::logging::client::JobStatus>::iterator it = states.begin();
   // vector<edg::workload::logging::client::JobStatus>::iterator it = states.begin();
   for ( int j = 0 ; j< status_number ; j++)  {
      if (  it==states.end()  ) break ;
      it++ ;
   }
   edg::workload::logging::client::JobStatus status_retrieved = *it;
   std::vector<pair<JobStatus::Attr, JobStatus::AttrType> > attrList = status_retrieved.getAttrs();
   return attrList.size() ;
}

int Status::size(){  return states.size() ; }
int Eve::size(){ return events.size() ;  }

int Eve::size(int event_number){
   error_code = false ;
   list<edg::workload::logging::client::Event>::iterator it = events.begin();
   for ( int j = 0 ; j< event_number ; j++, it++)  if (  it==events.end()  ) break ;
   edg::workload::logging::client::Event event_retrieved = *it ;
   std::vector<pair<Event::Attr,Event::AttrType> > attrList = event_retrieved.getAttrs();
   return attrList.size() ;
}

void Eve::log_error ( const std::string& err) {    error_code = true ; error = err ;};
int Eve::get_error (std::string& err) {
   if (error_code ){
         err = error ;
         error = "" ;
         return 1 ;
   }
   err = "" ;
   return 0 ;
};
void Status::log_error ( const std::string& err) { error_code = true ; error = err ;};
int Status::get_error (std::string& err)  {
   if (error!= "" ){
         err = error ;
         error = "" ;
         return 1 ;
   }
   err = "" ;
   return 0 ;
};
int Status::getStatus (const string& jobid , int level) {
	error_code = false ;
	edg::workload::logging::client::JobStatus status ;
	try{
		lbJob = edg::workload::common::jobid::JobId( jobid  ) ;
		if (level!= 0 ) level =  edg::workload::logging::client::Job::STAT_CLASSADS ;
		status = lbJob.status( level  | edg::workload::logging::client::Job::STAT_CHILDSTAT  ) ;
		states.push_back( status );
		return status.getAttrs().size() ;
	}catch (exception &exc){  log_error ("Unable to retrieve the status for: " + jobid +"\n" + string (exc.what() ) ) ;
	} catch (...){  error_code= true; error = "Fatal Error: Unpredictalbe exception thrown by swig wrapper"; }
	return 0 ;
} ;



 int Eve::queryEvents (
 	// Lb Server address
	const std::string& host , int port ,
	// User Tags parameters:
	const std::vector<std::string>& tagNames,
	const std::vector<std::string>& tagValues,
	// Issuer value (if -all selected):
	std::string issuer,
	// --from and --to options:
	int from , int to ,
	// Verbosity Level
	int ad )
{
		ServerConnection server ;
		server.setQueryServer( host , port );
		vector <Event> events_vector  ;
		// cout << "Eve::queryStates> Query Making " << endl ;
		try{
			/*
			vector<QueryRecord> cond , empty;
			createQuery (cond ,tagNames , tagValues , issuer , from , to );
			cout << "Eve::queryEvents: performing queryEventsList....... " << endl ;
			events = server.queryEventsList (cond , empty ) ;
			cout << "Eve::queryEvents: performed query....... DONE lengst: " <<  events.size()   <<endl ;
			events_vector = server.queryEvents( cond , empty ) ;
			// Iterate over the vector to fill the "states" list:
			cout << "Eve::queryEvents: performed query, length: " <<  events_vector.size() <<  endl ;
			for (unsigned int i = 0 ; i< events_vector.size() ;i++){
				cout << "Eve::queryEvents: pushing back..." << endl ;
				events.push_back(  events_vector[i]  ) ;
			}
			*/
		}catch (Exception &exc){
			cout << "Eve::queryStates common Error: " << exc.getExceptionName() << ": " << exc.printStackTrace() << endl ;
			log_error ("Unable to retrieve bookkeeping information from: " + host + ": " +string (exc.what() ) ) ;
		}catch (exception &exc){
			log_error ("Unable to retrieve bookkeeping information from: " + host + ": " +string (exc.what() ) ) ;
		} catch (...){  error_code= true; error = "Fatal Error: Unpredictalbe exception thrown by swig wrapper"; }
	return events.size() ;
}



 int Status::queryStates (
 	// Lb Server address
	const std::string& host , int port ,
	// User Tags parameters:
	const std::vector<std::string>& tagNames,
	const std::vector<std::string>& tagValues,
	// Includes Excludes parameters
	const std::vector<int>& excludes,
	const std::vector<int>& includes,
	// Issuer value (if -all selected):
	std::string issuer,
	// --from and --to options:
	int from , int to ,
	// Verbosity Level
	int ad )
{
		ServerConnection server ;
		server.setQueryServer( host , port );
		int FLAG  = 0 ;
		if (ad!=0) FLAG =   EDG_WLL_STAT_CLASSADS  ;
		vector <JobStatus> states_v ;		
		try{
			vector <vector<QueryRecord> >cond ;
			createQuery (cond ,tagNames , tagValues , excludes , includes, issuer , from , to );
			// the Server will fill the result vector anyway, even when exception is raised
			if ( ! getenv ( "EDG_WL_QUERY_RESULTS") ) server.setParam (EDG_WLL_PARAM_QUERY_RESULTS , 3 ) ;
			server.queryJobStates (cond, FLAG | EDG_WLL_STAT_CHILDSTAT , states_v ) ;
		}catch (Exception &exc){
			// cout << "Status::queryStates common Error: " << exc.getExceptionName() << ": " << exc.printStackTrace() << endl ;
			log_error ("Unable to retrieve status query information from: " + host + ": " +string (exc.what() ) ) ;
		}catch (exception &exc){
			log_error ("Unable to retrieve status query information from: " + host + ": " +string (exc.what() ) ) ;
		} catch (...){  error_code= true; error = "Fatal Error: Unpredictalbe exception thrown by swig wrapper"; }
	for ( unsigned int i = 0 ; i< states_v.size() ; i++ )
		states.push_back( states_v[i]  ) ;
	return states.size() ;
}


// Retrieve all the states of the user
int Status::getStates ( const string& host , int port , int level ){
   error_code = false ;
   vector <JobStatus> states_vector ;
   try{
     ServerConnection server ;
     server.setQueryServer( host , port);
     server.userJobStates(states_vector) ;
     // Iterate over the vector to fill the "states" list:
     for (unsigned int i = 0 ; i< states_vector.size() ;i++){
           states.push_back(  states_vector[i] ) ;
     }
     }catch (exception &exc){ log_error ("Unable to retrieve the user's Job States "+string (exc.what() )  ) ;
     } catch (...){  error_code= true; error = "Fatal Error: Unpredictalbe exception thrown by swig wrapper"; }
   return states.size();
} ;
int Eve::getEvents (const  std::string& jobid) {
     error_code = false ;
     vector<Event> events_vector;
     try{
       lbJob = edg::workload::common::jobid::JobId ( jobid );
       // edg::workload::logging::client::Job    lbJob ( j )   ;
       events_vector = lbJob.log();
       for (unsigned int i = 0 ; i< events_vector.size() ;i++)
           events.push_back(  events_vector[i] ) ;
       return events_vector.size() ;
     }catch (edg::workload::common::utilities::Exception &exc){
         log_error ("Unable to retrieve the Job Events for: " +jobid+"\n" + string (exc.what() ) ) ;
     } catch (...){  error_code= true; error = "Fatal Error: Unpredictalbe exception thrown by swig wrapper"; }
     return 0 ;
} ;

std::string Eve::getVal (int field , string& result , int event_number) {
	error_code = false ;
	list<edg::workload::logging::client::Event>::iterator it = events.begin();
	for ( int j = 0 ; j< event_number ; j++, it++)  if (  it==events.end()  ) break ;
	edg::workload::logging::client::Event event_retrieved = *it ;
	string attrName = "" ;
	int EVENT = 48 ;
	if ( field == EVENT) {
		result = event_retrieved.name() ;
		return "event" ;
	}
	Event::Attr fieldAttr = (Event::Attr) field ;
	if (fieldAttr < Event::ATTR_MAX)
	    attrName = event_retrieved.getAttrName(fieldAttr) ;
	char tmp [1024] ;
try{
	switch ( fieldAttr ){
		/** Integer Values*/
		case Event::LEVEL:
		result = string (  edg_wll_LevelToString ( (edg_wll_Level) event_retrieved.getValInt(fieldAttr) ) ) ;
			break;
		case Event::STATUS_CODE:
			result = string (  edg_wll_DoneStatus_codeToString ((edg_wll_DoneStatus_code)event_retrieved.getValInt(fieldAttr) ) ) ;
			break;
		case Event::EXIT_CODE:
		case Event::PRIORITY:
		case Event::NSUBJOBS:
		case Event::SVC_PORT:		
		case Event::PERMISSION: // Not avaliable in DAG branch
		case Event::PERMISSION_TYPE: // Not avaliable in DAG branch
		case Event::USER_ID: // Not avaliable in DAG branch
		case Event::USER_ID_TYPE: // Not avaliable in DAG branch
			sprintf (tmp , "%d" , event_retrieved.getValInt(fieldAttr) ) ;
			result = string (tmp) ;
			// cout << event_retrieved.getValInt(fieldAttr) << endl;
			break;
		case Event::SOURCE:
		case Event::DESTINATION:
		case Event::FROM:
			result = string (  edg_wll_SourceToString ( (edg_wll_Source )event_retrieved.getValInt(fieldAttr) ) ) ;
			// cout << event_retrieved.getValInt(fieldAttr) << endl;
			break;
		case Event::SRC_ROLE:
		case Event::HOST:
		case Event::SEQCODE:
		case Event::USER:
		case Event::SRC_INSTANCE:
		case Event::VALUE:
		case Event::RESULT:
		case Event::NAME:
		case Event::REASON:
		case Event::DESCR:
		case Event::SVC_HOST:
		case Event::SVC_NAME:
		case Event::FROM_INSTANCE:
		case Event::FROM_HOST:
		case Event::LOCAL_JOBID:
		case Event::QUEUE:
		case Event::NODE:
		case Event::JDL:
		case Event::SEED:
		case Event::NS:
		case Event::JOB:
		case Event::RETVAL:
		case Event::CLASSAD:
		case Event::TAG:
		case Event::HELPER_PARAMS:
		case Event::HELPER_NAME:
		case Event::DEST_ID:
		case Event::DEST_HOST:
		case Event::DEST_JOBID:
		case Event::DEST_INSTANCE:
			result = event_retrieved.getValString(fieldAttr) ;
			break;
		// case Event::ARRIVED: // NOT AVAILABLE IN HEAD (ONLY DAG BRANCH)
		case Event::TIMESTAMP:{
				timeval t = event_retrieved.getValTime(fieldAttr);
				sprintf (tmp , "%d",  (int )t.tv_sec ) ;
				result = string (tmp) ;
		}
		break;
		case Event::JOBID:
		case Event::PARENT:{
			if(   ((edg::workload::common::jobid::JobId)event_retrieved.getValJobId(fieldAttr)).isSet()   )
			result =  event_retrieved.getValJobId(fieldAttr).toString() ;
		}
		break;
		default      : 	// something is wrong
			//cout << "\n\n\nSomething has gone bad for: " << event_retrieved.getAttrName(fieldAttr) << endl;
		log_error("Something is wrong for " +attrName ) ;
		break;
	} // end switch
} catch ( exception &exc){
    // cout << "Eve::getVal-> Fatal Error. Exception caught, please check field and number: " << field <<" , " << event_number<< endl << exc.what() << endl << flush;
    // log_error("Fatal Error\n" + string (exc.what() )  );
} catch (...){  error_code= true; error = "Fatal Error: Unpredictalbe exception thrown by swig wrapper"; }
	return attrName ;
};


string Status::getVal (int field , string& result, int status_number ) {
	// Retrieve the status to be investigated
	error_code = false ;
	list<edg::workload::logging::client::JobStatus>::iterator it = states.begin();
	// vector<edg::workload::logging::client::JobStatus>::iterator it = states.begin();
	for ( int j = 0 ; j< status_number ; j++,it++ )  {   if (  it==states.end()  ) break ; }
	edg::workload::logging::client::JobStatus status_retrieved = *it;  //TBD use pointers!!!
	// Special FIELD Retrieval:
	char tmp [1024] ;//TBD could be not enough for JobStatus list
	int STATUS = 35 ;
	int STATUS_CODE = 36 ;
 	if (field == STATUS ){
		result = status_retrieved.name() ;
		return "Status" ;
	}
	else  if (field == STATUS_CODE ){
		sprintf (tmp , "%d" , status_retrieved.status ) ;
		result = string(tmp) ;
		return "status_code" ;
	}
	std::vector<pair<JobStatus::Attr, JobStatus::AttrType> > attrList = status_retrieved.getAttrs(); //TBD NEEDED???
	JobStatus::Attr fieldAttr = (JobStatus::Attr) field ;
	string attrName ;
	if (fieldAttr < JobStatus::ATTR_MAX)
	    attrName = status_retrieved.getAttrName(fieldAttr) ;
try{
	switch (fieldAttr){
		case JobStatus::JOBTYPE:
		case JobStatus::DONE_CODE:
		case JobStatus::CPU_TIME:
		case JobStatus::EXIT_CODE:
		case JobStatus::CHILDREN_NUM:
			sprintf (tmp , "%d" , status_retrieved.getValInt(fieldAttr) ) ;
			//cout <<"getting var INT_T: " <<status_retrieved.getValInt(fieldAttr)<< endl << flush;
			result = string (tmp) ;
			break;
		case JobStatus::SUBJOB_FAILED:
		case JobStatus::EXPECT_UPDATE:
		case JobStatus::RESUBMITTED:
		case JobStatus::CANCELLING:
			sprintf (tmp , "%d" , status_retrieved.getValBool(fieldAttr) );
			result = string (tmp) ;
			break;
		case JobStatus::MATCHED_JDL:
		case JobStatus::CANCEL_REASON:
		case JobStatus::CONDOR_ID:;
		case JobStatus::GLOBUS_ID:;
		case JobStatus::LOCAL_ID:
		case JobStatus::LOCATION:
		case JobStatus::RSL:
		case JobStatus::ACL:
		case JobStatus::CONDOR_JDL:
		case JobStatus::CE_NODE:
		case JobStatus::EXPECT_FROM:
		case JobStatus::NETWORK_SERVER:
		case JobStatus::REASON:
		case JobStatus::SEED:
		case JobStatus::JDL:
		case JobStatus::DESTINATION:
		case JobStatus::OWNER:
			result = status_retrieved.getValString(fieldAttr) ;
			//cout <<"getting var STRING_T: " <<status_retrieved.getValString(fieldAttr)<< endl << flush;
			break;
		case JobStatus::STATE_ENTER_TIME:
		case JobStatus::LAST_UPDATE_TIME:{
			timeval t = status_retrieved.getValTime(fieldAttr);
			//cout <<"getting var TIMEVAL_T: " <<  t.tv_sec<< endl << flush;
			// sprintf (tmp , "%d%s%d", (int)  t.tv_sec , "." , (int) t.tv_usec ) ;
			sprintf (tmp , "%d", (int)  t.tv_sec) ;
			result = string (tmp) ;
			}
			break;
		case JobStatus::PARENT_JOB:
		case JobStatus::JOB_ID:
			if(((edg::workload::common::jobid::JobId)status_retrieved.getValJobId(fieldAttr)).isSet()){
				result = status_retrieved.getValJobId(fieldAttr).toString()  ;
				//cout <<"getting var JOBID_T: " << status_retrieved.getValJobId(fieldAttr).toString()<< endl << flush;
			}
			break;
		case JobStatus::CHILDREN_HIST:
		case JobStatus::STATE_ENTER_TIMES:{
			std::vector<int> v = status_retrieved.getValIntList(fieldAttr);
			for(unsigned int j=0; j < v.size(); j++){
				//cout << "  " << v[j]  << flush ; //TBD
				sprintf (tmp , "%s%s%s%d%s", tmp , edg_wll_StatToString((edg_wll_JobStatCode) j ), "=" , v[j] , " ") ;
			}
			result = string (tmp) ;
			}
			break;
		case JobStatus::USER_TAGS:{
			std::vector<std::pair<std::string,std::string> >  v = status_retrieved.getValTagList( fieldAttr  ) ;
			for (unsigned int i = 0 ; i < v.size() ; i++ ){
				result +=v[i].first + "=" + v[i].second +";" ;
			}
		}
		break ;
		// case JobStatus::USER_VALUES: DEPRECATED
		case JobStatus::CHILDREN:{
			std::vector<std::string> v = status_retrieved.getValStringList(fieldAttr);
			for(unsigned int j=0; j < v.size(); j++)
				result += v[j] ;
			}
			break;
		case JobStatus::CHILDREN_STATES:{
			std::vector<JobStatus> v = status_retrieved.getValJobStatusList(fieldAttr);
			for(unsigned int i=0; i < v.size(); i++) ; //TBD  //TBD
			//cout <<"getting var:  STSLIST_T Jobstates: TBD !!!"<< endl << flush;
			}
			break;
		default:
			cout << "\n\n\nWarning!!!!! Something has gone bad!\nField Attribute=" << fieldAttr<<  "contact the developer!! LbWrapper line "<<__LINE__ << endl ;
			log_error("Something is wrong" ) ;
			//break;
	}
} catch ( exception &exc){
    // cout << "Status::getVal-> Fatal Error. Exception caught, please check: " <<exc.what() << endl << flush;
    // log_error("Fatal Error\n" + string (exc.what() ) );
} catch (...){  error_code= true; error = "Fatal Error: Unpredictalbe exception thrown by swig wrapper"; }
  return attrName;
  } ;

  
void push_status( JobStatus status_retrieved , std::vector<std::string>& result , int hierarchy ){
	int VECT_OFFSET = result.size() ;
	int VECT_DIM = 38 ;
	result.resize( VECT_DIM + VECT_OFFSET ) ;
	char  tmp [1024];
	// push back additional Status information: STATUS
	int STATUS = VECT_DIM  - 3 ;
	result[VECT_OFFSET + STATUS  ] = status_retrieved.name()  ;
	// push back additional Status information:STATUS_CODE
	int STATUS_CODE = VECT_DIM - 2  ;
	sprintf (tmp , "%d" ,  status_retrieved.status ) ;
	result[VECT_OFFSET +STATUS_CODE ] =  string( tmp ) ;
	// push back additional Status information:HIERARCHY
	int HIERARCHY  =VECT_DIM - 1  ;
	sprintf (tmp , "%d" , hierarchy  ) ;
	result[VECT_OFFSET + HIERARCHY ] =  string( tmp ) ;
	// Iterate over the attributes
	std::vector<pair<JobStatus::Attr, JobStatus::AttrType> > attrList = status_retrieved.getAttrs();
	for (unsigned i=0; i < attrList.size(); i++ ) {
		switch (attrList[i].second) {
			case JobStatus::INT_T :{
				sprintf (tmp , "%d" , status_retrieved.getValInt(attrList[i].first) );
				result[VECT_OFFSET + attrList[i].first ] =  string ( tmp) ;
			}
			break;
			case JobStatus::BOOL_T    :{
				sprintf (tmp , "%d" , status_retrieved.getValBool(attrList[i].first  ) );
				result[VECT_OFFSET +  attrList[i].first ] =  string( tmp );
			}
			break;
			case JobStatus::STRING_T  :{
				result[VECT_OFFSET +  attrList[i].first ] = status_retrieved.getValString(attrList[i].first)  ;
			}
			break;
			case JobStatus::JOBID_T    :{
				if(((edg::workload::common::jobid::JobId)status_retrieved.getValJobId(attrList[i].first)).isSet())
					result[VECT_OFFSET +  attrList[i].first ] = status_retrieved.getValJobId(attrList[i].first).toString()  ;
			}
			break;
			case JobStatus::STSLIST_T    :{
				// Do Nothing, JobStatus sons have been already processed
			}
			break;
			case JobStatus::STRLIST_T    :{
				std::vector<std::string> v = status_retrieved.getValStringList(attrList[i].first);
				for(unsigned int j=0; j < v.size(); j++)
					result[VECT_OFFSET +  attrList[i].first ] =  v[j]  ;
			}
			break;
			case JobStatus::TAGLIST_T    :{
				std::vector<std::pair<std::string,std::string> >  v = status_retrieved.getValTagList(   attrList[i].first   ) ;
				string res ;
				for (unsigned int j = 0 ; j < v.size() ; j++ )
					res +=v[j].first + "=" + v[j].second +";" ;
				result[VECT_OFFSET +  attrList[i].first ] =  res ;
			break;
			}
			case JobStatus::INTLIST_T    :{
				std::vector<int> v = status_retrieved.getValIntList(attrList[i].first);
				for(unsigned int j=0; j < v.size(); j++)
					sprintf (tmp , "%s%s%s%d%s", tmp , edg_wll_StatToString((edg_wll_JobStatCode) j ), "=" , v[j] , " ") ;
				result[VECT_OFFSET +  attrList[i].first ] = string(tmp) ;
			}
			break;
			case JobStatus::TIMEVAL_T    :{
				timeval t = status_retrieved.getValTime(attrList[i].first);
				sprintf (tmp , "%d", (int)  t.tv_sec) ;
				result[VECT_OFFSET +  attrList[i].first ] = string(tmp) ;
			}
			break;
			default : /* something is wrong */
				cout << "\n\nFATAL ERROR!!! Something has gone bad for " << status_retrieved.getAttrName(attrList[i].first) << flush;
			break;
		}// end switch
	}// end for	
	// Upload info from Sub Jobs ( if present )
	std::vector<JobStatus> v = status_retrieved.getValJobStatusList(  JobStatus::CHILDREN_STATES );
	for(unsigned int i=0; i < v.size(); i++)  push_status( v[i] , result , hierarchy+1 );

}
std::vector< std::string  > Status::loadStatus( int status_number )  {
	// Retrieve the selected status
	list<JobStatus>::iterator it = states.begin();
	// vector<JobStatus>::iterator it = states.begin();
	for ( int j = 0 ; j< status_number ; j++,it++ )  {   if (  it==states.end()  ) break ; }
	// found related status. Now iterating over the tree
	std::vector< std::string  > result ;
	push_status (*it , result , 0 ) ;
	return result ;
}
std::vector<edg::workload::logging::client::JobStatus>   Status::getAllStates (const std::string& host , int port , int level){
		// ServerConnection Approach:
		ServerConnection server ;
		server.setQueryServer( host , port );
		int FLAG  = 0 ;
		if (level!=0) FLAG =   EDG_WLL_STAT_CLASSADS  ;
		vector<QueryRecord> cond ;
		cond.push_back(   QueryRecord   (   QueryRecord::JOBID , QueryRecord::EQUAL, "https://ibm139.cnaf.infn.it:9000/KoZ9ARK8w2EvNJy4NiuOUA" )   ) ;
		// cond.push_back(   QueryRecord   (   QueryRecord::JOBID , QueryRecord::EQUAL, "https://ibm139.cnaf.infn.it:9000/NkwOxy0NySq3VaNKLpR6yA" )   ) ;
		// cond.push_back(   QueryRecord   (   QueryRecord::JOBID , QueryRecord::EQUAL, "https://ibm139.cnaf.infn.it:9000/1gtxVzr0DZbKPK3-hA-gpg" )   ) ;
		try{
			return server.queryJobStates ( cond , FLAG | EDG_WLL_STAT_CHILDSTAT ) ;
		}catch (exception &exc){  log_error ("Unable to retrieve status query information from: " + host + ": " +string (exc.what() ) ) ;
		} catch (...){  error_code= true; error = "Fatal Error: Unpredictalbe exception thrown by swig wrapper"; }
		std::vector<edg::workload::logging::client::JobStatus> empty ;
		return empty ;
}
