#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <iostream>


#include <unistd.h> // getpid method
#include "edg/workload/userinterface/jcommon/edg_workload_userinterface_jclient_Api.h"
// Network Server:
#include "edg/workload/networkserver/client/NSClient.h"
#include "edg/workload/common/logger/common.h"
#include "edg/workload/common/utilities/Exceptions.h"
// LB:
#include "edg/workload/logging/client/consumer.h" // TBD is possible?
#include "edg/workload/logging/client/producer.h"
#include "edg/workload/logging/client/JobStatus.h"
#include "edg/workload/logging/client/Job.h"
#include "edg/workload/common/jobid/JobId.h"
#include "edg/workload/common/jobid/manipulation.h"  //replace jobid name
//Open Ssl safe thread
#include <globus_common.h>
#include "edg/workload/common/ssl_helpers/ssl_pthreads.h"
#include "edg/workload/common/ssl_helpers/ssl_inits.h"

#define EDG_WLU_SOURCE_NS "NetworkServer"
#define EDG_WL_LOG_DESTINATION "EDG_WL_LOG_DESTINATION"

#define LB_CTX true
#define NS_CTX false

using namespace std ;


/*  static variables */
pthread_mutex_t  mutex;
vector <edg_wll_Context* > lbVect ;
edg::workload::logging::client::Job lbJob ;
string nsHost ;
int nsPort ;
int nsLevel ;

/*
non-native methods:
*/

	int cmp_by_timestamp(const void* first, const void* second) {
		timeval f = ((edg_wll_Event *) first)->any.timestamp;
		timeval s = ((edg_wll_Event *) second)->any.timestamp;
		if (( f.tv_sec > s.tv_sec ) ||
		(( f.tv_sec == s.tv_sec ) && ( f.tv_usec > s.tv_usec )))
		return 1;
		if (( f.tv_sec < s.tv_sec ) ||
		(( f.tv_sec == s.tv_sec ) && ( f.tv_usec < s.tv_usec )))
		return -1;
		return 0;
	}

	const char* error_message ( edg_wll_Context ctx , const char* api  ){
		char* error_message =(char*) malloc ( 1024 ) ;
		char *msg, *dsc ;
		edg_wll_Error(  ctx , &msg , &dsc ) ;
		sprintf ( error_message , "%s%s%s%s%s%s%s", api,
		getenv ( EDG_WL_LOG_DESTINATION) , "\n" , msg , " (" , dsc , " )" )  ;
		return error_message ;
	} ;

	void lock() {     pthread_mutex_lock( &mutex);      /* LOCK RESOURCES    */ };
	void unlock(){   pthread_mutex_unlock( &mutex);  /*UNLOCK RESOURCES*/ };

	void log_error (JNIEnv *env , string error) {
		// jclass newExcCls = env->FindClass("edg/workload/userinterface/client/ApiException");
		jclass newExcCls = env->FindClass("java/lang/RuntimeException");
		if (newExcCls == 0) /* Unable to find the new exception class, give up */ return ;
		if ( error.empty() )
			env->ThrowNew( newExcCls , "(No message available)" );
		else
			env->ThrowNew( newExcCls , error.c_str() );
	}

	int getCtx ( JNIEnv *env , jobject obj , bool value) {
		// TBD can be done statically?
		jclass cls = env->GetObjectClass(obj);
		jmethodID getx = env->GetMethodID ( cls , "getCtx" , "(Z)I" ) ;
		if (getx==0) /** then */ log_error (env , "Fatal Error: Unable to find Api.getCtx method\n" ) ;
		return env->CallIntMethod(obj, getx, value) ;
	};





	/*
	* Class:     edg_workload_userinterface_jclient_Api
	* Method:    getPid
	* Signature: ()I
	*/
	JNIEXPORT jint JNICALL Java_edg_workload_userinterface_jclient_Api_getPid
	(JNIEnv *env, jclass){  return getpid();    }

	/*
	* Class:     edg_workload_userinterface_jclient_Api
	* Method:    shadow
	* Signature: (Ljava/lang/String;)I
	*/
	JNIEXPORT jint JNICALL Java_edg_workload_userinterface_jclient_Api_shadow
	(JNIEnv *env, jclass, jstring unique){
		const char *nm = env->GetStringUTFChars(unique, 0);
		int result = system( nm  ) ;
		env->ReleaseStringUTFChars(unique, nm);
		return result ;
	}

	/*
	* Class:     edg_workload_userinterface_jclient_Api
	* Method:    getEnv
	* Signature: (Ljava/lang/String;)Ljava/lang/String;
	*/
	JNIEXPORT jstring JNICALL Java_edg_workload_userinterface_jclient_Api_getEnv
	(JNIEnv *env, jclass, jstring name ){
		const char *nm = env->GetStringUTFChars(name, 0);
		char* value = getenv (nm) ;
		env->ReleaseStringUTFChars(name, nm);
		return env->NewStringUTF( value  );
	}

	/*
	* Class:     edg_workload_userinterface_jclient_Api
	* Method:    getUid
	* Signature: ()I
	*/
	JNIEXPORT jint JNICALL Java_edg_workload_userinterface_jclient_Api_getUid
	(JNIEnv *env, jclass){   return  getuid() ;    }

	/*
	* Class:     edg_workload_userinterface_jclient_Api
	* Method:    setEnv
	* Signature: (Ljava/lang/String;Ljava/lang/String;)V
	*/
	JNIEXPORT void JNICALL Java_edg_workload_userinterface_jclient_Api_setEnv
	(JNIEnv *env, jclass, jstring name, jstring value ){
		const char *nm = env->GetStringUTFChars(name, 0);
		const char *vl = env->GetStringUTFChars(value, 0);
		unsetenv(nm) ;
		setenv( nm , vl, 0);
		env->ReleaseStringUTFChars(name, nm);
		env->ReleaseStringUTFChars(value, vl);
	}


        /*
         * Class:     edg_workload_userinterface_jclient_Api
         * Method:    unsetEnv
         * Signature: (Ljava/lang/String;)V
         */
         JNIEXPORT void JNICALL Java_edg_workload_userinterface_jclient_Api_unsetEnv
         (JNIEnv *env, jclass, jstring name){
                 const char *nm = env->GetStringUTFChars(name, 0);
                 unsetenv(nm) ;
                 env->ReleaseStringUTFChars(name, nm);
         }


	/*
	* Class:     edg_workload_userinterface_jclient_Api
	* Method:    initialise
	* Signature: ()V
	*/
	JNIEXPORT void JNICALL Java_edg_workload_userinterface_jclient_Api_initialise
	(JNIEnv *env, jclass ){
		edg_wlc_SSLInitialization();
		if ( globus_module_activate(GLOBUS_COMMON_MODULE) != GLOBUS_SUCCESS )
			log_error (env , "Unable to use safe multi threading for Open SSL" );
		else if (edg_wlc_SSLLockingInit() != 0)
			log_error (env , "Unable to use safe multi threading for Open SSL" );
	}

	/*
	* Class:     edg_workload_userinterface_jclient_Api
	* Method:    genetareId
	* Signature: (Ljava/lang/String;I)Ljava/lang/String;
	*/
	JNIEXPORT jstring JNICALL Java_edg_workload_userinterface_jclient_Api_generateId
	(JNIEnv *env, jclass , jstring lbHost, jint lbPort){
		const char *host = env->GetStringUTFChars( lbHost, 0);
		try{
			edg::workload::common::jobid::JobId jobid ;
			jobid.setJobId( host,  lbPort );
			env->ReleaseStringUTFChars( lbHost, host);
			return env->NewStringUTF( jobid.getUnique().c_str() ) ;
		}catch(edg::workload::common::utilities::Exception &exc){  log_error (env , exc.what() ) ;
		}catch(exception &exc){  log_error (env , exc.what() ) ;
		}catch (...){ log_error (env, "Fatal Error: Unpredictalbe exception thrown by JNI wrapper");    }
	}

	/*
	* Class:     edg_workload_userinterface_jclient_Api
	* Method:    ns_init
	* Signature: (Ljava/lang/String;II)V
	* This Metdhos is synchronized. no more lock unlock are needed
	*/
	JNIEXPORT void JNICALL Java_edg_workload_userinterface_jclient_Api_ns_1init
	(JNIEnv *env, jobject obj, jstring host , jint port , jint  logLevel){
		const char *ns = env->GetStringUTFChars( host, 0);
		nsHost = string (ns) ;   nsPort = port ;     nsLevel = logLevel ;
		env->ReleaseStringUTFChars( host , ns);
	}

	/*
	* Class:     edg_workload_userinterface_jclient_Api
	* Method:    ns_submit
	* Signature: (Ljava/lang/String;Ljava/lang/String;I)V
	*/
	JNIEXPORT void JNICALL Java_edg_workload_userinterface_jclient_Api_ns_1submit
	(JNIEnv *env, jobject obj, jstring  jdl , jstring jnsHost, jint jnsPort){
		const char *jd = env->GetStringUTFChars(jdl, 0);
		const char *ct = env->GetStringUTFChars(jnsHost, 0);
		char str_port [1024];
		// sprintf (str_port , "%d" , jnsPort );
		sprintf (str_port , "%s%s%d" , nsHost.c_str(),":",nsPort );
		edg_wll_Context ctx = *lbVect[  getCtx( env, obj , LB_CTX  ) ]  ;
		try{
lock() ;
		edg::workload::networkserver::client::NSClient nsClient ( ct , jnsPort , ( edg::workload::common::logger::level_t)nsLevel ) ;
		nsClient.jobSubmit( jd   );
		if (edg_wll_LogTransferOK(ctx, EDG_WLL_SOURCE_NETWORK_SERVER , ct, str_port , jd, "","") )
			cerr << error_message (ctx , "edg_wll_LogTransferOK") << endl ;
unlock();
		}catch (exception &exc){    unlock();      /*Call the dgLog Transfer method  for failure (No Thread Safe Method)*/
lock() ;
		if (edg_wll_LogTransferFAIL(ctx ,  EDG_WLL_SOURCE_NETWORK_SERVER , ct, str_port , jd, exc.what(),""))
			cerr << error_message(ctx , "edg_wll_LogTransferFAIL") << endl ;
		if (edg_wll_LogAbort ( ctx  , exc.what() ))
			cerr <<  error_message(ctx, "edg_wll_LogAbort") << endl ;
		// edg_wll_FreeContext(*lbVect[  getCtx( env, obj , LB_CTX  ) ] ) ; // needed before throwing the exception sure? TBD
unlock() ;
		log_error (env , "Unable to submit the job: " + string (exc.what() )  ) ;
		}catch (...){ unlock();   log_error (env, "Fatal Error: Unpredictalbe exception thrown by JNI wrapper");    }
		env->ReleaseStringUTFChars( jdl, jd);
		env->ReleaseStringUTFChars( jnsHost, ct);
	}

	/*
	* Class:     edg_workload_userinterface_jclient_Api
	* Method:    ns_purge
	* Signature: (Ljava/lang/String;)V
	*/
	JNIEXPORT void JNICALL Java_edg_workload_userinterface_jclient_Api_ns_1purge
	(JNIEnv *env, jobject obj, jstring jobid){
	const char *id = env->GetStringUTFChars(jobid, 0);
		try{
			edg::workload::common::jobid::JobId jid ( id )  ;
			edg::workload::networkserver::client::NSClient nsClient ( nsHost , nsPort,  ( edg::workload::common::logger::level_t)nsLevel ) ;
			nsClient.jobPurge( id )  ;
		}catch(edg::workload::common::utilities::Exception &exc){  log_error (env , exc.what() ) ;
		}catch(exception &exc){  log_error (env , exc.what() ) ;
		}catch (...){ log_error (env, "Fatal Error: Unpredictalbe exception thrown by JNI wrapper");    }
		env->ReleaseStringUTFChars( jobid , id);
	}

	/*
	* Class:     edg_workload_userinterface_jclient_Api
	* Method:    ns_cancel
	* Signature: (Ljava/lang/String;)I
	*/
	JNIEXPORT jint JNICALL Java_edg_workload_userinterface_jclient_Api_ns_1cancel
	(JNIEnv *env, jobject obj, jstring jobid){
		const char *id = env->GetStringUTFChars(jobid, 0);
		try{
			edg::workload::common::jobid::JobId jid ( id )  ;
			edg::workload::networkserver::client::NSClient nsClient ( nsHost , nsPort,  ( edg::workload::common::logger::level_t)nsLevel ) ;
			nsClient.jobCancel( list<string> ( 1   ,  id) );
		}catch(exception &exc){  log_error (env , exc.what() ) ;
		}catch (...){ log_error (env, "Fatal Error: Unpredictalbe exception thrown by JNI wrapper");    }
		env->ReleaseStringUTFChars( jobid , id);
	}

	/*
	* Class:     edg_workload_userinterface_jclient_Api
	* Method:    ns_get_root
	* Signature: (Ljava/lang/String;)Ljava/lang/String;
	*/
	JNIEXPORT jstring JNICALL Java_edg_workload_userinterface_jclient_Api_ns_1get_1root
	(JNIEnv *env, jobject obj, jstring jobId ){
	const char *id = env->GetStringUTFChars(jobId, 0);
		try{
			edg::workload::networkserver::client::NSClient nsClient ( nsHost , nsPort,  ( edg::workload::common::logger::level_t)nsLevel  ) ;
			string path =  nsClient.getSandboxRootPath() ;
			string sep = "/" ;
			path += sep + edg::workload::common::jobid::to_filename(edg::workload::common::jobid::JobId(id) )  + sep + "output" + sep  ;
			return env->NewStringUTF( path.c_str() ) ;
		}catch (exception &exc){ log_error (env ,  exc.what() ) ;
		}catch (...){ log_error (env, "Fatal Error: Unpredictalbe exception thrown by JNI wrapper");    }
		env->ReleaseStringUTFChars( jobId, id);
		return env->NewStringUTF (""); //Some error occurred or the Api are faked
	}

	/*
	* Class:     edg_workload_userinterface_jclient_Api
	* Method:    ns_multi
	* Signature: ()V
	*/
	JNIEXPORT void JNICALL Java_edg_workload_userinterface_jclient_Api_ns_1multi
	(JNIEnv *env, jobject obj){
		jclass cls = env->GetObjectClass(obj);
		jmethodID appStr = env->GetMethodID ( cls , "appendString" ,  "(ILjava/lang/String;)V" ) ;
		try{
			if (appStr==0)    log_error (env , "Fatal Error while retrieving multi attribute info. Unable to find Api.appStr method\n" ) ;
			edg::workload::networkserver::client::NSClient nsClient ( nsHost , nsPort ,  ( edg::workload::common::logger::level_t)nsLevel  ) ;
			vector<string> multi ;
			nsClient.getMultiattributeList(multi) ;
			for  (  vector<string>::iterator it  =  multi.begin() ; it !=multi.end() ; it++ )
				env->CallVoidMethod(obj, appStr, (jint) 0 , env->NewStringUTF( it->c_str() ) );
		}catch (exception &exc){ log_error (env ,  exc.what() ) ;
		}catch (...){ log_error (env, "Fatal Error: Unpredictalbe exception thrown by JNI wrapper");    }
	}

	/*
	* Class:     edg_workload_userinterface_jclient_Api
	* Method:    ns_output
	* Signature: (Ljava/lang/String;)V
	*/

	JNIEXPORT void JNICALL Java_edg_workload_userinterface_jclient_Api_ns_1output
	(JNIEnv *env, jobject obj, jstring   jobid){
		jclass cls = env->GetObjectClass(obj);
		jmethodID appStr = env->GetMethodID ( cls , "appendString" ,  "(ILjava/lang/String;)V" ) ;
		try{
			const char *jid = env->GetStringUTFChars(jobid, 0);
			if (appStr==0)    log_error (env , "Fatal Error while retrieving outputsandbox file. Unable to find Api.appStr method\n" ) ;
			edg::workload::networkserver::client::NSClient nsClient ( nsHost , nsPort ,  ( edg::workload::common::logger::level_t)nsLevel  ) ;
			vector<string> outputfiles ;
			nsClient.getOutputFilesList ( jid ,outputfiles  ) ;
			for  (  vector<string>::iterator it  =   outputfiles.begin() ; it !=outputfiles.end() ; it++ )
				env->CallVoidMethod(obj, appStr, (jint) 0 , env->NewStringUTF( it->c_str() ) );
		}catch (exception &exc){ log_error (env ,  exc.what() ) ;
		}catch (...){ log_error (env, "Fatal Error: Unpredictalbe exception thrown by JNI wrapper");    }
	};


	/*
	* Class:     edg_workload_userinterface_jclient_Api
	* Method:    ns_match
	* Signature: (Ljava/lang/String;I)V
	*/
	JNIEXPORT void JNICALL Java_edg_workload_userinterface_jclient_Api_ns_1match
	(JNIEnv *env, jobject obj, jstring jobad){
		const char *id = env->GetStringUTFChars( jobad, 0);
		jclass cls = env->GetObjectClass(obj);
		jmethodID appStr = env->GetMethodID ( cls , "appendString" ,  "(ILjava/lang/String;)V" ) ;
		if (appStr==0)
		log_error (env , "Fatal Error while retrieving matching ce for the job: "+string(id) +".\nUnable to find Api.appStr method\n" ) ;
		try{
			// vector<string> resources ;
			std::vector<string> resources ;
			edg::workload::networkserver::client::NSClient nsClient ( nsHost , nsPort,  ( edg::workload::common::logger::level_t)nsLevel  ) ;
			// cout << "Calling list match with: " << id << endl  ;
			nsClient.listJobMatch( string ( id ) , resources )  ;
			// cout << "Found resources: " <<  resources.size()  <<endl ;
			for (unsigned int i  = 0 ; i< resources.size() ; i++){
				// cout << "Found: " << resources[i].first << endl << flush ;
				//env->CallVoidMethod(obj, appStr, (int) resources[i].second   , env->NewStringUTF( resources[i].first.c_str() ) );
                                env->CallVoidMethod(obj, appStr, i  , env->NewStringUTF( resources[i].c_str() ) );
				// env->CallVoidMethod(obj, appInt, i  , resources[i].second ) ;
			}
		}catch(exception &exc){  log_error (env , exc.what() ) ;
		}catch (...){ log_error (env, "Fatal Error: Unpredictable exception thrown by JNI wrapper");    }
		env->ReleaseStringUTFChars( jobad, id);
	}

	/*
	* Class:     edg_workload_userinterface_jclient_Api
	* Method:    ns_free
	* Signature: ()V
	*/
	JNIEXPORT void JNICALL Java_edg_workload_userinterface_jclient_Api_ns_1free
	(JNIEnv *env, jobject obj){     cout << "JNI ns_free: do nothing (Deprecated)" << endl ; /*  Deprecated */              }


	/*
	* Class:     edg_workload_userinterface_jclient_Api
	* Method:    lb_init
	* Signature: (Ljava/lang/String;)V
	*/
	JNIEXPORT void JNICALL Java_edg_workload_userinterface_jclient_Api_lb_1init
	(JNIEnv *env, jobject obj, jstring  nsAddr ){
		const char *lb = env->GetStringUTFChars(nsAddr, 0);
		jclass cls = env->GetObjectClass(obj);
		jmethodID appInt = env->GetMethodID ( cls , "appendInt" ,  "(II)V" ) ;
		if (appInt==0)
			log_error (env , "Fatal Error: unable to find Api.appInt method\n" ) ;
		edg_wll_Context * ctx =  (edg_wll_Context*) malloc (sizeof(edg_wll_Context*) ) ;
		env->CallVoidMethod(obj, appInt, 1 , (jint) lbVect.size() );
		lbVect.push_back( ctx) ;
		if ( edg_wll_InitContext( ctx ) ) log_error (env , "Unable to Initialise LB context") ;
		if (edg_wll_SetParam( *ctx, EDG_WLL_PARAM_SOURCE, EDG_WLL_SOURCE_USER_INTERFACE ) ) log_error (env , "Unable to set LB source parameter" ) ;
		if ( ! getenv ( EDG_WL_LOG_DESTINATION) ){
			if (edg_wll_SetParamString( *ctx, EDG_WLL_PARAM_DESTINATION, lb ) ) log_error (env , "Unable to set LB destination parameter") ;
			char edg_log_dest [1024];
			int DEF_LOG_PORT = 9002 ;
			sprintf (  edg_log_dest , "%s%s%d", lb , ":" , DEF_LOG_PORT ) ;
			unsetenv( EDG_WL_LOG_DESTINATION  ) ;
			setenv (  EDG_WL_LOG_DESTINATION, edg_log_dest , 0 );
		}
		env->ReleaseStringUTFChars( nsAddr, lb);
	}

	/*
	* Class:     edg_workload_userinterface_jclient_Api
	* Method:    lb_register
	* Signature: (Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)V
	*/
	JNIEXPORT void JNICALL Java_edg_workload_userinterface_jclient_Api_lb_1register
	(JNIEnv *env, jobject obj, jstring jobId, jstring jdl, jstring nsAddr){
		const char *jd = env->GetStringUTFChars(jdl, 0);
		const char *id = env->GetStringUTFChars(jobId, 0);
		const char *ns = env->GetStringUTFChars(nsAddr, 0);
		edg_wlc_SSLInitialization();
		try{
			edg::workload::common::jobid::JobId jid ( id );
lock() ;
			if ( edg_wll_RegisterJobSync ( *lbVect[  getCtx( env, obj , LB_CTX  ) ]  , jid.getId() ,  EDG_WLL_JOB_SIMPLE ,    jd, ns, 0 ,   NULL, NULL) ){
				char error_message [1024];
				char *msg, *dsc ;
				edg_wll_Error(   *lbVect[  getCtx( env, obj , LB_CTX  ) ]  , &msg , &dsc ) ;
				sprintf ( error_message , "%s%s%s%s%s%s%s%s%s", "Unable to Register the Job:\n", jid.toString().c_str() ,"\nto the LB logger at: ",
				getenv ( EDG_WL_LOG_DESTINATION) , "\n" , msg , " (", dsc , " )" )  ;
				log_error (env , error_message) ;
			}
unlock() ;
		}catch(exception &exc){  log_error (env , exc.what() ) ;
		}catch (...){ log_error (env, "Fatal Error: Unpredictalbe exception thrown by JNI wrapper") ;   }
		env->ReleaseStringUTFChars( jdl, jd);
		env->ReleaseStringUTFChars( jobId, id);
		env->ReleaseStringUTFChars( nsAddr, ns);
	}

	/*
	* Class:     edg_workload_userinterface_jclient_Api
	* Method:    lb_logSync
	* Signature: (Ljava/lang/String;)V
	*/
	JNIEXPORT void JNICALL Java_edg_workload_userinterface_jclient_Api_lb_1logSync
	(JNIEnv *env, jobject obj, jstring jdl){
		const char *jd = env->GetStringUTFChars(jdl, 0);
		if ( edg_wll_LogEventSync( *lbVect[  getCtx( env, obj , LB_CTX  ) ] , EDG_WLL_EVENT_CHKPT , EDG_WLL_FORMAT_CHKPT , "1" , jd ) )
			log_error (env , "Unable to log the sync event to LB" ) ;
		env->ReleaseStringUTFChars( jdl, jd);
	}


	/*
	* Class:     edg_workload_userinterface_jclient_Api
	* Method:    lb_log_output
	* Signature: (Ljava/lang/String;)V
	*/
	JNIEXPORT void JNICALL Java_edg_workload_userinterface_jclient_Api_lb_1log_1output
	(JNIEnv *env, jobject obj, jstring jobId){
		const char *id = env->GetStringUTFChars(jobId, 0);
		try{
			edg::workload::common::jobid::JobId jid ( id )  ;
		}catch(exception &exc){  log_error (env , exc.what() ) ;
		}catch (...){ log_error (env, "Fatal Error: Unpredictalbe exception thrown by JNI wrapper");    }
		env->ReleaseStringUTFChars( jobId, id);
	}

	/*
	* Class:     edg_workload_userinterface_jclient_Api
	* Method:    lb_log_start
	* Signature: (Ljava/lang/String;Ljava/lang/String;I)V
	*/
	JNIEXPORT void JNICALL Java_edg_workload_userinterface_jclient_Api_lb_1log_1start
	(JNIEnv *env, jobject obj, jstring jdl, jstring host , jint port ){
		const char *addr = env->GetStringUTFChars(host, 0);
		const char *jd = env->GetStringUTFChars(jdl, 0);
		char str_port [1024];
		sprintf (str_port , "%s%s%d" , nsHost.c_str(),":",nsPort );
		// sprintf (str_port , "%d" , port );
		// cout << "JNI::Starting...." << endl << flush ;
		lock();
		if (edg_wll_LogTransferSTART( *lbVect[  getCtx( env, obj , LB_CTX  ) ] ,
		EDG_WLL_SOURCE_NETWORK_SERVER ,
		addr, str_port , jd, "", "" ))  cerr << "\n\n\nJNI::edg_wll_LogTransferSTART failed"<<flush;
		unlock();
		// cout << "JDN::Starting....DONE" << endl << flush ;
		env->ReleaseStringUTFChars(host, addr);
		env->ReleaseStringUTFChars( jdl, jd);
	}

	/*
	* Class:     edg_workload_userinterface_jclient_Api
	* Method:    lb_free
	* Signature: ()V
	*/
	JNIEXPORT void JNICALL Java_edg_workload_userinterface_jclient_Api_lb_1free
	(JNIEnv *env, jobject obj){
		cout << "JNI lb_free: do nothing (Deprecated)" << endl ;
	}

	/*
	* Class:     edg_workload_userinterface_jclient_Api
	* Method:    lb_jobs
	* Signature: (Ljava/lang/String;I)V
	*/
	JNIEXPORT void JNICALL Java_edg_workload_userinterface_jclient_Api_lb_1jobs
	(JNIEnv *env, jobject obj, jstring lbHost, jint port){
		using namespace edg::workload::logging::client ;
		const char *lb = env->GetStringUTFChars(lbHost, 0);
		//Defining non-native methods:
		jclass cls = env->GetObjectClass(obj);
		jmethodID appStr = env->GetMethodID ( cls , "appendString" ,  "(ILjava/lang/String;)V" ) ;
		if (appStr==0)
			log_error (env , "Fatal Error while retrieving user's jobs.\nUnable to find Api.appStr method\n" ) ;
		try{
			ServerConnection server ;
			vector<edg::workload::common::jobid::JobId>  jobs ;
			server.setQueryServer( lb , port ) ;
			server.userJobs(jobs) ;
			for  (  vector<edg::workload::common::jobid::JobId>::iterator it  =  jobs.begin() ; it !=jobs.end() ; it++ )
				env->CallVoidMethod(obj, appStr, (jint) 0 , env->NewStringUTF( it->toString().c_str() ) );
		}catch(exception &exc){     log_error (env , exc.what() ) ;
		}catch (...){ log_error (env, "Fatal Error: Unpredictalbe exception thrown by JNI wrapper\n");    }
		env->ReleaseStringUTFChars(lbHost, lb);
	}


	/*
	* Class:     edg_workload_userinterface_jclient_Api
	* Method:    lb_getSequence
	* Signature: ()Ljava/lang/String;
	*/
	JNIEXPORT jstring JNICALL Java_edg_workload_userinterface_jclient_Api_lb_1getSequence
	(JNIEnv *env, jobject obj){   return  env->NewStringUTF ( edg_wll_GetSequenceCode(*lbVect[  getCtx( env, obj , LB_CTX  ) ] )  );  }

	/**
	* Method : LOAD STATUS
	* This method Load the c++ JobStatus into a Java JobStatus instance **/
	void loadStatus(  JNIEnv *env, const jobject &obj, const edg::workload::logging::client::JobStatus  &status ){
		// cout << "JNI::loadStatus Call load Status" << endl ;
		//Defining non-native methods
		using namespace edg::workload::logging::client ;
		jclass cls = env->GetObjectClass(obj);
		jmethodID appStr = env->GetMethodID ( cls , "appendString" ,  "(ILjava/lang/String;)V" ) ;
		if (appStr==0)   log_error (env , "Fatal Error while retrieving status information\nUnable to find Api.appStr method\n" ) ;
		jmethodID appInt = env->GetMethodID ( cls , "appendInt" ,  "(II)V" ) ;
		if (appInt==0)   log_error (env , "Fatal Error while retrieving status information.\nUnable to find Api.appInt method\n"   ) ;

		// Iterate over attributes
		std::vector<pair<JobStatus::Attr, JobStatus::AttrType> > attrList = status.getAttrs();
		int STATUS = 35;
		env->CallVoidMethod(obj, appStr, (jint) STATUS , env->NewStringUTF( status.name().c_str() ) );
		int STATUS_CODE = 36;
		env->CallVoidMethod(obj, appInt, (jint) STATUS_CODE , (jint)status.status );
		char time [1024];

		for (unsigned i=0; i < attrList.size(); i++ ) {
			switch (attrList[i].second) {
				case JobStatus::INT_T :{
					env->CallVoidMethod(obj, appInt,  attrList[i].first , (jint) status.getValInt(attrList[i].first) );
				}
				break;
				case JobStatus::BOOL_T    :{
					env->CallVoidMethod(obj, appInt,  attrList[i].first , (jint)  status.getValBool(attrList[i].first) );
				}
				break;
				case JobStatus::STRING_T  :{
					// cout << "Appending for: " << attrList[i].first << ") " << status.getValString(attrList[i].first) << endl<< flush;
					env->CallVoidMethod(obj, appStr,   attrList[i].first , env->NewStringUTF( status.getValString(attrList[i].first).c_str() ) );
				}
				break;
				case JobStatus::JOBID_T    :{
					if(((edg::workload::common::jobid::JobId)status.getValJobId(attrList[i].first)).isSet())
						env->CallVoidMethod(obj, appStr,  attrList[i].first , env->NewStringUTF( status.getValJobId(attrList[i].first).toString().c_str() ) ) ;
				}
				break;
				case JobStatus::STSLIST_T    :{
					// cout <<"\n\n\nFOUND STSLIST_T!!!!!!!!!!!!\n\n\n" << endl ;
					// for(unsigned int i=0; i < v.size(); i++) {};
				}
				break;
				case JobStatus::STRLIST_T    :{
					std::vector<std::string> v = status.getValStringList(attrList[i].first);
					for(unsigned int j=0; j < v.size(); j++){
						env->CallVoidMethod(obj, appStr,attrList[i].first , env->NewStringUTF( v[j].c_str() ) );
					}
				}
				break;
				case JobStatus::TAGLIST_T    :{
					std::vector<std::pair<std::string,std::string> >  v = status.getValTagList(   attrList[i].first   ) ;
					for (unsigned int j = 0 ; j < v.size() ; j++ ){
						// cout << v[i].first << " - " << v[i].second << " ;   " << flush ;
						env->CallVoidMethod( obj, appStr,attrList[i].first ,   env->NewStringUTF( v[j].first.c_str() )  );
						env->CallVoidMethod( obj, appStr,  attrList[i].first ,   env->NewStringUTF( v[j].second.c_str() )  );
					}
				break;
				}
				case JobStatus::INTLIST_T    :{
					std::vector<int> v = status.getValIntList(attrList[i].first);
					for(unsigned int j=0; j < v.size(); j++){
						sprintf (time , "%d%s" , v[j] , ".0 s") ;
						env->CallVoidMethod(obj, appStr,attrList[i].first , env->NewStringUTF( time ) );
					}
				}
				break;
				case JobStatus::TIMEVAL_T    :{
					timeval t = status.getValTime(attrList[i].first);
					// cout << t.tv_sec << "." << t.tv_usec << " s " << endl;
					sprintf (time , "%d%s%d%s" ,(int)t.tv_sec , ".", (int)t.tv_usec," s") ;
					// cout << "\nTIME CHECKED:" << t.tv_sec <<":" <<t.tv_usec<< "----->" <<time << endl << flush;
					env->CallVoidMethod(obj, appStr,  attrList[i].first , env->NewStringUTF( time ) );
					// env->CallVoidMethod(obj, appInt, (jint) attrList[i].first , (jint)  t.tv_sec );  // TBD
				}
				break;
				default : /* something is wrong */
					cerr << "\n\nFATAL ERROR!!! Something has gone bad for " << status.getAttrName(attrList[i].first) << flush;
				break;
			}
		}  // end For
		// Sub Jobs Implementation recoursive CALL
		std::vector<JobStatus> v = status.getValJobStatusList(  JobStatus::CHILDREN_STATES );
		for(unsigned int i=0; i < v.size(); i++)  loadStatus (env, obj, v[i]  );
	}



	/*
	* Class:     edg_workload_userinterface_jclient_Api
	* Method:    lb_user_status
	* Signature: (Ljava/lang/String;I)V
	*/
	JNIEXPORT void JNICALL Java_edg_workload_userinterface_jclient_Api_lb_1user_1status
	(JNIEnv *env, jobject obj, jstring  lbHost, jint port){
	using namespace edg::workload::logging::client ;
	const char *lb = env->GetStringUTFChars(lbHost, 0);
		try{
			ServerConnection server ;
			server.setQueryServer( lb , port ) ;
			vector<JobStatus> states = server.userJobStates()  ;
			for (unsigned int j = 0 ; j< states.size() ; j++) {
				loadStatus(  env, obj, states[j] ) ;
			}  // end for (unsigned i=0; i < attrList.size(); i++ )
		} catch (exception &exc){     log_error (env , exc.what() ) ;
		}catch (...){ log_error (env, "Fatal Error: Unpredictalbe exception thrown by JNI wrapper\n");    }
		env->ReleaseStringUTFChars(lbHost, lb);
	}


	/*
	* Class:     edg_workload_userinterface_jclient_Api
	* Method:    lb_status
	* Signature: (Ljava/lang/String;)V
	*/
	JNIEXPORT void JNICALL Java_edg_workload_userinterface_jclient_Api_lb_1status
	(JNIEnv *env, jobject obj, jstring jobId ){
		const char *id = env->GetStringUTFChars(jobId, 0);
		using namespace edg::workload::logging::client ;
		string err ;
		try{
			edg::workload::common::jobid::JobId jid (id)  ;
			err = "- " + jid.toString()  +":\n";
			lock() ;
			lbJob = jid ;
			JobStatus  status =   lbJob.status( Job::STAT_CLASSADS | Job::STAT_CHILDSTAT      ) ;
			unlock() ;
			// Load the Status
			loadStatus(  env, obj, status );
		}catch( exception &exc){
			unlock() ;
			log_error (env , err +exc.what()  ) ;
		}catch (...){
			unlock() ;
			log_error (env, "Fatal Error: Unpredictalbe exception thrown by JNI wrapper");
		}
		env->ReleaseStringUTFChars(jobId, id);
	}

	/*
	* Class:     edg_workload_userinterface_jclient_Api
	* Method:    lb_log
	* Signature: (Ljava/lang/String;)V
	*/
	JNIEXPORT void JNICALL Java_edg_workload_userinterface_jclient_Api_lb_1log
	(JNIEnv *env, jobject obj, jstring jobId){
		const char *id = env->GetStringUTFChars(jobId, 0);
		using namespace edg::workload::logging::client ;
		//Defining non-native methods:
		jclass cls = env->GetObjectClass(obj);
		jmethodID appStr = env->GetMethodID ( cls , "appendString" ,  "(ILjava/lang/String;)V" ) ;
		if (appStr==0)
			log_error (env , "Fatal Error while retrieving logging information for the job: "+string(id) +".\nUnable to find Api.appStr method\n" ) ;
		jmethodID appInt = env->GetMethodID ( cls , "appendInt" ,  "(II)V" ) ;
		if (appInt==0)
			log_error (env , "Fatal Error while retrieving logging information for the job: "+string(id)+".\nUnable to find Api.appInt method\n"   ) ;
		try{
			edg::workload::common::jobid::JobId jid (id)  ;
			Job    lbJob (jid)   ;
			vector <Event> events= lbJob.log();
			char time [1024] ;
			for (unsigned i=0; i < events.size(); i++ ) {
				std::vector<pair<Event::Attr,Event::AttrType> > attrList = events[i].getAttrs();
				// cout << "Event : "  << events[i].name() << endl ;
				const int EVENT = 48 ;  // Do not change this value unless change the Event.java code
				env->CallVoidMethod(obj, appStr, (jint) EVENT , env->NewStringUTF( events[i].name().c_str() ) );
				const int EVENT_CODE = 49 ;  // Do not change this value unless change the Event.java code
				env->CallVoidMethod(obj, appInt, (jint) EVENT_CODE , (int)events[i].type ) ;
				// cout << (int) events[i].type << endl << flush ;
				for (unsigned j=0; j < attrList.size(); j++ ) {
					// cout << "parsing :" <<  j << ")"<< attrList[j].first << "]" <<events[i].getAttrName(attrList[j].first) << endl;
					switch (attrList[j].first){
					/* Int attributes*/
					case Event::JOBTYPE:
					case Event::NSUBJOBS:
					case Event::EXIT_CODE:
					case Event::PRIORITY:
					case Event::SVC_PORT:
					case Event::PERMISSION: // Not avaliable in DAG branch
					case Event::PERMISSION_TYPE: // Not avaliable in DAG branch
					case Event::USER_ID: // Not avaliable in DAG branch
					case Event::USER_ID_TYPE: // Not avaliable in DAG branch
						env->CallVoidMethod(obj, appInt, (jint) attrList[j].first , (jint) events[i].getValInt(attrList[j].first) );
						break;
					case Event::LEVEL:
						env->CallVoidMethod(obj, appStr, (jint) attrList[j].first , env->NewStringUTF(  edg_wll_LevelToString ( (edg_wll_Level) events[i].getValInt(attrList[j].first) ) ) );
						break;
					case Event::STATUS_CODE:
						env->CallVoidMethod(obj, appStr, (jint) attrList[j].first , env->NewStringUTF(   (  edg_wll_DoneStatus_codeToString ((edg_wll_DoneStatus_code) events[i].getValInt(attrList[j].first)     ) )  ) );
						break;
					case Event::SOURCE:
					case Event::DESTINATION:
					case Event::FROM:
						env->CallVoidMethod(obj, appStr, (jint) attrList[j].first , env->NewStringUTF( edg_wll_SourceToString ( (edg_wll_Source )events[i].getValInt(attrList[j].first)    ) ) );
						break;
					case Event::SRC_ROLE:
					case Event::HOST:
					case Event::SEQCODE:
					case Event::USER:
					case Event::SRC_INSTANCE:
					case Event::VALUE:
					case Event::RESULT:
					case Event::NAME:
					case Event::REASON:
					case Event::DESCR:
					case Event::SVC_HOST:
					case Event::SVC_NAME:
					case Event::FROM_INSTANCE:
					case Event::FROM_HOST:
					case Event::LOCAL_JOBID:
					case Event::QUEUE:
					case Event::NODE:
					case Event::JDL:
					case Event::SEED:
					case Event::NS:
					case Event::JOB:
					case Event::RETVAL:
					case Event::CLASSAD:
					case Event::TAG:
					case Event::HELPER_PARAMS:
					case Event::HELPER_NAME:
					case Event::DEST_ID:
					case Event::DEST_HOST:
					case Event::DEST_JOBID:
					case Event::DEST_INSTANCE:
						env->CallVoidMethod(obj, appStr, (jint) attrList[j].first , env->NewStringUTF( events[i].getValString(attrList[j].first).c_str() ) );
						break;
					case Event::JOBID:
					case Event::PARENT:{
						if(   ((edg::workload::common::jobid::JobId)events[i].getValJobId(attrList[j].first)).isSet()   )
						env->CallVoidMethod(obj, appStr, (jint) attrList[j].first , env->NewStringUTF( events[i].getValJobId(attrList[j].first).toString().c_str() ) ) ;
						}
						break;
					case Event::TIMESTAMP:
						{
							timeval t = events[i].getValTime(attrList[j].first);
							// cout << t.tv_sec << "." << t.tv_usec << " s " << endl;
							sprintf (time , "%d%s%d%s" ,(int)t.tv_sec , ".", (int)t.tv_usec," s") ;
							// cout << "\nTIME CHECKED:" << t.tv_sec <<":" <<t.tv_usec<< "----->" <<time << endl ;
							env->CallVoidMethod(obj, appStr, (jint) attrList[j].first , env->NewStringUTF( time ) );
							// cout << t.tv_sec << "." << t.tv_usec << " s " << endl;
						}
						break;
					default:
						cout << "\n\n\nSomething has gone bad.Please check field: " << j << ")"<< attrList[j].first << "]" <<events[i].getAttrName(attrList[j].first) << endl;
						break;
					}
				} // end of switch
				// end of the event: append a new one
				// cout << "\n\nFuori un altro" << endl <<flush;
				// env->CallVoidMethod(obj, next );
			} //end for events
		}catch( edg::workload::common::utilities::Exception &exc){
		cout  << "JNI::getLogInfo-> error caught" << exc.dbgMessage() << endl << flush ;
		log_error (env ,  exc.what() ) ;
		}catch(exception &exc){  log_error (env ,  exc.what() ) ;
		}catch (...){ log_error (env, "Fatal Error: Unpredictalbe exception thrown by JNI wrapper\n");    }
		env->ReleaseStringUTFChars(jobId, id);
	}

	/*
	* Class:     edg_workload_userinterface_jclient_Api
	* Method:    lb_log_listener
	* Signature: (Ljava/lang/String;Ljava/lang/String;I)V
	*/
	JNIEXPORT void JNICALL Java_edg_workload_userinterface_jclient_Api_lb_1log_1listener
	(JNIEnv *env, jobject obj, jstring jobid , jstring host , jint port){
		const char *addr = env->GetStringUTFChars(host, 0);
		const char *id = env->GetStringUTFChars(jobid, 0);
		if ( string(id).length() >1 ){
			try{
				edg::workload::common::jobid::JobId jid ( id )  ;
				if ( edg_wll_SetLoggingJob(*lbVect[  getCtx( env, obj , LB_CTX  ) ]  , jid.getId() , NULL , EDG_WLL_SEQ_DUPLICATE) )
					log_error (env , "Error found while performing edg_wll_SetLoggingJob LB method") ;
			}catch(exception &exc){   log_error (env , exc.what() ) ;  return ;
			}catch (...){ log_error (env, "Fatal Error: Unpredictalbe exception thrown by JNI wrapper\n");   return ; }
		}
		if ( edg_wll_LogListener(  *lbVect[  getCtx( env, obj , LB_CTX  ) ]  , "InteractiveListener" , string(addr).c_str() ,   (uint16_t) port) ){
			char error_message [1024];
			char *msg, *dsc ;
			edg_wll_Error(   *lbVect[  getCtx( env, obj , LB_CTX  ) ]   , &msg , &dsc ) ;
			sprintf ( error_message , "%s%s%s%s%s%s%s","Unable to perform edg_wll_LogListener  at: ",
			getenv ( EDG_WL_LOG_DESTINATION) , "\n" , msg , " (" , dsc , " )" )  ;
			// cout << "JNI: listener error: " << error_message << endl << flush ;
			log_error (env ,   error_message ) ;
		}
		env->ReleaseStringUTFChars(host, addr);
		env->ReleaseStringUTFChars(jobid, id);
	}

/*
 * Class:     edg_workload_userinterface_jclient_Api
 * Method:    lb_log_query
 * Signature: (Ljava/lang/String;I)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_edg_workload_userinterface_jclient_Api_lb_1log_1query
  (JNIEnv *env, jobject obj, jstring jobid_str, jint step){
	using namespace edg::workload::logging::client ;
	const char *id = env->GetStringUTFChars(jobid_str, 0);
	int                 i, cnt, error;
	edg_wlc_JobId       jobid;
	edg_wll_Event       *events = NULL;
	edg_wll_QueryRec    jc[2];
	edg_wll_QueryRec    ec[2];
	// parse the jobID string
	error = edg_wlc_JobIdParse(id, &jobid);
	if ( error )   { log_error (env , "Unable to Initialise LB context") ; return  env->NewStringUTF( "" ) ; }
	memset(jc,0,sizeof jc);
	memset(ec,0,sizeof ec);
	// job condition: JOBID = jobid
	jc[0].attr = EDG_WLL_QUERY_ATTR_JOBID;
	jc[0].op = EDG_WLL_QUERY_OP_EQUAL;
	jc[0].value.j = jobid;
	// event condition: Event type = CHKPT
	ec[0].attr = EDG_WLL_QUERY_ATTR_EVENT_TYPE;
	ec[0].op = EDG_WLL_QUERY_OP_EQUAL;
	/******************************************************************
	* This is a workaround. The meaning of the step parameter depends on
	* its value: if greater than zero is a Checkpointing step,
	* otherwise retrieves the first Logging listener Event
	******************************************************************/
	if  (step<0)   ec[0].value.i = EDG_WLL_EVENT_LISTENER ; else
	//Normal behaviour: chkpt
	ec[0].value.i = EDG_WLL_EVENT_CHKPT;
	// cout << "Job::getState > Performing query" << endl << flush ;
	error = edg_wll_QueryEvents(  *lbVect[  getCtx( env, obj , LB_CTX  ) ] , jc, ec, &events );
	if ( error == ENOENT ) {  log_error (env , "No events found: ENOENT") ; return  env->NewStringUTF( "" ) ; }
	if ( error ) {  log_error (env , "Query failed") ;   return  env->NewStringUTF( "" ) ; }
	for ( cnt=0; events[cnt].type; cnt++ ); // counts the number of events
	if ( !cnt ) { log_error (env , "Empty Events vector returned") ;  return  env->NewStringUTF( "" ) ; }
	// sort the events vector using the timestamp
	qsort(events, cnt, sizeof(edg_wll_Event), &cmp_by_timestamp);
	// the last state in the array is the most recent
	if ((int) step>= (int)cnt ){ log_error (env , "Number of requested step bigger than chkpt logged events") ; 	return  env->NewStringUTF( "" ) ; }
	string state ;

	/******************************************************************
	* This is a workaround. The meaning of the step parameter depends on
	* its value: if greater than zero is a Checkpointing step,
	* otherwise retrieves the first Logging listener Event
	******************************************************************/
	if  (step<0){
		char
		str_port  [20] ;
		sprintf ( str_port  , "%d" ,  events[0].listener.svc_port ) ;
		// cout << "JNI:: converted logged port:" << str_port << endl ;
		state = string( events[0].listener.svc_host )  + ":"  + string( str_port  );  ;
		// cout << "JNI:: final String:" << state << endl ;
	}else
		//Normal behaviour: chkpt
		state = string( events[cnt-1-step].chkpt.classad );
	for ( i = 0; i < cnt; i++)
		edg_wll_FreeEvent( events+i );
	free(events);
	env->ReleaseStringUTFChars(jobid_str, id);
	return  env->NewStringUTF(state.c_str()) ;
}

// EOF


