#include "edg/workload/logging/client/Event.h"
#include "edg/workload/common/jobid/JobId.h"
#include "edg/workload/common/utilities/Exceptions.h"
#include "edg/workload/userinterface/client/Job.h"
#include <iostream>

using namespace std ;
using namespace edg::workload::logging::client ;
using namespace edg::workload::common::utilities ;


void printEvent( vector<Event> &events, int level) {
		for (unsigned i=0; i < events.size(); i++ ) {
			std::vector<pair<Event::Attr,Event::AttrType> > attrList = events[i].getAttrs();
			cout << "\n -Logged Event: "  << events[i].name()<<"\n---" << flush;
			if (level>0)
			  for (unsigned j=0; j < attrList.size(); j++ ) {
				cout << events[i].getAttrName(attrList[j].first) << ": ";
				switch (attrList[j].second) {
				case Event::INT_T:
				case Event::LOGSRC_T:
				case Event::PORT_T:
					cout << events[i].getValInt(attrList[j].first) << endl;
					break;
				case Event::STRING_T:
					cout << events[i].getValString(attrList[j].first) << endl;
					break;
				case Event::JOBID_T    :{
					if(((edg::workload::common::jobid::JobId)events[i].getValJobId(attrList[j].first)).isSet())
					cout << events[i].getValJobId(attrList[j].first).toString()  ;
					}
				break;
				case Event::TIMEVAL_T:
					{
						timeval t = events[i].getValTime(attrList[j].first);
						cout << t.tv_sec << "." << t.tv_usec << " s " << endl;
					}
					break;
				default      : 	/* something is wrong */
					break;
				}
			}
			cout << endl;
		} //end for events
	     // } //end vect interator
}

int main(int argc,char *argv[]){
try{
   int level = 0 ;
   if (argc < 2 ||  strcmp(argv[1],"--help") == 0)  {
		cout << "Usage : " << argv[0] << "  <job id > [<verbosity level>]" << endl;
		return 1;
   }
   if (argc ==3)
           level = atoi (argv[2]);
   edg::workload::common::jobid::JobId jid ( argv[1] );
   edg::workload::userinterface::Job   job(jid);
   cout << "Retrieving event information from LB, please wait..." << endl <<  flush;
   vector<Event> events = job.getLogInfo() ;
   printEvent(events, level) ;
   return 0 ;

}catch  (Exception &exc){
         cout << "Operation not performed\n" ;
         cout <<  exc.printStackTrace();
}
return 1 ;
}

