#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "edg/workload/logging/client/JobStatus.h"
#include "edg/workload/common/jobid/JobId.h"
#include "edg/workload/common/utilities/Exceptions.h"
#include "edg/workload/userinterface/client/Job.h"
#include "edg/workload/userinterface/client/JobCollection.h"
#include "edg/workload/logging/client/Job.h"
#include <iostream>

#ifdef HAVE_STRINGSTREAM
#include <sstream>
#else
#include <strstream>
#endif

using namespace std ;
using namespace edg::workload::common::utilities ;
using namespace edg::workload::common::jobid ;
using namespace edg::workload::logging::client ;

void printStatus(JobStatus &stat, int level) {
	std::vector<pair<JobStatus::Attr, JobStatus::AttrType> > attrList = stat.getAttrs();
        cout << "\nJob Status Information:\n - Current Status =  " << stat.name() << "\n   ---" << endl<< flush ;	
	if (level>0)
	   for (unsigned i=0; i < attrList.size(); i++ ) {
		cout << stat.getAttrName(attrList[i].first) << " == " ;
		switch (attrList[i].second) {
			case JobStatus::INT_T     :
				cout << stat.getValInt(attrList[i].first) << endl<<flush;
				break;
			case JobStatus::STRING_T  :
				cout << stat.getValString(attrList[i].first) << endl<<flush;
				break;
			case JobStatus::TIMEVAL_T    :{
				timeval t = stat.getValTime(attrList[i].first);
				cout << t.tv_sec << "." << t.tv_usec << " s " << endl<<flush;}
				break;
			case JobStatus::BOOL_T    :
				cout << stat.getValBool(attrList[i].first) << endl<<flush;
				break;
			case JobStatus::JOBID_T    :{
				if(((edg::workload::common::jobid::JobId)stat.getValJobId(attrList[i].first)).isSet())
				cout << stat.getValJobId(attrList[i].first).toString() << endl<<flush;
				}
		break;
			case JobStatus::STSLIST_T    :{
				std::vector<JobStatus> v = stat.getValJobStatusList(attrList[i].first);
				for(unsigned int i=0; i < v.size(); i++) ; //TBD
			}
			break ;
			case JobStatus::TAGLIST_T :{
				std::vector<std::pair<std::string,std::string> >  v = stat.getValTagList(  attrList[i].first ) ;
				for (unsigned int i = 0 ; i < v.size() ; i++ )
					cout << v[i].first  << " - " << v[i].second << " ;   " << flush ;
				}
			break;
			case JobStatus::INTLIST_T    :{
				std::vector<int> v = stat.getValIntList(attrList[i].first);
				for(unsigned int j=0; j < v.size(); j++)
					cout << "  " << v[j]  << flush ;
				cout << endl<<flush;
			}
			break;
			case JobStatus::STRLIST_T    :{
				std::vector<std::string> v = stat.getValStringList(attrList[i].first);
				for(unsigned int j=0; j < v.size(); j++)
					cout << v[j] ;
				cout << endl<<flush;
			}
			break ;
			default : /* something is wrong */
				break;
		}
	}
	cout << endl<<flush;
}

void printResult (edg::workload::userinterface::CollectionResult res) {
   cout << "\n\nPrinting  collection Result: " << endl << flush ;
   for (unsigned int i = 0; i < res.result.size() ; i ++ ){
       cout  <<"   - - - \n"<< i << ") " << res.result[i].first << endl << flush  ;
       if (  res.result[i].second.get() == edg::workload::common::utilities::SUCCESS  ){
          cout << "Success!" << endl << flush;
	  JobStatus *status = res.result[i].second.getStatus()  ;
	  cout << "Current State: " << status->name()  << endl << flush ;
       }else
          cout << "Failure: "  << res.result[i].second.getError() << endl << flush ;
   }
   cout << "   - - -\n\n" << endl << flush ;
}

int main(int argc,char *argv[]){
   using namespace edg::workload::userinterface ;
   int level = 0 ;
   if (argc < 2 ||  strcmp(argv[1],"--help") == 0)  {
		cout << "Usage : " << argv[0] << "  <job id > [<verbose level>]" << endl<<flush;
		return 1;
   }
   if (argc ==3){
	   level = atoi (argv[2]);
   }
   JobCollection collect ;
#ifdef HAVE_STRINGSTREAM
        stringstream buffer ;
#else
        strstream buffer;
#endif
        filebuf fb;
        fb.open (  argv[1]  ,ios::in);
        istream jdl_in(&fb);
        string result = "" ;
        int blank, leng = 2048;
        char tmpChar [leng];
        cout << "Adding JobIds..." << endl << flush ;
        while (  jdl_in.getline(tmpChar,leng,'\n')  ){
          //find the first no-blank character
          blank = 0 ;
          while (tmpChar[blank]==' ')
               blank++;
          //Check For Commented lines:
          if (tmpChar[blank] =='#'){
              //header += "\n" +string(tmpChar);
              //Python Comment: do nothing
          }else if (tmpChar[blank] =='/' && tmpChar[blank+1] =='/'){
              //header += "\n" +string(tmpChar);
              //C/C++ Comment: do nothing
          }else  try{
	      collect.insert ( edg::workload::userinterface::Job ( JobId ( tmpChar ) )  ) ;
	      cout << collect.size() << ") " << tmpChar << endl << flush ;
	  }catch  (Exception &exc){  cout << exc.what() ;  }
        }
        fb.close() ;



try{
   // std::vector <std::pair< edg::workload::userinterface::Job ,  edg::workload::logging::client::JobStatus > >     statusVector ;
  printResult  ( collect.getStatus(    )  );

   return 0 ;
}catch  (Exception &exc){
         cout << "Operation not performed\n" ;
         cout <<  exc.printStackTrace();
}
return 1 ;
};
