#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "edg/workload/logging/client/JobStatus.h"
#include "edg/workload/common/jobid/JobId.h"
#include "edg/workload/common/requestad/JobAd.h"
// #include "edg/workload/common/requestad/JobAd.h"
#include "edg/workload/common/utilities/Exceptions.h"
#include "edg/workload/userinterface/client/Job.h"
#include "edg/workload/userinterface/client/JobCollection.h"
#include "edg/workload/logging/client/Job.h"

#include <iostream>

#ifdef HAVE_STRINGSTREAM
#include <sstream>
#else
#include <strstream>
#endif

using namespace std ;
using namespace edg::workload::common::utilities ;
using namespace edg::workload::common::requestad ;
using namespace edg::workload::common::jobid ;
using namespace edg::workload::logging::client ;

void printResult (edg::workload::userinterface::CollectionResult res) {
   cout << "\n\nPrinting  collection Result: " << endl << flush ;
   for (unsigned int i = 0; i < res.result.size() ; i ++ ){
       cout  <<"   - - - \n"<< i << ") " << res.result[i].first << ":\n" << flush  ;
       if (  res.result[i].second.get() == edg::workload::common::utilities::SUCCESS  ){
          cout << "Success" <<endl << flush;
       }else
          cout << "Failure: "  << res.result[i].second.getError() << endl << flush ;
   }
   cout << "   - - -\n\n" << endl << flush ;
}

int main(int argc,char *argv[]){
   using namespace edg::workload::userinterface ;
   if (argc < 2 ||  strcmp(argv[1],"--help") == 0)  {
		cout << "Usage : " << argv[0] << "  <config test file> <jdl file 1> <jdl file 2> <jdl file 3> ..." << endl
		<< "where config test file is a classAd containing the following info:\n " << endl
		<< "[\n nsHost = \"<host>\";\n nsPort = <port> ;\n LbHosts = { \"<lb host 1>\" , \"<lb host 2>\" ,... } ;\n LbPorts = { \"<lb port 1>\" , \"<lb port2>\" ,... };\nce_id = \"<ceId>\";\n]\n" << flush;
		return 1 ;
   }
   cout << "edg-wl-jobs-submit test: reading conf file..." << endl << flush ;
   Ad conf ;
   string nsHost  ;
   int nsPort ;
   string ce_id ="";
   vector < pair < string , int > > lbAddrs ;
   try{
      conf.fromFile ( argv[1] );
      cout << conf.toLines () << flush;
      nsHost = conf.getStringValue ("nsHost")[0] ;
      nsPort = conf.getIntValue ("nsPort")[0] ;
      vector <string> lbHosts = conf.getStringValue("lbHosts") ;
      vector <int> lbPorts = conf.getIntValue ("lbPorts") ;
      for (unsigned int i = 0 ; i < lbHosts.size() ; i++ )
            lbAddrs.push_back(  pair <string , int >  (lbHosts[i] , lbPorts[i] ) );
   }catch  (Exception &exc){
      cout << "edg-wl-jobs-submit test: Exception while Reading conf file.\n" ;
      cout <<  exc.printStackTrace();
      return 1 ;
   }
   cout << "\nedg-wl-jobs-submit test: Inserting JobAd(s) in the collection..." <<endl << flush ;
   JobCollection collect ;
   for ( int i = 2 ; i < argc ; i++ )try{
         cout << collect.size()  <<"\nedg-wl-jobs-submit test: Reading file: "<<  argv[i] << endl << flush ;
         JobAd jobad ;
         jobad.fromFile ( argv[i] ) ;
         cout << jobad.toLines() << endl << flush;
         collect.insert ( edg::workload::userinterface::Job ( jobad) ) ;
   }catch  (Exception &exc){
      cout << "edg-wl-jobs-submit test: Exception while inserting Jobad:\n" ;
      cout <<  exc.what();
   }


try{
   cout << "edg-wl-jobs-submit test:  Performing a submission.."<<endl << flush ;
   //collect.setMaxThreadNumber(1) ;
   collect.setLoggerLevel( 6 ) ;
   CollectionResult res = collect.submit ( nsHost , nsPort , lbAddrs , ce_id ) ;
   printResult (  res  );
   return 0 ;
}catch  (Exception &exc){
         cout << "edg-wl-jobs-submit test: Operation not performed, here follows the stack trace: " << endl ;
         cout <<  exc.printStackTrace() << endl << flush ;
}
return 1 ;
};
