#ifndef  EDG_WORKLOAD_COMMON_REQUESTAD_AD_H
#define EDG_WORKLOAD_COMMON_REQUESTAD_AD_H
/*
 * Ad.h
 * Copyright (c) 2001 The European Datagrid Project - IST programme, all rights reserved.
 * Contributors are mentioned in the code where appropriate.
 */
#include  "edg/workload/common/requestad/requestad_namespace.h"
#include "classad_distribution.h"

COMMON_REQUESTAD_NAMESPACE_BEGIN  //Defining UserInterFace NameSpace
/**
 * Provides a common interface for all Ad components.
 * It allows the user to create a valid ClassAd instance utilizing native classes and
 * retrieve any kind of information from it. It is utilised as a superclass for JobAd class
 * @see JoibAd
 *
 *
 * @version 0.1
 * @date 15 April 2002
 * @author Alessandro Maraschini <alessandro.maraschini@datamat.it> */
class Ad : protected classad::ClassAd{
	public :
					/**@name Constructors*/
		//@{
		/**
		* default Constructor   */
		Ad() ;
		/**
		* Default Destructor   */
		virtual ~Ad() throw();
		/**
		* Constructor by ClassAd
		@param classAd the classad source where to create the Ad instance from*/
		Ad(const classad::ClassAd& classAd );
		/**
		* Constructor by string
		@param jdl_string the Ad string representation */
		Ad(const std::string& jdl_string) ;
		/**Deep copy of Ad.
		@return a ClassAd pointer representing a copy of all Ad attributes**/
		classad::ClassAd* ad() {  return Copy() ;  } ;
		//@}
					/**@name From methods*/
		//@{
		/**
		* Create an Ad instance from a string
		@param jdl_string the ad string representation*/
		virtual void fromString(const std::string& jdl_string) ;

		/**
		* Create an Ad instacne from a file
		@param jdl_file the string representing the path containing the jdl to be parsed */
		virtual void fromFile(const std::string& jdl_file) ;
		//@}
					/**@name To methods*/
		//@{
		/**
		* Convert the Ad Instance into a single line string representation */
		virtual std::string toString();
		/**Convert an Ad attribute into its string representation
		@param attr_name the attribute to be looked up
		@return return the attrbute string representation*/
		virtual std::string toString( const std::string& attr_name );
		/**
		* Convert the Ad Instance into a multi line indented string representation
		*@return as in toString() methods but splits the string one line per attribute */
		virtual std::string toLines();
		/** Check whether the JobAd has been initialised
		*@return whether the JobAd has been initialised(true) or not (false)*/
		bool isSet() ;
		/**Reset the JobAd Instance. All the previous existing attributes will be deleted*/
		void clear();
		//@}
					/**@name has methods*/
		//@{
		/** Check If the specified attribute has already been set
		* @param attr_nam The name of the attibute to be looked for
		* @return @a true if the attribute has been found, @a false otherwise  */
		bool hasAttribute(const std::string& attr_name);
		/**
		* Delete an Attribute. It fails if the attribute doesn't exist
		* @param attr_nam The name of the attibute to be deleted */
		virtual classad::ExprTree* delAttribute(const std::string& attr_name) ;

		/**
		* Check if the specified value is present in the specified attribute  */
		bool hasAttribute(const std::string& attr_name, const std::string& attr_value);
		/**
		* Return the list of all attributes inside classad
		*/
		std::vector<std::string> attributes() ;
		//@}

                           /**@name Get Methods*/
		//@{
		enum {
		TYPE_UNKNOWN = classad::Value::ERROR_VALUE,
		/**Attribute Integer type value
		@see #getType*/
		TYPE_INTEGER =classad::Value::INTEGER_VALUE,
		/**Attribute Boolean type value
		@see #getType*/
		TYPE_BOOLEAN =classad::Value::BOOLEAN_VALUE,
		/**Attribute String type value
		@see #getType*/
		TYPE_STRING =classad::Value::STRING_VALUE,
		/**Attribute Real/Double type value
		@see #getType*/
		TYPE_REAL =classad::Value::REAL_VALUE,
		/**Attribute Expression type value
		@see #getType*/
		TYPE_EXPRESSION =classad::Value::UNDEFINED_VALUE
		};
		/**
		* Retrieve the type of the value specified for attr_name
		*@param attr_name the name of the attribute
		*@return an integer representing the type of the attribute
		*@exception AdEmptyException if the attribute is not present in the JobAd instance
		*@see #TYPE_UNKNOWN
		*@see #TYPE_INTEGER
		*@see #TYPE_BOOLEAN
		*@see #TYPE_STRING
		*@see #TYPE_REAL
		*@see #TYPE_EXPRESSION */
		int getType (const std::string& attr_name )   ;



		/**
		* Allow adding a value to an already set attribute of the JobAd instance
		* (i.e. it transforms it in a list attribute). if used on a non-set attribute the corresponding setAttribute method is automatically called.
		*  @param attr_name a string representing the attribute name
		*  @param attr_value -  The value of the attribute to be added
		* @exception AdMismatchException - The type of value is not allowed for the specified attribute name
		* @exception AdSyntaxException - Syntax error catched while trying to add the attribute */
		void  addAttribute (const std::string& attr_name, int attr_value)          ;

		/**
		* Allow adding a value to an already set attribute of the JobAd instance
		* (i.e. it transforms it in a list attribute). if used on a non-set attribute the corresponding setAttribute method is automatically called.
		*  @param attr_name a string representing the attribute name
		*  @param attr_value -  The value of the attribute to be added
		* @exception AdMismatchException - The type of value is not allowed for the specified attribute name
		* @exception AdSyntaxException - Syntax error catched while trying to add the attribute */
		void  addAttribute (const std::string& attr_name, double attr_value)       ;
		/**
		* Allow adding a value to an already set attribute of the JobAd instance
		* (i.e. it transforms it in a list attribute). if used on a non-set attribute the corresponding setAttribute method is automatically called.
		*  @param attr_name a string representing the attribute name
		*  @param attr_value -  The value of the attribute to be added
		* @exception AdMismatchException - The type of value is not allowed for the specified attribute name
		* @exception AdSyntaxException - Syntax error catched while trying to add the attribute*/
		void  addAttribute (const std::string& attr_name, bool attr_value)         ;
		/**
		* Allow adding a value to an already set attribute of the JobAd instance
		* (i.e. it transforms it in a list attribute). if used on a non-set attribute the corresponding setAttribute method is automatically called.
		*  @param attr_name a string representing the attribute name
		*  @param attr_value -  The value of the attribute to be added
		* @exception AdMismatchException - The type of value is not allowed for the specified attribute name
		* @exception AdFormatException The type of value is not allowed for the specified attribute name
		* @exception AdSyntaxException - Syntax error catched while trying to add the attribute  */
		void  addAttribute (const std::string& attr_name, const std::string& attr_value);

		/**
		* Allow adding a value to an already set attribute of the JobAd instance
		* (i.e. it transforms it in a list attribute). if used on a non-set attribute the corresponding setAttribute method is automatically called.
		*  @param attr_name a string representing the attribute name
		*  @param attr_value -  The value of the attribute to be added
		* @exception AdMismatchException - The type of value is not allowed for the specified attribute name
		* @exception AdFormatException The type of value is not allowed for the specified attribute name
		* @exception AdSyntaxException - Syntax error catched while trying to add the attribute  */
		void  addAttribute (const std::string& attr_name, Ad* attr_value);


		/**
		* Add The specified Expression Attribute to the jdl istance
		* @param  attr_name - The Name of the attribute to be added
		* @param  attr_value - The string expression of the attribute to be added
		* @exception AdMismatchException - The type of value is not allowed for the specified attribute name
		* @exception AdClassAdException - a classAd method raised an error
		* @exception AdEmptyException - The attribute  attr_name had been already set */
		virtual void  setAttributeExpr (const std::string& attr_name, const std::string& attr_value);


		/**
		* Add The specified Integer Attribute to the jdl istance
		* @param  attr_name - The Name of the attribute to be added
		* @param  attr_value - The value of the attribute to be added
		* @exception AdMismatchException - The type of value is not allowed for the specified attribute name
		* @exception AdSyntaxException - Syntax error catched while trying to add the attribute
		* @exception AdEmptyException - The attribute  attr_name had been already set   */
		virtual void  setAttribute (const std::string& attr_name, int attr_value) ;
		/**
		* Add The specified String Attribute to the jdl istance
		* @param  attr_name - The Name of the attribute to be added
		* @param  attr_value - The value of the attribute to be added
		* @exception AdMismatchException - The type of value is not allowed for the specified attribute name
		* @exception AdFormatException The type of value is not allowed for the specified attribute name   */
		virtual void  setAttribute (const std::string& attr_name, const std::string& attr_value) ;
		/**
		* Add The specified String Attribute to the jdl istance
		* @param  attr_name - The Name of the attribute to be added
		* @param  attr_value - The value of the attribute to be added
		* @exception AdMismatchException - The type of value is not allowed for the specified attribute name
		* @exception AdSyntaxException - Syntax error catched while trying to add the attribute
		* @exception AdEmptyException - The attribute  attr_name had been already set   */
		virtual void  setAttribute (const std::string& attr_name, double attr_value)   ;
		/**
		* Add The specified String Attribute to the jdl istance
		* @param  attr_name - The Name of the attribute to be added
		* @param  attr_value - The value of the attribute to be added
		* @exception AdMismatchException - The type of value is not allowed for the specified attribute name
		* @exception AdSyntaxException - Syntax error catched while trying to add the attribute
		* @exception AdEmptyException - The attribute  attr_name had been already set   */
		virtual void  setAttribute (const std::string& attr_name, bool attr_value)   ;
		/**
		* Add The specified Integer Attribute to the jdl istance
		* @param  attr_name - The Name of the attribute to be added
		* @param  attr_value - The value of the attribute to be added
		* @exception AdMismatchException - The type of value is not allowed for the specified attribute name
		* @exception AdSyntaxException - Syntax error catched while trying to add the attribute
		* @exception AdEmptyException - The attribute  attr_name had been already set   */
		virtual void  setAttribute (const std::string& attr_name, Ad* attr_value) ;
		/**
		* Retreive the value of the specified attribute
		*  @param attr_name The name of the attribute name to be retrieved
		* @return the string representation of this attribute
		* @exception AdEmptyException - The checked attribute has not been set yet
		* @exception AdMismatchException - The type of retrieved value is not allowed for the specified attribute name   */
		virtual std::string  getAttributeExpr (const std::string& attr_name) 	;

		/**
		* Retreive the value of the specified attribute
		*  @param attr_name The name of the attribute name to be retrieved
		* @return a vector cantaining the values listed in the specified attribute ,  (1-size vector if the attribute has a single value)
		* @exception AdEmptyException - The checked attribute has not been set yet
		* @exception AdMismatchException - The type of retrieved value is not allowed for the specified attribute name   */
		std::vector<int> getIntValue(const std::string& attr_name) ;
		/**
		* Retreive the value of the specified attribute
		*  @param attr_name The name of the attribute  to be retrieved
		* @return a vector cantaining the values listed in the specified attribute ,  (1-size vector if the attribute has a single value)
		* @exception AdEmptyException - The checked attribute has not been set yet
		* @exception AdMismatchException - The type of retrieved value is not allowed for the specified attribute name   */
		std::vector<double> getDoubleValue(const std::string& attr_name) ;
		/**
		* Retreive the value of the specified attribute
		*  @param attr_name The name of the attribute  to be retrieved
		* @return a vector cantaining the values listed in the specified attribute ,  (1-size vector if the attribute has a single value)
		* @exception AdEmptyException - The checked attribute has not been set yet
		* @exception AdMismatchException - The type of retrieved value is not allowed for the specified attribute name   */
		std::vector<bool> getBoolValue(const std::string& attr_name) ;
		/**
		* Retreive the value of the specified attribute
		*  @param attr_name The name of the attribute  to be retrieved
		* @return a vector cantaining the values listed in the specified attribute ,  (1-size vector if the attribute has a single value)
		* @exception AdEmptyException - The checked attribute has not been set yet
		* @exception AdMismatchException - The type of retrieved value is not allowed for the specified attribute name   */
		std::vector<std::string> getStringValue(const std::string& attr_name) ;
		/**
		* Retreive the value of the specified attribute
		*  @param attr_name The name of the attribute  to be retrieved
		* @return a vector cantaining the a vector of strings  (1-size vector if the attribute has a single value)
		* @exception AdEmptyException - The checked attribute has not been set yet
		* @exception AdMismatchException - The type of retrieved value is not allowed for the specified attribute name   */
		std::vector<std::vector<std::string> > getStringList(const std::string& attr_name) ;
		//@}

	protected:
		/**Convert the string into lower case
		*@param src the source string
		@return the lower representation of the source string
		*/
		std::string toLower ( const std::string &src) ;
		/**
		* Add a value to a list (if already present) or set the first value of this attribute
		*  @param attr_name a string representing the attribute name
		*  @param val   The value of the attribute to be added
		* @exception AdMismatchException  The type of value is not allowed for the specified attribute name
		* @exception AdFormatException The type of value is not allowed for the specified attribute name
		*/
		virtual void addAttribute(std::string attr_name , classad::Value val) ;
		/**
		* Append a value to a list
		*  @param attr_name a string representing the attribute name
		*  @param val -  The value of the attribute to be added
		* @param tree the current value (before appending) of the attribute
		*/
		virtual void appendValue(classad::ExprTree* tree , const classad::Value& val, const std::string& attr_name) ;
		/**
		* Insert a classad Exptression inside the Ad instance
		*  @param attr_name a string representing the attribute name
		*  @param val -  The value of the attribute to be added
		*/
		virtual void insertAttribute(const std::string& attr_name , classad::ExprTree* val);
		/**
		* Insert a classad Exptression inside the Ad instance
		*  @param attr_name a string representing the attribute name
		*  @param val -  The value of the attribute to be added
		*/
		virtual void insertAttribute(const std::string& attr_name , classad::Value val);
		/**
		* Retrieve the Value of the specified attribute
		*  @param attr_name a string representing the attribute name
		*  @return the Value of the attribute inside the Ad instance
		*/
		virtual classad::Value getTypeValue(const std::string& attr_name);
		/**Check whether a value is present inside a classad Expression
		* @param tree the expression to be checked
		* @ attr_value the value to be checked 
		* @return true (if the value is present) false otherwise */
		bool hasAttribute( classad::ExprTree* tree ,  const std::string& attr_value);
	private:
		// meaningless: utilised for future implementation (like DAG)
		friend class ExpDagAd ;
		virtual classad::ClassAd* fromString(const std::string& father , std::string& jdl_string , std::string& fxm  ) ;
		virtual classad::ExprList* fromList(const std::string& father , std::string& jdl_string, std::string& fxm) ;
		virtual classad::ExprTree* readValue(const std::string& father , std::string& jdl_string, std::string& fxm) ;
};
}  // requestad namespace
COMMON_NAMESPACE_END } //Closing  UserInterFace NameSpace
#endif

//EOF

