#ifndef  EDG_WORKLOAD_COMMON_REQUESTAD_JDLATTRIBUTELIST_H
#define EDG_WORKLOAD_COMMON_REQUESTAD_JDLATTRIBUTELIST_H
/*
 * JdlAttributeList.h
 * Copyright (c) 2001 The European Datagrid Project - IST programme, all rights reserved.
 * Contributors are mentioned in the code where appropriate.
 *
 */
#include <list>
#include <string>
#include <vector>
#include <stdlib.h>
#include <algorithm>
#include  "edg/workload/common/requestad/requestad_namespace.h"
// #include  "edg/workload/common/common_namespace.h"
#include "classad_distribution.h"


#define JDL_ARRAY 30
#define JDLATTR_SET_METHOD(method) string METHOD = method

COMMON_REQUESTAD_NAMESPACE_BEGIN
// COMMON_NAMESPACE_BEGIN{
// namespace requestad{
/**
* JobAd Manipulation attribute class
* This class allow the user to know the names of all Jobad attributes and
* it is used principally by JobAd in order to perform the requested checks needed
* ro prepare for submission
*/
class  JdlAttributeList{
public:
    //Constuctor
    JdlAttributeList();
    virtual ~JdlAttributeList() throw() {};

    //Find Methods
    bool  findBool(const std::string& attr_name)    {return findAttr (attr_name , boolAttrs  );   };
    bool  findInt(const std::string& attr_name)       {return findAttr (attr_name , intAttrs   );   };
    bool  findString(const std::string& attr_name)  {return findAttr (attr_name , stringAttrs);   };
    bool  findDouble(const std::string& attr_name){return findAttr (attr_name , doubleAttrs);   };
    bool  findAd(const std::string& attr_name)      {return findAttr (attr_name , adAttrs);   };
    bool  findList(const std::string& attr_name)     {return findAttr (attr_name , listAttrs  );   };
    bool  findExpr(const std::string& attr_name)   {return findAttr (attr_name , exprAttrs  );   };
    bool  findCheck(const std::string& attr_name){ return findAttr (attr_name , checkAttrs  );   };
    //This funcion finds a string out of a list of strings
    bool  findAttribute(const std::string& attr_name) ;
    bool  findAttr (const std::string& attr_name , std::vector<std::string> ) const ;
    static std::string toLower ( const std::string & source) ;

    // bool  findString(const std::string& attr_name , std::list<std::string> vect );  //TBD DEPRECATED
                       /**@name Check Methods*/
 //@{
     /**Check if the couple attribute/value is admitted
     * @param attr_name - the name of the attribute
     * @param attr_value - the value of the attribute
     * @exception AdMismatchException - The type of value is not allowed for the specified attribute name     */
     void checkInt(const std::string& attr_name, int attr_value)  ;
     /**Check if the couple attribute/value is admitted
     * @param attr_name - the name of the attribute
     * @param attr_value - the value of the attribute
     * @exception AdMismatchException - The type of value is not allowed for the specified attribute name     */
     void checkDouble(const std::string& attr_name, const double& attr_value)  ;
     /**Check if the couple attribute/value is admitted
     * @param attr_name - the name of the attribute
     * @param attr_value - the value of the attribute
     * @exception AdMismatchException - The type of value is not allowed for the specified attribute name     */
     void checkBool(const std::string& attr_name, const bool& attr_value)    ;
     /**Check if the couple attribute/value is admitted
     * @param attr_name - the name of the attribute
     * @param attr_value - the value of the attribute
     * @exception AdMismatchException - The type of value is not allowed for the specified attribute name     */
     void checkString(const std::string& attr_name, const std::string& attr_value) ;
     /**Check if the couple attribute/value is admitted
     * @param attr_name - the name of the attribute
     * @param attr_value - the value of the attribute
     * @exception AdMismatchException - The type of value is not allowed for the specified attribute name     */
     void checkAd(const std::string& attr_name, classad::ClassAd* attr_value) ;
     /**Check if the couple attribute/value is admitted
     * @param attr_name - the name of the attribute
     * @param attr_value - the value of the attribute
     * @exception AdMismatchException - The type of value is not allowed for the specified attribute name
     * @exception AdFormatException - The type of value is not allowed for the specified attribute name     */
     void checkExpr(const std::string& attr_name, const std::string& attr_value)  ;
     enum Attr {
        JOBTYPE_MPICH_REQ_RTE,
        JOBTYPE_MPICH_REQ_CPU,
        JOBTYPE_MPICH_RANK_FREE,
        REQ_DEFAULT,
        RANK_DEFAULT,
        MAX_ATTR
     } ;
     void setDefault ( Attr attr_name , const std::string& attr_value)  ; 

 private:
    std::vector<std::string> defaultAttrs ;
    std::vector<std::string> stringAttrs ;
    std::vector<std::string>  intAttrs ;
    std::vector<std::string>  doubleAttrs;
    std::vector<std::string>  boolAttrs;
    std::vector<std::string>  adAttrs;
    std::vector<std::string>  listAttrs;
    std::vector<std::string>  exprAttrs;
    std::vector<std::string>  checkAttrs;

    //This function is called by Find methods
    // bool  findAttr (const std::string& attr_name , std::string lis[]) const ;
    bool checking ;
    // JobAd is allowed to access to all JdlAttributeList members
    friend class JobAd ; //TBD needed????
};
}   // requestad namespace
COMMON_NAMESPACE_END } //Closing  UserInterFace NameSpace
#endif



