#ifndef  EDG_WORKLOAD_COMMON_REQUESTAD_JOBAD_H
#define EDG_WORKLOAD_COMMON_REQUESTAD_JOBAD_H
/*
 * JobAd.h
 * Copyright (c) 2001 The European Datagrid Project - IST programme, all rights reserved.
 * Contributors are mentioned in the code where appropriate.
 */
#include "edg/workload/common/requestad/Ad.h"
#include "edg/workload/common/requestad/JobAdSchema.h"
#include "classad_distribution.h"
#include "edg/workload/common/requestad/JdlAttributeList.h"

COMMON_REQUESTAD_NAMESPACE_BEGIN  //Defining UserInterFace NameSpace

// class JdlAttributeList ;
/**
  * Provides a representation of the job description in the JDL language
 * and the functions for building and manipulating it. Basically the
 * JDL is the Condor ClassAd language, so it is legitimate the direct
 * use of the Condor API library for creating, modifying, deleting a
 * job description. However the JobAd class extends the ClassAd class
 * of the Condor ClassAd library additionally providing some helper methods
 * that ease the construction of job descriptions being fully compliant to
 * WP1 WMS specification.
 *
 * @brief Provides a representation of the job description in the JDL language
 *
 * @ingroup UserInterface
 * @version 0.1
 * @date 15 April 2002
 * @author Alessandro Maraschini <alessandro.maraschini@datamat.it>
*/

// class JobAd : private classad::ClassAd{
// class JobAd : public edg::workload::common::requestad::Ad{
class JobAd : public edg::workload::common::requestad::Ad{
	public:
				/**@name Constructors/Destructor */
 //@{
		/** Instantiates an  empty  JobAd object */
		JobAd() ;
		/** JobAd destructor */
		virtual ~JobAd() throw();
		/**Instantiates a JobAd object from the given ClassAd-jdl string
		* @param  jdl_string  A string representig the description of the job
		* @exception AdMismatchException - A value is not of the right type for an attribute name in the JobAd
		* @exception AdFormatException  A value is not in the right format for the  an attribute name in the JobAd
		* @exception AdSyntaxException  Syntax error catched while trying to add an attribute
		* @exception AdClassAdException  a classAd method raised an error
		* @exception AdListException  A list has been mad with a non-list value attribute   */
		JobAd( const std::string& jdl_string) ;
		/**
		* Constructor with a ClassAd instance
		* @param classAd the ClassAd instance where to build the JobAd from
		*/
		JobAd( const classad::ClassAd& classAd);
		/** Copy constructor
		*@param jobad a JobAd instance to be copied  */
		JobAd::JobAd(const JobAd& jobad);
		/** Operator **/
		void JobAd::operator=(const JobAd& jobad);





 //@}
				/**@name String and Stream Constructor/Destructor */
 //@{
		/**
		* Update the JobAd object with the given input stream.
		* @param  jdl_in  this is the job description passed in the form of a generic input stream
		so that it can be taken from a terminal input stream, file input stream, string streams etc.
		* @exception AdMismatchException  A value is not of the right type for an attribute name in the JobAd
		* @exception AdFormatException  A value is not in the right format for the  an attribute name in the JobAd
		* @exception AdSyntaxException  Syntax error catched while trying to add an attribute
		* @exception AdClassAdException  a classAd method raised an error
		* @exception AdListException  A list has been mad with a non-list value attribute   */
		void fromStream(std::istream& jdl_in) ;
		/**
		* Convert the JobAd Instance into a single line string representation */
		std::string toString();
		/** Retrieve the string representation of an attribute
		* @param attr_name the attribute to be retrieved
		* @return the string representation of the attribute  */
		std::string toString( const std::string & attr_name  );

		/**
		* Convert the JobAd Instance into a multi line indented string representation */
		// std::std::string toLines();
		/**
		* Convert the JobAd Instance into a single line string representation ready for submission
		* @return the string reoresentation that goes to the NetworkServer */
		std::string toSubmissionString();
		/** Put the JobAd Instance as a string into a file
		* @param file_path the string representation of the file where to copy the JobAd to
		*/
		void toFile(const std::string& file_path) ;
 //@}
					/**@name Insertion Methods*/
 //@{
		/**
		* Set the default value for Rank attribute
		* (take in consideration if not specified in JDL)
		* @param attr_value the value to be set to the default rank
		*/
		void setDefaultRank (const std::string& attr_value ) ;

		/**
		* Set the default value for Requirements attribute
		* (take in consideration if not specified in JDL)
		* @param attr_value the value to be set to the default requirements
		*/
		void setDefaultReq   (const std::string& attr_value ) ;

		/**
		* Add The specified Expression Attribute to the jdl istance
		* @param  attr_name  The Name of the attribute to be added
		* @param  attr_value  The string expression of the attribute to be added
		* @exception AdMismatchException  The type of value is not allowed for the specified attribute name
		* @exception AdClassAdException  a classAd method raised an error   */
		void  setAttributeExpr (const std::string& attr_name, const std::string& attr_value);
 //@}
					/**@name Retrieval Methods*/
 //@{
		/**
		* Retreive the value of the specified attribute
		*  @param attr_name The name of the attribute name to be retrieved
		* @return the string representng the value of the specified attribute
		* @exception AdEmptyException  The checked attribute has not been set yet
		* @exception AdMismatchException  The type of retrieved value is not allowed for the specified attribute name   */
		std::string getAd(const std::string& attr_name);
		/**
		* Retreive the value of the specified attribute
		*  @param attr_name The name of the attribute name to be retrieved
		* @return the value of the specified attribute
		* @exception AdEmptyException  The checked attribute has not been set yet
		* @exception AdMismatchException  The type of retrieved value is not allowed for the specified attribute name   */
		std::string getString(const std::string& attr_name) ;
		/**
		* Retreive the value of the specified attribute
		*  @param attr_name The name of the attribute name to be retrieved
		* @return the  value of the specified attribute
		* @exception AdEmptyException  The checked attribute has not been set yet
		* @exception AdMismatchException  The type of retrieved value is not allowed for the specified attribute name   */
		int getInt(const std::string& attr_name) ;
		/**
		* Retreive the value of the specified attribute
		*  @param attr_name The name of the attribute name to be retrieved
		* @return the  value of the specified attribute
		* @exception AdEmptyException  The checked attribute has not been set yet
		* @exception AdMismatchException  The type of retrieved value is not allowed for the specified attribute name   */
		double getDouble(const std::string& attr_name) ;
		/**
		* Retreive the value of the specified attribute
		*  @param attr_name The name of the attribute name to be retrieved
		* @return the  value of the specified attribute
		* @exception AdEmptyException  The checked attribute has not been set yet
		* @exception AdMismatchException  The type of retrieved value is not allowed for the specified attribute name   */
		bool getBool(const std::string& attr_name) ;
 //@}


                        /**@name Miscellaneous Methods*/
//@{
		/**Check if the couple attribute/value is admitted
		* @param attr_name  the name of the attribute
		* @param attr_value  a pointer to an ExprTree representing the value to be checkted
		* @exception AdMismatchException  The type of value is not allowed for the specified attribute name   */
		void checkSyntax( const std::string& attr_name , classad::ExprTree* attr_value  )   ;
		/**Check if the Member/isMember expression is properly used in rank and requirements attributes expressions
		* @param multi a vector containing all the attributes that could be of multiple value type
		* @exception   AdSyntaxException when the Member/IsMember expression is badly used*/
		void checkMultiAttribute ( const std::vector<std::string> &multi ) ;
		/**
		* Delete an Attribute. It fails if the attribute doesn't exist
		* @param attr_nam The name of the attibute to be deleted
		* @exception AdEmptyException  The attribute has not been set yet  */
		classad::ExprTree* delAttribute(const std::string& attr_name) ;
		/**
		* Check the JobAd instance for both syntax and semanthic errors
		* @exception AdMismatchException  A value is not of the right type for an attribute name in the JobAd
		* @exception AdFormatException  A value is not in the right format for the  an attribute name in the JobAd
		* @exception AdSyntaxException  Syntax error catched while trying to add an attribute
		* @exception AdClassAdException  a classAd method raised an error
		* @exception AdListException  A list has been mad with a non-list value attribute
		*/
		void check(bool restore=true) ;
//@}
	protected:
		// Not yet implemented, left for future implementation like DAG
		virtual void checkInputSandbox( std::vector<std::string>& extracted );
		virtual void insertAttribute(const std::string& attr_name , classad::ExprTree* val);
		std::vector<std::string> extractFiles(const std::string& attr_name , const std::string& path , std::vector<std::string> &extracted) ;
		/** user classAd: calssad containing all the attributes that could be possibly changed by the check method.
		This member is utilized in order to  ripristinate (restore method ) the old classad situation*/
		classad::ClassAd user ;
	private:
		JobAdSchema* schema ;
		JdlAttributeList jdlist ;
		// Check Only the syntax of the Jdl
		void checkJobType(const std::vector<std::string> & attr_value) ;
		void checkDataManagement( ) ;
		void checkOutputData(const std::vector<Ad> & attr_value) ;
		void checkSyntax( const std::string& attr_name , classad::Value attr_value )  ;
		// This method is not yet implemented, used for future implementation (like DagAd)
		virtual void checkSpecials( ) ;
		// Default special attributes:
		classad::ExprTree *defaultRank ;
		classad::ExprTree  *defaultReq ;
		// Check Jdl semantic co-existance rules
		void checkSemantic() ;
		// Add an attribute, this method is used by all public addAttribute methods
		void restore () ;

		//  Extract all files from a specified path

		bool checking ;
		//This Variable is utilized to iterate over the threads
		friend class Job ;
		friend class ExpDagAd ;
		static bool isAbsolutePath(const std::string &path , bool platform_dependence = true) ;
		static const std::string getName (const std::string &path , bool platform_dependence = true)  ;
};

}  // requestad namespace
COMMON_NAMESPACE_END } //Closing  UserInterFace NameSpace
#endif

//EOF
