#ifndef  EDG_WORKLOAD_COMMON_CLIENT_JOBADEXCEPTIONS_H
#define EDG_WORKLOAD_COMMON_CLIENT_JOBADEXCEPTIONS_H

/*
 * JobAdExceptions.h
 * Copyright (c) 2001 The European Datagrid Project - IST programme, all rights reserved.
 * Contributors are mentioned in the code where appropriate.
 */
#include "edg/workload/common/utilities/Exceptions.h"
COMMON_NAMESPACE_BEGIN{ //Defining UserInterFace NameSpace
namespace requestad{
/**
 * JobAdException
 * This Exception is thrown when a bad usage of the JobAd class is made
 * @ingroup Common
 * @version 0.1
 * @date 15 April 2002
 * @author Alessandro Maraschini <alessandro.maraschini@datamat.it>
*/
class JobAdException : public edg::workload::common::utilities::Exception {
protected:
   /**
   * Update all mandatory Exception Information
   */
   JobAdException (std::string file,
                                   int line,
                                  std::string method,
                                   int code,
                                  std::string exception_name) ;
  virtual ~JobAdException() throw () {};
  std::string what() throw();
protected:
  std::string          error_description ;

};//End CLass  JobAdException
/**
* AdClassAdException - raised when ClassAd error is checked during add/set methods
*/
class AdClassAdException   : public JobAdException {
public:
  AdClassAdException::AdClassAdException (std::string file,
                                      int line,
                                     std::string method,
                                      int code ,
                                     std::string method_name ,
                                     std::string error_description = "");
};//End CLass  AdClassAdException



/**
* AdSyntaxException - raised when syntax error is checked during add/set methods
*/
class AdSyntaxException   : public JobAdException {
public:
  AdSyntaxException::AdSyntaxException (std::string file,
                                      int line,
                                     std::string method,
                                      int code ,
                                     std::string attr_name );
};//End CLass  AdSyntaxException



/**
* AdSemanticException - raised while checking the whole constructed JobAd with the check() method
*/
class AdSemanticException : public JobAdException {
public:
  /**
   * Raised when mandatory attribute is missing, unable to find a specified path, wrong attribute coexistence
  */
  AdSemanticException::AdSemanticException (std::string file,
                                      int line,
                                     std::string method,
                                      int code,
                                     std::string exception_name );
};//End CLass  AdSemanticException

     /**
     * AdSemanticMandatoryException - raised when a mandatoty attribute is missing to the classAd
     */
     class AdSemanticMandatoryException : public AdSemanticException {
        public:
        /**
        * Raised when mandatory attribute is missing, unable to find a specified path, wrong attribute coexistence
        */
        AdSemanticMandatoryException (std::string file,
                                        int line,
                                       std::string method,
                                        int code,
                                       std::string attr_name );
     };//End CLass  AdSemanticMandatoryException


     /**
     * AdSemanticPathException - raised when a mandatoty attribute is missing to the classAd
     */
     class AdSemanticPathException : public AdSemanticException {
        public:
        /**
        * Raised when Path attribute is missing, unable to find a specified path, wrong attribute coexistence
        */
        AdSemanticPathException (std::string file,
                                        int line,
                                       std::string method,
                                        int code,
                                       std::string attr_name,
                                       std::string path_name );
     };//End CLass  AdSemanticPathException

     /**
     * AdSemanticGroupException - raised when a mandatoty attribute is missing to the classAd
     */
     class AdSemanticGroupException : public AdSemanticException {
        public:
        /**
        * Raised when Group attribute is missing, unable to find a specified path, wrong attribute coexistence
        */
        AdSemanticGroupException (std::string file,
                                        int line,
                                       std::string method,
                                        int code,
                                       std::string attrs_name );
     };//End CLass  AdSemanticGroupException

/**
*  AdAttributeException - raised when a not admitted value is added/set to the attribute
*/
class AdAttributeException : public JobAdException {
   public:
      AdAttributeException::AdAttributeException (std::string file,
                                      int line,
                                     std::string method,
                                      int code,
                                     std::string exception_name );
};//End CLass  AdAttibuteException

   /**
   *  AdMismatchException - raised when a not admitted value is added/set to the jdl
   */
   class AdMismatchException : public AdAttributeException {
        public:
        /**
        * AdMismatchException  Constructor
        */
        AdMismatchException (std::string file,
                                        int line,
                                       std::string method,
                                        int code,
                                       std::string attr_name,
                                       std::string error_description = ""  );

   };//End CLass  AdMismatchException

   /**
   *   AdFormatException Class
   */
   class AdFormatException : public AdAttributeException {
        public:
        /**
        *  AdFormatException  Constructor
        */
         AdFormatException(std::string file,
                                        int line,
                                       std::string method,
                                        int code,
                                       std::string attr_name,
                                       std::string format ="");
   };//End CLass  AdFormatException
   /**
   *  AdListException - raised when an add method is made on a non-list attribute
   */
   class AdListException : public AdAttributeException {
        public:
        /**
        *  AdListException  Constructor
        */
         AdListException(std::string file,
                                        int line,
                                       std::string method,
                                        int code,
                                       std::string attr_name );

   };//End CLass  AdListException


   /**
   *  AdEmptyException - raised when an set method is made on a empty attribute
   */
   class AdEmptyException : public AdAttributeException {
        public:
        /**
        *  AdEmptyException  Constructor
        */
         AdEmptyException(std::string file,
                                        int line,
                                       std::string method,
                                        int code,
                                       std::string attr_name );
   };//End CLass  AdEmptyException

// EWC_END_NAMESPACE; //Close the NameSpace
}  // requestad namespace
} COMMON_NAMESPACE_END
//USERINTERFACE_NAMESPACE_END  //Closing  UserInterFace NameSpace
#endif
