#ifndef EDG_WORKLOAD_JOBCONTROL_LOGMONITOR_CONDORMONITOR_H
#define EDG_WORKLOAD_JOBCONTROL_LOGMONITOR_CONDORMONITOR_H

#include <cstdio>

#include <string>
#include <fstream>

#include <boost/shared_ptr.hpp>

#include "edg/workload/common/common_namespace.h"
#include "edg/workload/jobcontrol/jobcontrol_namespace.h"
#include "edg/workload/jobcontrol/common/constants.h"

#include "Timer.h"
#include "SizeFile.h"

COMMON_SUBNAMESPACE_CLASS( jobid, JobId );

class ReadUserLog;
class ULogEvent;
class ExecuteEvent;

JOBCONTROL_NAMESPACE_BEGIN {

namespace jccommon {
  class EventLogger;
  class IdContainer;
}

namespace logmonitor {

class AbortedContainer;

struct MonitorData {
  MonitorData( AbortedContainer *aborted, jccommon::IdContainer *container, jccommon::EventLogger *logger = NULL );

  jccommon::EventLogger   *md_logger;
  jccommon::IdContainer   *md_container;
  AbortedContainer        *md_aborted;
};

class CondorMonitor {
public:
  enum status_t { no_events, event_read, event_error };

  CondorMonitor( const char *filename, MonitorData &data );
  CondorMonitor( const std::string &filename, MonitorData &data );

  ~CondorMonitor( void );

  /*
    Copy constructor and copy operator generated
    by the compiler...
  */

  inline bool file_completed( void ) const { return this->cm_sizefile->completed(); }
  inline bool got_last( void ) const { return this->cm_sizefile->size_field().last(); }
  inline unsigned int pending_jobs( void ) const { return this->cm_sizefile->size_field().pending(); }
  inline const std::string &filename( void ) const { return this->cm_filename; }

  status_t process_next_event( void );

  inline static void recycle_directory( const std::string &dir ) { cm_s_recycleDirectory.assign( dir ); }
  inline static const std::string &recycle_directory( void ) { return cm_s_recycleDirectory; }

private:
  void openFile( void );
  void doRecycle( void );
  void writeCurrentPosition( void );
  void processEvent( ULogEvent *event, bool removeTimer = true );
  void processGenericEvent( const std::string &condorid, int code, const std::string &message );
  void readRSL( const std::string &edgId, std::string &full, std::string &globus );
  void resubmitJob( int laststatus, const std::string &edgId, const std::string &sequence );
  void updateSequenceCode( jccommon::IdContainer::iterator &position, int status, int last = jccommon::undefined_status );
  ULogEvent *instantiateCondorEvent( int event, int condorid, time_t time );
  ULogEvent *instantiateGenericEvent( jccommon::generic_event_t evn, int condorid, time_t epoch );

  static std::string timerFileName( const std::string &filename );

  jccommon::EventLogger            *cm_logger;
  jccommon::IdContainer            *cm_container;
  AbortedContainer                 *cm_aborted;
  std::string                       cm_filename;
  boost::shared_ptr<SizeFile>       cm_sizefile;
  boost::shared_ptr<ReadUserLog>    cm_logfile;
  boost::shared_ptr<Timer>          cm_timer;

  static std::string    cm_s_recycleDirectory;
};

}; // Namespace logmonitor

} JOBCONTROL_NAMESPACE_END;

#endif /* EDG_WORKLOAD_JOBCONTROL_LOGMONITOR_CONDORMONITOR_H */

// Local Variables:
// mode: c++
// End:
