#ifndef __EDG_WORKLOAD_LOGGING_CLIENT_EVENTS_H__
#define __EDG_WORKLOAD_LOGGING_CLIENT_EVENTS_H__

/**
 * \file edg/workload/logging/client/events.h
 * \brief contains definition of event type codes for use both by lbapi.h and dglog.h
 */

#ident "$Header: /local/reps/lcgware/workload/logging/client/events.h.T,v 1.25 2003/01/17 07:27:56 honik Exp $"
/*

  !! Automatically generated file
  !! Do not edit, your changes will be discarded upon build
  !! Change the corresponding template file ./logging/client/events.h.T

*/

#include <sys/time.h>
#include <inttypes.h>

#include "edg/workload/common/jobid/jobid.h"
#include "edg/workload/logging/client/context.h"

#ifdef __cplusplus
extern "C" {
#endif


/**
 * Predefined type for ULM string 
 */
typedef char *edg_wll_LogLine;


/**
 * \typedef edg_wll_EventCode
 * Predefined event types 
 */
typedef enum _edg_wll_EventCode {
/** invalid code, e.g. uninitialized variable */
        EDG_WLL_EVENT_UNDEF = 0,
	EDG_WLL_EVENT_TRANSFER,	/**< Start, success, or failure of job transfer to another component */
	EDG_WLL_EVENT_ACCEPTED,	/**< Accepting job (successful couterpart to Transfer) */
	EDG_WLL_EVENT_REFUSED,	/**< Refusing job (unsuccessful couterpart to Transfer) */
	EDG_WLL_EVENT_ENQUEUED,	/**< The job has been enqueued in an inter-component queue */
	EDG_WLL_EVENT_DEQUEUED,	/**< The job has been dequeued from an inter-component queue */
	EDG_WLL_EVENT_HELPERCALL,	/**< Helper component is called */
	EDG_WLL_EVENT_HELPERRETURN,	/**< Helper component is returning the control */
	EDG_WLL_EVENT_RUNNING,	/**< Executable started */
	EDG_WLL_EVENT_RESUBMISSION,	/**< Result of resubmission decision */
	EDG_WLL_EVENT_DONE,	/**< Execution terminated (normally or abnormally) */
	EDG_WLL_EVENT_CANCEL,	/**< Cancel operation has been attempted on the job */
	EDG_WLL_EVENT_ABORT,	/**< Job aborted by system */
	EDG_WLL_EVENT_CLEAR,	/**< Job cleared, output sandbox removed */
	EDG_WLL_EVENT_PURGE,	/**< Job is purged from bookkepping server */
	EDG_WLL_EVENT_MATCH,	/**< Matching CE found */
	EDG_WLL_EVENT_PENDING,	/**< No match found yet */
	EDG_WLL_EVENT_REGJOB,	/**< New job registration */
	EDG_WLL_EVENT_CHKPT,	/**< Application-specific checkpoint record */
	EDG_WLL_EVENT_LISTENER,	/**< Listening network port for interactive control */
	EDG_WLL_EVENT_CURDESCR,	/**< current state of job processing (optional event) */
	EDG_WLL_EVENT_USERTAG,	/**< user tag -- arbitrary name=value pair */
	EDG_WLL_EVENT_CHANGEACL,	/**< Management of ACL stored on bookkepping server */
        EDG_WLL_EVENT__LAST,	/**< last currently supported event code */
} edg_wll_EventCode;

/**
 * \fn edg_wll_EventCode edg_wll_StringToEvent(char *name)
 * \param name		a string event name (e.g. "JobTransfer")
 * \return corresponding numeric code (edg_wll_EventCode)
 * \brief convert a string event name to the corresponding numeric code
 */

extern edg_wll_EventCode edg_wll_StringToEvent(char *);

/**
 * \fn char *edg_wll_EventToString(edg_wll_EventCode event)
 * \param event		an event numeric code (edg_wll_EventCode)
 * \return corresponding string (e.g. "JobTransfer")
 * \brief convert an event numeric code to the corresponding string
 */

extern char *edg_wll_EventToString(edg_wll_EventCode);


/**
 * \typedef edg_wll_KeyNameCode
 * Predefined ULM key types
 */
typedef enum _edg_wll_KeyNameCode {
	UNDEFINED,		/**< undefined */
	EDG_WLL_EVNT,		/**< event type */
	ULM_DATE,		/**< timestamp of event generation */
	ULM_HOST,		/**< hostname of the machine where the event was generated */
	ULM_LVL,		/**< logging level (system, debug, ...) */
	EDG_WLL_COMMON_PRIORITY,		/**< message priority (yet 0 for asynchronous and 1 for synchronous transfers) */
	EDG_WLL_COMMON_JOBID,		/**< DataGrid job id of the source job */
	EDG_WLL_COMMON_SEQCODE,		/**< sequence code assigned to the event */
	EDG_WLL_COMMON_USER,		/**< identity (cert. subj.) of the generator */
	EDG_WLL_COMMON_SOURCE,		/**< source (WMS component) which generated this event */
	EDG_WLL_COMMON_SRC_INSTANCE,		/**< instance of WMS component (e.g. service communication endpoint) */
	EDG_WLL_TRANSFER_DESTINATION,	/**< destination where the job is being transfered to */
	EDG_WLL_TRANSFER_DEST_HOST,	/**< destination hostname */
	EDG_WLL_TRANSFER_DEST_INSTANCE,	/**< destination instance */
	EDG_WLL_TRANSFER_JOB,	/**< job description in receiver language */
	EDG_WLL_TRANSFER_RESULT,	/**< result of the attempt */
	EDG_WLL_TRANSFER_REASON,	/**< detailed description of transfer, especially reason of failure */
	EDG_WLL_TRANSFER_DEST_JOBID,	/**< destination internal jobid */
	EDG_WLL_ACCEPTED_FROM,	/**< where was the job received from */
	EDG_WLL_ACCEPTED_FROM_HOST,	/**< sending component hostname */
	EDG_WLL_ACCEPTED_FROM_INSTANCE,	/**< sending component instance */
	EDG_WLL_ACCEPTED_LOCAL_JOBID,	/**< new jobId (Condor, Globus ...) assigned by the receiving component */
	EDG_WLL_REFUSED_FROM,	/**< where was the job received from */
	EDG_WLL_REFUSED_FROM_HOST,	/**< sending component hostname */
	EDG_WLL_REFUSED_FROM_INSTANCE,	/**< sending component instance */
	EDG_WLL_REFUSED_REASON,	/**< reason of refusal */
	EDG_WLL_ENQUEUED_QUEUE,	/**< destination queue */
	EDG_WLL_ENQUEUED_JOB,	/**< job description in receiver language */
	EDG_WLL_ENQUEUED_RESULT,	/**< result of the attempt */
	EDG_WLL_ENQUEUED_REASON,	/**< detailed description of transfer, especially reason of failure */
	EDG_WLL_DEQUEUED_QUEUE,	/**< queue name */
	EDG_WLL_DEQUEUED_LOCAL_JOBID,	/**< new jobId assigned by the receiving component */
	EDG_WLL_HELPERCALL_HELPER_NAME,	/**< name of the called component */
	EDG_WLL_HELPERCALL_HELPER_PARAMS,	/**< parameters of the call */
	EDG_WLL_HELPERCALL_SRC_ROLE,	/**< whether the logging component is called or calling one */
	EDG_WLL_HELPERRETURN_HELPER_NAME,	/**< name of the called component */
	EDG_WLL_HELPERRETURN_RETVAL,	/**< returned data */
	EDG_WLL_HELPERRETURN_SRC_ROLE,	/**< whether the logging component is called or calling one */
	EDG_WLL_RUNNING_NODE,	/**< worker node where the executable is run */
	EDG_WLL_RESUBMISSION_RESULT,	/**< result code */
	EDG_WLL_RESUBMISSION_REASON,	/**< reason for the decision */
	EDG_WLL_RESUBMISSION_TAG,	/**< value of the attribute on which the decision is based */
	EDG_WLL_DONE_STATUS_CODE,	/**< way of termination */
	EDG_WLL_DONE_REASON,	/**< reason for the change */
	EDG_WLL_DONE_EXIT_CODE,	/**< process exit code */
	EDG_WLL_CANCEL_STATUS_CODE,	/**< classification of the cancel */
	EDG_WLL_CANCEL_REASON,	/**< detailed description */
	EDG_WLL_ABORT_REASON,	/**< reason of abort */
	EDG_WLL_CLEAR_REASON,	/**< why the job was cleared */
	EDG_WLL_MATCH_DEST_ID,	/**< Id of the destination CE/queue */
	EDG_WLL_PENDING_REASON,	/**< why matching CE cannot be found */
	EDG_WLL_REGJOB_JDL,	/**< job description */
	EDG_WLL_REGJOB_NS,	/**< NetworkServer handling the job */
	EDG_WLL_REGJOB_PARENT,	/**< jobid of parent job */
	EDG_WLL_REGJOB_JOBTYPE,	/**< job type */
	EDG_WLL_REGJOB_NSUBJOBS,	/**< number of subjobs */
	EDG_WLL_REGJOB_SEED,	/**< seed for subjob id generation */
	EDG_WLL_CHKPT_TAG,	/**< checkpoint tag */
	EDG_WLL_CHKPT_CLASSAD,	/**< checkpoint value */
	EDG_WLL_LISTENER_SVC_NAME,	/**< port instance name */
	EDG_WLL_LISTENER_SVC_HOST,	/**< hostname */
	EDG_WLL_LISTENER_SVC_PORT,	/**< port number */
	EDG_WLL_CURDESCR_DESCR,	/**< description of current job transformation (output of helper) */
	EDG_WLL_USERTAG_NAME,	/**< tag name */
	EDG_WLL_USERTAG_VALUE,	/**< tag value */
	EDG_WLL_CHANGEACL_USER_ID,	/**< DN or VOMS parameter (in format VO:group) */
	EDG_WLL_CHANGEACL_USER_ID_TYPE,	/**< type of information given in user_id (DN or VOMS) */
	EDG_WLL_CHANGEACL_PERMISSION,	/**< ACL permission to change (currently only READ) */
	EDG_WLL_CHANGEACL_PERMISSION_TYPE,	/**< type of permission requested ('allow', 'deny') */
	EDG_WLL_CHANGEACL_OPERATION,	/**< operation requested to perform with ACL (add, remove) */
} edg_wll_KeyNameCode;

/**
 * \fn edg_wll_KeyNameCode edg_wll_StringToKeyName(char *name)
 * \param name		a string ULM key name (e.g. "DG.JOB.TRANSFER.DEST")
 * \return corresponding numeric code (edg_wll_KeyNameCode)
 * \brief convert a string ULM key name to the corresponding numeric code
 */

extern edg_wll_KeyNameCode edg_wll_StringToKeyName(char *);

/**
 * \fn char *edg_wll_KeyNameToString(edg_wll_KeyNameCode key)
 * \param key		a ULM key name numeric code (edg_wll_KeyNameCode)
 * \return corresponding string (e.g. "DG.JOB.TRANSFER.DEST")
 * \brief convert a ULM key name numeric code to the corresponding string
 */

extern char *edg_wll_KeyNameToString(edg_wll_KeyNameCode);


/**
 * Predefined _code_ types and
 * related StringTo_code and _code_ToString function prototypes
 */

/**
 * \enum enum edg_wll_Level
 * Level codes 
 */
enum edg_wll_Level {
	EDG_WLL_LEVEL_UNDEFINED,	/**< undefined code */ 
	EDG_WLL_LEVEL_EMERGENCY,	/**< emergency */ 
	EDG_WLL_LEVEL_ALERT,	/**< alert */ 
	EDG_WLL_LEVEL_ERROR,	/**< error */ 
	EDG_WLL_LEVEL_WARNING,	/**< warning */ 
	EDG_WLL_LEVEL_AUTH,	/**< authentication */ 
	EDG_WLL_LEVEL_SECURITY,	/**< security */ 
	EDG_WLL_LEVEL_USAGE,	/**< usage */ 
	EDG_WLL_LEVEL_SYSTEM,	/**< system */ 
	EDG_WLL_LEVEL_IMPORTANT,	/**< important */ 
	EDG_WLL_LEVEL_DEBUG,	/**< debug */ 
};

/**
 * \fn enum edg_wll_Level edg_wll_StringToLevel(char *name);
 * \param name		a string representing Level code (e.g. "ALERT")
 * \return corresponding numeric code (enum edg_wll_Level)
 * \brief converts a string Level code to corresponding numeric code
 */
extern enum edg_wll_Level edg_wll_StringToLevel(char *name);

/**
 * \fn char *edg_wll_LevelToString(enum edg_wll_Level code);
 * \param code		a Level numeric code (enum edg_wll_Level)
 * \return corresponding string (e.g. "ALERT")
 * \brief converts a Level numeric code to corresponding string
 */
extern char *edg_wll_LevelToString(enum edg_wll_Level code);


/**
 * \enum enum edg_wll_TransferResult
 * Result codes of the Transfer event
 */
enum edg_wll_TransferResult {
	EDG_WLL_TRANSFER_UNDEFINED,	/**< undefined code */ 
	EDG_WLL_TRANSFER_START,	/**< the sending component has started or is about to start the transfer */ 
	EDG_WLL_TRANSFER_OK,	/**< job was sent successfully */ 
	EDG_WLL_TRANSFER_REFUSED,	/**< job was refused by the other component */ 
	EDG_WLL_TRANSFER_FAIL,	/**< transfer failed for other reason than explicit refusal (eg. network timeout) */ 
};

/**
 * \fn enum edg_wll_TransferResult edg_wll_StringToTransferResult(char *name);
 * \param name		a string representing Transfer Result code (e.g. "OK")
 * \return corresponding numeric code (enum edg_wll_TransferResult)
 * \brief converts a string Transfer Result code to corresponding numeric code
 */
extern enum edg_wll_TransferResult edg_wll_StringToTransferResult(char *name);

/**
 * \fn char *edg_wll_TransferResultToString(enum edg_wll_TransferResult code);
 * \param code		a Transfer Result numeric code (enum edg_wll_TransferResult)
 * \return corresponding string (e.g. "OK")
 * \brief converts a Transfer Result numeric code to corresponding string
 */
extern char *edg_wll_TransferResultToString(enum edg_wll_TransferResult code);


/**
 * \enum enum edg_wll_EnQueuedResult
 * Result codes of the EnQueued event
 */
enum edg_wll_EnQueuedResult {
	EDG_WLL_ENQUEUED_UNDEFINED,	/**< undefined code */ 
	EDG_WLL_ENQUEUED_START,	/**< the sending component has started or is about to start the transfer */ 
	EDG_WLL_ENQUEUED_OK,	/**< job was sent successfully */ 
	EDG_WLL_ENQUEUED_REFUSED,	/**< job was refused by the other component */ 
	EDG_WLL_ENQUEUED_FAIL,	/**< transfer failed for other reason than explicit refusal (eg. network timeout) */ 
};

/**
 * \fn enum edg_wll_EnQueuedResult edg_wll_StringToEnQueuedResult(char *name);
 * \param name		a string representing EnQueued Result code (e.g. "OK")
 * \return corresponding numeric code (enum edg_wll_EnQueuedResult)
 * \brief converts a string EnQueued Result code to corresponding numeric code
 */
extern enum edg_wll_EnQueuedResult edg_wll_StringToEnQueuedResult(char *name);

/**
 * \fn char *edg_wll_EnQueuedResultToString(enum edg_wll_EnQueuedResult code);
 * \param code		a EnQueued Result numeric code (enum edg_wll_EnQueuedResult)
 * \return corresponding string (e.g. "OK")
 * \brief converts a EnQueued Result numeric code to corresponding string
 */
extern char *edg_wll_EnQueuedResultToString(enum edg_wll_EnQueuedResult code);


/**
 * \enum enum edg_wll_HelperCallSrc_role
 * Src_role codes of the HelperCall event
 */
enum edg_wll_HelperCallSrc_role {
	EDG_WLL_HELPERCALL_UNDEFINED,	/**< undefined code */ 
	EDG_WLL_HELPERCALL_CALLING,	/**< the logging component is caller */ 
	EDG_WLL_HELPERCALL_CALLED,	/**< the logging component is callee */ 
};

/**
 * \fn enum edg_wll_HelperCallSrc_role edg_wll_StringToHelperCallSrc_role(char *name);
 * \param name		a string representing HelperCall Src_role code (e.g. "CALLED")
 * \return corresponding numeric code (enum edg_wll_HelperCallSrc_role)
 * \brief converts a string HelperCall Src_role code to corresponding numeric code
 */
extern enum edg_wll_HelperCallSrc_role edg_wll_StringToHelperCallSrc_role(char *name);

/**
 * \fn char *edg_wll_HelperCallSrc_roleToString(enum edg_wll_HelperCallSrc_role code);
 * \param code		a HelperCall Src_role numeric code (enum edg_wll_HelperCallSrc_role)
 * \return corresponding string (e.g. "CALLED")
 * \brief converts a HelperCall Src_role numeric code to corresponding string
 */
extern char *edg_wll_HelperCallSrc_roleToString(enum edg_wll_HelperCallSrc_role code);


/**
 * \enum enum edg_wll_HelperReturnSrc_role
 * Src_role codes of the HelperReturn event
 */
enum edg_wll_HelperReturnSrc_role {
	EDG_WLL_HELPERRETURN_UNDEFINED,	/**< undefined code */ 
	EDG_WLL_HELPERRETURN_CALLING,	/**< the logging component is caller */ 
	EDG_WLL_HELPERRETURN_CALLED,	/**< the logging component is callee */ 
};

/**
 * \fn enum edg_wll_HelperReturnSrc_role edg_wll_StringToHelperReturnSrc_role(char *name);
 * \param name		a string representing HelperReturn Src_role code (e.g. "CALLED")
 * \return corresponding numeric code (enum edg_wll_HelperReturnSrc_role)
 * \brief converts a string HelperReturn Src_role code to corresponding numeric code
 */
extern enum edg_wll_HelperReturnSrc_role edg_wll_StringToHelperReturnSrc_role(char *name);

/**
 * \fn char *edg_wll_HelperReturnSrc_roleToString(enum edg_wll_HelperReturnSrc_role code);
 * \param code		a HelperReturn Src_role numeric code (enum edg_wll_HelperReturnSrc_role)
 * \return corresponding string (e.g. "CALLED")
 * \brief converts a HelperReturn Src_role numeric code to corresponding string
 */
extern char *edg_wll_HelperReturnSrc_roleToString(enum edg_wll_HelperReturnSrc_role code);


/**
 * \enum enum edg_wll_ResubmissionResult
 * Result codes of the Resubmission event
 */
enum edg_wll_ResubmissionResult {
	EDG_WLL_RESUBMISSION_UNDEFINED,	/**< undefined code */ 
	EDG_WLL_RESUBMISSION_WILLRESUB,	/**< will be resubmitted */ 
	EDG_WLL_RESUBMISSION_WONTRESUB,	/**< will not be resubmitted */ 
};

/**
 * \fn enum edg_wll_ResubmissionResult edg_wll_StringToResubmissionResult(char *name);
 * \param name		a string representing Resubmission Result code (e.g. "WONTRESUB")
 * \return corresponding numeric code (enum edg_wll_ResubmissionResult)
 * \brief converts a string Resubmission Result code to corresponding numeric code
 */
extern enum edg_wll_ResubmissionResult edg_wll_StringToResubmissionResult(char *name);

/**
 * \fn char *edg_wll_ResubmissionResultToString(enum edg_wll_ResubmissionResult code);
 * \param code		a Resubmission Result numeric code (enum edg_wll_ResubmissionResult)
 * \return corresponding string (e.g. "WONTRESUB")
 * \brief converts a Resubmission Result numeric code to corresponding string
 */
extern char *edg_wll_ResubmissionResultToString(enum edg_wll_ResubmissionResult code);


/**
 * \enum enum edg_wll_DoneStatus_code
 * Status_code codes of the Done event
 */
enum edg_wll_DoneStatus_code {
	EDG_WLL_DONE_UNDEFINED,	/**< undefined code */ 
	EDG_WLL_DONE_OK,	/**< terminated by itself */ 
	EDG_WLL_DONE_FAILED,	/**< disappeared from LRMS */ 
	EDG_WLL_DONE_CANCELLED,	/**< cancelled by user request */ 
};

/**
 * \fn enum edg_wll_DoneStatus_code edg_wll_StringToDoneStatus_code(char *name);
 * \param name		a string representing Done Status_code code (e.g. "FAILED")
 * \return corresponding numeric code (enum edg_wll_DoneStatus_code)
 * \brief converts a string Done Status_code code to corresponding numeric code
 */
extern enum edg_wll_DoneStatus_code edg_wll_StringToDoneStatus_code(char *name);

/**
 * \fn char *edg_wll_DoneStatus_codeToString(enum edg_wll_DoneStatus_code code);
 * \param code		a Done Status_code numeric code (enum edg_wll_DoneStatus_code)
 * \return corresponding string (e.g. "FAILED")
 * \brief converts a Done Status_code numeric code to corresponding string
 */
extern char *edg_wll_DoneStatus_codeToString(enum edg_wll_DoneStatus_code code);


/**
 * \enum enum edg_wll_CancelStatus_code
 * Status_code codes of the Cancel event
 */
enum edg_wll_CancelStatus_code {
	EDG_WLL_CANCEL_UNDEFINED,	/**< undefined code */ 
	EDG_WLL_CANCEL_REQ,	/**< request acknowledged */ 
	EDG_WLL_CANCEL_REFUSE,	/**< request declined by this component */ 
	EDG_WLL_CANCEL_DONE,	/**< request completed by whole WMS */ 
	EDG_WLL_CANCEL_ABORT,	/**< request refused by whole WMS */ 
};

/**
 * \fn enum edg_wll_CancelStatus_code edg_wll_StringToCancelStatus_code(char *name);
 * \param name		a string representing Cancel Status_code code (e.g. "REFUSE")
 * \return corresponding numeric code (enum edg_wll_CancelStatus_code)
 * \brief converts a string Cancel Status_code code to corresponding numeric code
 */
extern enum edg_wll_CancelStatus_code edg_wll_StringToCancelStatus_code(char *name);

/**
 * \fn char *edg_wll_CancelStatus_codeToString(enum edg_wll_CancelStatus_code code);
 * \param code		a Cancel Status_code numeric code (enum edg_wll_CancelStatus_code)
 * \return corresponding string (e.g. "REFUSE")
 * \brief converts a Cancel Status_code numeric code to corresponding string
 */
extern char *edg_wll_CancelStatus_codeToString(enum edg_wll_CancelStatus_code code);


/**
 * \enum enum edg_wll_ClearReason
 * Reason codes of the Clear event
 */
enum edg_wll_ClearReason {
	EDG_WLL_CLEAR_UNDEFINED,	/**< undefined code */ 
	EDG_WLL_CLEAR_USER,	/**< user retrieved output sandbox */ 
	EDG_WLL_CLEAR_TIMEOUT,	/**< timed out, resource purge forced */ 
	EDG_WLL_CLEAR_NOOUTPUT,	/**< no output was generated */ 
};

/**
 * \fn enum edg_wll_ClearReason edg_wll_StringToClearReason(char *name);
 * \param name		a string representing Clear Reason code (e.g. "TIMEOUT")
 * \return corresponding numeric code (enum edg_wll_ClearReason)
 * \brief converts a string Clear Reason code to corresponding numeric code
 */
extern enum edg_wll_ClearReason edg_wll_StringToClearReason(char *name);

/**
 * \fn char *edg_wll_ClearReasonToString(enum edg_wll_ClearReason code);
 * \param code		a Clear Reason numeric code (enum edg_wll_ClearReason)
 * \return corresponding string (e.g. "TIMEOUT")
 * \brief converts a Clear Reason numeric code to corresponding string
 */
extern char *edg_wll_ClearReasonToString(enum edg_wll_ClearReason code);


/**
 * \enum enum edg_wll_RegJobJobtype
 * Jobtype codes of the RegJob event
 */
enum edg_wll_RegJobJobtype {
	EDG_WLL_REGJOB_UNDEFINED,	/**< undefined code */ 
	EDG_WLL_REGJOB_SIMPLE,	/**< simple job */ 
	EDG_WLL_REGJOB_DAG,	/**< dag (containing static set of subjobs) */ 
	EDG_WLL_REGJOB_PARTITIONABLE,	/**< partitionable (may become partitioned) */ 
	EDG_WLL_REGJOB_PARTITIONED,	/**< partitioned (dynamically created dag) */ 
};

/**
 * \fn enum edg_wll_RegJobJobtype edg_wll_StringToRegJobJobtype(char *name);
 * \param name		a string representing RegJob Jobtype code (e.g. "DAG")
 * \return corresponding numeric code (enum edg_wll_RegJobJobtype)
 * \brief converts a string RegJob Jobtype code to corresponding numeric code
 */
extern enum edg_wll_RegJobJobtype edg_wll_StringToRegJobJobtype(char *name);

/**
 * \fn char *edg_wll_RegJobJobtypeToString(enum edg_wll_RegJobJobtype code);
 * \param code		a RegJob Jobtype numeric code (enum edg_wll_RegJobJobtype)
 * \return corresponding string (e.g. "DAG")
 * \brief converts a RegJob Jobtype numeric code to corresponding string
 */
extern char *edg_wll_RegJobJobtypeToString(enum edg_wll_RegJobJobtype code);



/**
 * common fields of all event types: 
 */

#define _EDG_WLL_EVENT_COMMON \
	edg_wll_EventCode		type;\
	struct timeval		timestamp;	\
	char *		host;	\
	int		level;	\
	int		priority;	\
	edg_wlc_JobId		jobId;	\
	char *		seqcode;	\
	char *		user;	\
	edg_wll_Source		source;	\
	char *		src_instance;	\


typedef struct _edg_wll_AnyEvent {
_EDG_WLL_EVENT_COMMON
} edg_wll_AnyEvent;


/**
 *  Event types specific structures: 
 */

/**
* \typedef edg_wll_TransferEvent
* structure definition for Transfer event 
*/
typedef struct _edg_wll_TransferEvent {
_EDG_WLL_EVENT_COMMON
	edg_wll_Source	destination;	/**< destination where the job is being transfered to */
	char *	dest_host;	/**< destination hostname */
	char *	dest_instance;	/**< destination instance */
	char *	job;	/**< job description in receiver language */
	enum edg_wll_TransferResult	result;	/**< result of the attempt */
	char *	reason;	/**< detailed description of transfer, especially reason of failure */
	char *	dest_jobid;	/**< destination internal jobid */
} edg_wll_TransferEvent;

/**
* \typedef edg_wll_AcceptedEvent
* structure definition for Accepted event 
*/
typedef struct _edg_wll_AcceptedEvent {
_EDG_WLL_EVENT_COMMON
	edg_wll_Source	from;	/**< where was the job received from */
	char *	from_host;	/**< sending component hostname */
	char *	from_instance;	/**< sending component instance */
	char *	local_jobid;	/**< new jobId (Condor, Globus ...) assigned by the receiving component */
} edg_wll_AcceptedEvent;

/**
* \typedef edg_wll_RefusedEvent
* structure definition for Refused event 
*/
typedef struct _edg_wll_RefusedEvent {
_EDG_WLL_EVENT_COMMON
	edg_wll_Source	from;	/**< where was the job received from */
	char *	from_host;	/**< sending component hostname */
	char *	from_instance;	/**< sending component instance */
	char *	reason;	/**< reason of refusal */
} edg_wll_RefusedEvent;

/**
* \typedef edg_wll_EnQueuedEvent
* structure definition for EnQueued event 
*/
typedef struct _edg_wll_EnQueuedEvent {
_EDG_WLL_EVENT_COMMON
	char *	queue;	/**< destination queue */
	char *	job;	/**< job description in receiver language */
	enum edg_wll_EnQueuedResult	result;	/**< result of the attempt */
	char *	reason;	/**< detailed description of transfer, especially reason of failure */
} edg_wll_EnQueuedEvent;

/**
* \typedef edg_wll_DeQueuedEvent
* structure definition for DeQueued event 
*/
typedef struct _edg_wll_DeQueuedEvent {
_EDG_WLL_EVENT_COMMON
	char *	queue;	/**< queue name */
	char *	local_jobid;	/**< new jobId assigned by the receiving component */
} edg_wll_DeQueuedEvent;

/**
* \typedef edg_wll_HelperCallEvent
* structure definition for HelperCall event 
*/
typedef struct _edg_wll_HelperCallEvent {
_EDG_WLL_EVENT_COMMON
	char *	helper_name;	/**< name of the called component */
	char *	helper_params;	/**< parameters of the call */
	enum edg_wll_HelperCallSrc_role	src_role;	/**< whether the logging component is called or calling one */
} edg_wll_HelperCallEvent;

/**
* \typedef edg_wll_HelperReturnEvent
* structure definition for HelperReturn event 
*/
typedef struct _edg_wll_HelperReturnEvent {
_EDG_WLL_EVENT_COMMON
	char *	helper_name;	/**< name of the called component */
	char *	retval;	/**< returned data */
	enum edg_wll_HelperReturnSrc_role	src_role;	/**< whether the logging component is called or calling one */
} edg_wll_HelperReturnEvent;

/**
* \typedef edg_wll_RunningEvent
* structure definition for Running event 
*/
typedef struct _edg_wll_RunningEvent {
_EDG_WLL_EVENT_COMMON
	char *	node;	/**< worker node where the executable is run */
} edg_wll_RunningEvent;

/**
* \typedef edg_wll_ResubmissionEvent
* structure definition for Resubmission event 
*/
typedef struct _edg_wll_ResubmissionEvent {
_EDG_WLL_EVENT_COMMON
	enum edg_wll_ResubmissionResult	result;	/**< result code */
	char *	reason;	/**< reason for the decision */
	char *	tag;	/**< value of the attribute on which the decision is based */
} edg_wll_ResubmissionEvent;

/**
* \typedef edg_wll_DoneEvent
* structure definition for Done event 
*/
typedef struct _edg_wll_DoneEvent {
_EDG_WLL_EVENT_COMMON
	enum edg_wll_DoneStatus_code	status_code;	/**< way of termination */
	char *	reason;	/**< reason for the change */
	int	exit_code;	/**< process exit code */
} edg_wll_DoneEvent;

/**
* \typedef edg_wll_CancelEvent
* structure definition for Cancel event 
*/
typedef struct _edg_wll_CancelEvent {
_EDG_WLL_EVENT_COMMON
	enum edg_wll_CancelStatus_code	status_code;	/**< classification of the cancel */
	char *	reason;	/**< detailed description */
} edg_wll_CancelEvent;

/**
* \typedef edg_wll_AbortEvent
* structure definition for Abort event 
*/
typedef struct _edg_wll_AbortEvent {
_EDG_WLL_EVENT_COMMON
	char *	reason;	/**< reason of abort */
} edg_wll_AbortEvent;

/**
* \typedef edg_wll_ClearEvent
* structure definition for Clear event 
*/
typedef struct _edg_wll_ClearEvent {
_EDG_WLL_EVENT_COMMON
	enum edg_wll_ClearReason	reason;	/**< why the job was cleared */
} edg_wll_ClearEvent;

/**
* \typedef edg_wll_PurgeEvent
* structure definition for Purge event 
*/
typedef struct _edg_wll_PurgeEvent {
_EDG_WLL_EVENT_COMMON
} edg_wll_PurgeEvent;

/**
* \typedef edg_wll_MatchEvent
* structure definition for Match event 
*/
typedef struct _edg_wll_MatchEvent {
_EDG_WLL_EVENT_COMMON
	char *	dest_id;	/**< Id of the destination CE/queue */
} edg_wll_MatchEvent;

/**
* \typedef edg_wll_PendingEvent
* structure definition for Pending event 
*/
typedef struct _edg_wll_PendingEvent {
_EDG_WLL_EVENT_COMMON
	char *	reason;	/**< why matching CE cannot be found */
} edg_wll_PendingEvent;

/**
* \typedef edg_wll_RegJobEvent
* structure definition for RegJob event 
*/
typedef struct _edg_wll_RegJobEvent {
_EDG_WLL_EVENT_COMMON
	char *	jdl;	/**< job description */
	char *	ns;	/**< NetworkServer handling the job */
	edg_wlc_JobId	parent;	/**< jobid of parent job */
	enum edg_wll_RegJobJobtype	jobtype;	/**< job type */
	int	nsubjobs;	/**< number of subjobs */
	char *	seed;	/**< seed for subjob id generation */
} edg_wll_RegJobEvent;

/**
* \typedef edg_wll_ChkptEvent
* structure definition for Chkpt event 
*/
typedef struct _edg_wll_ChkptEvent {
_EDG_WLL_EVENT_COMMON
	char *	tag;	/**< checkpoint tag */
	char *	classad;	/**< checkpoint value */
} edg_wll_ChkptEvent;

/**
* \typedef edg_wll_ListenerEvent
* structure definition for Listener event 
*/
typedef struct _edg_wll_ListenerEvent {
_EDG_WLL_EVENT_COMMON
	char *	svc_name;	/**< port instance name */
	char *	svc_host;	/**< hostname */
	uint16_t	svc_port;	/**< port number */
} edg_wll_ListenerEvent;

/**
* \typedef edg_wll_CurDescrEvent
* structure definition for CurDescr event 
*/
typedef struct _edg_wll_CurDescrEvent {
_EDG_WLL_EVENT_COMMON
	char *	descr;	/**< description of current job transformation (output of helper) */
} edg_wll_CurDescrEvent;

/**
* \typedef edg_wll_UserTagEvent
* structure definition for UserTag event 
*/
typedef struct _edg_wll_UserTagEvent {
_EDG_WLL_EVENT_COMMON
	char *	name;	/**< tag name */
	char *	value;	/**< tag value */
} edg_wll_UserTagEvent;

/**
* \typedef edg_wll_ChangeACLEvent
* structure definition for ChangeACL event 
*/
typedef struct _edg_wll_ChangeACLEvent {
_EDG_WLL_EVENT_COMMON
	char *	user_id;	/**< DN or VOMS parameter (in format VO:group) */
	int	user_id_type;	/**< type of information given in user_id (DN or VOMS) */
	int	permission;	/**< ACL permission to change (currently only READ) */
	int	permission_type;	/**< type of permission requested ('allow', 'deny') */
	int	operation;	/**< operation requested to perform with ACL (add, remove) */
} edg_wll_ChangeACLEvent;

#undef _EDG_WLL_EVENT_COMMON


/**
 * \union edg_wll_Event
 * \brief All event types union
 */
typedef union _edg_wll_Event {
        edg_wll_EventCode           type; /* it is probably never used */
        edg_wll_AnyEvent            any;
	edg_wll_TransferEvent	transfer;
	edg_wll_AcceptedEvent	accepted;
	edg_wll_RefusedEvent	refused;
	edg_wll_EnQueuedEvent	enQueued;
	edg_wll_DeQueuedEvent	deQueued;
	edg_wll_HelperCallEvent	helperCall;
	edg_wll_HelperReturnEvent	helperReturn;
	edg_wll_RunningEvent	running;
	edg_wll_ResubmissionEvent	resubmission;
	edg_wll_DoneEvent	done;
	edg_wll_CancelEvent	cancel;
	edg_wll_AbortEvent	abort;
	edg_wll_ClearEvent	clear;
	edg_wll_PurgeEvent	purge;
	edg_wll_MatchEvent	match;
	edg_wll_PendingEvent	pending;
	edg_wll_RegJobEvent	regJob;
	edg_wll_ChkptEvent	chkpt;
	edg_wll_ListenerEvent	listener;
	edg_wll_CurDescrEvent	curDescr;
	edg_wll_UserTagEvent	userTag;
	edg_wll_ChangeACLEvent	changeACL;
} edg_wll_Event;


/**
 * Initialise an event structure
 * \return pointer to initialised event structure
 */
extern edg_wll_Event *edg_wll_InitEvent(edg_wll_EventCode eventcode);


/**
 * Free the contents of event structure
 * \param IN event structure to be freed
 * \warning As event structures are likely to be allocated in arrays,
 *      the structure itself is not freed.
 *      Its the responsibility of the caller to call free(event)
 *      if appropriate.
 */
void edg_wll_FreeEvent(
	edg_wll_Event * event
);


#ifdef __cplusplus
}
#endif

#endif /* __EDG_WORKLOAD_LOGGING_CLIENT_EVENTS_H__ */
