/*
 * File: CommandFactory.h
 *
 * Copyright (c) 2002 EU DataGrid.
 * For license conditions see http://www.eu-datagrid.org/license.html
 */

#ifndef _CommandFactory_h_
#define _CommandFactory_h_

#include <boost/scoped_ptr.hpp>
#include "edg/workload/networkserver/commands/Command.h"

namespace edg {
namespace workload {
namespace networkserver {
namespace commands {

class CommandFactoryImpl;

/**
 * The command factory class template.
 * 
 * @version 1.0
 * @date September 16 2002
 * @author Salvatore Monforte, Marco Pappalardo.
 */
template<class implementation>
class CommandFactory 
{
 public:
   CommandFactory() : impl( new implementation ) {}
  
  /**
   * Creates a Command by name.
   * @return a pointer to new Command.
   */
  Command* create(const std::string& name) { return impl -> create( name ); }
  
private:
  boost::scoped_ptr<CommandFactoryImpl> impl;
};

} // namespace commands
} // namespace networkserver
} // namespace workload
} // namespace edg

#endif
