/*
 * File: NS2WMProxy.h
 * Author: Marco Pappalardo
 * Copyright (c) 2002 EU DataGrid.
 * For license conditions see http://www.eu-datagrid.org/license.html
 */

// $Id: NS2WMProxy.h,v 1.10 2004/11/24 17:04:15 dhsmith Exp $
#ifndef _NS2WMProxy_h_
#define _NS2WMProxy_h_

#include <string>
#include <boost/utility.hpp>
#include <boost/scoped_ptr.hpp>
#include "edg/workload/common/utilities/FileList.h"
#include "edg/workload/common/utilities/FileListLock.h"
#include "edg/workload/common/jobid/jobid.h"
#include "edg/workload/logging/client/producer.h"

namespace classad {
  class ClassAd;
}

namespace edg {
namespace workload {
namespace networkserver {
  namespace commands {
    class Command;
  }
namespace daemon {

namespace commands=edg::workload::networkserver::commands;

typedef boost::scoped_ptr< edg::workload::common::utilities::FileList<std::string> > FileListPtr;
typedef boost::scoped_ptr< edg::workload::common::utilities::FileListMutex> FileListMutexPtr;

class NS2WMProxy
{
  FileListPtr m_filelist;
  FileListMutexPtr m_mutex;

public:
  /*
   * Contructor.
   */
  NS2WMProxy();
  /*
   * Destructor.
   */
  virtual ~NS2WMProxy();
  
  /*
   * Initialized the NS to WM Proxy using the given FileList.
   * @param filename the pathname of FileList file.
   */
  void init(const std::string &filename);

  /*
   * Forwards a submit command to the WM through the FileList.
   * @param cmd the Command object related to the edg_job_submit as provided by NS.
   */
  virtual void submit(classad::ClassAd*);
  /*
   * Forwards a cancel command to the WM through the FileList.
   * @param cmd the Command object related to the edg_job_cancel as provided by NS.
   */  
  virtual void cancel(classad::ClassAd*);

private:
  /*
   * Stores the sequence code for Purger activities, in case of submit.
   * @param cmd the Command classad object related to the edg_job_submit as provided by NS.
   */
  void storeSequenceCode(classad::ClassAd*, edg_wll_Context*);
  /*
   * Stores the user id for Purger quota activities.
   * @param cmd the Command classad object related to the edg_job_submit as provided by NS.
   */
  void storeQuotaId(classad::ClassAd*);
  /*
   * Converts NS inner protocol-based info into conversation protocol common
   * to NS and WM.
   * @param the Command object related to the command as provided by NS.
   * @return the converted common-protocol-based string.
   */
  std::string convertProtocol(classad::ClassAd*); 

  /*
   * Used to detminte whether to specify async running event logging
   */
  int get_max_retry_count();
  int get_max_shallow_count();

};

} // daemon namespace closure
} // networkserver namespace closure
} // workload namespace closure
} // edg namespace closure

#endif

// Local Variables:
// mode: c++
// End:
