/*
 * File: NetworkServerReal.h
 * Author: Monforte Salvatore <Salvatore.Monforte@ct.infn.it>
 * Author: Marco Pappalardo <Marco.Pappalardo@ct.infn.it>
 * Copyright (c) 2001 EU DataGrid.
 * For license conditions see http://www.eu-datagrid.org/license.html
 */
 
#ifndef _NetworkServerReal_h_
#define _NetworkServerReal_h_


#include <boost/shared_ptr.hpp>
#include <boost/scoped_ptr.hpp>
#include <boost/thread/condition.hpp>
#include <boost/thread/mutex.hpp>

#include "edg/workload/networkserver/daemon/NetworkServerImpl.h"

namespace edg {
namespace workload {

namespace common {
namespace configuration {
  class Configuration;
  class NSConfiguration;
} 
namespace socket_pp {
class GSISocketServer;
}
}

namespace networkserver {
namespace daemon {

namespace socket_pp = workload::common::socket_pp;
namespace configuration = workload::common::configuration;

typedef boost::shared_ptr<socket_pp::GSISocketServer> GSISocketServerPtr;
 
class NetworkServerReal : public NetworkServerImpl  
{
  // non-copyable
  NetworkServerReal(const NetworkServerReal& nsi);
  NetworkServerReal& operator=(const NetworkServerReal& nsi);
  
public:
  NetworkServerReal();
  virtual ~NetworkServerReal();
  
  void run();
  void init();
  void shutdown();
  void drop_privileges();
  const configuration::NSConfiguration& configuration();

public:
  GSISocketServerPtr UIserver;
  const configuration::NSConfiguration* NSconf;

private:
  boost::condition  do_shutdown;
  boost::mutex     mtx_shutdown;
  std::auto_ptr<configuration::Configuration>    conf;
};

} // namespace daemon
} // namespace networkserver
} // namespace workload
} // namespace edg

#endif

