/******************************************************************************
 *
 *  filename  : DataLocationInterfaceSOAP.h
 *  author    : Heinz Stockinger     <Heinz.Stockinger@cern.ch>
 *
 *  Copyright (c) 2004 CERN and INFN on behalf of the EU DataGrid.
 *  For license conditions see LICENSE file or
 *  http://www.edg.org/license.html
 *
 *****************************************************************************/

// $Id: DataLocationInterfaceSOAP.h,v 1.2 2004/10/27 15:59:17 dhsmith Exp $

#ifndef EDG_WORKLOAD_PLANNING_BROKER_DLI_DATALOCATIONINTERFACESOAP_H
#define EDG_WORKLOAD_PLANNING_BROKER_DLI_DATALOCATIONINTERFACESOAP_H

#define M_SOAP ((struct soap *)m_soap)

#include <string>
#include <vector>

namespace edg {
namespace workload {
namespace planning {
namespace broker {
namespace dli {


class DataLocationInterfaceSOAP {
public:
  /**
   * Constructor for DataLocationInterface
   *
   * @param vo       Virtual Organisation 
   * @param endpoint SOAP endpoint (URL) of the remote catalogue
   *                 example: http://localhost:8085/
   */
  DataLocationInterfaceSOAP(std::string vo,
			    std::string endpoint);

  /**
   * List all replicas of a given InputDataType. A replica needs to contain
   * a valid SEId that is registered with the Information Service.
   *
   * @param inputDataType Defines one of the following InputDataTypes:
   *                      lfn   ... LogicalFileName
   *                      guid  ... GUID Global Unique Idenifier
   *                      lds   ... LogicalDataSet
   *                      query ... generic query to the catalogue
   *        Further InputDataTypes can be extended in the future but need to
   *        be understood by the remote catalogue. 
   *        Note that a catalogue does not need to implement all of the four
   *        InputDataTypes but is free to support any subset.
   * @param inputData     Actutual InputData variable
   *
   * @returns a vector of URLs that represent the locations of where
   *          the InputData is located. The URL can either be a full URL
   *          of the form    protocol://hostname/pathname
   *          or             hostname
   *          where hostname is a registered SEId.
   */
  virtual std::vector<std::string> listReplicas(std::string inputDataType,
						std::string inputData)
    throw (char *);

  /**
   * Destructor: clean up the SOAP environment
   */
  virtual ~DataLocationInterfaceSOAP();

private:
  DataLocationInterfaceSOAP(){};

  void *m_soap;     // pointer for gSOAP structure for message exchange
  std::string m_endpoint; // endpoint (URL) of the data catalogue to contact
};

typedef DataLocationInterfaceSOAP* create_t(const std::string&, 
					    const std::string&);
typedef void destroy_t(DataLocationInterfaceSOAP*);


} // namespace dli
} // namespace broker
} // namespace planning
} // namespace workload
} // namespace edg

#endif // EDG_WORKLOAD_PLANNING_BROKER_DLI_DATALOCATIONINTERFACESOAP_H
