/*
 * File: BrokerInfo.h
 * Author: Monforte Salvatore <Salvatore.Monforte@ct.infn.it>
 * Copyright (c) 2001 EU DataGrid.
 * For license conditions see http://www.eu-datagrid.org/license.html
 */

// $Id: brokerinfo.h,v 1.12 2005/10/03 11:47:08 dhsmith Exp $

#ifndef _EDG_WORKLOAD_PLANNING_BROKER_MATCHMAKING_BROKERINFO_H_
#define _EDG_WORKLOAD_PLANNING_BROKER_MATCHMAKING_BROKERINFO_H_

#include <boost/utility.hpp>
#include <boost/scoped_ptr.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/thread/tss.hpp>
#include "edg/workload/planning/broker/matchmaking/BrokerInfoData.h"

namespace classad
{
  class ClassAd;
}

namespace edg {
namespace workload {
namespace planning {
namespace broker {
namespace matchmaking {
namespace brokerinfo {

template <class implementation> class BrokerInfo;

class BrokerInfoImpl : boost::noncopyable
{
public:
  virtual void retrieveCloseSAsInfo (const BrokerInfoData::VO_name_type&, BrokerInfoData&, std::vector<std::string>* = 0) = 0;
  virtual void retrieveCloseSEsInfo (const BrokerInfoData::CEid_type&, BrokerInfoData&, std::vector<std::string>* = 0)   = 0;
  virtual void retrieveSEsInfo      (const classad::ClassAd& requestAd, BrokerInfoData&)  = 0;
  virtual void retrieveSFNsInfo     (const classad::ClassAd& requestAd, BrokerInfoData&)  = 0;
  virtual std::string findLCGApplicationDir(const std::string &ceid, BrokerInfoData&) = 0;
  virtual ~BrokerInfoImpl() {}
};

template<class implementation>
class BrokerInfo : boost::noncopyable
{ 
public:
 
  BrokerInfo() : m_data( new BrokerInfoData ), m_impl( new implementation ) {} 
  ~BrokerInfo() {}
  
  void retrieveCloseSAsInfo (const BrokerInfoData::VO_name_type& vo, std::vector<std::string>* attrs = 0)
  {
    m_impl -> retrieveCloseSAsInfo(vo, *this -> m_data, attrs);	  
  }
  void retrieveCloseSEsInfo (const BrokerInfoData::CEid_type& CEid, std::vector<std::string>* attrs = 0) 
  { 
    m_impl -> retrieveCloseSEsInfo(CEid, *this -> m_data, attrs); 
  }
  
  void retrieveSEsInfo (const classad::ClassAd& requestAd) 
  { 
    m_impl -> retrieveSEsInfo(requestAd, *this -> m_data); 
  }
  
  void retrieveSFNsInfo     (const classad::ClassAd& requestAd) 
  { 
    m_impl -> retrieveSFNsInfo(requestAd, *this -> m_data); 
  }

  std::string findLCGApplicationDir (const std::string &ceid)
  {
    return m_impl -> findLCGApplicationDir(ceid, *this -> m_data);
  }
  
  classad::ClassAd* asClassAd() const
  {
    return m_data -> asClassAd();		  	
  }
  const BrokerInfoData* operator->() const { return m_data.get(); }

  bool is_impl(BrokerInfoImpl *impl) const { return m_impl.get() == impl; }

  static boost::thread_specific_ptr<boost::shared_ptr<BrokerInfo<implementation> *> > tp_BIp;

private:
  boost::scoped_ptr<BrokerInfoData> m_data;
  boost::scoped_ptr<BrokerInfoImpl> m_impl;
};

}; // namespace brokerinfo
}; // namespace matchmaking
}; // namespace broker
}; // namespace planning
}; // workload
}; // edg

#endif
