// File: brokerinfoGlueImpl.h
// Author: Salvatore Monforte <Salvatore.Monforte@ct.infn.it>
// Copyright (c) 2002 EU DataGrid.
// For license conditions see http://www.eu-datagrid.org/license.html

// $Id: brokerinfoGlueImpl.h,v 1.16 2005/10/03 11:47:08 dhsmith Exp $

#ifndef _EDG_WORKLOAD_PLANNING_BROKER_MATCHMAKING_BROKERINFOGLUEIMPL_H
#define _EDG_WORKLOAD_PLANNING_BROKER_MATCHMAKING_BROKERINFOGLUEIMPL_H

#include <boost/utility.hpp>
#include <boost/scoped_ptr.hpp>
#include <boost/thread/mutex.hpp>

#include "edg/workload/common/ldif2classad/LDAPQuery.h"
#include "edg/workload/common/ldif2classad/LDAPSynchConnection.h"
#include "edg/workload/common/ldif2classad/LDAPForwardIterator.h"
#include "edg/workload/common/ldif2classad/exceptions.h"
#include "edg/workload/common/configuration/Configuration.h"
#include "edg/workload/common/configuration/NSConfiguration.h"
#include "edg/workload/common/utilities/ii_attr_utils.h"

#include "edg/workload/planning/broker/matchmaking/brokerinfo.h"

#include "edg/workload/planning/broker/rls/ReplicaServiceReal.h"
#include "edg/workload/planning/broker/dli/DataLocationInterfaceSOAP.h"

namespace edg {
namespace workload {
namespace planning {

namespace rls         = broker::rls;
namespace dli         = broker::dli;

namespace broker {
namespace matchmaking { 
namespace brokerinfo {
	
class brokerinfoGlueImpl : public BrokerInfoImpl
{
 public:
  brokerinfoGlueImpl();
  ~brokerinfoGlueImpl();
  void retrieveCloseSAsInfo(const BrokerInfoData::VO_name_type& VO, BrokerInfoData& bid, std::vector<std::string>* = 0);
  void retrieveCloseSEsInfo(const BrokerInfoData::CEid_type& CEId, BrokerInfoData& bid,std::vector<std::string>* = 0);
  void retrieveSEsInfo     (const classad::ClassAd& requestAd, BrokerInfoData& bid);
  void retrieveSFNsInfo    (const classad::ClassAd& requestAd, BrokerInfoData& bid);
  std::string findLCGApplicationDir( const std::string &ceid, BrokerInfoData& bid );

private:
  /**
   * Contact the Information Service (IS) and check if the given SE (SEid)
   * is registered in the IS. If yes, the SE is valid and 0 is returned, 
   * otherwise -1. 
   */
  int  validSE(std::string SEid, BrokerInfoData& bid);

  /**
   * Contact the Information Service (IS) and return the URL (endpoint) of the
   * server the provides the DataLocationInterface(DLI).
   * If no service is found, "" is returned.
   */
  std::string getDLIurl(std::string vo, BrokerInfoData& bid);

  /*
   * Check the configuration file of the Networkserver if DLI is used for
   * a certain VO. In case the DLI is used, 0 is returned, Otherwise -1.
   */
  int checkDliUsage(std::string vo);

  /*
   * Check the configuration file of the Networkserver if RLS is used for
   * a certain VO. In case the RLS is used, 0 is returned, Otherwise -1.
   */
  int checkRlsUsage(std::string vo);

  /*
   * retrieves the SE information for all CEs
   */
  void retrieveCloseSEsInfo(std::map<BrokerInfoData::CEid_type, BrokerInfoData::CloseSEInfo_map_type> &ce_to_ses,
                            BrokerInfoData& bid,std::vector<std::string>* = 0);

  /*
   * retrieves the SA information for all SEs
   */
  void retrieveCloseSAsInfo(const BrokerInfoData::VO_name_type& VO, std::vector<std::string>* = 0);

  /* to keep the SE information for all CEs */
  std::map<BrokerInfoData::CEid_type, BrokerInfoData::CloseSEInfo_map_type> m_ce_to_close_se_info;
  std::vector<std::string> m_close_se_extra_attrs;
  bool m_close_se_info_avail;

  BrokerInfoData::CloseSEInfo_map_type     m_se_to_sa_info;
  std::vector<std::string> m_sa_extra_attrs;
  BrokerInfoData::VO_name_type m_sa_vo;
  bool m_sa_info_avail;

  /* Mutex for plugin loader */
  static boost::mutex mtx;

  /*
   * Handles for the pluggins
   */
  static dli::create_t* createDli;
  static dli::destroy_t* destroyDli;
  static void *dliLibHandle;
  static rls::create_t* createRls;
  static rls::destroy_t* destroyRls;
  static void *rlsLibHandle;

  /* Connection to II */
  boost::scoped_ptr<common::ldif2classad::LDAPConnection> m_IIconnection;
};

} // namespace brokerinfo
} // namespace matchmaking
} // namespace broker
} // namespace planning
} // namespace workload
} // namespace edg

#endif
