/***************************************************************************
 **  filename  : ReplicaServiceReal.h
 **  authors   : Elisabetta Ronchieri <elisabetta.ronchieri@cnaf.infn.it>
 **  Copyright (c) 2002 CERN and INFN on behalf of the EU DataGrid.
 **  For license conditions see LICENSE file or
 **  http://www.edg.org/license.html
 ****************************************************************************/

// $Id: ReplicaServiceReal.h,v 1.9 2004/10/25 17:54:34 dhsmith Exp $

#ifndef EDG_WORKLOAD_PLANNING_BROKER_RLS_REPLICASERVICEREAL_H
#define EDG_WORKLOAD_PLANNING_BROKER_RLS_REPLICASERVICEREAL_H

#ifndef EDG_WORKLOAD_STRING
#define EDG_WORKLOAD_STRING
#include <string>
#endif

#ifndef EDG_WORKLOAD_VECTOR
#define EDG_WORKLOAD_VECTOR
#include <vector>
#endif

#include "edg/workload/common/utilities/triple.h"

namespace EdgReplicaManager {
  class ReplicaManagerImpl;
}

namespace utilities = edg::workload::common::utilities;

namespace replicamanager = EdgReplicaManager;

namespace edg {
namespace workload {
namespace planning {
namespace broker {
namespace rls {

typedef utilities::triple<std::string, float, float> access_cost_info_type;
typedef std::vector<access_cost_info_type> access_cost_info_container_type;
 
class ReplicaServiceReal
{
public:

  ReplicaServiceReal(const std::string& vo);

  virtual ~ReplicaServiceReal(void);

public:

  virtual std::vector<std::string> listReplica(const std::string& lfn);

  virtual access_cost_info_container_type getAccessCost(const std::vector<std::string>& lfns,
		                                 const std::vector<std::string>& ces,
						 const std::vector<std::string>& protocols);
  
private:

  replicamanager::ReplicaManagerImpl* m_rm;
}; 

// Types for plug-in
//
typedef ReplicaServiceReal* create_t(const std::string&);
typedef void destroy_t(ReplicaServiceReal*);

} // namespace rls
} // namespace broker
} // namespace planning
} // namespace workload
} // namespace edg

#endif // EDG_WORKLOAD_PLANNING_BROKER_RLS_REPLICASERVICEREAL_H

/*
  Local Variables:
  mode: c++
  End:
*/

