// File: HelperFactory.h
// Author: Francesco Giacomini <Francesco.Giacomini@cnaf.infn.it>
// Copyright (c) 2002 EU DataGrid.
// For license conditions see http://www.eu-datagrid.org/license.html

// $Id: HelperFactory.h,v 1.6 2003/09/02 14:32:18 giaco Exp $

#ifndef EDG_WORKLOAD_PLANNING_HELPER_HELPERFACTORY_H
#define EDG_WORKLOAD_PLANNING_HELPER_HELPERFACTORY_H

#ifndef EDG_WORKLOAD_X_VECTOR
#define EDG_WORKLOAD_X_VECTOR
#include <vector>
#endif
#ifndef EDG_WORKLOAD_X_BOOST_UTILITY_HPP
#define EDG_WORKLOAD_X_BOOST_UTILITY_HPP
#include <boost/utility.hpp>
#endif

namespace edg {
namespace workload {
namespace planning {
namespace helper {

class HelperImpl;

class HelperFactory: boost::noncopyable
{
  class Impl;

  Impl* m_impl;

  static HelperFactory* s_instance;

  HelperFactory();

  typedef HelperImpl product_type;
  typedef product_type* (*product_creator_type)();

public:
  static HelperFactory* instance();
  ~HelperFactory();

public:

  bool register_helper(std::string const& id, product_creator_type creator);
  bool unregister_helper(std::string const& id);
  product_type* create_helper(std::string const& id);
  std::vector<std::string> list() const;
};

}}}} // edg::workload::planning::helper

#endif
