// File: DispatcherFactory.h
// Author: Francesco Giacomini <Francesco.Giacomini@cnaf.infn.it>
// Copyright (c) 2002 EU DataGrid.
// For license conditions see http://www.eu-datagrid.org/license.html

// $Id: DispatcherFactory.h,v 1.1 2003/03/27 14:27:29 giaco Exp $

#ifndef EDG_WORKLOAD_PLANNING_MANAGER_DISPATCHERFACTORY_H
#define EDG_WORKLOAD_PLANNING_MANAGER_DISPATCHERFACTORY_H

#ifndef EDG_WORKLOAD_X_BOOST_UTILITY_HPP
#define EDG_WORKLOAD_X_BOOST_UTILITY_HPP
#include <boost/utility.hpp>
#endif
#ifndef EDG_WORKLOAD_X_BOOST_SCOPED_PTR_HPP
#define EDG_WORKLOAD_X_BOOST_SCOPED_PTR_HPP
#include <boost/scoped_ptr.hpp>
#endif
#ifndef EDG_WORKLOAD_PLANNING_MANAGER_DISPATCHING_UTILS_H
#include "edg/workload/planning/manager/dispatching_utils.h"
#endif

namespace edg {
namespace workload {
namespace planning {
namespace manager {

class DispatcherImpl;

class DispatcherFactory: boost::noncopyable
{
  class Impl;

  boost::scoped_ptr<Impl> m_impl;

  static DispatcherFactory* s_instance;

  DispatcherFactory();

  typedef DispatcherImpl product_type;
  typedef product_type* (*product_creator_type)();

public:
  static DispatcherFactory* instance();
  ~DispatcherFactory();

public:

  bool register_dispatcher(dispatcher_type const& id, product_creator_type creator);
  bool unregister_dispatcher(dispatcher_type const& id);
  product_type* create_dispatcher(dispatcher_type const& id);
};

}}}} // edg::workload::planning::manager

#endif

// Local Variables:
// mode: c++
// End:
