// File: dispatching_utils.h
// Author: Francesco Giacomini <Francesco.Giacomini@cnaf.infn.it>
// Copyright (c) 2002 EU DataGrid.
// For license conditions see http://www.eu-datagrid.org/license.html

// $Id: dispatching_utils.h,v 1.3 2003/09/15 10:13:28 giaco Exp $

#ifndef EDG_WORKLOAD_PLANNING_MANAGER_DISPATCHING_UTILS_H
#define EDG_WORKLOAD_PLANNING_MANAGER_DISPATCHING_UTILS_H

#ifndef EDG_WORKLOAD_X_STRING
#define EDG_WORKLOAD_X_STRING
#include <string>
#endif
#ifndef EDG_WORKLOAD_PLANNING_MANAGER_PIPEDEFS_H
#include "edg/workload/planning/manager/pipedefs.h"
#endif
#ifndef EDG_WORKLOAD_COMMON_TASK_H
#include "edg/workload/common/task/Task.h"
#endif

namespace edg {
namespace workload {
namespace planning {
namespace manager {

typedef std::string dispatcher_type;

enum PPResult {
  INVALID_REQUEST,
  NOTHING_TO_DO,
  FORWARD_TO_WM,
  QUIT
};

boost::mutex& submit_cancel_mutex();

PPResult preprocess_submit(ClassAdPtr command_ad);
PPResult preprocess_resubmit(ClassAdPtr command_ad);
PPResult preprocess_cancel(ClassAdPtr command_ad);
PPResult preprocess_quit(ClassAdPtr command_ad);
PPResult preprocess(ClassAdPtr command_ad);

class InvalidRequest: public std::exception
{
  class Impl;
  boost::shared_ptr<Impl> m_impl;
public:
  InvalidRequest(std::string const& s);
  ~InvalidRequest() throw();
  std::string str() const;
  char const* what() const throw();
};

bool process(
  std::string const& ad_str,
  PostProcessFunction f,
  common::task::PipeWriteEnd<pipe_value_type>& write_end
);



}}}} // edg::workload::planning::manager

#endif

// Local Variables:
// mode: c++
// End:
